-- example-start clist clist.e
class CLIST
inherit
   GTK_CONSTANTS
   VEGTK_MAIN
   VEGTK_CALLBACK_HANDLER
creation
   make
feature {ANY} -- Creation
   make is
   local
      vbox, hbox : GTK_BOX;
      button_add, button_clear, button_hide_show : GTK_BUTTON;
      titles : ARRAY [STRING];
      window : GTK_WINDOW;
   do
      titles := <<"Ingredients", "Amount" >>;
      visible := True;

      vegtk_init

      !!window.make (Gtk_window_toplevel);
      window.set_usize(300, 150);
      window.set_title("GtkCList Example");

      signal_connect (window, "destroy",$destroy);
      !GTK_VBOX!vbox.make (False, 5);
      vbox.set_border_width (5);
      window.add(vbox);
      vbox.show;

      -- Create the GtkCList. For this example we use 2 columns
      !!clist.make_with_titles( 2, titles);

      -- When a selection is made, we want to know about it. The callback
      -- used is selection_made, and its code can be found further down
      signal_connect(clist, "select_row",$selection_made);

      -- It isn't necessary to shadow the border, but it looks nice :)
      clist.set_shadow_type (Gtk_shadow_out);

      -- What however is important, is that we set the column widths as
      -- they will never be right otherwise. Note that the columns are
      -- numbered from 0 and up (to 1 in this case).

      clist.set_column_width (0, 150);

      -- Add the GTK_CLIST widget to the vertical box and show it.
      vbox.pack_start(clist, True, True, 0);
      clist.show;

      -- Create the buttons and add them to the window. See the button
      -- tutorial for more examples and comments on this.

      !GTK_HBOX!hbox.make(False, 0);
      vbox.pack_start (hbox, False, True, 0);
      hbox.show;

      !!button_add.make_with_label("Add List");
      !!button_clear.make_with_label("Clear List");
      !!button_hide_show.make_with_label("Hide/Show titles");

      hbox.pack_start (button_add, True, True, 0);
      hbox.pack_start (button_clear, True, True, 0);
      hbox.pack_start (button_hide_show, True, True, 0);

      -- Connect our callbacks to the three buttons
      signal_connect (button_add,"clicked", $button_add_clicked);
      signal_connect (button_clear, "clicked", $button_clear_clicked);
      signal_connect (button_hide_show, "clicked", $button_hide_show_clicked);

      button_add.show;
      button_clear.show;
      button_hide_show.show;

      -- The interface is completely set up so we show the window and
      -- enter the gtk_main loop.

      window.show;
      gtk_main;
end -- make
feature {NONE} -- Callbacks

   destroy is
   do
      gtk_main_quit;
   end

   button_add_clicked  is
      -- User clicked the "Add List" button.
   local
      dummy : INTEGER;
   do
      dummy := clist.append (<<"Milk","3 Oz">>);
      dummy := clist.append (<<"Water","6 l">>);
      dummy := clist.append (<<"Carrots", "2">>);
      dummy := clist.append (<<"Snakes",  "55">>);
   end -- button_add_clicked

   button_clear_clicked is
      -- User clicked the "Clear List" button.
   do
      -- Clear the list using clear. This is much faster than
      -- calling remove once for each row.

      clist.clear;
   end
   button_hide_show_clicked is
      -- The user clicked the "Hide/Show titles" button.
   do
       if visible then
          -- Hide the titles and set the flag to False
          clist.column_titles_hide;
          visible := False;
       else
          -- Show the titles and reset flag to True
          clist.column_titles_show;
          visible := True;
       end
   end -- button_hide_show_clicked

   text : STRING;

   selection_made (data: ANY; cb_data : VEGTK_CALLBACK_DATA) is
      -- If we come here, then the user has selected a row in the list.
   local
      row,column : INTEGER_REF;
      args : ARRAY[ANY];
   do
      args := cb_data.get_args;
      row ?= args.item(1);
      column ?= args.item(2);
      -- Get the text that is stored in the selected row and column
      -- which was clicked in. We will receive it as a pointer in the
      -- argument text.
      text := clist.get_text(row.item, column.item);

      -- Just prints some information about the selected row
      print ("You selected row ")
      print (row);
      print (". More specifically you clicked in column ");
      print (column);
      print (", and the text in this cell is ");
      print (text);
      print ("%N%N");
   end

feature {NONE} -- Implementation
   visible : BOOLEAN; -- Just a flag to remember the status.
                   --  True - currently visible.
   clist : GTK_CLIST -- our GTK_CLIST widget;
end -- class CLIST

-- example-end
