class LIST
inherit
   GTK_CONSTANTS
   VEGTK_MAIN
   VEGTK_CALLBACK_HANDLER
creation
   make
feature {ANY} -- Creation
   make is
   local
      window : GTK_WINDOW;
      separator : GTK_HSEPARATOR;
      vbox : GTK_VBOX;
      scrolled_window : GTK_SCROLLED_WINDOW;
      frame : GTK_FRAME;
      gtklist : GTK_LIST;
      list_item : GTK_LIST_ITEM;
      i : INTEGER;
      buffer : STRING;
      button : GTK_BUTTON;
      label : GTK_LABEL;
      alist : ARRAY[GTK_LIST_ITEM];
   do
      -- Initialize gtk+ (and subsequently gdk)
      vegtk_init

      -- Create a window to put all the widgets in
      -- connect gtk_main_quit() to the "destroy" event of
      -- the window to handle window manager close-window-events
      !!window.make (Gtk_window_toplevel);
      window.set_title ("GtkList Example");
      window.set_policy (True, True, False);
      signal_connect (window, "destroy",$destroy);


      -- Inside the window we need a box to arrange the widgets
      -- vertically
      !!vbox.make (False, 5);
      vbox.set_border_width (5);
      window.add (vbox);
      vbox.show;
      
      -- This is the scrolled window to put the GtkList widget inside
      !!scrolled_window.make (Void, Void);
      scrolled_window.set_usize (250, 150);
      vbox.add (scrolled_window);
      scrolled_window.show;
      
      -- Create the GtkList widget.
      -- Connect the sigh_print_selection() signal handler
      -- function to the "selection_changed" signal of the GtkList
      -- to print out the selected items each time the selection
      -- has changed
      !!gtklist.make;
      scrolled_window.add_with_viewport (gtklist);
      gtklist.show;
      signal_connect(gtklist, "selection_changed", $sigh_print_selection);
      
      -- We create a "Prison" to put a list item in ;)
      !!frame.make ("Prison");
      frame.set_usize (200, 50);
      frame.set_border_width (5);
      frame.set_shadow_type (Gtk_shadow_out);
      vbox.add (frame);
      frame.show;
      
      -- Connect the sigh_button_event() signal handler to the GtkList
      -- which will handle the "arresting" of list items
      signal_connect_with_data (gtklist, "button_release_event",
                                $sigh_button_event, frame);
      
      -- Create a separator
      !!separator.make;
      vbox.add (separator);
      separator.show;
      
      -- Finally create a button and connect its "clicked" signal
      -- to the procedure of the destruction the window
      !!button.make_with_label ("Close");
      vbox.add (button);
      button.show;
      signal_connect_with_data (button, "clicked", $quit_app, window);
      
      
      -- Now we create 5 list items, each having its own
      -- label and add them to the GtkList using procedure add()
      -- Also we query the text string from the label and
      -- associate it with the list_item_data_key for each list item
      from
         i :=1;
      until
         i >5
      loop
         !!buffer.make_from_string ("ListItemContainer with Label #");
         buffer.append_string (i.out);
         !!label.make (buffer);
         !!list_item.make;
         list_item.add (label);
         label.show;
         gtklist.add (list_item);
         list_item.show;
         i := i+1;
      end -- loop
      -- Here, we are creating another 5 labels, this time
      -- we use GTK_LIST_ITEM::make_with_label for the creation
      -- For adding of the list items we put them all into a 
      -- ARRAY [GTK_LIST_ITEM], and then add them by a single call to
      -- GTK_LIST::append_items().

      !!alist.make (1,5);
      from
         i := 6;
      until
         i >10
      loop
         !!buffer.make_from_string ("ListItemContainer with Label #");
         buffer.append_string (i.out);
         !!list_item.make_with_label (buffer);
         alist.put (list_item, i-5);
         list_item.show;
         i := i+1;
      end -- loop
      gtklist.append_items_array (alist);
      
      -- Finally we want to see the window, don't we? ;
      window.show;
      
      -- Fire up the main event loop of gtk
      gtk_main;
      
      -- We get here after gtk_main_quit() has been called which
      -- happens if the main window gets destroyed
   end -- make  

feature {NONE} -- Callbacks

   destroy is
   do
      gtk_main_quit;
   end

   quit_app (data : ANY) is
   local
      window : GTK_WINDOW;
   do
      window ?= data;
      window.destroy;
   end

   sigh_button_event (data : ANY; cb_data : VEGTK_CALLBACK_DATA) is
      -- This is the signal handler that got connected to button
      -- press/release events of the GtkList
   local
      gtklist : GTK_LIST
      event : GDK_EVENT_BUTTON;
      frame : GTK_FRAME;
      alist : ARRAY[GTK_LIST_ITEM];
      alist1 : ARRAY[GTK_WIDGET];
      new_prisoner : GTK_LIST_ITEM;
      i : INTEGER;
   do
      event ?= cb_data.get_event;
      -- We only do something if the third (rightmost mouse button
      -- was released
      if event.type.is_equal (Gdk_button_release) and event.button = 3 then
         gtklist ?= cb_data.get_widget;
         frame ?= data;
         -- Fetch the currently selected list item which
         -- will be our next prisoner ;)
          alist := gtklist.selection_array;
          if alist /= Void and then not alist.is_empty then
             new_prisoner := alist.item(1);
          end
          -- Look for already imprisoned list items, we
          -- will put them back into the list.
   
          alist1 := frame.children_array;
          if alist1 /= Void and then not alist.is_empty then
             from
                i := 1;
             until 
                i > alist1.upper
             loop
                alist1.item (i).reparent(gtklist)
                i := i+1;
             end
          end
          
          -- If we have a new prisoner, remove him from the
          -- GtkList and put him into the frame "Prison".
          -- We need to unselect the item first.
   
          if new_prisoner /= Void then
              gtklist.unselect_child (new_prisoner);
              new_prisoner.reparent (frame);
          end
      end
   end

   sigh_print_selection (data : ANY; cb_data : VEGTK_CALLBACK_DATA) is
      -- This is the signal handler that gets called if GtkList
      -- emits the "selection_changed" signal
   local
      gtklist : GTK_LIST;
      alist : ARRAY[GTK_LIST_ITEM];
      str : STRING;
      list_item : GTK_LIST_ITEM;
      label : GTK_LABEL;
      i : INTEGER;
   do 
      gtklist ?= cb_data.get_widget;
      -- Fetch the array of selected items of the GtkList
      alist := gtklist.selection_array;
      
      -- If there are no selected items there is nothing more
      -- to do than just telling the user so
   
      if alist = Void or else alist.is_empty then 
         print("Selection cleared%N");
      else
         -- Ok, we got a selection and so we print it
         print("The selection is a ");
         
         -- Get the list item from the doubly linked list
         -- and then query the data associated with list_item_data_key.
         -- We then just print it */
         from
            i := 1;
         until 
            i > alist.upper
         loop
            list_item := alist.item (i);
            label ?= list_item.child;
            str := label.get;
            print(str);
            print(" ");
            i := i+1;
         end
         print("%N");
      end -- if
   end
end -- class LIST
