-- example-start rangewidgets rangewidgets.e
class RANGEWIDGETS
inherit
   -- All constants we need are defined in the class below.
   -- eg gtk_window_toplevel etc.
   GTK_CONSTANTS
   -- This class contains common features like vegtk_init and so
   VEGTK_MAIN
   -- If we need to handle callbacks we must inherit from the below
   -- class
   VEGTK_CALLBACK_HANDLER
creation
   make
feature -- Callback handlers
   close_application is
   -- This callback quits the program
   do
       gtk_main_quit;
   end
   cb_pos_menu_select (data : ANY; cb_data : VEGTK_CALLBACK_DATA) is
   local
      pos : GTK_POSITION_TYPE;
   do
       pos ?= data;
       -- Set the value position on both scale widgets
       hscale.set_value_pos (pos);
       vscale.set_value_pos (pos);
   end

   cb_update_menu_select (data : ANY) is
   local
      policy : GTK_UPDATE_TYPE;
   do
       policy ?= data;
       -- Set the update policy for both scale widgets
       hscale.set_update_policy (policy);
       vscale.set_update_policy (policy);
   end

   cb_digits_scale (data : ANY; cb_data : VEGTK_CALLBACK_DATA) is
   local
      adj : GTK_ADJUSTMENT;
   do
       adj ?= cb_data.get_widget;
       -- Set the number of decimal places to which adj.value is rounded
       hscale.set_digits (adj.value.rounded);
       vscale.set_digits (adj.value.rounded);
   end

   cb_page_size (data : ANY; cb_data : VEGTK_CALLBACK_DATA) is
   local
      get,set : GTK_ADJUSTMENT;
--      signals : expanded GTK_SIGNAL_EXTERNALS;
   do
       get ?= cb_data.get_widget;
       set ?= data;
       -- Set the page size and page increment size of the sample
       -- adjustment to the value specified by the "Page Size" scale

       set.set_page_size (get.value);
       set.set_page_increment (get.value);
       -- Now emit the "changed" signal to reconfigure all the widgets that
       -- are attached to this adjustment
       set.changed;
--       signals.gtk_signal_emit_by_name(set.to_external,("changed").to_external);
   end

   cb_draw_value (data : ANY; cb_data : VEGTK_CALLBACK_DATA) is
   local
      button : GTK_CHECK_BUTTON;
   do
       button ?= cb_data.get_widget;
       -- Turn the value display on the scale widgets off or on depending
       --  on the state of the checkbutton
       hscale.set_draw_value (button.get_active);
       vscale.set_draw_value (button.get_active);
   end
feature -- Convenience procedures

   make_menu_item(name : STRING; callback : POINTER ;
                  data : ANY) : GTK_MENU_ITEM is
   do
       !!Result.make_with_label(name);
       signal_connect_with_data (Result, "activate",    callback, data);
       Result.show;
   end

   scale_set_default_values (scale : GTK_SCALE) is
   do
       scale.set_update_policy (Gtk_update_continuous);
       scale.set_digits (1);
       scale.set_value_pos (Gtk_pos_top);
       scale.set_draw_value (True);
   end

   create_range_controls is
      -- makes the sample window
   local
       window : GTK_WINDOW;
       box1, box2, box3 : GTK_BOX;
       button : GTK_BUTTON;
       check_button : GTK_CHECK_BUTTON;
       scrollbar : GTK_HSCROLLBAR;
       separator : GTK_HSEPARATOR;
       opt : GTK_OPTION_MENU
       menu : GTK_MENU;
       item : GTK_MENU_ITEM;
       label : GTK_LABEL;
       scale : GTK_HSCALE;
       adj1, adj2 : GTK_ADJUSTMENT;
   do
       -- Standard window-creating stuff
       !!window.make(Gtk_window_toplevel);
       signal_connect (window, "destroy",$close_application);
       window.set_title ("range controls");

       !GTK_VBOX!box1.make (False, 0);
       window.add (box1);
       box1.show;

       !GTK_HBOX!box2.make (False, 10);
       box2.set_border_width (10);
       box1.pack_start (box2, True, True, 0);
       box2.show;

       -- calue, lower, upper, step_increment, page_increment, page_size
       -- Note that the page_size value only makes a difference for
       -- scrollbar widgets, and the highest value you'll get is actually
       -- (upper - page_size).
       !!adj1.make(0.0, 0.0, 101.0, 0.1, 1.0, 1.0);

       !!vscale.make (adj1);
       scale_set_default_values (vscale);
       box2.pack_start (vscale, True, True, 0);
       vscale.show;

       !GTK_VBOX!box3.make (False, 10);
       box2.pack_start (box3, True, True, 0);
       box3.show;

       -- Reuse the same adjustment
       !!hscale.make(adj1);
       hscale.set_usize (200, 30);
       scale_set_default_values (hscale);
       box3.pack_start (hscale, True, True, 0);
       hscale.show;

       -- Reuse the same adjustment again
       !!scrollbar.make (adj1);
       -- Notice how this causes the scales to always be updated
       -- continuously when the scrollbar is moved
       scrollbar.set_update_policy (Gtk_update_continuous);
       box3.pack_start (scrollbar, True, True, 0);
       scrollbar.show;

       !GTK_HBOX!box2.make (False, 10);
       box2.set_border_width (10);
       box1.pack_start (box2, True, True, 0);
       box2.show;

       -- A checkbutton to control whether the value is displayed or not
       !!check_button.make_with_label("Display value on scale widgets");
       check_button.set_active (True);
       signal_connect (check_button, "toggled",$cb_draw_value);
       box2.pack_start ( check_button, True, True, 0);
       check_button.show;

       !GTK_HBOX!box2.make (False, 10);
       box2.set_border_width (10);

       -- An option menu to change the position of the value
       !!label.make ("Scale Value Position:");
       box2.pack_start (label, False, False, 0);
       label.show;

       !!opt.make;
       !!menu.make;

       item := make_menu_item ("Top",$cb_pos_menu_select,Gtk_pos_top);
       menu.append (item);

       item := make_menu_item ("Bottom",$cb_pos_menu_select,Gtk_pos_bottom);
       menu.append (item);

       item := make_menu_item ("Left", $cb_pos_menu_select,Gtk_pos_left);
       menu.append (item);

       item := make_menu_item ("Right", $cb_pos_menu_select,Gtk_pos_right);
       menu.append (item);

       opt.set_menu (menu);
       box2.pack_start (opt, True, True, 0);
       opt.show;

       box1.pack_start (box2, True, True, 0);
       box2.show;

       !GTK_HBOX!box2.make (False, 10);
       box2.set_border_width (10);

       -- Yet another option menu, this time for the update policy of the
       -- scale widgets
       !!label.make ("Scale Update Policy:");
       box2.pack_start (label, False, False, 0);
       label.show;

       !!opt.make;
       !!menu.make;

       item := make_menu_item ("Continuous",$cb_update_menu_select,
                               Gtk_update_continuous);
       menu.append (item);

       item := make_menu_item ("Discontinuous",$cb_update_menu_select,
                               Gtk_update_discontinuous);
       menu.append (item);

       item := make_menu_item ("Delayed",$cb_update_menu_select,
                              Gtk_update_delayed);
       menu.append (item);

       opt.set_menu (menu);
       box2.pack_start (opt, True, True, 0);
       opt.show;

       box1.pack_start (box2, True, True, 0);
       box2.show;

       !GTK_HBOX!box2.make (False, 10);
       box2.set_border_width (10);

       -- A GTK_HSCALE widget for adjusting the number of digits on the
       -- sample scales.
       !!label.make ("Scale Digits:");
       box2.pack_start (label, False, False, 0);
       label.show;

       !!adj2.make (1.0, 0.0, 5.0, 1.0, 1.0, 0.0);
       signal_connect (adj2, "value_changed",$cb_digits_scale);
       !!scale.make (adj2);
       scale.set_digits (0);
       box2.pack_start (scale, True, True, 0);
       scale.show;

       box1.pack_start (box2, True, True, 0);
       box2.show;

       !GTK_HBOX!box2.make (False, 10);
       box2.set_border_width (10);

       -- And, one last GTK_HSCALE widget for adjusting the page size of the
       -- scrollbar.
       !!label.make ("Scrollbar Page Size:");
       box2.pack_start (label, False, False, 0);
       label.show;

       !!adj2.make (1.0, 1.0, 101.0, 1.0, 1.0, 0.0);
       signal_connect_with_data (adj2, "value_changed",$cb_page_size, adj1);
       !!scale.make(adj2);
       scale.set_digits (0);
       box2.pack_start (scale, True, True, 0);
       scale.show;

       box1.pack_start (box2, True, True, 0);
       box2.show;

       !!separator.make;
       box1.pack_start (separator, False, True, 0);
       separator.show;

       !GTK_VBOX!box2.make (False, 10);
       box2.set_border_width (10);
       box1.pack_start (box2, False, True, 0);
       box2.show;

       !!button.make_with_label ("Quit");
       signal_connect (button, "clicked",$close_application)

       box2.pack_start (button, True, True, 0);
       button.set_flags (Gtk_can_default);
       button.grab_default;
       button.show;

       window.show;
   end -- create_range_controls

feature -- Creation
   make is
   do
      -- This is called in all GTK applications. Arguments are parsed
      -- from the command line and are returned to the application.
      vegtk_init;

      create_range_controls;

      gtk_main;
   end -- make
feature {NONE}
   hscale : GTK_HSCALE;
   vscale : GTK_HSCALE;
end -- class RANGEWIDGETS
-- example-end
