/**********************************************************************
* 
* Visual Eiffel run-time library
* 
* basicio.c
* 
* Copyright (c) 1995-1998, Object Tools. All rights reserved.
* 
*
*     Features declaration for class BASIC_IO
*
*     Functions:
*           _io_get_boolean
*           _io_get_character
*           _io_get_bool
*           _io_get_char
*           _io_get_double
*           _io_get_int
*           _io_get_real
*           _io_get_string
*           _io_put_char
*           _io_put_double
*           _io_put_int
*           _io_put_newline
*           _io_put_real
*           _io_put_string
**********************************************************************/

#include <windows.h>
#include <stdlib.h>
#include <basic_io.h>
#include "wb_io.h"

#define DEFAULT_DIGITS        8

/*---------------------------------------------------------------------------
*
*     winIOAttach
*
*     Registers WinIO class
*
*     Args:    Instance handle
*
*     Return:  On seccess, returns TRUE
*              On error, returns FALSE
*
*     Notes:   Registers window class and gets module file name
*/
IOEXPIMP EIF_BOOLEAN IOAttach (HINSTANCE hInstance, HANDLE *phSupHeap, DWORD param)
{
 initialize_wb_io ((HWND) param, 2212);
 return TRUE;
}

/*---------------------------------------------------------------------------
*
*     winIODetach
*
*     Destroys IO window if it requires
*
*     Args:    None
*
*     Return:  None
*
*     Notes:
*/
IOEXPIMP void IODetach (void)
{
 uninitialize_wb_io ();
}

/*---------------------------------------------------------------------------
*
*     _io_get_newline
*
*     Consume input until newline is found, then consume newline character
*
*     Args:    Points to a current
*
*     Return:  None
*
*     Notes:
*/

#pragma warning (disable:4100)
IOEXPIMP EIF_BOOLEAN   _io_get_newline (EIF_REFERENCE current)
{
 // The function does not supported in WorkBench input/output but we
 // supply it for compatibility with IO interface
 return TRUE;
}
#pragma warning (default:4100)

/*---------------------------------------------------------------------------
*
*     printError
*
*     Prints error message
*
*     Args:    Points to a null-terminated error message
*
*     Return:  None
*
*     Notes:
*/

IOEXPIMP EIF_BOOLEAN printError (char *msg)
{
 MessageBox (NULL, msg, NULL, MB_OK|MB_ICONSTOP|MB_SETFOREGROUND);
 return TRUE;
}

/*---------------------------------------------------------------------------
*
*     _io_get_boolean
*
*     Read a boolean from the standard input.
*
*     Args:    Points to a current
*
*     Return:  None
*
*     Notes:
*/

#pragma warning (disable:4100)
IOEXPIMP EIF_BOOLEAN   _io_get_boolean (EIF_REFERENCE current, EIF_BOOLEAN *dest_bool)
{
 // The function does not supported in WorkBench input/output but we
 // supply it for compatibility with IO interface
 return TRUE;
}
#pragma warning (default:4100)

/*---------------------------------------------------------------------------
*
*     _io_get_character
*
*     Read a character from the standard input.
*
*     Args:    Points to a current
*
*     Return:  None
*
*     Notes:   Empty input is interpreted as '%N'
*/

#pragma warning (disable:4100)
IOEXPIMP EIF_BOOLEAN   _io_get_character (EIF_REFERENCE current, EIF_CHARACTER *dest_ch)
{
 // The function does not supported in WorkBench input/output but we
 // supply it for compatibility with IO interface
 return TRUE;
}
#pragma warning (default:4100)

/*---------------------------------------------------------------------------
*
*     _io_get_string
*
*     Read a string from the standard input and assign it to 'last_string'
*
*     Args:    Points to a current
*
*     Return:  None
*
*     Notes:   Every call will allocate a new string
*              ! This string exist until next call to _io_get_string
*/

#pragma warning (disable:4100)
IOEXPIMP char * _io_get_string (EIF_REFERENCE current)
{
 return wb_io_get_str ();
}
#pragma warning (default:4100)

/*---------------------------------------------------------------------------
*
*     _io_get_int
*
*     Read an integer from the standard input and assign it to 'last_int'
*
*     Args:    Points to a current
*
*     Return:  None
*
*     Notes:
*/

#pragma warning (disable:4100)
IOEXPIMP EIF_BOOLEAN _io_get_int (EIF_REFERENCE current, EIF_INTEGER *dest_int)
{
 // The function does not supported in WorkBench input/output but we
 // supply it for compatibility with IO interface
 *dest_int = 0;
 return TRUE;
}
#pragma warning (default:4100)

/*---------------------------------------------------------------------------
*
*     _io_get_real
*
*     Read a real from the standard input and assign it to 'last_real'
*
*     Args:    Points to a current
*
*     Return:  None
*
*     Notes:
*/

#pragma warning (disable:4100)
IOEXPIMP EIF_BOOLEAN _io_get_real (EIF_REFERENCE current, EIF_REAL *dest_real)
{
 // The function does not supported in WorkBench input/output but we
 // supply it for compatibility with IO interface
 *dest_real = 0.0;
 return TRUE;
}
#pragma warning (default:4100)

/*---------------------------------------------------------------------------
*
*     _io_get_double
*
*     Read a double from the standard input and assign it to 'last_double'
*
*     Args:    Points to a current
*
*     Return:  None
*
*     Notes:
*/

#pragma warning (disable:4100)
IOEXPIMP EIF_BOOLEAN _io_get_double (EIF_REFERENCE current, EIF_DOUBLE *dest_double)
{
 // The function does not supported in WorkBench input/output but we
 // supply it for compatibility with IO interface
 *dest_double = 0.0;
 return TRUE;
}
#pragma warning (default:4100)

/*---------------------------------------------------------------------------
*
*     _io_get_bool
*
*     Read a boolean from the standard input and assign it to 'last_bool'
*
*     Args:    Points to a current
*
*     Return:  None
*
*     Notes:
*/

#pragma warning (disable:4100)
IOEXPIMP EIF_BOOLEAN _io_get_bool (EIF_REFERENCE current)
{
 // The function does not supported in WorkBench input/output but we
 // supply it for compatibility with IO interface
 return TRUE;
}
#pragma warning (default:4100)

/*---------------------------------------------------------------------------
*
*     _io_get_char
*
*     Read a character from the standard input and assign it to 'last_char'
*
*     Args:    Points to a current
*
*     Return:  None
*
*     Notes:   Empty input is interpreted as '%N'
*/

#pragma warning (disable:4100)
IOEXPIMP EIF_CHARACTER _io_get_char (EIF_REFERENCE current)
{
 // The function does not supported in WorkBench input/output but we
 // supply it for compatibility with IO interface
 return (EIF_CHARACTER) 0;
}

#pragma warning (default:4100)

/*---------------------------------------------------------------------------
*
*     _io_put_string
*
*     Output string to current output device
*
*     Args:    Points to a current
*
*     Return:  None
*
*     Notes:
*/

#pragma warning (disable:4100)
IOEXPIMP EIF_BOOLEAN _io_put_string (
   EIF_REFERENCE current, char *string, EIF_INTEGER length
)
{
 wb_io_put_str (string);
 return TRUE;
}
#pragma warning (default:4100)

/*---------------------------------------------------------------------------
*
*     _io_put_int
*
*     Output integer value to current output device
*
*     Args:    Points to a current
*
*     Return:  None
*
*     Notes:
*/

#pragma warning (disable:4100)
IOEXPIMP EIF_BOOLEAN _io_put_int (EIF_REFERENCE current, EIF_INTEGER integer)
{
 char buf [64];
 _ltoa (integer, buf, 10);
 wb_io_put_str (buf);
 return TRUE;
}
#pragma warning (default:4100)

/*---------------------------------------------------------------------------
*
*     _io_put_real
*
*     Output real value to current output device
*
*     Args:    Points to a current
*
*     Return:  None
*
*     Notes:
*/

#pragma warning (disable:4100)
IOEXPIMP EIF_BOOLEAN _io_put_real (EIF_REFERENCE current, EIF_REAL real)
{
 char buf [64];
 _gcvt (real, DEFAULT_DIGITS, buf);
 wb_io_put_str (buf);
 return TRUE;
}
#pragma warning (default:4100)

/*---------------------------------------------------------------------------
*
*     _io_put_double
*
*     Output double value to current output device
*
*     Args:    Points to a current
*
*     Return:  None
*
*     Notes:
*/

#pragma warning (disable:4100)
IOEXPIMP EIF_BOOLEAN _io_put_double (EIF_REFERENCE current, EIF_DOUBLE dbl)
{
 char buf [64];
 _gcvt (dbl, DEFAULT_DIGITS, buf);
 wb_io_put_str (buf);
 return TRUE;
}
#pragma warning (default:4100)

/*---------------------------------------------------------------------------
*
*     _io_put_char
*
*     Output character value to current output device
*
*     Args:    Points to a current
*
*     Return:  None
*
*     Notes:
*/

#pragma warning (disable:4100)
IOEXPIMP EIF_BOOLEAN _io_put_char (EIF_REFERENCE current, EIF_INTEGER ch)
{
 char buf [2];
 buf [0] = (char)ch;
 buf [1] = '\0';
 wb_io_put_str (buf);
 return TRUE;
}
#pragma warning (default:4100)

/*---------------------------------------------------------------------------
*
*     _io_put_newline
*
*     Output newline character
*
*     Args:    Points to a current
*
*     Return:  None
*
*     Notes:
*/

#pragma warning (disable:4100)
IOEXPIMP EIF_BOOLEAN _io_put_newline (EIF_REFERENCE current)
{
 wb_io_put_str ("\n");
 return TRUE;
}
#pragma warning (default:4100)
