/**********************************************************************
* 
* Visual Eiffel run-time library
* 
* cecil.h
* 
* Copyright (c) 1995-2000, Object Tools. All rights reserved.
* 
*
*     Visual Eiffel Cecil library (C-Eiffel Call-In Library)
*     Function prototypes and specific definitions
*
*     Notes:   Cecil library has no support for routine calls with names
*              like 'eif_dyn_...' and for bits types yet
*
**********************************************************************/

#ifndef _EX_CECIL_H
#define _EX_CECIL_H

#ifdef __cplusplus
extern "C"
{
#endif				// __cplusplus

#ifdef EXPORT_ENTRY
#define _EXPFUNC                __declspec (dllexport)
#else
#define _EXPFUNC
#endif

//------------ "C" equivalents for Visual Eiffel basic types ----------------
  typedef unsigned short EIF_TYPE_ID;
  typedef unsigned char EIF_BOOLEAN;
  typedef char EIF_CHARACTER;
  typedef long EIF_INTEGER;
  typedef double EIF_REAL;
  typedef double EIF_DOUBLE;
  typedef char *EIF_POINTER;
  typedef void *EIF_RTN;
  typedef EIF_TYPE_ID EIF_OBJ_HEADER;
  typedef EIF_OBJ_HEADER *EIF_REFERENCE;

  typedef void (*EIF_PROC) (EIF_REFERENCE,...);
  typedef EIF_BOOLEAN (*EIF_FN_BOOL) (EIF_REFERENCE,...);
  typedef EIF_CHARACTER (*EIF_FN_CHAR) (EIF_REFERENCE,...);
  typedef EIF_INTEGER (*EIF_FN_INT) (EIF_REFERENCE,...);
  typedef EIF_REAL (*EIF_FN_REAL) (EIF_REFERENCE,...);
  typedef EIF_DOUBLE (*EIF_FN_DOUBLE) (EIF_REFERENCE,...);
  typedef EIF_POINTER (*EIF_FN_POINTER) (EIF_REFERENCE,...);
  typedef EIF_REFERENCE (*EIF_FN_REF) (EIF_REFERENCE,...);

/*---------------------------------------------------------------------------
*
*     eif_type_id
*
*     Refers to Visual Eiffel types
*
*     Args:    Class name (as null-terminated ASCII string)
*
*     Return:  On success, returns the type identificator (magic number)
*
*     Notes:   eif_type_id refers to Visual Eiffel types to get from
*              Visual Eiffel side some type identificator (handle)
*              corresponding to interesting type.
*/

  _EXPFUNC EIF_TYPE_ID eif_type_id (char *class_name);

/*---------------------------------------------------------------------------
*
*     eif_is_expanded_type
*
*
*
*     Args:    The type identificator (magic number)
*
*     Return:  Return TRUE if type identificator is type identificator
*              of expanded object
*
*/

  _EXPFUNC EIF_BOOLEAN eif_is_expanded_type (EIF_TYPE_ID type_id);

/*---------------------------------------------------------------------------
*
*     eif_type_id_by_reference
*
*     Refers to Visual Eiffel types
*
*     Args:    Handle of the Visual Eiffel object
*              The Visual Eiffel object must be reference type (not expanded).
*
*     Return:  On success, returns the type identificator (magic number)
*
*     Notes:   eif_type_id_by_reference refers to Visual Eiffel types
*              to get from Visual Eiffel side some type identificator (handle)
*              corresponding to interesting object
*/

  _EXPFUNC EIF_TYPE_ID eif_type_id_by_reference (EIF_REFERENCE obj);

/*---------------------------------------------------------------------------
*
*     eif_create
*
*     Creates an Visual Eiffel object
*
*     Args:    The type identificator (magic number)
*
*     Return:  On success, returns handle of the Visual Eiffel object
*                (not the pointer to the one)
*/

  _EXPFUNC EIF_REFERENCE eif_create (EIF_TYPE_ID type_id);

/*---------------------------------------------------------------------------
*
*     eif_access
*
*     Checks access to an Visual Eiffel object
*
*     Args:    Handle of the Visual Eiffel object
*
*     Return:  On success, returns the pointer to the Visual Eiffel object
*
*     Notes:   You can directly manipulate with object with help
*              of pointer returned from this function
*/

  _EXPFUNC char *eif_access (EIF_REFERENCE obj);
  _EXPFUNC char *eif_access_ec (EIF_REFERENCE obj, EIF_TYPE_ID type_id);

/*---------------------------------------------------------------------------
*
*     eif_adopt
*
*     Adopts the Visual Eiffel object
*
*     Args:    Handle of the Visual Eiffel object
*
*     Return:  None
*
*     Notes:   In some cases garbage collector (GC) can discard any
*              Visual Eiffel object and reclaim corresponding space.
*              To prevent such behavior of GC (and corresponding conflicts
*              with eif_access () returns pointer) you can use this procedure
*
*/

  _EXPFUNC void eif_adopt (EIF_REFERENCE obj);
  _EXPFUNC void eif_adopt_ec (EIF_REFERENCE obj, EIF_TYPE_ID type_id);

/*---------------------------------------------------------------------------
*
*     eif_wean
*
*     Releases adopted Visual Eiffel object
*
*     Args:    Handle of the Visual Eiffel object
*
*     Return:  None
*
*     Notes:   This function releases previously adopted (by eif_adopt ())
*              Visual Eiffel object
*/

  _EXPFUNC void eif_wean (EIF_REFERENCE obj);
  _EXPFUNC void eif_wean_ec (EIF_REFERENCE obj, EIF_TYPE_ID type_id);

/*---------------------------------------------------------------------------
*
*     eif_field_address
*
*     Field access from Visual Eiffel object
*
*     Args:    Points to an Visual Eiffel object
*              Attribute name (as null-terminated ASCII string)
*
*     Return:  On success, returns the pointer to the field address
*
*     Notes:   You can access any field from any Visual Eiffel object.
*              To do this you must get field address after that you can
*              directly manipulate with corresponding value (using type
*              casting)
*
*     Warning: Don't forget to adopt your object for safely working with
*              the one
*/

  _EXPFUNC char *eif_field_address (char *obj_ptr, char *some_attribute_name);
  _EXPFUNC char *eif_field_address_ec (char *obj_ptr, EIF_TYPE_ID type_id, char *some_attribute_name);

/*---------------------------------------------------------------------------
*
*     eif_field
*
*     Gets direct value vs. reference to the one
*
*     Args:    Routine name (as null-terminated ASCII string)
*              Class type identificator (class magic number)
*
*     Return:  On success, returns the pointer to the routine
*
*/

#define eif_field(obj,att_name,att_type) \
   *(att_type *) eif_field_address (eif_access (obj), att_name)

#define eif_field_ec(obj,type_id,att_name,att_type) \
   *(att_type *) eif_field_address_ec (eif_access_ec (obj, type_id), type_id, att_name)

  _EXPFUNC EIF_TYPE_ID eif_attribute_type_id (EIF_TYPE_ID type_id, char *attribute_name);

/*---------------------------------------------------------------------------
*
*     eif_routine
*
*     Gets pointer to the Visual Eiffel routine
*
*     Args:    Routine name (as null-terminated ASCII string)
*              Class type identificator (class magic number)
*
*     Return:  On success, returns the pointer to the routine
*
*/

  _EXPFUNC EIF_RTN eif_routine (char *routine_name, EIF_TYPE_ID type_id);
  _EXPFUNC EIF_RTN eif_dynamic_routine (EIF_REFERENCE obj, char *routine_name);

/*---------------------------------------------------------------------------
*
*     eif_release_routine
*
*     Releases pointer to the Visual Eiffel routine
*
*     Args:    pointer to the routine produced by 'eif_routine'
*
*     Return:
*
*     Notes:
*/
  _EXPFUNC void eif_release_routine (EIF_RTN p);

/*---------------------------------------------------------------------------
*
*     eif_proc
*
*     Gets address of Visual Eiffel procedure
*
*     Args:    Procedure name (as null-terminated ASCII string)
*              Class type identificator (class magic number)
*
*     Return:  On success, returns the address of procedure
*
*     Notes:   This call does not support dynamic binding
*/

#define eif_proc(routine_name,type_id) \
      ((EIF_PROC)(eif_routine(routine_name,type_id)))

/*---------------------------------------------------------------------------
*
*     eif_fn_bool
*
*     Gets address of Visual Eiffel routine, returning a result
*     of type EIF_BOOLEAN
*
*     Args:    Routine name (as null-terminated ASCII string)
*              Class type identificator (class magic number)
*
*     Return:  On success, returns the address of routine
*
*     Notes:   This call does not support dynamic binding
*/

#define eif_fn_bool(routine_name,type_id) \
      ((EIF_FN_BOOL)(eif_routine(routine_name,type_id)))

/*---------------------------------------------------------------------------
*
*     eif_fn_char
*
*     Gets address of Visual Eiffel routine, returning a result
*     of type EIF_CHARACTER
*
*     Args:    Routine name (as null-terminated ASCII string)
*              Class type identificator (class magic number)
*
*     Return:  On success, returns the address of routine
*
*     Notes:   This call does not support dynamic binding
*/

#define eif_fn_char(routine_name,type_id) \
      ((EIF_FN_CHAR)(eif_routine(routine_name,type_id)))

/*---------------------------------------------------------------------------
*
*     eif_fn_int
*
*     Gets address of Visual Eiffel routine, returning a result
*     of type EIF_INTEGER
*
*     Args:    Routine name (as null-terminated ASCII string)
*              Class type identificator (class magic number)
*
*     Return:  On success, returns the address of routine
*
*     Notes:   This call does not support dynamic binding
*/

#define eif_fn_int(routine_name,type_id) \
      ((EIF_FN_INT)(eif_routine(routine_name,type_id)))

/*---------------------------------------------------------------------------
*
*     eif_fn_real
*
*     Gets address of Visual Eiffel routine,
*        returning a result of type EIF_REAL
*
*     Args:    Routine name (as null-terminated ASCII string)
*              Class type identificator (class magic number)
*
*     Return:  On success, returns the address of routine
*
*     Notes:   This call does not support dynamic binding
*/

#define eif_fn_real(routine_name,type_id) \
      ((EIF_FN_REAL)(eif_routine(routine_name,type_id)))

/*---------------------------------------------------------------------------
*
*     eif_fn_double
*
*     Gets address of Visual Eiffel routine,
*        returning a result of type EIF_DOUBLE
*
*     Args:    Routine name (as null-terminated ASCII string)
*              Class type identificator (class magic number)
*
*     Return:  On success, returns the address of routine
*
*     Notes:   This call does not support dynamic binding
*/

#define eif_fn_double(routine_name,type_id) \
      ((EIF_FN_DOUBLE)(eif_routine(routine_name,type_id)))

/*---------------------------------------------------------------------------
*
*     eif_fn_pointer
*
*     Gets address of Visual Eiffel routine, returning a result
*     of type EIF_POINTER
*
*     Args:    Routine name (as null-terminated ASCII string)
*              Class type identificator (class magic number)
*
*     Return:  On success, returns the address of routine
*
*     Notes:   This call does not support dynamic binding
*/

#define eif_fn_pointer(routine_name,type_id) \
      ((EIF_FN_POINTER)(eif_routine(routine_name,type_id)))

/*---------------------------------------------------------------------------
*
*     eif_fn_ref
*
*     Gets address of Visual Eiffel routine, returning a result
*     of type EIF_REFERENCE
*
*     Args:    Routine name (as null-terminated ASCII string)
*              Class type identificator (class magic number)
*
*     Return:  On success, returns the address of routine
*
*     Notes:   This call does not support dynamic binding
*/

#define eif_fn_ref(routine_name,type_id) \
      ((EIF_FN_REF)(eif_routine(routine_name,type_id)))

#ifdef __cplusplus
}
#endif				// __cplusplus
#endif				// _EX_CECIL_H
