/*---------------------------------------------------------------------------
*
*     WINIO.C        Copyright (c) 1996 by SiG Computer
*
*     Workbench output
*
*     Version:       0.00
*
*     WinIO functions
*
*     Functions:
*           initialize_wb_io
*           uninitialize_wb_io
*           wb_io_get_str
*           wb_io_put_str
*
*     Notes:
*/

#include <windows.h>
#include <stdlib.h>
#include "wb_io.h"

//---------------------------------------------------------------------------
#define MAX_STRING 8192
#define CR '\x0d'
#define LF '\x0a'

//---------------------------------------------------------------------------
static CRITICAL_SECTION WbIOCriticalSection; // WbIO critical-section object
static HWND  hWbWnd = NULL;                  // Wb Application window handle
static DWORD IDM_OUTPUT_STRING_IS_READY;

//---------------------------------------------------------------------------
static struct _WB_IO_STRING;
static struct _WB_IO_STRING {
   char * s;
   struct _WB_IO_STRING* next;
};

typedef struct _WB_IO_STRING WB_IO_STRING;

//---------------------------------------------------------------------------
static WB_IO_STRING* first;
static WB_IO_STRING* last;
static char * last_string;
static char * buffer_string;
static char * buffer_tail;

//---------------------------------------------------------------------------
void initialize_wb_io (HWND _hWbWnd, DWORD CallBackCmd) {

   hWbWnd = _hWbWnd;
   IDM_OUTPUT_STRING_IS_READY = CallBackCmd;
   InitializeCriticalSection (&WbIOCriticalSection);
   init_list ();
   buffer_string = (char *) LocalAlloc (LMEM_FIXED, MAX_STRING);
   buffer_tail   = NULL;
}

//---------------------------------------------------------------------------
void uninitialize_wb_io (void) {

   destroy_list ();
   DeleteCriticalSection (&WbIOCriticalSection);
   LocalFree ((HLOCAL) buffer_string);
}

//---------------------------------------------------------------------------
char* wb_io_get_str (void) {
   char * result;
   EnterCriticalSection (&WbIOCriticalSection);
   result = get_string_from_list ();
   LeaveCriticalSection (&WbIOCriticalSection);
   return result;
}

//---------------------------------------------------------------------------
void wb_io_put_str (char * s) {

   if (s == NULL) return;

   append_to_buffer (s);
   put_string ();
}

/*---------------------------------------------------------------------------
*
* List of string
*
*/
void init_list (void) {
   first       = NULL;
   last        = NULL;
   last_string = NULL;
}

//---------------------------------------------------------------------------
void destroy_list (void) {

   while ( first != NULL ) {
      WB_IO_STRING* tmp = first;

      first = first->next;

      LocalFree ( (HLOCAL) tmp->s );
      LocalFree ( (HLOCAL) tmp );
   }
   if (last_string != NULL)
      LocalFree ( (HLOCAL) (last_string) );
}

//---------------------------------------------------------------------------
void put_string_to_list (char * s) {

   WB_IO_STRING* new_elem = (WB_IO_STRING*)
      LocalAlloc (LMEM_FIXED, sizeof(WB_IO_STRING)/sizeof(char) );
   char * new_string = (char*)
      LocalAlloc (LMEM_FIXED, strlen(s)+1 );
   memcpy (new_string, s, strlen(s)+1 );

   new_elem->s = new_string;
   new_elem->next = NULL;
   if (last == NULL) { // empty list
      first = new_elem;
      last  = new_elem;
   } else {           // nonempty list
      last->next = new_elem;
      last       = new_elem;
   }
}

//---------------------------------------------------------------------------
char * get_string_from_list (void) {

   if (first != NULL) { // nonempty list
      WB_IO_STRING* to_be_deleted = first;

      if (last_string != NULL)
         LocalFree ((HLOCAL) last_string);

      last_string = first->s;
      first = first->next;
      if(first == NULL) last = NULL; // say list is empty
      LocalFree ((HLOCAL) to_be_deleted);
      return last_string;
   }
   return NULL;
}

//---------------------------------------------------------------------------
void put_non_empty_string (char *s) {
   EnterCriticalSection (&WbIOCriticalSection);
   put_string_to_list (s);
   LeaveCriticalSection (&WbIOCriticalSection);
   PostMessage (hWbWnd, WM_COMMAND, (WPARAM)IDM_OUTPUT_STRING_IS_READY, 0 );
}

//---------------------------------------------------------------------------
void append_to_buffer (char * s) {
   if (buffer_tail==NULL) { // buffer is empty
      strcpy (buffer_string, s);
      buffer_tail = buffer_string + strlen(s);
   } else {
      strcpy (buffer_tail, s);
      buffer_tail = buffer_tail + strlen(s);
   }
   *buffer_tail = 0;
}

//---------------------------------------------------------------------------
BOOL is_empty_string (char* s) {
   if(s == NULL) return TRUE;
   if(*s == 0)   return TRUE;
   return FALSE;
}


//---------------------------------------------------------------------------
void put_string (void) {

   if (is_empty_string (buffer_string) == FALSE) {
      put_non_empty_string (buffer_string);
   }
   if (buffer_tail) {
      buffer_tail = NULL;
   }
}

