indexing

   title:       "Simple I/O facilities for all the demos",
                "available in cluster Examples\Containers";
   project:     "Visual Eiffel";
   date:        "$Date: 2005/02/14 11:56:17 $";
   revision:    "$Revision: 1.1 $";
   copyright:   "Copyright (C) 1996-2005 Object Tools Group";
   license:     "http://visual-eiffel.com/license";
   disclaimer:  "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions:"[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

class INPUT_OUTPUT

   inherit

      SIMPLE_DEMO_IO
         end


   feature -- Operations

      get_a_string (msg: STRING) is
            -- reads a string from the standard input and assigns it to
            -- 'the_string' and then parses, checks it and assigns the
            -- tokens to 'the_character', 'the_first', 'the_second' - if
            -- errors were found outputs some prompts
            --
         local
            done:  BOOLEAN;
            messg: STRING;
            index,
            tries: INTEGER

         do
            from
               tries := 0;
               !!messg.make_from_string (msg)
            until
               done
            loop
               if tries > 0 and then messg.item (1) = '%N' then
                  messg.prune ('%N')
               end;

               io.put_string (messg);
               io.read_line;
               the_string := io.last_string;

               -- from
               --   !!the_string.make (0);
               --   io.read_character
               -- until
               --   io.last_character = '%N'
               -- loop
               --   if io.last_character.code >= 32 and then
               --      io.last_character.code < 128          then
               --
               --      the_string.append_character (io.last_character);
               -- end;
               --
               --   io.read_character
               -- end;

               -- the above loop could be substituted by the following lines:
               --
               --    io.read_line;
               --    the_string := io.last_string;
               --


               if the_string.count > 0 then

                  the_string.to_lower;
                  index := get_comma_position (the_string);
                  if index >= 0 then

                     the_character := the_string.item (1);
                     the_first  := 0;
                     the_second := 0;
                     if index = 0 then
                        if the_string.count > 1 then
                           the_first := get_integer
                              (the_string.substring (2, the_string.count));
                           done := true
                        else
                           if the_character = '?' then
                              help
                           else
                              done := true
                           end
                        end
                     else
                        the_first := get_integer
                                        (the_string.substring (2, index - 1));
                        the_second := get_integer
                                        (the_string.substring (index + 1,
                                                       the_string.count));
                        done := true
                     end;

                     if the_character /= 'x' and then the_first > 21 then
                        error_the_first;
                        done := false
                     end;

                     if the_second > the_second_upper_bound then
                        error_the_second;
                        done := false
                     end
                  else
                     error_message;
                     tries := 0
                  end
               else
                  -- if nothing were typed ...
                  tries := tries + 1;
                  inspect
                     tries
                  when 1 .. 3, 5 .. 7, 9, 10 then
                     -- nothing
                  when 4 then
                     io.put_string (prompt);
                     io.put_string ("you could type something ...%N")
                  when 8 then
                     help
                  when 11 then
                     the_character := 'q';
                     done := true
                  else
                     tries := 0
                  end
               end
            end
         end


   feature -- Queries

      the_character: CHARACTER;
      the_first:     INTEGER;
      the_second:    INTEGER;
      the_string:    STRING


   feature {NONE} -- Implementation

      prompt: STRING is once Result := "Demo:  " end;

      the_second_upper_bound: INTEGER is
            -- gives the maximum possible value of 'the_second'
         once
         end;

      error_the_first is
            -- prints the error message (concerning 'the_first') ...
         do
            io.put_string (prompt);
            io.put_string ("a sample number must be in range 1 .. 21 only%N")
         end;


      error_the_second is
            -- prints the error message (concerning 'the_second') ...
         do
            io.put_string (prompt);
            io.put_string ("only one number is possible%N")
         end;

      error_message is
            -- prints the error message in case of an unqualified error ...
         do
            io.put_string (prompt);
            io.put_string ("the first symbol typed must be a letter%
                           % following one optional number%N")
         end;

      get_comma_position (src: STRING): INTEGER is
            -- finds a comma position in 'src' and concurrently makes checks
            -- if the first symbol of 'src' is a letter, '?' or '/' and all
            -- the other symbols - if they exist - are digits with an optional
            -- comma between them ...
         local
            index:  INTEGER;
            symbol: CHARACTER;
            found:  BOOLEAN

         do
            from
               index := 1
            until
               index > src.count or else Result < 0
            loop
               symbol := src.item (index);

               inspect
                  index
               when 1 then
                  if symbol < 'a' or else symbol > 'z' then
                     Result := - index
                  else
                     index := index + 1
                  end
               when 2 then
                  if symbol < '0' or else symbol > '9' then
                     Result := - index
                  else
                     index := index + 1
                  end
               else
                  if not found and then symbol = ',' then
                     if index < src.count then
                        Result := index;
                        found  := true;
                        index  := index + 1
                     else
                        Result := - index
                     end
                  elseif symbol >= '0' and then symbol <= '9' then
                     index := index + 1
                  else
                     Result := - index
                  end
               end
            end;

            if src.count = 1 and then (symbol = '?' or else symbol = '/') then
               Result := 0
            end
         end;

      adjust_right (src: STRING): STRING is
            -- moves blanks from the right side of 'src' to the left one
            --
         require
            proper_input: src /= Void and then src.count > 0
         local
            i: INTEGER

         do
            !!Result.make_from_string (src);
            Result.right_adjust;
            i := src.count - Result.count;
            if i > 0 then
               from until i = 0 loop
                  Result.precede (' ');
                  i := i - 1
               end
            end
         end;

         print_list (src: LIST_[SAMPLE]) is
               -- prints all items of 'src'
            require
               valid_list: src /= Void
            local
               index:   INTEGER;
               int_csr: CURSOR_

            do
               from
                  int_csr := src.cursor
               until
                  int_csr.is_finished
               loop
                  if index = 0 then
                     io.put_string ("Demo: ")
                  elseif index \\ 7 = 0 then
                     io.put_string ("%NDemo: ");
                  end;

                  io.put_string (src.item_at (int_csr).base);
                  io.put_string (" ");
                  index := index + 1;
                  int_csr.forth
               end
            end;

      help is
            -- simply prints some help ...
         do
            io.put_string (prompt);
            io.put_string ("You can use all the commands put on the right side%
                           % of your screen.%N");

            io.put_string (prompt);
            io.put_string ("Every command starts with a letter ignoring case%
                           % with the exception of%N");

            io.put_string (prompt);
            io.put_string ("Help command which consists only of one symbol '?'%
                           % and Refresh - '/'.%N");

            io.put_string (prompt);
            io.put_string ("%N");

            io.put_string (prompt);
            io.put_string ("Besides the letter some commands requires one or%
                           % two optional numbers%N");

            io.put_string (prompt);
            io.put_string ("indicating the corresponding sample(s) which might%
                           % be used to fill into%N");

            io.put_string (prompt);
            io.put_string ("the container. All possible samples put on the%
                           % left side of the screen.%N");

            io.put_string (prompt);
            io.put_string ("%N");

            io.put_string (prompt);
            io.put_string ("All the samples are taken equal if their first%
                           % symbols are equal - for%N");

            io.put_string (prompt);
            io.put_string ("example 'Annie' and 'Amy' are identical here -%
                           % it's done with the only %N");

            io.put_string (prompt);
            io.put_string ("target - to enable you to see how duplicates%
                           % are stored and handled ...%N");

            io.put_string (prompt);
            io.put_string ("%N");

            io.put_string (prompt);
            io.put_string ("On the right bottom the results of the queries%
                           % which are executed every%N");

            io.put_string (prompt);
            io.put_string ("time after you enter any valid command are%
                           % shown. Commands 'cursor' and%N");

            io.put_string (prompt);
            io.put_string ("'cursor_...' create a cursor if you're running any%
                           % list ot table demo.%N");

            io.put_string (prompt);
            io.put_string ("If it's not 'stop'ed - an arrow'll be pointing to%
                           % the current item and%N");

            io.put_string (prompt);
            io.put_string ("'is_protected = true' will be on. Commands 'back',%
                           % 'first', 'forth' and%N");

            io.put_string (prompt);
            io.put_string ("'last' relates to the cursor created ...%N");

            io.put_string (prompt);
            io.put_string ("%N");

            io.put_string (prompt);
            io.put_string ("Command 'merge' uses samples with numbers 7, 14,%
                           % 21 - mind duplicates!%N");

            io.put_string (prompt);
            io.put_string ("%N");

            io.put_string (prompt);
            io.put_string ("Type / to come back to the main picture ...%N");
         end

end -- class INPUT_OUTPUT


class SIMPLE_DEMO_IO

   feature -- Convertors ...

      get_concatenated (width: INTEGER; src: ARRAY [ANY]): STRING is
            -- unites all the items of 'src' in one string using 'width'
            -- digits for representing any integer in 'src' ...
         require
            valid_source: src /= Void;
            valid_width:  width >= 0
         local
            a : ANY;
            i : INTEGER;
            pb: BOOLEAN_REF;
            pc: CHARACTER_REF;
            pi: INTEGER_REF;
            ps,
            st: STRING

         do
            if src = Void then
               Result := " (void) "
            else
               from
                  i := src.lower
               until
                  i > src.upper
               loop
                  a  := src.item (i);
                  st := Void;

                  -- boolean ?
                  pb ?= a;
                  if pb /= Void then
                     if pb.item then
                        !!st.make_from_string ("true ")
                     else
                        !!st.make_from_string ("false")
                     end
                  else
                     -- character ?
                     pc ?= a;
                     if pc /= Void and then pc.item /= '%U' then
                        !!st.make_filled (pc.item, 1)
                     else
                        -- integer ?
                        pi ?= a;
                        if pi /= Void and then width /= 0 then
                           st := int_to_str (width, pi.item)
                        else
                           -- string ?
                           ps ?= a;
                           if ps /= Void then
                              !!st.make_from_string (ps)
                           end
                        end
                     end
                  end;

                  if i = src.lower or else Result = Void then
                     Result := st
                  elseif st /= Void then
                     Result.append_string (st)
                  end;
                  i := i + 1

               end
            end
         end;

      get_fixed (width: INTEGER; src: STRING): STRING is
            -- delivers 'src' exactly in 'width' characters ...
         require
            valid_string: src /= Void;
            real_width:   width > 0
         local
            i: INTEGER
         do
            !!Result.make_from_string (src);
            i := width - src.count;
            if i > 0 then
               from until i = 0 loop
                  Result.precede (' ');
                  i := i - 1
               end
            end
         end;

      get_hash_code (symbol: CHARACTER): INTEGER is
            -- ...
         do
            Result := symbol.code
         end;

      get_integer (src: STRING): INTEGER is
            -- ...
         require
            valid_string: src /= Void
         do
            Result := src.to_integer
         end;

      get_united (str_a, str_b: STRING): STRING is
            -- unites two strings ...
         require
            valid_str_a: str_a /= Void;
            valid_str_b: str_b /= Void
         do
            !!Result.make_from_string (str_a);
            Result.append_string (str_b)
         end

      int_to_str (width, number: INTEGER): STRING is
            -- ...
         require
            real_width: width /= 0
         local
            left: BOOLEAN;
            size: INTEGER

         do
            if width < 0 then
               left := True;
               size:= - width
            else
               size := width
            end;

            Result := number.out;

            from
               size := size - Result.count
            until
               size <= 0
            loop
               if left then
                  Result.append_character (' ')
               else
                  Result.precede (' ')
               end;

               size := size - 1
            end
         end

end -- class SIMPLE_DEMO_IO


class SAMPLE_SUPPLIER

   creation
      make


   feature -- Operations

      make is
            -- ...
         do
            pool_a := << "Annie    ", "Amy      ", "Bridget  ",
                         "Colette  ", "Diana    ", "Doris    ",
                         "Felicia  ", "Fidelia  ", "Flo      ",
                         "Flora    ", "Flossie  ", "Floy     ",
                         "Freda    ", "Greta    ", "Gwendolin",
                         "Harriet  ", "Hatty    ", "Helen    ",
                         "Sarah    ", "Susan    ", "Yoko     "  >>;

            pool_b := << "A        ", "B        ", "C        ",
                         "D        ", "E        ", "F        ",
                         "G        ", "H        ", "I        ",
                         "J        ", "K        ", "L        ",
                         "M        ", "N        ", "O        ",
                         "P        ", "Q        ", "R        ",
                         "S        ", "T        ", "Z        "  >>
         end


   feature -- Queries

      sample_set_a: ARRAY [SAMPLE] is
            -- ....
         do
            Result := sample_set (pool_a)
         end;

      sample_set_b: ARRAY [SAMPLE] is
            -- ...
         do
            Result := sample_set (pool_b)
         end


   feature -- Implementation

      pool_a: ARRAY [STRING];
      pool_b: ARRAY [STRING];

      sample_set (pool: ARRAY [STRING]): ARRAY [SAMPLE] is
            -- ...
         local
            i: INTEGER;
            s: SAMPLE
         do
            !!Result.make (1, 21);
            from
               i := 1
            until
               i > pool.count
            loop
               !!s.make (pool.item (i));
               Result.put (s, i);
               i := i + 1
            end
        end

end -- class SAMPLE_SUPPLIER


class SAMPLE
   inherit
      COMPARABLE
         redefine
            is_equal
         end;

      HASHABLE
         redefine
            is_equal
         end

   creation
      make


   feature -- Operations

      make (init: STRING) is
            -- the creation procedure
         do
            base := init
         end


   feature -- Queries

      base: STRING;

      hash_code: INTEGER is
            -- defines hash value to be equal to
            -- the ASCII code of the first character
         local
            hs: SIMPLE_DEMO_IO
         do
            !!hs;
            Result := hs.get_hash_code (base.item (1))
         end;

      infix "<" (other: like Current): BOOLEAN is
            -- all other functions of COMPARABLE are
            -- derived from this one and are effective
            --
         local
            head1, head2 : CHARACTER
         do
            head1 := Current.base.item (1);
            head2 :=   other.base.item (1);
            Result := (head1 < head2)
         end;

      is_equal (other: like Current): BOOLEAN is
            -- are the Current's and 'other's first
            -- characters of 'base' identical ?
            --
         local
            head1, head2 : CHARACTER
         do
            head1  := Current.base.item (1);
            head2  :=   other.base.item (1);
            Result := (head1 = head2)
         end

end -- class SAMPLE
