indexing

   title:       "An example of how QUEUE_ might be used";
   project:     "Visual Eiffel";
   date:        "$Date: 2005/02/18 15:34:22 $";
   revision:    "$Revision: 1.2 $";
   copyright:   "Copyright (C) 1996-2005 Object Tools Group";
   license:     "http://visual-eiffel.com/license";
   disclaimer:  "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions:"[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

class QUEUE_DEMO

inherit

   INPUT_OUTPUT
      redefine
         help
      end

creation

   make


feature -- Operations

   make is
         -- procedure ...
      local
         err: BOOLEAN;
         slr: SAMPLE_SUPPLIER

      do
         !!slr.make;
         source := slr.sample_set_a;

         !!target;   -- creating a queue ...

         from
            show;
            io.put_string ("%NDemo: ...%N");
            get_a_string ("Demo> ")

         until
            the_character = 'q'

         loop
            message := o_k_;
            err := false;

            inspect
               the_character

            when 'p' then                                            -- put
               if the_first > 0 then
                  target.put (source.item (the_first))
               else
                  err := true
               end

            when 'r' then                                         -- remove
               if target.empty then
                  message := queue_is_empty
               else
                  target.remove
               end

            when '/' then
               -- used only to put the picture on the screen again ...

            else
               message := unknown_command
            end;

            if err then
               message := syntax_error
            end;

            show; -- printing the main picture with the results ...

            io.put_string ("%NDemo: ");
            io.put_string (the_string);
            io.put_string (" -> ");

            io.put_string (message);
            get_a_string ("%NDemo> ")
         end
      end


feature {NONE} -- Implementation

   o_k_:      STRING is "done";
   true_str:  STRING is "true ";
   false_str: STRING is "false";

   empty_str: STRING is "         ";

   queue_is_empty:    STRING is "queue is empty ...";
   unknown_command:   STRING is "unknown command ...";
   syntax_error:      STRING is "syntax error ...";

   message: STRING;
   pool:    ARRAY [STRING];
   source:  ARRAY [SAMPLE];
   target:  QUEUE_[SAMPLE];

   help is
         -- simply print some help ...
      do
         io.put_string (prompt);   io.put_string
            ("Hmm ... It seems to be nothing to add ...%N");
      end;

   show is
         -- print the main picture ...
      local
         idx: INTEGER;
         tmp,
         txt: STRING;
         csr: CURSOR_;
         set: LIST_[SAMPLE]

      do
         set := target.list_of_elements;
         io.put_string ("---- Samples ---");
         from
            idx := 1;
            csr := set.cursor;
            if csr.is_finished then
               io.put_string ("         Your queue         ")
            else
               io.put_string ("   *** Items in queue ***   ")
            end;
            io.put_string ("------------- Notes -------------%N")

         until
            idx > source.count

         loop
            tmp := get_concatenated (4, <<idx, " - ",
                      source.item (idx).base, "  ">>);

            if idx > 1 then
               if not csr.is_finished then
                  if idx = 2 then
                     tmp := get_united (tmp, "Head => ")
                  elseif idx /= 2 and then idx = set.count + 1 then
                     tmp := get_united (tmp, "Tail => ")
                  else
                     tmp := get_united (tmp, "        ")
                  end;
                  tmp := get_concatenated (2, <<tmp, set.item_at (csr).base,
                            "         ">>);
                  csr.forth
               else
                  tmp := get_united (tmp, "                          ")
               end
            end;

            io.put_string (tmp);

            inspect
               idx
            when 1 then
               -- nothing to put here ...
            when 2 then
               if target.empty then
                  io.put_string (" You can play with Your queue by");
               else
                  io.put_string (" Head is the only %"active entry%"")
               end
            when 3 then
               if target.empty then
                  io.put_string (" using the commands placed below")
               elseif target.count = 1 then
                  io.put_string ("               of the container")
               else
                  io.put_string (" Tail is shown only for fun ...")
               end
            when 4 then
               -- nothing to put here ...
            when 5 then
               -- nothing to put here ...
            when 6 then
               io.put_string ("------------ Commands ------------")
            when 7 then
               -- nothing to put here ...
            when 8 then
               io.put_string ("? = Help,   / = Refresh,  q = Quit")
            when 9 then
               -- nothing to put here ...
            when 10then
               -- nothing to put here ...
            when 11 then
               io.put_string ("X below: a number in range 1 .. 21")
            when 12 then
               io.put_string (" corresponding the sample selected")
            when 13 then
               -- nothing to put here ...
            when 14 then
               io.put_string ("pX - put (X: G)")
            when 15 then
               io.put_string ("r  - remove")
            when 16 then
               -- nothing to put here ...
            when 17 then
               -- nothing to put here ...
            when 18 then
               io.put_string ("----------- QUEUE LIVE -----------")
            when 19 then
               -- nothing to put here ...
            when 20 then
               io.put_string ("item: ");
               if target.item /= Void then
                  io.put_string (target.item.base)
               else
                  io.put_string (empty_str)
               end;
               io.put_string ("     empty = ");
               if target.empty then
                  txt := true_str
               else
                  txt := false_str
               end;
               io.put_string (txt)
            when 21 then
               io.put_string ("                    count = ");
               io.put_string (get_concatenated (2, <<target.count>>))
            end;

            io.put_string ("%N");
            idx := idx + 1
         end;

         io.put_string ("--------------------------------------------");
         io.put_string ("                 capacity = ");
         io.put_string (get_concatenated (2, <<target.capacity>>));

         if not csr.is_finished then
            csr.stop
         end
      end

end -- class QUEUE_DEMO