-- example-start colorsel colorsel.e
class COLORSEL
inherit
   GTK_CONSTANTS
   VEGTK_MAIN
   VEGTK_CALLBACK_HANDLER
creation
   make

feature -- Creation

   make is
   do
      -- Initialize the toolkit, remove gtk-related commandline stuff
      vegtk_init

      -- Create toplevel window, set title and policies
      !!window.make(Gtk_window_toplevel)
      window.set_title("Color selection test");
      window.set_policy (True, True, True);
      -- Attach to the "delete" and "destroy" events so we can exit
      signal_connect(window,"destroy",$destroy_window)
      -- Create drawingarea, set size and catch button events

      !!drawingarea.make;

      drawingarea.size (200, 200);

      drawingarea.set_events (Gdk_button_press_mask);

      signal_connect (drawingarea, "event",$area_event);

      -- Add drawingarea to window, then show them both

      window.add (drawingarea);

      drawingarea.show;
      window.show;

      -- Enter the gtk main loop (this never returns

      gtk_main;

   end
feature {NONE} -- Implementation
   window : GTK_WINDOW;
   colorseldlg : GTK_COLOR_SELECTION_DIALOG;
   drawingarea : GTK_DRAWING_AREA;
feature {NONE} -- Callbacks

   color_changed_cb (data : ANY; cb_data : VEGTK_CALLBACK_DATA) is
      -- Color changed handler
   local
     color : ARRAY[DOUBLE];
     gdk_color : GDK_COLOR;
     colormap : GDK_COLORMAP;
     colorsel : GTK_COLOR_SELECTION;
     dummy : BOOLEAN;
   do
     colorsel ?= cb_data.get_widget;
     -- Get drawingarea colormap

     colormap := drawingarea.window.get_colormap;

     -- Get current color

     color := colorsel.get_color;

     -- Fit to a unsigned 16 bit integer (0..65535) and
     -- insert into the GdkColor structure */
     !!gdk_color.make_with_values((color.item(1)*65535).rounded,
                                  (color.item(2)*65535).rounded,
                                  (color.item(3)*65535).rounded);

     -- Allocate color

     dummy := colormap.alloc_color (gdk_color,True,True);

     -- Set window background color

     drawingarea.window.set_background (gdk_color);

     -- Clear window

     drawingarea.window.clear;
   end


   area_event (data : ANY; cb_data : VEGTK_CALLBACK_DATA) is
      -- Drawingarea event handler
   local
     handled  : BOOLEAN;
     event : GDK_EVENT_BUTTON;
     colorsel : GTK_COLOR_SELECTION;
   do
      -- Check if we've received a button pressed event
      event ?= cb_data.get_event
      if event /= Void and then colorseldlg = Void then
          -- Yes, we have an event and there's no colorseldlg yet!

          handled := True;

          -- Create color selection dialog

          !!colorseldlg.make("Select background color");

          -- Get the GtkColorSelection widget

          colorsel := colorseldlg.colorsel;

          -- Connect to the "color_changed" signal

          signal_connect(colorsel, "color_changed",$color_changed_cb);

          -- Show the dialog

          colorseldlg.show;
        end -- if
    --  return handled;
   end


   destroy_window is
   -- Close down and exit handler
   do
      gtk_main_quit;
   end
end -- class COLORSEL
-- example-end
