-- example-start helloworld helloworld.e
class HELLOWORLD
inherit
   -- All constants we need are defined in the class below.
   -- eg gtk_window_toplevel etc.
   GTK_CONSTANTS
   -- This class contains common features like vegtk_init and so
   VEGTK_MAIN
   -- If we need to handle callbacks we must inherit from the below
   -- class
   VEGTK_CALLBACK_HANDLER
creation
   make
feature -- Callback handlers
   hello (data : ANY; cb_data : VEGTK_CALLBACK_DATA) is
   -- This is a callback function. The data arguments are ignored
   -- in this example. More on callbacks below.
   do
      print("Hello World%N");
   end
   delete_event (data : ANY; cb_data : VEGTK_CALLBACK_DATA) is
      -- If you return False in the "delete_event" signal handler,
      -- GTK will emit the "destroy" signal. Returning True means
      -- you don't want the window to be destroyed.
      -- This is useful for popping up 'are you sure you want to quit?'
      -- type dialogs.
   do
       print ("delete event occurred%N");
       -- Change True to False and the main window will be destroyed with
       -- a "delete_event".
       cb_data.set_return_value_boolean(True);
   end

   destroy (data : ANY; cb_data : VEGTK_CALLBACK_DATA) is
   do
       gtk_main_quit;
   end

   close (data : ANY; cb_data : VEGTK_CALLBACK_DATA) is
   local
      window : GTK_WINDOW;
   do
      window ?= data;
      if window /= Void then
         window.destroy;
      end
   end
feature -- Creation
   make is
   local
      window : GTK_WINDOW;
      button : GTK_BUTTON;
   do
      -- This is called in all GTK applications. Arguments are parsed
      -- from the command line and are returned to the application.
      vegtk_init;

      !!window.make(Gtk_window_toplevel)

      -- When the window is given the "delete_event" signal (this is given
      -- by the window manager, usually by the 'close' option, or on the
      -- titlebar), we ask it to call the delete_event function
      -- as defined above.
      signal_connect (window, "delete_event",$delete_event);

     -- Here we connect the "destroy" event to a signal handler.
     -- This event occurs when we call gtk_widget_destroy on the window,
     -- or if we return 'FALSE' in the "delete_event" callback.
      signal_connect (window, "destroy", $destroy);

      -- Sets the border width of the window.
      window.set_border_width (10);

      -- Creates a new button with the label "Hello World".
      !!button.make_with_label ("Hello World");

      -- When the button receives the "clicked" signal, it will call the
      -- procedure hello. The hello procedure is defined above.
      signal_connect (button, "clicked",$hello);

      -- This will cause the window to be destroyed by calling
      -- window.destroy from procedure close when "clicked".
      --  Again, the destroy signal could come from there,
      -- or the window manager. Also note using data in this call
      -- and in the procedure close .
      signal_connect_with_data (button, "clicked",$close,window);

      -- This packs the button into the window (a gtk container).
      window.add (button);

      -- The final step is to display this newly created widget.
      button.show;

      -- and the window
      window.show;

      -- All GTK applications must have a gtk_main. Control ends here
      -- and waits for an event to occur (like a key press or
      -- mouse event).
      gtk_main
      print ("End%N");
   end
end -- class HELLOWORLD
-- example-end
