-- example-start packbox packbox.e
class PACKBOX
inherit
   -- All constants we need are defined in the class below.
   -- eg gtk_window_toplevel etc.
   GTK_CONSTANTS
   -- This class contains common features like vegtk_init and so
   VEGTK_MAIN
   -- If we need to handle callbacks we must inherit from the below
   -- class
   VEGTK_CALLBACK_HANDLER
creation
   make
feature -- Callback handlers
   delete_event is
   do
       gtk_main_quit;
   end
   close (data : ANY; cb_data : VEGTK_CALLBACK_DATA) is
   local
      window : GTK_WINDOW;
   do
      window ?= data;
      if window /= Void then
         window.destroy;
         gtk_main_quit;
      end
   end
feature -- support
   make_box(homogeneous : BOOLEAN; spacing : INTEGER; expand :BOOLEAN;
            fill : BOOLEAN; padding :INTEGER ) : GTK_HBOX is
   -- Make a new hbox filled with button-labels. Arguments for the
   -- variables we're interested are passed in to this function.
   -- We do not show the box, but do show everything inside. */
   local
      button : GTK_BUTTON;
      paddstr : STRING;
   do

      -- Create a new hbox with the appropriate homogeneous
      -- and spacing settings
      !!Result.make(homogeneous, spacing);

      -- Create a series of buttons with the appropriate settings
      !!button.make_with_label ("gtk_box_pack");
      Result.pack_start (button, expand, fill, padding);
      button.show;

      !!button.make_with_label ("(box,");
      Result.pack_start (button, expand, fill, padding);
      button.show;

      !!button.make_with_label ("button,");
      Result.pack_start (button, expand, fill, padding);
      button.show;

      -- Create a button with the label depending on the value of
      -- expand.
      if expand then
         !!button.make_with_label ("True,");
      else
         !!button.make_with_label ("False,");
      end

      Result.pack_start (button, expand, fill, padding);
      button.show;

      if fill then
         !!button.make_with_label ("True,");
      else
         !!button.make_with_label ("False,");
      end

      Result.pack_start (button, expand, fill, padding);
      button.show;

      paddstr := padding.out;
      paddstr.append_string(");");

      !!button.make_with_label(paddstr);
      Result.pack_start (button, expand, fill, padding);
      button.show;
   end -- make_box

feature -- Creation
   make is
   local
      window : GTK_WINDOW;
      button : GTK_BUTTON;
      quitbox,box1,box2   : GTK_BOX;
      separator : GTK_HSEPARATOR;
      label : GTK_LABEL;
      which : INTEGER;
      args : ARRAY[STRING];
   do
      -- Our init, don't forget this!
      vegtk_init
      -- get modified list of program arguments. It doesn't contain any
      -- gtk/gdk recognized arguments.
      args := vegtk_args;
      if args.count /= 2 then
         print ("usage: packbox num, where num is 1, 2, or 3.%N");
      else

         which := args.item(1).to_integer

         -- Create our window
         !!window.make (Gtk_window_toplevel);

         -- You should always remember to connect the delete event to the
         -- main window. This is very important for proper intuitive
         -- behavior
         signal_connect (window, "delete_event",$delete_event);
         window.set_border_width (10);

         -- We create a vertical box (vbox) to pack the horizontal boxes into.
         -- This allows us to stack the horizontal boxes filled with buttons one
         -- on top of the other in this vbox.
         !GTK_VBOX!box1.make (False, 0);

         -- which example to show. These correspond to the pictures above.
         inspect
            which
         when 1 then
            -- create a new label.
            !!label.make ("gtk_hbox_new (False, 0);");

            -- Align the label to the left side.  We'll discuss this function and
            -- others in the section on Widget Attributes.
            label.set_alignment (0, 0);

            -- Pack the label into the vertical box (vbox box1).  Remember that
            -- widgets added to a vbox will be packed one on top of the other in
            -- order.
            box1.pack_start (label, False, False, 0);

            -- Show the label
            label.show;

            -- Call our make box function - homogeneous = False, spacing = 0,
            -- expand = False, fill = False, padding = 0
            box2 := make_box (False, 0, False, False, 0);
            box1.pack_start (box2, False, False, 0);
            box2.show;

            -- Call our make box function - homogeneous = False, spacing = 0,
            -- expand = True, fill = False, padding = 0
            box2 := make_box (False, 0, True, False, 0);
            box1.pack_start (box2, False, False, 0);
            box2.show;

            -- Args are: homogeneous, spacing, expand, fill, padding
            box2 := make_box (False, 0, True, True, 0);
            box1.pack_start (box2, False, False, 0);
            box2.show;

            -- Creates a separator, we'll learn more about these later,
            -- but they are quite simple.
            !!separator.make;

            -- Pack the separator into the vbox. Remember each of these
            -- widgets are being packed into a vbox, so they'll be stacked
            -- vertically.
            box1.pack_start (separator, False, True, 5);
            separator.show;

            -- Create another new label, and show it.
            !!label.make("gtk_hbox_new (True, 0);");
            label.set_alignment (0, 0);
            box1.pack_start (label, False, False, 0);
            label.show;

            -- Args are: homogeneous, spacing, expand, fill, padding
            box2 := make_box (True, 0, True, False, 0);
            box1.pack_start (box2, False, False, 0);
            box2.show;

            -- Args are: homogeneous, spacing, expand, fill, padding
            box2 := make_box (True, 0, True, True, 0);
            box1.pack_start (box2, False, False, 0);
            box2.show;

            -- Another new separator.
            !!separator.make;
            -- The last 3 arguments to gtk_box_pack_start are:
            -- expand, fill, padding.
            box1.pack_start (separator, False, True, 5);
            separator.show;

         when 2 then

            -- Create a new label, remember box1 is a vbox as created
            -- near the beginning of make
            !!label.make ("gtk_hbox_new (False, 10);");
            label.set_alignment (0, 0);
            box1.pack_start (label, False, False, 0);
            label.show;

            -- Args are: homogeneous, spacing, expand, fill, padding
            box2 := make_box (False, 10, True, False, 0);
            box1.pack_start (box2, False, False, 0);
            box2.show;

            -- Args are: homogeneous, spacing, expand, fill, padding
            box2 := make_box (False, 10, True, True, 0);
            box1.pack_start (box2, False, False, 0);
            box2.show;

            !!separator.make;
            -- The last 3 arguments to gtk_box_pack_start are:
            -- expand, fill, padding.
            box1.pack_start (separator, False, True, 5);
            separator.show;

            !!label.make("gtk_hbox_new (False, 0);");
            label.set_alignment (0, 0);
            box1.pack_start (label, False, False, 0);
            label.show;

            -- Args are: homogeneous, spacing, expand, fill, padding
            box2 := make_box (False, 0, True, False, 10);
            box1.pack_start (box2, False, False, 0);
            box2.show;

            -- Args are: homogeneous, spacing, expand, fill, padding
            box2 := make_box (False, 0, True, True, 10);
            box1.pack_start (box2, False, False, 0);
            box2.show;

            !!separator.make;
            -- The last 3 arguments to gtk_box_pack_start are:
            -- expand, fill, padding.
            box1.pack_start (separator, False, True, 5);
            separator.show;
         when 3 then
            -- This demonstrates the ability to use pack_end() to
            -- right justify widgets. First, we create a new box as before.
            box2 := make_box (False, 0, False, False, 0);

            -- Create the label that will be put at the end.
            !!label.make ("end");
            -- Pack it using pack_end(), so it is put on the right
            -- side of the hbox created in the make_box() call. */
            box2.pack_end (label, False, False, 0);
            -- Show the label.
            label.show;

            -- Pack box2 into box1 (the vbox remember ? :)
            box1.pack_start (box2, False, False, 0);
            box2.show;

            -- A separator for the bottom.
            !!separator.make;
            -- This explicitly sets the separator to 400 pixels wide by 5 pixels
            -- high. This is so the hbox we created will also be 400 pixels wide,
            -- and the "end" label will be separated from the other labels in the
            -- hbox. Otherwise, all the widgets in the hbox would be packed as
            -- close together as possible.
            -- separator.set_usize (400, 5);
            -- pack the separator into the vbox (box1) created near the start
            -- of make
            box1.pack_start (separator, False, True, 5);
            separator.show;
         end -- inspect

         -- Create another new hbox.. remember we can use as many as we need! */
         !GTK_HBOX!quitbox.make(False, 0);

         -- Our quit button.
         !!button.make_with_label ("Quit");

         -- Setup the signal to destroy the window. Remember that this will send
         -- the "destroy" signal to the window which will be caught by our signal
         -- handler as defined above. */
         signal_connect_with_data (button, "clicked",$close,window);
         -- Pack the button into the quitbox.
         -- The last 3 arguments to gtk_box_pack_start are:
         -- expand, fill, padding.
         quitbox.pack_start (button, True, False, 0);
         -- pack the quitbox into the vbox (box1)
         box1.pack_start (quitbox, False, False, 0);

         -- Pack the vbox (box1) which now contains all our widgets, into the
         -- main window.
         window.add (box1);

         -- And show everything left
         button.show;
         quitbox.show;

         box1.show;
         -- Showing the window last so everything pops up at once.
         window.show;

         -- And of course, our main function.
         gtk_main;

         -- Control returns here when gtk_main_quit() is called, but not when
         -- gtk_exit is used.
      end -- if
   end -- make
end -- class PACKBOX
-- example-end
