-- example-start paned paned.e
class PANED
inherit
   GTK_CONSTANTS
   VEGTK_MAIN
   VEGTK_CALLBACK_HANDLER
creation
   make

feature -- Creation

   make is
   local
      window : GTK_WINDOW;
      vpaned : GTK_VPANED;
      text,list : GTK_WIDGET;
   do
      -- Initialise GTK
      vegtk_init
      -- Create a new window
      !!window.make (Gtk_window_toplevel);
      window.set_title("Paned Windows");
      -- Here we connect the "destroy" event to a signal handler
      signal_connect (window, "destroy",$destroy);
      -- Sets the border width of the window.
      window.set_border_width (10);

      window.set_usize (450, 400);

      -- create a vpaned widget and add it to our toplevel window

      !!vpaned.make;
      window.add (vpaned);
      vpaned.set_handle_size (10);
      vpaned.set_gutter_size (15);
      vpaned.show;

      -- Now create the contents of the two halves of the window

      list := create_list;
      vpaned.add1 (list);
      list.show;

      text := create_text;
      vpaned.add2 (text);
      text.show;
      window.show;
      gtk_main;
   end -- make
feature {NONE} -- Support
   create_list : GTK_SCROLLED_WINDOW  is
      -- Create the list of "messages"
   local
      scrolled_window : GTK_SCROLLED_WINDOW;
      list : GTK_LIST;
      list_item : GTK_LIST_ITEM;
      i : INTEGER;
      buffer : STRING;
   do
       -- Create a new scrolled window, with scrollbars only if needed
      !!scrolled_window.make (Void,Void);
      scrolled_window.set_policy (Gtk_policy_automatic, Gtk_policy_automatic);

      -- Create a new list and put it in the scrolled window
      !!list.make;
      scrolled_window.add_with_viewport (list);
      list.show;

      -- Add some messages to the window
      from
         i :=0
      until
        i >= 10
      loop
          !!buffer.make_from_string("Message # ");
          buffer.append_string(i.out);
          !!list_item.make_with_label (buffer);
          list.add (list_item);
          list_item.show;
          i := i+1;
      end

      Result := scrolled_window;
   end -- create_list
create_text  : GTK_WIDGET is
   -- Create a scrolled text area that displays a "message"
local
   table : GTK_TABLE;
   text : GTK_TEXT;
   hscrollbar : GTK_HSCROLLBAR;
   vscrollbar : GTK_VSCROLLBAR;
do
   -- Create a table to hold the text widget and scrollbars */
   !!table.make (2, 2, False);

   -- Put a text widget in the upper left hand corner. Note the use of
   -- GTK_SHRINK in the y direction */
   !!text.make (Void, Void);
   table.attach (text, 0, 1, 0, 1,
                 Gtk_fill and Gtk_expand,
                 Gtk_fill and Gtk_expand and Gtk_shrink, 0, 0);
   text.show;

   -- Put a HScrollbar in the lower left hand corner
   !!hscrollbar.make(text.hadj);
   table.attach (hscrollbar, 0, 1, 1, 2,
                 Gtk_expand and Gtk_fill, Gtk_fill, 0, 0);
   hscrollbar.show;

   -- And a VScrollbar in the upper right
   !!vscrollbar.make (text.vadj);
   table.attach (vscrollbar, 1, 2, 0, 1,
                 Gtk_fill, Gtk_expand and Gtk_fill and Gtk_shrink, 0, 0);
   vscrollbar.show;

   -- Add a handler to put a message in the text widget when it is realized
   signal_connect (text, "realize",$realize_text);
   Result := table;
end -- create_text
feature {NONE} -- Callbacks
   destroy is
   do
       gtk_main_quit;
   end
   realize_text (data : ANY; cb_data : VEGTK_CALLBACK_DATA) is
      -- Add some text to our text widget - this is a callback that is invoked
      -- when our window is realized. We could also force our window to be
      -- realized with gtk_widget_realize, but it would have to be part of
      -- a hierarchy first
   local
      text : GTK_TEXT;
   do
      text ?= cb_data.get_widget;
      text.freeze;
      text.insert (Void,text.get_style.black,Void,
      "From: pathfinder@nasa.gov%N%
      %To: mom@nasa.gov%N%
      %Subject: Made it!%N%
      %%N%
      %We just got in this morning. The weather has been%N%
      %great - clear but cold, and there are lots of fun sights.%N%
      %Sojourner says hi. See you soon.%N%
      % -Path%N");

      text.thaw;
   end -- realize_text

end -- class PANED
-- example-end
