-- example-start scrolledwin scrolledwin.e
class SCROLLEDWIN
inherit
   GTK_CONSTANTS
   VEGTK_MAIN
   VEGTK_CALLBACK_HANDLER
creation
   make

feature -- Creation

   make is
   local
      window : GTK_DIALOG;
      scrolled_window : GTK_SCROLLED_WINDOW;
      table : GTK_TABLE;
      button : GTK_BUTTON;
      buffer : STRING;
      i,j : INTEGER;
   do
      -- Initialise GTK
      vegtk_init
      -- Create a new window
      -- Create a new dialog window for the scrolled window to be
      -- packed into. A dialog is just like a normal window except it has a
      -- vbox and a horizontal separator packed into it. It's just a shortcut
      -- for creating dialogs */
      !!window.make;
      window.set_title("GtkScrolledWindow example");

      -- Here we connect the "destroy" event to a signal handler
      signal_connect (window, "destroy",$destroy);

      -- Sets the border width of the window.
      window.set_border_width (0);
      window.set_usize(300, 300);

      -- create a new scrolled window.
      !!scrolled_window.make(Void, Void);

      scrolled_window.set_border_width (10);

      -- the policy is one of GTK_POLICY AUTOMATIC, or GTK_POLICY_ALWAYS.
      -- GTK_POLICY_AUTOMATIC will automatically decide whether you need
      -- scrollbars, whereas GTK_POLICY_ALWAYS will always leave the scrollbars
      -- there.  The first one is the horizontal scrollbar, the second,
      -- the vertical.
      scrolled_window.set_policy (Gtk_policy_automatic, Gtk_policy_always);
      -- The dialog window is created with a vbox packed into it.
      window.vbox.pack_start(scrolled_window, True, True, 0);
      scrolled_window.show;

      -- create a table of 10 by 10 squares.
      !!table.make (10, 10, False);

      -- set the spacing to 10 on x and 10 on y
      table.set_row_spacings (10);
      table.set_col_spacings (10);

      -- pack the table into the scrolled window
      scrolled_window.add_with_viewport (table);
      table.show;

      -- this simply creates a grid of toggle buttons on the table
      -- to demonstrate the scrolled window.
      from
         i := 0
      until
         i >= 10
      loop
         from
            j := 0
         until
            j >= 10
         loop
            !!buffer.make_from_string("button (");
            buffer.append_string(i.out);
            buffer.append_string(",");
            buffer.append_string(j.out);
            buffer.append_string(")%N");
            !!button.make_with_label (buffer);
            table.attach_defaults (button,i, i+1, j, j+1);
            button.show;
            j := j+1;
         end -- loop
         i := i+1;
      end -- loop

      -- Add a "close" button to the bottom of the dialog
      !!button.make_with_label ("close");
      signal_connect_with_data (button, "clicked",$close_application,window);

      -- this makes it so the button is the default.

      button.set_flags (Gtk_can_default);
      window.action_area.pack_start (button, True, True, 0);

      -- This grabs this button to be the default button. Simply hitting
      -- the "Enter" key will cause this button to activate.
      button.grab_default;
      button.show;

      window.show;

      gtk_main;
   end -- make
feature {NONE} -- Callbacks
   close_application (data : ANY) is
   local
      window : GTK_OBJECT;
   do
       window ?= data;
       window.destroy;
   end
   destroy is
   do
       gtk_main_quit;
   end
end -- class SCROLLEDWIN
-- example-end
