-- example-start text text.e
class TEXT
inherit
   GTK_CONSTANTS
   VEGTK_MAIN
   VEGTK_CALLBACK_HANDLER
creation
   make
feature {ANY} -- Creation
   make is
   local
      window : GTK_WINDOW;
      box1, box2, hbox : GTK_BOX;
      button : GTK_BUTTON;
      checkb : GTK_CHECK_BUTTON;
      separator : GTK_HSEPARATOR;
      table : GTK_TABLE;
      vscrollbar : GTK_VSCROLLBAR;
      text : GTK_TEXT;
      cmap : GDK_COLORMAP;
      colour : GDK_COLOR;
      fixed_font : GDK_FONT;
      infile : FILE;
   do
      -- Initialise GTK
      vegtk_init;

      !!window.make (Gtk_window_toplevel);
      window.set_usize (600, 500);
      window.set_policy (True, True, False);

      signal_connect (window, "destroy",$destroy);

      window.set_title ("Text Widget Example");
      window.set_border_width (0);

      !GTK_VBOX!box1.make (False, 0);
      window.add (box1);
      box1.show;

      !GTK_VBOX!box2.make (False, 10);
      box2.set_border_width (10);
      box1.pack_start (box2, True, True, 0);
      box2.show;

      !!table.make (2, 2, False);
      table.set_row_spacing (0, 2);
      table.set_col_spacing (0, 2);
      box2.pack_start (table, True, True, 0);
      table.show;

      -- Create the GtkText widget
      !!text.make (Void,Void);
      text.set_editable (True);
      table.attach (text, 0, 1, 0, 1,
                    Gtk_expand and Gtk_shrink and Gtk_fill,
                    Gtk_expand and Gtk_shrink and Gtk_fill, 0, 0);
      text.show;

      -- Add a vertical scrollbar to the GtkText widget
      !!vscrollbar.make (text.vadj);
      table.attach (vscrollbar, 1, 2, 0, 1,
                    Gtk_fill, Gtk_expand and Gtk_shrink and Gtk_fill, 0, 0);
      vscrollbar.show;

      -- Get the system colour map and allocate the colour red
      !!cmap.make_system;
      !!colour.make_with_values (65535, 0, 0);
      if not cmap.alloc_color (colour, False,True) then
         print("couldn't allocate colour%N");
      else
         -- Load a fixed font
         !!fixed_font.load ("-misc-fixed-medium-r-*-*-*-140-*-*-*-*-*-*");

         -- Realizing a widget creates a window for it,
         -- ready for us to insert some text
         text.realize;

         -- Freeze the text widget, ready for multiple updates
         text.freeze;

         -- Insert some coloured text
         text.insert (Void,text.get_style.black, Void,"Supports ");
         text.insert (Void,colour, Void, "colored ");
         text.insert (Void,text.get_style.black, Void, "text and different ");
         text.insert (fixed_font, text.get_style.black, Void, "fonts%N%N");

         -- Load the file text.c into the text window

         !!infile.make_open_read("text.e");

         if infile.is_open_read then
            from
               infile.read_line
            until
               infile.end_of_file
            loop
               text.insert(fixed_font, Void, Void, infile.last_string);
               text.insert(fixed_font, Void, Void, "%N");
               infile.read_line
            end -- loop
            infile.close
         end -- if
         -- Thaw the text widget, allowing the updates to become visible
         text.thaw;

         !GTK_HBUTTON_BOX!hbox.make;
         box2.pack_start (hbox, False, False, 0);
         hbox.show;

         !!checkb.make_with_label("Editable");
         hbox.pack_start (checkb, False, False, 0);
         signal_connect_with_data (checkb, "toggled", $text_toggle_editable,
                                   text);
         checkb.set_active(TRUE);
         checkb.show;
         !!checkb.make_with_label("Wrap Words");
         hbox.pack_start (checkb, False, True, 0);
         signal_connect_with_data (checkb, "toggled", $text_toggle_word_wrap,
                                   text);
         checkb.set_active (False);
         checkb.show;

         !!separator.make;
         box1.pack_start (separator, False, True, 0);
         separator.show;

         !GTK_VBOX!box2.make (FALSE, 10);
         box2.set_border_width (10);
         box1.pack_start (box2, False, True, 0);
         box2.show;

         !!button.make_with_label ("close");
         signal_connect_with_data (button, "clicked", $quit_app, window);
         box2.pack_start (button, True, True, 0);
         button.set_flags (Gtk_can_default);
         button.grab_default;
         button.show;

         window.show;

         gtk_main;
      end -- if
   end
   text_toggle_editable (data : ANY; cb_data : VEGTK_CALLBACK_DATA) is
   local
      checkbutton : GTK_CHECK_BUTTON;
      text : GTK_TEXT;
   do
      checkbutton ?= cb_data.get_widget;
      text ?= data;
      text.set_editable(checkbutton.get_active);
   end

   text_toggle_word_wrap (data : ANY; cb_data : VEGTK_CALLBACK_DATA) is
   local
      checkbutton : GTK_CHECK_BUTTON;
      text : GTK_TEXT;
   do
      checkbutton ?= cb_data.get_widget;
      text ?= data;
      text.set_word_wrap(checkbutton.get_active);
   end
   destroy is
   do
      gtk_main_quit;
   end
   quit_app (data : ANY) is
   local
      window : GTK_WINDOW;
   do
      window ?= data;
      window.destroy;
   end
end -- class TEXT
-- example-end
