-- example-start toolbar toolbar.e
class TOOLBAR
inherit
   GTK_CONSTANTS
   VEGTK_MAIN
   VEGTK_CALLBACK_HANDLER
creation
   make
feature {NONE} -- Callbacks
   delete_event is
      -- This callback procedure is connected to the Close button or
      -- closing the window from the WM
   do
      gtk_main_quit;
   end
   radio_event (data : ANY) is
      -- that's easy... when one of the buttons is toggled, we just
      -- check which one is active and set the style of the toolbar
      -- accordingly
      -- ATTENTION: our toolbar is passed as data to callback !
   local
      toolbar :  GTK_TOOLBAR;
   do
      toolbar ?= data;
      if text_button.get_active then
        toolbar.set_style(Gtk_toolbar_text);
      elseif icon_button.get_active then
        toolbar.set_style(Gtk_toolbar_icons);
      elseif (both_button.get_active) then
        toolbar.set_style(Gtk_toolbar_both);
      end
   end

   toggle_event (data : ANY; cb_data : VEGTK_CALLBACK_DATA) is
      -- even easier, just check given toggle button and enable/disable
      -- tooltips
   local
      toolbar : GTK_TOOLBAR;
      button : GTK_TOGGLE_BUTTON;
   do
     button ?= cb_data.get_widget;
     toolbar ?= data;

     toolbar.set_tooltips (button.get_active );
   end

feature {NONE} -- Implementation

close_button : GTK_BUTTON; -- This button will emit signal to close
                           -- application
tooltips_button : GTK_TOGGLE_BUTTON; -- to enable/disable tooltips

-- radio buttons for toolbar style
text_button,icon_button,both_button : GTK_RADIO_BUTTON;

entry : GTK_ENTRY; -- a text entry to show packing any widget into
                   -- toolbar */
feature {ANY} -- Creation
   make is
   local
      -- Here is our main window (a dialog) and a handlebox
      dialog : GTK_DIALOG;
      handlebox : GTK_HANDLE_BOX;

      -- Ok, we need a toolbar, an icon with a mask (one for all of
      -- the buttons) and an icon widget to put this icon in (but
      -- we'll create a separate widget for each button)
      toolbar : GTK_TOOLBAR;
      icon : GDK_PIXMAP;
      iconw : GTK_PIXMAP;
   do
      -- this is called in all GTK application.
      vegtk_init

      -- create a new window with a given title, and nice size
      !!dialog.make;
      dialog.set_title ("GTKToolbar Tutorial");
      dialog.set_usize (600, 300);
      dialog.set_policy (True,True,False);

      -- typically we quit if someone tries to close us
      signal_connect (dialog, "delete_event",$delete_event);

      -- we need to realize the window because we use pixmaps for
      -- items on the toolbar in the context of it
      dialog.realize;

      -- to make it nice we'll put the toolbar into the handle box,
      -- so that it can be detached from the main window */
      !!handlebox.make;
      dialog.vbox.pack_start (handlebox, False, False, 5 );
      -- toolbar will be horizontal, with both icons and text, and
      -- with 5pxl spaces between items and finally,
      -- we'll also put it into our handlebox */
      !!toolbar.make (Gtk_orientation_horizontal, Gtk_toolbar_both );
      toolbar.set_border_width (5);
      toolbar.set_space_size (5);
      handlebox.add (toolbar);

      -- now we create icon mask: we'll reuse it to create
      -- icon widgets for toolbar items */
      !!icon.make_from_xpm_d (dialog.window,dialog.get_style.white,gtk_xpm);
      !!iconw.make(icon, icon.mask);
      -- our first item is <close> button
      toolbar.append_item ( "Close",               -- button label
                            "Closes this app",     -- this button's tooltip
                            "Private",             -- tooltip private info
                             iconw,                -- icon widget
                             Current,
                             $delete_event,        -- callback procedure
                             Void);
      close_button ?= toolbar.last_added_element
      toolbar.append_space; -- space after item
      -- now, let's make our radio buttons group...
      !!iconw.make( icon, icon.mask );
      toolbar.append_element(
                        Gtk_toolbar_child_radiobutton, -- a type of element */
                        Void,                          -- pointer to widget */
                        "Icon",                        -- label
                        "Only icons in toolbar",       -- tooltip
                        "Private",                     -- tooltip private string */
                        iconw,                         -- icon
                        Current,
                        $radio_event,                  -- callback procedure
                        toolbar);                      -- data for signal
      icon_button ?= toolbar.last_added_element
      toolbar.append_space;
      -- following radio buttons refer to previous ones
      !!iconw.make( icon, icon.mask );
      toolbar.append_element(Gtk_toolbar_child_radiobutton,
                             icon_button,
                             "Text",
                             "Only texts in toolbar",
                             "Private",
                             iconw,
                             Current,
                             $radio_event,
                             toolbar);
      text_button ?= toolbar.last_added_element
      toolbar.append_space;

      !!iconw.make ( icon, icon.mask );
      toolbar.append_element(Gtk_toolbar_child_radiobutton,
                             text_button,
                             "Both",
                             "Icons and text in toolbar",
                             "Private",
                             iconw,
                             Current,
                             $radio_event,
                             toolbar);
      both_button ?= toolbar.last_added_element
      toolbar.append_space;
      both_button.set_active(True);
      -- here we have just a simple toggle button
      !!iconw.make ( icon, icon.mask );
      toolbar.append_element(Gtk_toolbar_child_togglebutton,
                             Void,
                             "Tooltips",
                             "Toolbar with or without tips",
                             "Private",
                             iconw,
                             Current,
                             $toggle_event,
                             toolbar);
      tooltips_button ?= toolbar.last_added_element
      toolbar.append_space;
      tooltips_button.set_active(True);
      -- to pack a widget into toolbar, we only have to
      -- create it and append it with an appropriate tooltip */
      !!entry.make;
      toolbar.append_widget (entry,
                             "This is just an entry",
                             "Private" );

      -- well, it isn't created within thetoolbar, so we must still show it
      entry.show;
      -- that's it ! let's show everything.
      toolbar.show;
      handlebox.show;
      dialog.show;

      -- rest in gtk_main and wait for the fun to begin!
      gtk_main;
  end -- make
feature {NONE} -- Implementation
    -- XPM
   gtk_xpm : ARRAY[STRING] is
   once
      Result := <<
         "32 39 5 1",
         ".      c none",
         "+      c black",
         "@      c #3070E0",
         "#      c #F05050",
         "$      c #35E035",
         "................+...............",
         "..............+++++.............",
         "............+++++@@++...........",
         "..........+++++@@@@@@++.........",
         "........++++@@@@@@@@@@++........",
         "......++++@@++++++++@@@++.......",
         ".....+++@@@+++++++++++@@@++.....",
         "...+++@@@@+++@@@@@@++++@@@@+....",
         "..+++@@@@+++@@@@@@@@+++@@@@@++..",
         ".++@@@@@@+++@@@@@@@@@@@@@@@@@@++",
         ".+#+@@@@@@++@@@@+++@@@@@@@@@@@@+",
         ".+##++@@@@+++@@@+++++@@@@@@@@$@.",
         ".+###++@@@@+++@@@+++@@@@@++$$$@.",
         ".+####+++@@@+++++++@@@@@+@$$$$@.",
         ".+#####+++@@@@+++@@@@++@$$$$$$+.",
         ".+######++++@@@@@@@++@$$$$$$$$+.",
         ".+#######+##+@@@@+++$$$$$$@@$$+.",
         ".+###+++##+##+@@++@$$$$$$++$$$+.",
         ".+###++++##+##+@@$$$$$$$@+@$$@+.",
         ".+###++++++#+++@$$@+@$$@++$$$@+.",
         ".+####+++++++#++$$@+@$$++$$$$+..",
         ".++####++++++#++$$@+@$++@$$$$+..",
         ".+#####+++++##++$$++@+++$$$$$+..",
         ".++####+++##+#++$$+++++@$$$$$+..",
         ".++####+++####++$$++++++@$$$@+..",
         ".+#####++#####++$$+++@++++@$@+..",
         ".+#####++#####++$$++@$$@+++$@@..",
         ".++####++#####++$$++$$$$$+@$@++.",
         ".++####++#####++$$++$$$$$$$$+++.",
         ".+++####+#####++$$++$$$$$$$@+++.",
         "..+++#########+@$$+@$$$$$$+++...",
         "...+++########+@$$$$$$$$@+++....",
         ".....+++######+@$$$$$$$+++......",
         "......+++#####+@$$$$$@++........",
         ".......+++####+@$$$$+++.........",
         ".........++###+$$$@++...........",
         "..........++##+$@+++............",
         "...........+++++++..............",
         ".............++++...............">>;
   end -- gtk_xpm
end -- class TOOLBAR
-- example-end
