-- example-start tree tree.e
class TREE
inherit
   GTK_CONSTANTS
   VEGTK_MAIN
   VEGTK_CALLBACK_HANDLER
creation
   make
feature {ANY} -- Creation
   make is
   local
      window : GTK_WINDOW;
      scrolled_win : GTK_SCROLLED_WINDOW;
      tree : GTK_TREE;
      itemnames : ARRAY [STRING];
      i,j : INTEGER;
      subtree : GTK_TREE;
      item, subitem : GTK_TREE_ITEM;
   do
      vegtk_init

      itemnames := <<"Foo", "Bar", "Baz", "Quux","Maurice">>;
      -- a generic toplevel window
      !!window.make (Gtk_window_toplevel);
      signal_connect (window, "destroy",$destroy);

      window.set_border_width (5);

      -- A generic scrolled window
      !!scrolled_win.make (Void, Void);
      scrolled_win.set_policy (Gtk_policy_automatic,
                               Gtk_policy_automatic);
      scrolled_win.set_usize (150, 200);
      window.add (scrolled_win);
      scrolled_win.show;

      -- Create the root tree
      !!tree.make;
      print ("root tree is ");
      print (tree.to_external);
      print ("%N");
      -- connect all GtkTree:: signals
      signal_connect (tree, "select_child",$cb_select_child);
      signal_connect (tree, "unselect_child",$cb_unselect_child);
      signal_connect (tree, "selection_changed",$cb_selection_changed);

      -- Add it to the scrolled window
      scrolled_win.add_with_viewport (tree);
      -- Set the selection mode
      tree.set_selection_mode (Gtk_selection_multiple);
      -- Show it
      tree.show;
      from
         i := 1;
      until
         i > 5
      loop

         -- Create a tree item
         !!item.make_with_label (itemnames.item(i));
         -- Connect all GtkItem:: and GtkTreeItem:: signals
         signal_connect_with_data (item, "select",$cb_itemsignal, "select");
         signal_connect_with_data (item, "deselect",$cb_itemsignal, "deselect");
         signal_connect_with_data (item, "toggle",$cb_itemsignal, "toggle");
         signal_connect_with_data (item, "expand", $cb_itemsignal, "expand");
         signal_connect_with_data (item, "collapse", $cb_itemsignal, "collapse");
         -- Add it to the parent tree
         tree.append (item);
         -- Show it - this can be done at any time
         item.show;
         -- Create this item's subtree
         !!subtree.make;

         print ("-> item ");
         print (itemnames.item(i));
         print ("->");
         print (item.to_external);
         print (", subtree ")
         print (subtree.to_external);
         print ("%N");

         -- This is still necessary if you want these signals to be called
         -- for the subtree's children.  Note that selection_change will be
         -- signalled for the root tree regardless.
         signal_connect (subtree, "select_child",$cb_select_child);
         signal_connect (subtree, "unselect_child",$cb_unselect_child);

         -- This has absolutely no effect, because it is completely ignored
         --   in subtrees
         subtree.set_selection_mode (Gtk_selection_single);
         -- Neither does this, but for a rather different reason - the
         -- view_mode and view_line values of a tree are propagated to
         -- subtrees when they are mapped.  So, setting it later on would
         -- actually have a (somewhat unpredictable) effect */
         subtree.set_view_mode (Gtk_tree_view_item);
         -- Set this item's subtree - note that you cannot do this until
         --  AFTER the item has been added to its parent tree!
         item.set_subtree (subtree);

         from
            j := 1;
         until
            j > 5
         loop

            -- Create a subtree item, in much the same way
            !!subitem.make_with_label (itemnames.item(j));
            -- Connect all GtkItem:: and GtkTreeItem:: signals
            signal_connect_with_data (subitem, "select",$cb_itemsignal, "select");
            signal_connect_with_data (subitem, "deselect",$cb_itemsignal, "deselect");
            signal_connect_with_data (subitem, "toggle",$cb_itemsignal, "toggle");
            signal_connect_with_data (subitem, "expand",$cb_itemsignal, "expand");
            signal_connect_with_data (subitem, "collapse",$cb_itemsignal, "collapse");

            --g_print ("-> -> item %s->%p\n", itemnames[j], subitem);
            print ("-> -> item ");
            print (itemnames.item(j));
            print ("->");
            print (subitem.to_external);
            print ("%N");

            -- Add it to the parent tree
            subtree.append (subitem);
            -- Show it
            subitem.show;
            j := j+1;
         end -- loop
         i := i+1;
      end -- loop
      -- Show the window and loop endlessly
      window.show;
      gtk_main;
   end -- make

feature {NONE} -- Callbacks

   cb_itemsignal (data : ANY; cb_data : VEGTK_CALLBACK_DATA) is
      -- for all the GtkItem:: and GtkTreeItem:: signals
   local
       item : GTK_TREE_ITEM;
       signame : STRING;
       name : STRING;
       label : GTK_LABEL;
       tree : GTK_TREE;
   do
      item ?= cb_data.get_widget;
      signame ?= data;
      -- It's a GtkBin, so it has one child, which we know to be a
      --   label, so get that */
      label ?= item.child;
      -- Get the text of the label
      name := label.get;
      -- Get the level of the tree which the item is in
      print (signame);
      print (" called for item ");
      print (name);
      print ("->")
      print (item.to_external);
      print (", level ");
      tree ?= item.parent;
      print (tree.level);
      print ("%N");
   end

   cb_unselect_child (data : ANY; cb_data : VEGTK_CALLBACK_DATA) is
      -- Note that this is never called
   local
      root_tree,child,subtree : GTK_TREE;
      args : ARRAY[ANY];
   do
      root_tree ?= cb_data.get_widget;
      args := cb_data.get_args;
      child ?= args.item(1);
      subtree ?= args.item(2);

      print ("unselect_child called for root tree ");
      print (root_tree.to_external);
      print (", subtree ");
      print (subtree.to_external)
      print (", child ");
      print (child.to_external);
      print ("%N");
   end

   cb_select_child  (data : ANY; cb_data : VEGTK_CALLBACK_DATA) is
      -- Note that this is called every time the user clicks on an item,
      -- whether it is already selected or not. */
   local
      root_tree : GTK_TREE;
      args : ARRAY[ANY];
      child : GTK_TREE_ITEM;
   do
      root_tree ?= cb_data.get_widget;
      args := cb_data.get_args;
      child ?= args.item(1);
--      subtree ?= args.item(2);

      print ("select_child called for root tree ");
      print (root_tree.to_external);
--      print (", subtree ");
--      print (subtree.to_external)
      print (", child ");
      print (child.to_external);
      print ("%N");
   end

   cb_selection_changed (data : ANY; cb_data : VEGTK_CALLBACK_DATA) is
   local
      tree,subtree : GTK_TREE;
      selection : ARRAY [GTK_TREE_ITEM];
      i : INTEGER;
      name : STRING;
      label : GTK_LABEL;
      item : GTK_BIN;

   do
      tree ?= cb_data.get_widget;

      print ("selection_change called for tree ");
      print (tree.to_external);
      print ("%N");

      print ("selected objects are:%N");

      selection := tree.selection_array;

      if selection /= Void then
         from
            i := selection.lower;
         until
            i > selection.upper
         loop
            item  := selection.item(i);
            label ?= item.child;
            name := label.get;
            print ("%T");
            print (name);
            print (" on level ");
            subtree ?= item.parent;
            print (subtree.level);
            print ("%N");
            i := i+1;
         end -- loop
      end -- if
   end -- cb_selection_changed

   destroy is
   do
      gtk_main_quit;
   end
end -- class TREE
-- example-end
