indexing
   title      : "Interactive data structures builder                         ",
                "Drivers for basic data structure demos                      ";
   cluster    : ISE_Eiffel_Base_Structures_Builder;
   project    : "EiffelBase Examples",
                "Based on EiffelBase examples for ISE Eiffel,                ",
                "used with permission.                                       ";
   copyright  : "(c) 1986-1997 Interactive Software Engineering Inc.         ",
                "Modifications and extensions:                               ",
                "(c) 1997-2002 Object Tools                                  ",
                "All rights reserved. Duplication and distribution prohibited",
                "May be used only with Visual Eiffel under terms of user     ",
                "license                                                     ";
   address    : "mail_to:support@object-tools.com                            ",
                "http://www.object-tools.com                                 ";

class DEMO_DRIVER


inherit

   STD_FILES
      rename
         get_string as STD_FILES_get_string
   end
   BASIC_ROUTINES
   end

creation
   make

feature

-- Constant attributes

   Max_height_of_menu_column: INTEGER is 10 ;
   Max_Number_of_Entries : INTEGER is 40 ;
   Max_Length_of_an_Entry: INTEGER is 75 ;
   Max_Length_of_a_tag   : INTEGER is 2 ;

-- Other attributes

   menu_entry: ARRAY [STRING];
   menu_help : ARRAY [STRING];
   menu_tag  : ARRAY [STRING];
   menu_flag : ARRAY [BOOLEAN];
   line: STRING ;
   comment: STRING ;
   menu_size: INTEGER ;
   last_entry: INTEGER;
   menu_height: INTEGER ;
   choice: INTEGER ;

-- Routines

   make is
      -- Create arrays to store menus.
   do
      -- set_error_default;
      !!menu_entry.make (1, Max_Number_of_Entries);
      !!menu_help.make (1, Max_Number_of_Entries);
      !!menu_tag.make (1, Max_Number_of_Entries);
      !!menu_flag.make (1, Max_Number_of_Entries);
      build_line
   end;

   exit is
      -- Null action in this class.
   do
   end;

   new_menu (new_comment: STRING) is
      -- Create a new menu with new_comment as banner.
   do
      comment := new_comment;
      if comment = Void then
         comment := "No name"
      end;
      choice := 1;
      menu_size := 0
   end;

   add_entry (entry: STRING ; help: STRING) is
      -- Add an entry to the menu; assign its index to last_entry.
      -- The first argument is the name of the entry. If it
      -- contains upper case letters, they will be used as
      -- a tag for recognizing the user's input.
      -- For example, Put_Right will be selected if the user
      -- types in 'pr'. The tag may not be more than
      -- Max_length+of_a_tag letters long .
      -- The string given by the second argument should
      -- explain what the entry does .
   require
      not_too_many_entries: menu_size < Max_Number_of_Entries ;
      name_not_void: entry /= Void ;
      real_help: help /= Void
   do
      menu_size := menu_size + 1 ;
      entry.keep_head (Max_Length_of_an_Entry); -- head (Max_Length_of_an_Entry);
      menu_entry.put (entry, menu_size) ;
      menu_help.put (help, menu_size) ;
      menu_tag.put (get_tag (entry), menu_size) ;
      last_entry := menu_size
   end;

   get_tag (entry: STRING): STRING is
      -- Tag of entry, i.e. upper case letters
   local
      length: INTEGER ;
      char: CHARACTER ;
      i: INTEGER
   do
      from
         length := entry.count;
         !!Result.make_empty -- make (Max_Length_of_a_Tag)
         i := 1
      until
         i > length or Result.count = Max_Length_of_a_Tag
      loop
         char := entry.item (i);
         --if char.is_upper then
         --if char.to_upper = char then
         if char.as_upper = char then
            Result.append_character (char)
         end;
         i := i + 1
      end
   end;

   print_menu is
      -- Print menu.
   local
      i: INTEGER
   do
      put_string (line);
      put_new_line;
      put_string (comment);
      put_new_line;
      put_string ("Select a command by typing the first two letters");
      put_new_line;
      put_string ("in upper- or lower-case (? for help)");
      put_new_line;
      put_string (line);
      put_new_line;
      menu_height := menu_size;
      from
         i := 1
      until
         i > menu_height
      loop
         print_entry (i);
         put_character ('%N') ;
         i := i + 1
      end;
      put_string (line);
      put_new_line
   end;

   print_entry (n: INTEGER) is
      -- Print `n'-th menu item.
   local
      i, length: INTEGER;
      entry: STRING
   do
      entry := menu_entry.item (n);
      length := entry.count;
      from
         i := 1
      until
         i > length
      loop
         put_character (entry.item (i)) ;
         i := i + 1
      end
   end;

   complete_menu is
      -- Add help command to menu
   do
      menu_size := menu_size + 1 ;
      menu_entry.put ("?  Help", menu_size) ;
      menu_help.put ("Sorry, no further help available", menu_size) ;
      menu_tag.put ("?", menu_size) ;
   end;

   get_choice: INTEGER is
      -- Once the menu has been completely defined using
      -- new_menu and add_menu routines, get_choice gets
      -- the user input. The result is the number of
      -- the entry as it has been entered by add_menu
      -- (first entered returns number 1, ...) .
   do
      put_string ("%NCOMMAND: ");
      choice := read_choice ;
      if choice = menu_size then
         print_menu;
         put_string ("%NHELP: ");
         choice := read_choice ;
         show_help;
         choice := get_choice
      end;
      Result := choice
   end;

   bell is
      -- Ring the bell.
   do
      put_string ("%/007/")
   end;

   build_line is
      -- Make a line with `-' characters.
   local
      i: INTEGER
   do
      from
         !!line.make_empty -- make (60)
      until
         i = 60
      loop
         line.append_character ('-');
         i := i + 1
      end
   end;

   show_help is
      -- Display a help message.
   do
      put_string (menu_entry.item (choice));
      put_string (" ==> %N     ");
      put_string (menu_help.item (choice));
      put_new_line
   end;

   get_string (what_is_it: STRING): STRING is
      -- Get a user string input for stdin.
      -- 'what_is_it' is the name of the requested value.
   do
      put_string("Please enter parameter %'") ;
      put_string(what_is_it) ;
      put_string("%' ") ;
      read_line;
      Result := last_string;
   end;

   get_integer (what_is_it: STRING): INTEGER is
      -- Same as get_string but reads an integer.
   do
      Result := get_string (what_is_it).to_integer;
   end;

   signal_error (message: STRING) is
      -- Display the error message in the top of the monitoring
      -- screen and ring a bell.
   do
      bell;
      if message = Void then
         put_string ("This operation is not allowed");
         put_new_line
      else
         put_string (message);
      end
   end;

   read_choice: INTEGER is
      -- Menu item corresponding to user's input
   local
      recognized_entry, number_char_read, i: INTEGER;
      tag: STRING;
      char: CHARACTER;
      bad_char: BOOLEAN
   do
      from
         i := 1
      until
         i > menu_size
      loop
         menu_flag.put (true, i);
         i := i + 1
      end;
      from
         recognized_entry := 0;
         number_char_read := 0
      until
         recognized_entry > 0
      loop
         read_character;
         char := last_character;
         number_char_read := number_char_read + 1;
         --if char.to_lower = char then
         if char.as_lower = char then
            --char := char.to_upper
            char := char.as_upper
         end;
         from
                 bad_char := true; i := 1
         until
                 i > menu_size or recognized_entry > 0
         loop
            if menu_flag.item (i) then
               tag := menu_tag.item (i);
               if tag.item (number_char_read) = char then
                  bad_char := false;
                  if tag.count = number_char_read then
                     recognized_entry := i
                  end
               else
                  menu_flag.put (false, i)
               end
            end;
            i := i + 1
         end;
         if bad_char then
            read_line; tag := last_string;
            number_char_read := 0;
            signal_error ("Bad choice. Try again: ");
            from
               i := 1
            until
               i > menu_size
            loop
               menu_flag.put (true, i);
               i := i + 1
            end
         end
      end;
      read_line;
      tag := last_string;
      Result := recognized_entry
   end;

   start_result is
      -- No default action.
   do
   end; -- start_result

   end_result is
      -- No default action.
   do
   end; -- end_result

end -- class DEMO_DRIVER
