indexing
   title      : "Interactive data structures builder                         ",
                "Demo class for sets                                         ";
   cluster    : ISE_Eiffel_Base_Structures_Builder;
   project    : "EiffelBase Examples",
                "Based on EiffelBase examples for ISE Eiffel,                ",
                "used with permission.                                       ";
   copyright  : "(c) 1986-1997 Interactive Software Engineering Inc.         ",
                "Modifications and extensions:                               ",
                "(c) 1997 Object Tools                                       ",
                "All rights reserved. Duplication and distribution prohibited",
                "May be used only with Visual Eiffel under terms of user     ",
                "license                                                     ";
   address    : "mail_to:support@object-tools.com                            ",
                "http://www.object-tools.com                                 ";

class SET_DEMO

inherit
   TOP_DEMO
      redefine
         cycle, execute, fill_menu
   end

creation
   make

feature

   Wipe_out, Empty, Item_count: INTEGER;
   Remove, Has, Put, Intersect: INTEGER;
   Merge, Subtract, Is_superset, Is_subset: INTEGER;
   Show, Quit: INTEGER;

   a, b, c:  MLS;

   make is
         -- Initialize and execute demonstration
      do
         !!driver.make
         driver.new_menu ("%N%N* LINKED SET DEMO*%N%N[XX] shows current item %N");
         fill_menu;
         !!a.make;
         !!b.make;
         !!c.make;
         cycle
      end;

   cycle is
         -- Basic user interaction process.
      local
         new_command: INTEGER
      do
         from
            driver.print_menu;
            driver.put_string (" ");
            driver.put_new_line;
            sets_trace;
            new_command := driver.get_choice
         until
            new_command = Quit
         loop
            execute (new_command);
            driver.put_new_line;
            driver.start_result;
            sets_trace;
            driver.end_result;
            new_command := driver.get_choice;
         end;
         driver.exit;
      end; -- cycle

   sets_trace is
         -- Display the 3 sets.
      do
         driver.put_string ("a:");
         a.display;
         driver.put_string ("b:");
         b.display;
         driver.put_string ("c:");
         c.display;
      end;

   fill_menu is
      do
         driver.add_entry ("PU (PUt): Insert item in set", "Put item in the set");
         Put := driver.last_entry;
         driver.add_entry ("IN (INtersection): Intersect with other set", "Remove from set1 all items not in set2");
         Intersect := driver.last_entry;
         driver.add_entry ("UN (UNion): Union with other set", "Add to set1 all items in set2");
         Merge := driver.last_entry;
         driver.add_entry ("SU (SUbtract): Subtract other set", "Remove from set1 all items in set2");
         Subtract := driver.last_entry;
         driver.add_entry ("RM (ReMove): Remove an item",  "Remove an item from the set");
         Remove :=driver.last_entry;
         driver.add_entry ("WO (Wipe Out): Empty the set", "Make the set empty");
         Wipe_out := driver.last_entry;
         driver.add_entry ("SP (SuPerset): Show if superset of other set", "Is set1 a superset of set2?");
         Is_superset := driver.last_entry;
         driver.add_entry ("SB (Subset): Show if subset of other set", "Is set1 a subset of set2?");
         Is_subset := driver.last_entry;
         driver.add_entry ("HA (Has): Test if item belongs to set", "Is item in the set?");
         Has := driver.last_entry;
         driver.add_entry ("CO (COunt): Show number of items in set", "Display number of items");
         Item_count := driver.last_entry;
         driver.add_entry ("EM (EMpty): Test if set is empty", "Is the set empty?");
         Empty := driver.last_entry;
         driver.add_entry ("SH (SHow): Show contents of sets", "Display contents of set");
         Show := driver.last_entry;
         driver.add_entry ("QU (QUit)", "Terminate this session");
         Quit := driver.last_entry;
         driver.complete_menu
      end;

   execute (new_command: INTEGER) is
         -- Execute command corresponding of user's request.
      require else
         new_command >= Put;
         new_command <= Quit
      local
         set1, set2: like a;
         element: INTEGER;
      do
         -- parse and perform action
         if new_command = Wipe_out then
            set1 := get_set;
            if set1 /= Void then
               set1.wipe_out
            end
         elseif new_command = Empty then
            set1 := get_set;
            if set1 /= Void then
               driver.put_boolean (set1.empty)
            end
         elseif new_command = Item_count then
            set1 := get_set;
            if set1 /= Void then
               driver.put_integer (set1.count)
            end
         elseif new_command = Remove then
            set1 := get_set;
            if set1 /= Void then
               element := driver.get_integer ("item");
               set1.prune (element)
            end
         elseif new_command = Has then
            set1 := get_set;
            if set1 /= Void then
               element := driver.get_integer ("item");
               driver.put_boolean (set1.has (element))
            end
         elseif new_command = Put then
            set1 := get_set;
            if set1 /= Void then
               element := driver.get_integer ("item");
               set1.put (element)
            end
         elseif new_command = Intersect then
            set1 := get_set;
            if set1 /= Void then
               set2 := get_set;
               if set2 /= Void then
                  set1.intersect (set2)
               end
            end
         elseif new_command = Merge then
            set1 := get_set;
            if set1 /= Void then
               set2 := get_set;
               if set2 /= Void then
                  set1.merge (set2)
               end
            end
         elseif new_command = Subtract then
            set1 := get_set;
            if set1 /= Void then
               set2 := get_set;
               if set2 /= Void then
                  set1.subtract (set2)
               end
            end
         elseif new_command = Is_superset then
            set1 := get_set;
            if set1 /= Void then
               set2 := get_set;
               if set2 /= Void then
                  driver.put_boolean (set1.is_superset (set2))
               end
            end
         elseif new_command = Is_subset then
            set1 := get_set;
            if set1 /= Void then
               set2 := get_set;
               if set2 /= Void then
                  driver.put_boolean (set1.is_subset (set2))
               end
            end
         elseif new_command /= Show then
            driver.signal_error ("Unknown command")
         end;
      end;

   get_set: like a is
      local
         s: STRING
      do
         s := driver.get_string ("which set");
         if s.is_equal ("a") then
            Result := a
         elseif s.is_equal ("b") then
            Result := b
         elseif s.is_equal ("c") then
            Result := c
         else
            driver.signal_error ("Unknown set")
         end
      end

end -- class SET_DEMO
