indexing
   title      : "Interactive data structures builder                         ";
   cluster    : ISE_Eiffel_Base_Structures_Builder;
   project    : "EiffelBase Examples",
                "Based on EiffelBase examples for ISE Eiffel,                ",
                "used with permission.                                       ";
   copyright  : "(c) 1986-1997 Interactive Software Engineering Inc.         ",
                "Modifications and extensions:                               ",
                "(c) 1997 Object Tools                                       ",
                "All rights reserved. Duplication and distribution prohibited",
                "May be used only with Visual Eiffel under terms of user     ",
                "license                                                     ";
   address    : "mail_to:support@object-tools.com                            ",
                "http://www.object-tools.com                                 ";

class STRUCTURES

inherit
   EXCEPTIONS;

creation
   make

feature

   a_two_way_tree_demo: TWO_WAY_TREE_DEMO;
   a_sort_demo: SORT_DEMO;
   a_binary_search_tree_demo: BINARY_SEARCH_TREE_DEMO;
   a_binary_tree_demo: BINARY_TREE_DEMO;
   --a_priority_queue_demo: PRIORITY_QUEUE_DEMO;
   a_set_demo: SET_DEMO;
   a_sorted_set_demo: SORTED_SET_DEMO;


   chosen_demo: TOP_DEMO;
   saved_demo_file: RAW_FILE;

   saved_file_name: STRING is "demo.S";

   make is
     do
        --no_message_on_failure;
        io.put_string ("%N%N     * Class demos from the Eiffel Data Structure Library *%N%N");
        session;
     end;

   one_demo is
         -- Execute a new session, possibly from a previously saved one.
         -- Then store it if requested.
      local
         resume_session: BOOLEAN
      do
         !!saved_demo_file.make (saved_file_name);
         if saved_demo_file.exists then
            io.put_string ("Do you want to retrieve the last saved demo (y/n)?: ");
            io.read_character
            if io.last_character = 'y' then
               !!chosen_demo;
               saved_demo_file.open_read
               chosen_demo ?= chosen_demo.retrieved (saved_demo_file);
               saved_demo_file.close
               io.to_next_line
               chosen_demo.cycle;
               resume_session := true
            else
               io.to_next_line
            end;
         end;
         if not resume_session then
            new_demo
         end;
         if saved_demo_file.is_creatable then
            io.put_new_line;
            io.put_string ("Save this demo to be resumed later? (y/n): ");
            io.read_character
            if io.last_character = 'y' then
               -- saved_demo_file.open_write;
               saved_demo_file.create_read_write;
               chosen_demo.general_store (saved_demo_file);
               saved_demo_file.close
               io.put_string ("%TDemo saved%N")
            end;
            io.to_next_line
         end
      end;

   session is
         -- Execute one or more demo sessions.
      local
         over: BOOLEAN
      do
         from
         until
            over
         loop
            one_demo;
            io.put_new_line;
            io.put_string ("Another demo? (y/n): ");
            io.read_character
            over := (io.last_character /= 'y')
            io.to_next_line
         end
      end;

   print_menu is
         -- Display all possible choices.
      do
         io.put_string (" 1: Sorted lists.");
         io.put_new_line;
         io.put_string (" 2: General trees.");
         io.put_new_line;
         io.put_string (" 3: Binary trees.");
         io.put_new_line;
         io.put_string (" 4: Binary search trees.");
         io.put_new_line;
         io.put_string (" 5: Priority queues.");
         io.put_new_line;
         io.put_string (" 6: Sets.");
         io.put_new_line;
         io.put_string (" 7: Sorted sets.");
         io.put_new_line;
         io.put_string ("");
         io.put_new_line;
         io.put_string ("Please choose demo number: ");
      end;

   new_demo is
         -- Set up a new demo session.
      local
         choice: INTEGER
      do
         print_menu;
         io.read_integer
         choice := io.last_integer
         inspect
            choice
         when 1 then
            !!a_sort_demo.make;
            chosen_demo := a_sort_demo;
         when 2 then
            !!a_two_way_tree_demo.make;
            chosen_demo := a_two_way_tree_demo
         when 3 then
            !!a_binary_tree_demo.make;
            chosen_demo := a_binary_tree_demo
         when 4 then
            !!a_binary_search_tree_demo.make;
            chosen_demo := a_binary_search_tree_demo
         when 5 then
            io.put_string ("This is temporarily disabled%N");
            --!!a_priority_queue_demo.make;
            --chosen_demo := a_priority_queue_demo
         when 6 then
            !!a_set_demo.make;
            chosen_demo := a_set_demo
         when 7 then
            !!a_sorted_set_demo.make;
            chosen_demo := a_sorted_set_demo
         else
            io.put_string ("Demo number should be between 1 and 7")
         end
      end;

end -- class STRUCTURES
