indexing
	description:
		"Test class to show different features of %
		%the regular expression library. "

class TEST

creation
	make

feature {NONE} -- Initialization

	make is
		do
			test_splitter
			test_extractor
			test_misspelled
			test_regexp_extractor
		end

feature {NONE} -- Implementation

	splitter: SPLITTER

feature -- Test features

	test_splitter is
			-- Demonstrate the use of the SPLITTER class.
		local
			r: REGEXP
			i, max: INTEGER
		do
			print ("Testing the splitter...%N")
			!! r.make ("{b}+")

			!! splitter.make (r)
			splitter.ignore_separators (True, True)

			splitter.split ("this is a splitter example")

			from
				i := splitter.lower
				max := splitter.upper
			until
				i > max
			loop
				print ("Word at position ")
				print (i)
				print (": ")
				print (splitter.item (i))
				print ("%N")
				i := i + 1
			end
			print ("Done.%N")
		end

	test_extractor is
			-- Demonstrate the use of the WILDCARD_EXTRACTOR class.
		local
			w: WILDCARD_EXTRACTOR
			i, max: INTEGER
			a_string: STRING
		do
			print ("Testing the wildcard extractor...%N")
			!! w.make ("*i*")
			if w.compiled then
				from
					i := splitter.lower
					max := splitter.upper
				until
					i > max
				loop
					a_string := splitter.item (i)
					w.match (a_string)
					if w.found then
						print ("Matched: ")
						print (a_string)
						print ("%N")
					end
					i := i + 1
				end
			end
			print ("Done.%N")
		end

	test_misspelled is
			-- Demonstrate the use of the MISSPELLED class.
		local
			m: MISSPELLED
			start_pos, end_pos: INTEGER
			search_string: STRING
			done: BOOLEAN
			s: SUBSTRING
			pattern: STRING
		do
			print ("Testing the substring and misspelled classes...%N")

			pattern := "misspelled"
			search_string := "Looking for misspelled, possibly misppelled"

			!! m.make_insensitive_misspelled (pattern, 2)
			from
				start_pos := 1
				end_pos := search_string.count
			until
				done
			loop
				m.search_forward (search_string, start_pos, end_pos)
				if m.found then
					print ("Found %"")
					print (search_string.substring (m.start_position, m.end_position))
					print ("%" at position ")
					print (m.start_position)
					print (" with ")
					print (m.mismatches)
					print (" mismatch(es).%N")

					start_pos := m.end_position + 1
				else
					done := True
				end
			end

			!! s.make (pattern)
			from
				start_pos := 1
				end_pos := search_string.count
				done := False
			until
				done
			loop
				s.search_forward (search_string, start_pos, end_pos)
				if s.found then
					print ("Found %"")
					print (search_string.substring (s.start_position, s.end_position))
					print ("%" at position ")
					print (s.start_position)
					print (".%N")

					start_pos := s.end_position + 1
				else
					done := True
				end
			end

			print ("Done.%N")
		end

	test_regexp_extractor is
			-- Demonstrate the use of the REGEXP_EXTRACTOR class.
		local
			r: REGEXP_EXTRACTOR
			s, host: STRING
			i, port: INTEGER
			a: ARRAY [STRING]
		do
			print ("Testing the regexp extractor...%N")

				-- Pattern will match "<hostname>:<port_number>"
			!!r.make("({l}{a}*):({d}+)");

			a := << "triton:234", "foo:d222", "thalassa:54", "bar:3434s">>

			from
				i := a.lower
			until
				i > a.upper
			loop
				s := a.item (i)
				r.match(s)
				if not r.found then
					print ("Invalid entry: %"")
					print (s)
					print ("%".%N")
				else
					host := r.matched_substring(1)
					port := r.matched_substring(2).to_integer

					print ("Host: ")
					print (host)
					print (", port: ")
					print (port)
					print (".%N")
				end
				i := i + 1
			end

			print ("Done.%N")
		end

end
