indexing

   title:      "Universal facilities of the guide for class LIST_";
   cluster:    "containers";
   project:    "Visual Eiffel";
   date:       "$Date: 2005/02/15 16:21:21 $";
   revision:   "$Revision: 1.2 $";
   copyright:  "Copyright (C) 1996-2005 Object Tools Group";
   license:    "http://visual-eiffel.com/license";
   disclaimer: "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

deferred class CHAIN_CORE_

   inherit
      SIMPLE_CHAIN_CORE_
         redefine
            put, remove
         end


   feature {SIMPLE_CONTAINER_} -- Operations

      put, frozen chain_put (pos: INTEGER) is
            -- puts the new entry in order of the guide
         do
            if entrance = 0 then
               next.put (- pos, pos)
               prev.put (out_marker, pos)
            else
               next.put (next.item (entrance), pos)
               next.put (pos, entrance)
               check
                  positive: next.item (entrance) > 0
               end
               prev.put (entrance, pos)
            end

            entrance := pos

            check
               negative: next.item (entrance) < 0
            end
         end

      remove, frozen chain_remove (pos: INTEGER) is
            -- brings the entry out of the order of the guide
         local
            prev_pos: INTEGER

         do
            prev_pos := prev.item (pos)

            check
               proper_prev:
                  (pos = - next.item (entrance) implies prev_pos <= 0) and then
                  (pos /= - next.item (entrance) implies prev_pos > 0)
            end

            if prev_pos <= 0 then
               -- the first item is removing ...

               if entrance /= pos then
                  -- there are some other items in the list

                  next.put (- next.item (pos), entrance)
                  prev.put (out_marker, next.item (pos))

               else
                  -- the first item is the only item in the list

                  entrance := 0
               end

            else
               -- not the first item is removing ...

               if entrance = pos then
                  -- the last item is to be removed

                  next.put (next.item (pos), prev_pos)
                  entrance := prev_pos

               else
                  -- not the last item is to be removed

                  next.put (next.item (pos), prev_pos)
                  prev.put (prev_pos, next.item (pos))
               end
            end

            put_in_stock (pos)
         end


   feature {SIMPLE_CONTAINER_} -- Queries

      back (pos: INTEGER): INTEGER is
            -- gives the position of the previous (about 'pos') entry
         require
            proper_position: pos > 0 and then pos <= capacity
            not_empty:       last > 0

         deferred
         ensure
            well_done: Result >= 0 and then Result <= capacity
         end

      last: INTEGER is
            -- gives the position of the last entry
         require
            real_pool: capacity > 0

         deferred
         ensure
            well_done: Result >= 0 and then Result <= capacity
         end


   feature {NONE} -- Implementation

      prev: ARRAY [INTEGER]; -- references to the previous entries

      out_marker: INTEGER is
            -- gives the non-positive value of the end marker (the reference
            -- to the previous entry of the first entry ...)
         deferred
         end


   invariant

      non_positive: out_marker <= 0

end
