indexing

   title:      "The supplier of the USCL specific error handling facilities";
   cluster:    "Containers";
   project:    "Visual Eiffel";
   date:       "$Date: 2005/02/15 16:21:21 $";
   revision:   "$Revision: 1.2 $";
   copyright:  "Copyright (C) 1996-2005 Object Tools Group";
   license:    "http://visual-eiffel.com/license";
   disclaimer: "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

class CONTAINER_ERROR_HANDLER_

   inherit
      ERROR_HANDLER_


   feature -- Operations

      stop (first, second: ANY; routine_name: STRING; error_code: INTEGER) is
            -- builds the proper "where it happened" report depending on
            -- 'first' and 'second' for subsequent calling 'err_report' to
            -- build the error report in full and stop the execution soon after
            --
         require
            valid_object:  first /= Void
            valid_routine: routine_name /= Void
            proper_error:  error_code > 0 and then
                           error_code <= sc_max_error_code
         local
            src_a, src_b: STRING
            injn1, injn2: CHARACTER

         do
            if first /= Void then
               src_a := first.generating_type
            end

            if second /= Void then
               src_b := second.generating_type
            end

            if src_a = Void or else src_a.count = 0 then
               src_a := unspecified
            end

            if src_b /= Void and then src_b.count > 0 then
               injn1 := '('
               injn2 := ')'
            end

            if not injn1.is_equal (injn1.default) then
               src_a.append_character (injn1)
               src_a.append_string (src_b)
               src_a.append_character (injn2)
            end

            src_a.append_character ('.')

            if routine_name /= Void then
               src_a.append_string (routine_name)
            else
               src_a.append_string (unspecified)
            end

            err_report (src_a, error_messages_.item (error_code),
                                                     error_code, first)
         end


   feature -- Queries

      sc_is_empty: INTEGER is 1
      sc_is_full: INTEGER is 2
      sc_unexpected: INTEGER is 3
      sc_is_protected: INTEGER is 4
      sc_key_not_found: INTEGER is 5
      sc_item_not_found: INTEGER is 6
      sc_key_duplication: INTEGER is 7
      sc_item_duplication: INTEGER is 8
      sc_loss_of_data: INTEGER is 9
      sc_out_of_range: INTEGER is 10

      sc_max_error_code: INTEGER is 10; -- the constant should be equal the
                                          -- biggest one in the list above
                                          -- even after any possible change ...
      error_messages_: ARRAY [STRING] is
         once
            Result := << "container is empty",
                         "container is full",
                         "unexpected error",
                         "container is protected",
                         "key is not found",
                         "item is not found",
                         "key duplucation",
                         "item duplication",
                         "possible loss of data",
                         "cursor is out of range" >>
         ensure
            well_done: Result.count = sc_max_error_code
         end


   feature {NONE} -- Implementation

      unspecified: STRING is once Result := "???" end

end
