indexing

   title:      "Hash lists: containers which store its one element entries",
               "widely using their ability to be associated with integers";
   cluster:    "containers";
   project:    "Visual Eiffel";
   date:       "$Date: 2005/02/15 16:21:21 $";
   revision:   "$Revision: 1.2 $";
   copyright:  "Copyright (C) 1996-2005 Object Tools Group";
   license:    "http://visual-eiffel.com/license";
   disclaimer: "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

class HASH_LIST_[G -> HASHABLE]

   inherit
      LIST_[G]
         rename
            make as list_make
         export {NONE}
            list_make
         redefine
            default_size, extend, find_first, find_next,
            get_counterweight, guide, list_make
         end

   creation
      make


   feature -- Operations

      make (is_unique: BOOLEAN; initial_capacity: INTEGER) is
            -- tells the container if its entries must be unigue and sets
            -- the initial amount of slots to be allocated for storing entries
         require
            proper_initial_slots: initial_capacity > 0

         do
            unique_items := is_unique
            if store = Void then
               default_size := initial_capacity
            end

            !!store.make (1, initial_capacity)
            !!guide.make (Current, initial_capacity)
         end


   feature {NONE} -- Implementation

      default_size: INTEGER; -- the amount of slots allocated at creation time

      guide: HASHING_ON_ITEMS_[G]; -- the ordering structure of the container

      extend (new_capacity: INTEGER) is
            -- allocates memory for new slots
         do
            if new_capacity = 0 then
               if not no_reallocation then
                  -- ...
                  restructure (capacity * 2)
               end
            else
               restructure (new_capacity)
            end
         end

      get_counterweight (new_capacity: INTEGER): INTEGER is
            -- defines a new counterweight for a new 'make' call ...
         do
           Result := new_capacity - default_size
         end

      list_make (is_unique: BOOLEAN) is
            -- ...
         do
            make (is_unique, default_size + counterweight)
         end

      find_first (x: G): INTEGER is
            -- finds the first copy of the entry in the list
         do
            Result := guide.search (x)
         end

      find_next (x: G; pos: INTEGER): INTEGER is
            -- finds the duplicates of the entry which is stored at 'pos'
            -- traversal is towards the end of the list
         local
            done: BOOLEAN
            item: G

         do
            from
               Result := pos
            until
               Result = 0 or else done
            loop
               Result := guide.forth (Result)

               if Result > 0 then
                  item := store.item (Result)
                  if x.hash_code /= item.hash_code then
                     Result := 0
                  elseif x.is_equal (item) then
                     done := true
                  end
               end
            end
         end

end
