indexing

   title:      "Hash tables: containers which store its two element entries",
               "(element or item itself and the associated key) widely",
               "using the ability of keys to be transformed into integers";
   cluster:    "containers";
   project:    "Visual Eiffel";
   date:       "$Date: 2005/02/15 16:21:21 $";
   revision:   "$Revision: 1.2 $";
   copyright:  "Copyright (C) 1996-2005 Object Tools Group";
   license:    "http://visual-eiffel.com/license";
   disclaimer: "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

class HASH_TABLE_[G, H -> HASHABLE]

   inherit
      TABLE_[G, H]
         rename
            make as table_make
         export {NONE}
            table_make
         redefine
            default_size, extend, find_next,
            get_counterweight, guide, table_make
         end

   creation
      make


   feature -- Operations

      make (is_unique: BOOLEAN; initial_capacity: INTEGER) is
            -- tells the container if its entries must be unigue and sets
            -- the initial amount of slots to be allocated for storing entries
         require
            proper_initial_slots: initial_capacity > 0

         do
            unique_keys := is_unique
            if store = Void then
               default_size := initial_capacity
            end
            !!store.make (1, initial_capacity)
            !!guide.make (Current, initial_capacity)
            count := 0
         end


   feature {NONE} -- Implementation

      default_size: INTEGER; -- the amount of slots allocated at creation time

      guide: HASHING_WITH_KEYS_[H]; -- the ordering structure of the container

      extend (new_capacity: INTEGER) is
            -- allocates memory for new slots
         do
            if new_capacity = 0 then
               if not no_reallocation then
                  -- ...
                  restructure (capacity * 2)
               end
            else
               restructure (new_capacity)
            end
         end

      find_next (k: H; pos: INTEGER): INTEGER is
            -- finds the next entry (the current one is storing at 'pos')
            -- the element or item of which is associated with key 'k'
            -- traversal is towards the end of the table
         local
            done: BOOLEAN
            key:  H

         do
            from
               Result := pos
            until
               Result = 0 or else done
            loop
               Result := guide.forth (Result)

               if Result > 0 then
                  key := guide.get_key (Result)
                  if k.hash_code /= key.hash_code then
                     Result := 0
                  elseif k.is_equal (key) then
                     done := true
                  end
               end
            end
         end

      get_counterweight (new_capacity: INTEGER): INTEGER is
            -- defines a new counterweight for a new 'make' call ...
         do
           Result := new_capacity - default_size
         end

      table_make (is_unique: BOOLEAN) is
            -- ..
         do
            make (is_unique, default_size + counterweight)
         end

end
