indexing

   title:      "Some universal common facilities of the guides for classes",
               "HASH_LIST_ and HASH_TABLE_";
   cluster:    "containers";
   project:    "Visual Eiffel";
   date:       "$Date: 2005/02/15 16:21:21 $";
   revision:   "$Revision: 1.2 $";
   copyright:  "Copyright (C) 1996-2005 Object Tools Group";
   license:    "http://visual-eiffel.com/license";
   disclaimer: "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

deferred class HASHING_[G -> HASHABLE]

   inherit
      POOL_[G]
         redefine
            count, first, forth, put, remove, extend
         end


   feature {SIMPLE_CONTAINER_} -- Operations

      extend (increment: INTEGER) is
            -- ... allocating memory for new slots is forbidden here !!!
         once
            target.halt (Current, target.f_extend)
         end

      put (pos: INTEGER) is
            -- puts the new entry in order of the guide
         local
            pos_code: INTEGER

         do
            pos_code := get_key (pos).hash_code \\ capacity + 1
            entrance := entry.item (pos_code)

            check
               proper_entrance: entrance >= 0 and then entrance <= capacity
            end

            chain_put (pos)
            entry.put (entrance, pos_code)
            count := count + 1
         end

      remove (pos: INTEGER) is
            -- brings the entry out of the order of the guide
         local
            pos_code: INTEGER

         do
            pos_code := get_key (pos).hash_code \\ capacity + 1
            entrance := entry.item (pos_code)

            check
               proper_entrance: entrance >= 0 and then entrance <= capacity
            end

            chain_remove (pos)
            entry.put (entrance, pos_code)
            count := count - 1
         end


   feature {SIMPLE_CONTAINER_} -- Queries

      count: INTEGER; -- the number of used slots

      search (x: G): INTEGER is
            -- gives the position of the entry if it exists
         do
            entrance := entry.item (x.hash_code \\ capacity + 1)

            from
               Result := chain_first
            invariant
               proper_step: Result >= 0 and then Result <= capacity
            until
               Result = 0 or else x.is_equal (get_key (Result))
            loop
               Result := chain_forth (Result)
            end
         end

      first: INTEGER is
            -- gives the position of the first entry
         do
            Result := find_down (1)
         end

      forth (pos: INTEGER): INTEGER is
            -- gives the position of the next (about 'pos') entry
         do
            Result := next.item (pos)
            if Result < 0 then
               Result := find_down (get_key (pos).hash_code \\ capacity + 2)
            end
         end

      last: INTEGER is
            -- gives the position of the last entry
         do
            Result := find_up (capacity)
         end

      back (pos: INTEGER): INTEGER is
            -- gives the position of the previous (about 'pos') entry
         do
            Result := prev.item (pos)
            if Result = 0 then
               Result := find_up (get_key (pos).hash_code \\ capacity)
            end
         end


   feature {NONE} -- Implementation

      entry: ARRAY [INTEGER]; -- the index of entry points ...

      find_down (slot: INTEGER): INTEGER is
            -- finds the next slot used
         require
            proper_slot: slot > 0 and then slot <= capacity + 1
         local
            pos: INTEGER

         do
            if count /= 0 and then slot <= capacity then

               from
                  pos := slot
               invariant
                  inside_the_range: pos > 0 and then pos <= capacity + 1
               variant
                  proper_step: capacity + 1 - pos
               until
                  pos > capacity or else entry.item (pos) /= 0
               loop
                  pos := pos + 1
               end

               if pos <= capacity then
                  entrance := entry.item (pos)
                  Result := chain_first
               end
            end

         ensure
            well_done: Result >= 0 and then Result <= capacity
         end

      find_up (slot: INTEGER): INTEGER is
            -- finds the previous slot used
         require
            proper_slot: slot >= 0 and then slot <= capacity
         local
            pos: INTEGER

         do
            if count /= 0 and then slot > 0 then

               from
                  pos := slot
               invariant
                  pos >= 0 and then pos <= capacity
               variant
                  capacity - (capacity - pos)
               until
                  pos = 0 or else entry.item (pos) /= 0
               loop
                  pos := pos - 1
               end

               if pos > 0 then
                  Result := entry.item (pos)
               end
            end

         ensure
            well_done: Result >= 0 and then Result <= capacity
         end

      out_marker: INTEGER is
            -- gives zero once and forever ...
         once
         end

      resizing (new_size: INTEGER) is
            -- ... changing the amount of slots allocated is forbidden here !!
         once
            target.halt (Current, "resizing")
         end

      subsequent_init (size: INTEGER) is
            -- initializes the internal arrays other than 'next' or 'right' ...
         do
            !!prev.make (1, size)
            !!entry.make (1, size)
         end


end
