indexing

   title:      "Alternative facilities for traversing the containers";
   cluster:    "containers";
   project:    "Visual Eiffel";
   date:       "$Date: 2005/02/15 16:21:21 $";
   revision:   "$Revision: 1.2 $";
   copyright:  "Copyright (C) 1996-2005 Object Tools Group";
   license:    "http://visual-eiffel.com/license";
   disclaimer: "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

deferred class ITERATOR_


   feature -- Operations

      action is
            -- you can redefine this procedure to put all the operations
            -- which should be done on every entry of the container being
            -- traversed; the current entry is accessible through queries
            -- 'item' and 'key' - the latter one is only for the tables
            --
         require
            proper_target: target /= Void
         do
            -- do nothing by default ...
         end

      do_all is
            -- procedure 'action' will be performed on every entry of the
            -- container being traversed unconditionally
         require
            proper_target: target /= Void
         do
            if target /= Void  then
               from
                  cs := target.cursor
               until
                  cs.is_finished
               loop
                  action
                  cs.forth
               end
            end
         end

      do_if is
            -- procedure 'action' will be performed on every entry of the
            -- container being traversed if it satisfies query 'test'
            -- "testing" is performing on every entry automatically ...
         require
            proper_target: target /= Void
         do
            if target /= Void  then
               from
                  cs := target.cursor
               until
                  cs.is_finished
               loop
                  if test then
                     action
                  end
                  cs.forth
               end
            end
         end

      do_until is
            -- procedure 'action' will be performed on every entry of the
            -- container being traversed - one after another - until query
            -- 'test' gives true; "testing" is performing on every entry
            -- automatically ...
            --
         require
            proper_target: target /= Void
         do
            if target /= Void  then
               from
                  cs := target.cursor
               until
                  test or else cs.is_finished
               loop
                  action
                  cs.forth
               end
            end
         end

      do_while is
            -- procedure 'action' will be performed on every entry of the
            -- container being traversed - one after another - while query
            -- 'test' is giving true; "testing" is performing on every entry
            -- automatically ...
            --
         require
            proper_target: target /= Void
         do
            if target /= Void  then
               from
                  cs := target.cursor
               until
                  not test or else cs.is_finished
               loop
                  action
                  cs.forth
               end
            end
         end


   feature -- Queries

      target: TRAVERSABLE_ is
            -- the container is being traversed
         deferred
         end

      test: BOOLEAN is
            -- you can redefine this query to put the conditions which should
            -- be applied to every entry of the container - one after another
            -- the current entry is accessible through queries 'item' and
            -- 'key' - the latter one is only for the tables
            --
         require
            proper_target: target /= Void
         do
            Result := true
         end


   feature {NONE} -- Implementation

      cs: CURSOR_

end
