indexing

   title:      "Lists: containers with one element entries",
               "implementing a first-in, first-out policy and";
               "supporting traversal in the both directions";
   cluster:    "containers";
   project:    "Visual Eiffel";
   date:       "$Date: 2005/02/15 16:21:21 $";
   revision:   "$Revision: 1.2 $";
   copyright:  "Copyright (C) 1996-2005 Object Tools Group";
   license:    "http://visual-eiffel.com/license";
   disclaimer: "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

class LIST_[G]

   inherit
      SIMPLE_LIST_[G]
         undefine
            cursor
         redefine
            item_at, cursor_at, guide, make
         end

      TRAVERSABLE_
         end

   creation
      make


   feature -- Operations

      make (is_unique: BOOLEAN) is
            -- tells the container if its entries must be unigue
         local
            init_size: INTEGER

         do
            unique_items := is_unique
            init_size := default_size + counterweight

            !!store.make (1, init_size)
            !CHAIN_!guide.make (Current, init_size)
         end


   feature -- Queries

      item_at (cs: CURSOR_): G is
            -- delivers the entry of the list the cursor is positioned to
         do
            Result := store.item (cs.position)
         end

      cursor_at (x: G): CURSOR_ is
            -- delivers a new cursor positioned to the entry; if there is no
            -- such entry currently stored in the list Void is delivered
         local
            pos: INTEGER

         do
            pos := find_first (x)

            if pos /= 0 then
               !!Result.make (Current)
               Result.set_position (pos)
               Result.activate
            end
         end


   feature {CURSOR_}

      last (cs: CURSOR_) is
            -- positions on the last item
         do
            if not empty then
               cs.set_position (guide.last)
            else
               cs.stop
            end

         ensure then
            well_done: not empty implies has_item (item_at (cs))
         end

      back (cs: CURSOR_) is
            -- moves one item backwards
         local
            pos: INTEGER

         do
            if not cs.is_finished then
               pos := guide.back (cs.position)
               if pos /= 0 then
                  cs.set_position (pos)
               else
                  cs.stop
               end

            else
               errhlg.stop (Current, cs, f_back, errhlg.sc_out_of_range)
            end

         ensure then
            well_done: not cs.is_finished implies has_item (item_at (cs))
         end


   feature {STACK_} -- Implementation

      remove_last is
            -- deletes the last entry of the list
         local
            index: INTEGER

         do
            if is_protected or else empty then
               fault (f_remove_last, errhlg.sc_unexpected)
            else
               index := guide.last
               guide.remove (index)
               store.put (species_default, index)
               count := count - 1
               shrink (0)
            end
         end


   feature {NONE} -- Implementation

      guide: CHAIN_CORE_ -- the ordering structure of the container

end
