indexing

   title:      "Priority queues: dispensers with one element entries",
               "in which only the largest element is accessed";
   cluster:    "containers";
   project:    "Visual Eiffel";
   date:       "$Date: 2005/02/15 16:21:22 $";
   revision:   "$Revision: 1.2 $";
   copyright:  "Copyright (C) 1996-2005 Object Tools Group";
   license:    "http://visual-eiffel.com/license";
   disclaimer: "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

class PRIORITY_QUEUE_[G -> COMPARABLE]

   inherit
      DISPENSER_[G]
         redefine
            pool
         end


   feature -- Operations

      put (x: G) is
            -- adds the new entry into the dispenser
         require else
            valid_item:  x /= Void
         do
            if pool = Void then
               !!pool.make (false)
               cs := pool.cursor
            end

            check cs.is_finished end

            pool.put (x)
            item := pool.last_head
         end

      remove is
            -- deletes the "active entry" and sets 'item' to the largest one
            -- among those still existing in the dispenser after the removal
            -- if there are no more entries then 'item' is set to Void
         local
            tail: G

         do
            if not empty then
               cs.last
               tail := pool.item_at (cs)
               cs.stop
               pool.remove (tail)
               item := pool.last_head
            else
               fault (f_remove, errhlg.sc_is_empty)
            end

         ensure then
            well_done: cs.is_finished
         end


   feature -- Queries

      list_of_elements: LIST_[G] is
            -- delivers all the entries of the dispenser in the order in
            -- which they would be repeatedly removed if it really happened
         local
            local_stack:   STACK_[G]
            current_item:  G
            pop_stack_out,
            push_on_stack: BOOLEAN

         do
            !!Result.make (false)
            if not empty then
               check pool /= Void and then count > 0 end

               from
                  !!local_stack
                  Result.allocate_manually (count)
                  cs.last
               until
                  Result.count = count
               loop
                  push_on_stack := true
                  pop_stack_out := false

                  if cs.is_finished then
                     pop_stack_out := true
                     push_on_stack := false
                  else
                     current_item := pool.item_at (cs)
                     cs.back

                     -- all duplicates are collected on 'local_stack' first
                     if not (local_stack.empty or else
                             current_item.is_equal (local_stack.item)) then

                        pop_stack_out := true
                     end
                  end

                  if pop_stack_out then
                     check not local_stack.empty end

                     from
                     until
                        local_stack.empty
                     loop
                        Result.put (local_stack.item)
                        local_stack.remove
                     end
                  end

                  if push_on_stack then
                     local_stack.put (current_item)
                  end
               end

               Result.allocate_automatically
            end

         ensure then
            well_done: cs /= Void implies cs.is_finished
         end


   feature {NONE} -- Implementation

      pool: SORTED_LIST_[G] -- the real receptacle for entries to be stored

end
