indexing

   title:      "Universal facilities of the guide for class SIMPLE_LIST_";
   cluster:    "containers";
   project:    "Visual Eiffel";
   date:       "$Date: 2005/02/15 16:21:22 $";
   revision:   "$Revision: 1.2 $";
   copyright:  "Copyright (C) 1996-2005 Object Tools Group";
   license:    "http://visual-eiffel.com/license";
   disclaimer: "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

deferred class SIMPLE_CHAIN_CORE_

   inherit
      SIMPLE_POOL_
         end


   feature {SIMPLE_CONTAINER_} -- Operations

      extend (increment: INTEGER) is
            -- allocates memory for new slots
         require
            positive_increment: increment > 0
         local
            used, pos: INTEGER

         do
            if increment > 0 then

               used := capacity
               resizing (used + increment)

               -- ... 'capacity' already keeps the updated value

               from
                  pos := capacity
               variant
                  increment + pos - capacity
               until
                  pos = used
               loop
                  next.put (pos - 1, pos)
                  pos := pos - 1
               end

               next.put (first_free, pos + 1)
               first_free := capacity

            else
               target.halt (Current, target.f_extend)
            end

         ensure
            well_done: capacity = old capacity + increment
         end

      put (pos: INTEGER) is
            -- puts the new entry in order of the guide
         do
            if pos = free then

               if entrance = 0 then
                  next.put (- pos, pos)

               else
                  next.put (next.item (entrance), pos)
                  next.put (pos, entrance)
                  check
                     positive: next.item (entrance) > 0
                  end
               end

               entrance := pos
               check
                  negative: next.item (entrance) < 0
               end

            else
               target.halt (Current, target.f_put)
            end
         end

      remove (pos: INTEGER) is
            -- brings the entry out of the order of the guide
         require
            proper_position: pos > 0 and then pos <= capacity
            not_empty:       count > 0
         local
            prev_pos, next_pos, counter: INTEGER

         do
            from
               next_pos := - next.item (entrance)
            invariant
               next_pos > 0 and then next_pos <= capacity
            variant
               count - counter
            until
               next_pos = pos
            loop
               prev_pos := next_pos
               next_pos := next.item (prev_pos)
               counter  := counter + 1
            end

            if prev_pos = 0 then
               -- the first item is removing ...

               if entrance /= pos then
                  -- there are some other items in the list

                  next.put (- next.item (pos), entrance)

               else
                  -- the first item is the only item in the list

                  entrance := 0
               end

            else
               -- not the first item is removing ...

               if entrance = pos then
                  -- the last item is to be removed

                  next.put (next.item (pos), prev_pos)
                  entrance := prev_pos

               else
                  -- not the last item is to be removed

                  next.put (next.item (pos), prev_pos)
               end
            end

            frozen_put_in_stock (pos)
         end


   feature {SIMPLE_CONTAINER_} -- Queries

      count, chain_count: INTEGER is
            -- gives the number of used slots
         local
            pos: INTEGER

         do
            from
               pos := first
            variant
               capacity - Result
            until
               pos = 0
            loop
               Result := Result + 1
               pos := forth (pos)
            end
         end

      first, frozen chain_first: INTEGER is
            -- gives the position of the first entry
         require
            real_pool: capacity > 0

         do
            if entrance /= 0 then
               Result := - next.item (entrance)
            else
               Result := 0
            end

         ensure
            well_done: Result >= 0 and then Result <= capacity
         end

      forth, frozen chain_forth (pos: INTEGER): INTEGER is
            -- gives the position of the next (about 'pos') entry
         require
            proper_position: pos > 0 and then pos <= capacity
            not_empty:       first > 0

         do
            Result := next.item (pos)

            if Result < 0 then
               Result := 0
            end

         ensure
            well_done: Result >= 0 and then Result <= capacity
         end


   feature {NONE} -- Implementation

      entrance: INTEGER; -- keeps the position of the last slot used

      put_in_stock, frozen frozen_put_in_stock (pos: INTEGER) is
            -- marks the slot as unused one
         require
            proper_position: pos > 0 and then pos <= capacity

         do
            next.put (first_free, pos)
            first_free := pos

         ensure
            well_done: stock >= old stock
         end


   invariant

      proper_count:    count >= 0 and then  count <= capacity
      proper_entrance: entrance >= 0 and then entrance <= capacity

end
