indexing

   title:      "The project-wide universal properties:",
               "the class is an ancestor to all the container classes";
   cluster:    "containers";
   project:    "Visual Eiffel";
   date:       "$Date: 2005/02/15 16:21:22 $";
   revision:   "$Revision: 1.2 $";
   copyright:  "Copyright (C) 1996-2005 Object Tools Group";
   license:    "http://visual-eiffel.com/license";
   disclaimer: "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

deferred class SIMPLE_CONTAINER_


   feature -- Operations

      make (is_unigue: BOOLEAN) is
            -- tells the container if its entries must be unigue
         deferred
         end


   feature -- Queries

      empty: BOOLEAN is
            -- is the container empty ?
         do
            Result := (count = 0)
         end

      count: INTEGER is
            -- gives the number of entries in the container or equally
            -- the number of the slots used for storing the entries
         deferred
         end

      capacity: INTEGER is
            -- gives the number of all the slots allocated in the container
            -- (all the containers may have not only slots occupied by the
            -- entries already put into but unused ones also)
         deferred
         end


   feature {SIMPLE_POOL_} -- Implementation

      -- the following constants are used to invoke 'fault' in
      -- the descendents and - 'halt' in the client/suppliers

      f_allocate:    STRING is once Result := "allocate"    end;
      f_back:        STRING is once Result := "back"        end;
      f_extend:      STRING is once Result := "extend"      end;
      f_forth:       STRING is once Result := "forth"       end;
      f_merge:       STRING is once Result := "merge"       end;
      f_put:         STRING is once Result := "put"         end;
      f_remove:      STRING is once Result := "remove"      end;
      f_remove_all:  STRING is once Result := "remove_all"  end;
      f_remove_last: STRING is once Result := "remove_last" end;
      f_restructure: STRING is once Result := "restructure" end;
      f_wipe_out:    STRING is once Result := "wipe_out"    end;

      halt (xobject: SIMPLE_POOL_; routine_name: STRING) is
            -- stops the execution issuing 'unexpected error' which usually
            -- happens as a result of an existing internal problem (bug) ...
         require
            valid_object: xobject /= Void
            valid_name:   routine_name /= Void

         do
            errhlg.stop (Current, xobject, routine_name, errhlg.sc_unexpected)
         end


   feature {NONE} -- Implementation

      errhlg: CONTAINER_ERROR_HANDLER_ is

         once
            !!Result
         end

      fault (routine_name: STRING; error_code: INTEGER) is
            -- stops the execution; usually it is a reply for any incorrect
            -- user's actions which might happen while handling a container
         require
            valid_name:  routine_name /= Void
            proper_code: error_code >= errhlg.sc_is_empty and then
                         error_code <= errhlg.sc_out_of_range
         do
            errhlg.stop (Current, Void, routine_name, error_code)
         end


   invariant

      no_miracle: count >= 0 and then count <= capacity

end
