indexing

   title:      "The project-wide universal properties: the class is an",
               "ancestor to the classes of The Ordering Machine (TOM) -",
               "the supplier of all kinds of linear orders used in USCL";
   cluster:    "containers";
   project:    "Visual Eiffel";
   date:       "$Date: 2005/02/15 16:21:22 $";
   revision:   "$Revision: 1.2 $";
   copyright:  "Copyright (C) 1996-2005 Object Tools Group";
   license:    "http://visual-eiffel.com/license";
   disclaimer: "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

deferred class SIMPLE_POOL_


   feature {SIMPLE_CONTAINER_} -- Operations

      get_free is
            -- finds a free slot and saves it in 'free'
         require
            real_pool: capacity > 0

         do
            free := first_free
            if first_free /= 0 then
               first_free := next.item (first_free)
            end

         ensure
            well_done: (free >= 0 and then free <= capacity) and then
                       (free /= 0 implies stock = old stock - 1)
         end

      make (host: SIMPLE_CONTAINER_; init_size: INTEGER) is
            -- sets initial values of the guide: 'host' is the structure
            -- the guide belongs to and 'init_size' - the initial amount
            -- of slots to be allocated
         require
            valid_target:  host /= Void
            positive_size: init_size > 0

         deferred
         ensure
            well_done: (target /= Void) and then
                       (stock = init_size) and then (stock = capacity)
         end

      put (pos: INTEGER) is
            -- puts the new entry in order of the guide
         require
            proper_position: pos > 0 and then pos <= capacity
            proper_sequence: pos = free

         deferred
         end


   feature {SIMPLE_CONTAINER_} -- Queries

      free: INTEGER;             -- set by 'get_free' most recently to be used
                                 -- by the 'put' routine which must follow ...

      target: SIMPLE_CONTAINER_; -- the container attached to the ordering
                                 -- structure (the guide) and which keeps
         -- real data (at least elements or items) but not keys in case of
         -- two element entries like in any tables ...

      capacity: INTEGER is
            -- gives the total amount of slots currently allocated in the guide
         do
            Result := next.count
         end

      stock: INTEGER is
            -- gives the amount of still unused slots in the guide
         require
            real_pool: capacity > 0

         local
            pos: INTEGER

         do
            from
               pos := first_free
            invariant
               pos >= 0 and then pos <= capacity
            variant
               capacity - Result
            until
               pos = 0
            loop
               Result := Result + 1
               pos := next.item (pos)
            end
         end


   feature {NONE} -- Implementation

      first_free: INTEGER;   -- keeps the position of the first free slot

      next: ARRAY [INTEGER]; -- references to free and used slots (the next
                             -- entries) in two different chains

      resizing (new_size: INTEGER) is
            -- changes the amount of slots allocated
         require
            proper_argument: new_size > capacity

         deferred
         ensure
            well_done: capacity = new_size
         end


   invariant

      proper_free:  first_free >= 0 and then  first_free <= capacity
      proper_stock: stock >= 0 and then  stock <= capacity

end
