indexing

   title:      "The project-wide universal properties: facilities for",
               "being traversable in one direction - towards the last entry";
   cluster:    "containers";
   project:    "Visual Eiffel";
   date:       "$Date: 2005/02/15 16:21:22 $";
   revision:   "$Revision: 1.2 $";
   copyright:  "Copyright (C) 1996-2005 Object Tools Group";
   license:    "http://visual-eiffel.com/license";
   disclaimer: "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

deferred class SIMPLE_TRAVERSABLE_


   feature -- Operations

      cursor: SIMPLE_CURSOR_ is
            -- delivers a new cursor to the beginning of the container
         do
            !!Result.make (Current)
            Result.first
         end


   feature -- Queries

      is_inside (cs: like cursor): BOOLEAN is
            -- is this cursor traversing the container ?
         require
            valid_argument: cs /= Void

         do
            if cs.target /= Void and then cs.target = Current then
               Result := not cs.is_finished
            end
         end

      is_protected: BOOLEAN is
            -- true if at least one cursor is traversing the container
            -- otherwise - false
         do
            Result := cs_counter > 0
         end


   feature {SIMPLE_CURSOR_}

      cs_counter: INTEGER; -- number of the cursors checked in but not yet out

      first (cs: SIMPLE_CURSOR_) is
            -- positions on the first item
         require
            proper_cursor: is_inside (cs)

         deferred
         end

      forth (cs: SIMPLE_CURSOR_) is
            -- moves one item forward
         require
            proper_cursor: is_inside (cs)

         deferred
         end

      check_in is
            -- used to keep trace of active cursors
         do
            cs_counter := cs_counter + 1
         end

      check_out is
            -- used to keep trace of active cursors
         require
            a_good_match: cs_counter > 0

         do
            cs_counter := cs_counter - 1
         end

end
