indexing

   title:      "Sorted lists: containers which store their one element",
               "entries in a total relation order (ascending) based on the",
               "operations of class COMPARABLE from the Eiffel Kernel";
   cluster:    "containers";
   project:    "Visual Eiffel";
   date:       "$Date: 2005/02/15 16:21:22 $";
   revision:   "$Revision: 1.2 $";
   copyright:  "Copyright (C) 1996-2005 Object Tools Group";
   license:    "http://visual-eiffel.com/license";
   disclaimer: "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

class SORTED_LIST_[G -> COMPARABLE]

   inherit
      LIST_[G]
         redefine
            find_first, find_next, guide, make
         end

   creation
      make


   feature -- Operations

      make (is_unique: BOOLEAN) is
            -- tells the container if its entries must be unigue
         local
            init_size: INTEGER
            no_duplicates: SIMPLE_ORDER_ON_ITEMS_[G]
            with_duplicates: ORDER_ON_ITEMS_[G]

         do
            unique_items := is_unique
            init_size := default_size + counterweight

            check
               positive_size: init_size > 0
            end

            !!store.make (1, init_size)

            if unique_items then
               !!no_duplicates.make (Current, init_size)
               guide := no_duplicates
            else
               !!with_duplicates.make (Current, init_size)
               guide := with_duplicates
            end
         end


   feature -- Queries

      cursor_after (x: G): CURSOR_ is
            -- delivers a new cursor positioned to the entry closest to and
            -- greater than 'x' - the presence of entry 'x' is not essential
            -- but if currently there are at all no such entries in the list
            -- then Void is delivered
         require
            valid_argument: x /= Void
         local
            pos: INTEGER

         do
            pos := guide.search (x)

            if pos > 0 then
               pos := guide.tree_forth (pos)
            elseif pos < 0 and then x > store.item (- pos) then
               pos := guide.tree_forth (- pos)
            else
               pos := - pos
            end

            check pos >= 0 end

            if pos > 0 then
               !!Result.make (Current)
               Result.set_position (pos)
               Result.activate
            end

         ensure
            well_done: Result /= Void implies is_protected
         end

      cursor_before (x: G): CURSOR_ is
            -- delivers a new cursor positioned to the entry closest to and
            -- less than 'x' - the presence of entry 'x' is not essential
            -- but if currently there are at all no such entries in the list
            -- then Void is delivered
         require
            valid_argument: x /= Void
         local
            pos: INTEGER

         do
            pos := guide.search (x)

            if pos > 0 then
               pos := guide.tree_back (pos)
            elseif pos < 0 and then x < store.item (- pos) then
               pos := guide.tree_back (- pos)
            else
               pos := - pos
            end

            check pos >= 0 end

            if pos > 0 then
               !!Result.make (Current)
               Result.set_position (pos)
               Result.activate
            end

         ensure
            well_done: Result /= Void implies is_protected
         end


   feature {PRIORITY_QUEUE_} -- Implementation

      last_head: G is
            -- delivers the largest entry in the list; if currently it's not
            -- unique (there are duplicates of it) then the one that has been
            -- in the list for the longest time
         do
            if not empty then
               Result := store.item (guide.tree_last)
            end
         end


   feature {NONE} -- Implementation

      guide: SIMPLE_ORDER_[G]; -- the ordering structure of the container

      find_first (x: G): INTEGER is
            -- finds the first copy of the entry in the list
         do
            Result := guide.search (x)
            if Result < 0 then
               Result := 0
            end
         end

      find_next (x: G; pos: INTEGER): INTEGER is
            -- finds the duplicates of the entry which is stored at 'pos'
            -- traversal is towards the end of the list
         do
            Result := guide.forth (pos)
            if Result /= 0 and then not store.item (Result).is_equal (x) then
               Result := 0
            end
         end

end
