indexing

   title:      "Sorted tables: containers which store their two element",
               "entries (element or item itself and the associated key) in",
               "a total relation order (ascending) based on the operations",
               "of class COMPARABLE (the Eiffel Kernel) being done on keys";
   cluster:    "containers";
   project:    "Visual Eiffel";
   date:       "$Date: 2005/02/15 16:21:22 $";
   revision:   "$Revision: 1.2 $";
   copyright:  "Copyright (C) 1996-2005 Object Tools Group";
   license:    "http://visual-eiffel.com/license";
   disclaimer: "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

class SORTED_TABLE_[G, H -> COMPARABLE]

   inherit
      TABLE_[G, H]
         redefine
            current_keys, find_next, guide, make
         end

   creation
      make


   feature -- Operations

      make (is_unique: BOOLEAN) is
            -- tells the container if its entries must be unigue
         local
            init_size: INTEGER
            no_duplicates: SIMPLE_ORDER_WITH_KEYS_[H]
            with_duplicates: ORDER_WITH_KEYS_[H]

         do
            unique_keys := is_unique

            init_size := default_size + counterweight
            check init_size > 0 end

            !!store.make (1, init_size)

            if unique_keys then
               !!no_duplicates.make (Current, init_size)
               guide := no_duplicates
            else
               !!with_duplicates.make (Current, init_size)
               guide := with_duplicates
            end
         end


   feature -- Queries

      current_keys: LIST_[H] is
            -- delivers all the keys (without duplicates) currently stored in
            -- the table
         local
            pos: INTEGER

         do
            !!Result.make (true)
            pos := guide.first

            if pos /= 0 then
               from
               invariant
                  pos >= 0 and then pos <= capacity
               until
                  pos = 0
               loop
                  Result.put (guide.get_key (pos))
                  pos := guide.tree_forth (pos)
               end
            end
         end

      cursor_after (k: H): CURSOR_ is
            -- delivers a new cursor positioned to the entry the key of which
            -- is closest to and greater than 'k' - the presence of entry with
            -- key 'k' is not essential but if currently there are at all no
            -- such entries in the table then Void is delivered
         require
            proper_key: k /= Void
         local
            pos: INTEGER

         do
            -- pos := find_first (k)
            pos := guide.search (k)

            if pos /= 0 then
               if pos < 0 then
                  pos := - pos
                  if guide.get_key (pos) < k then
                     pos := guide.tree_forth (pos)
                  end
               else
                  pos := guide.tree_forth (pos)
               end

               if pos /= 0 then
                  !!Result.make (Current)
                  Result.set_position (pos)
                  Result.activate
               end
            end

         ensure
            well_done: Result /= Void implies is_protected
         end

      cursor_before (k: H): CURSOR_ is
            -- delivers a new cursor positioned to the entry the key of which
            -- is closest to and less than 'k' - the presence of entry with
            -- key 'k' is not essential but if currently there are at all no
            -- such entries in the table then Void is delivered
         require
            proper_key: k /= Void
         local
            pos: INTEGER

         do
            -- pos := find_first (k)
            pos := guide.search (k)

            if pos /= 0 then
               if pos < 0 then
                  pos := - pos
                  if guide.get_key (pos) > k then
                     pos := guide.tree_back (pos)
                  end
               else
                  pos := guide.tree_back (pos)
               end

               if pos /= 0 then
                  !!Result.make (Current)
                  Result.set_position (pos)
                  Result.activate
               end
            end

         ensure
            well_done: Result /= Void implies is_protected
         end


   feature {KEY_PRIORITY_QUEUE_} -- Implementation

      item_at_last_key_head: G is
            -- delivers the element or item that has been in the table for the
            -- longest time and associated with the largest key in the table
            -- if the table is empty then Void is delivered
         do
            if not empty then
               Result := store.item (guide.tree_last)
            end
         end

      last_key_head: H is
            -- delivers the largest key currently stored in the table
            -- if the table is empty then Void is delivered
         do
            if not empty then
               Result := guide.get_key (guide.tree_last)
            end
         end


   feature {NONE} -- Implementation

      find_next (k: H; pos: INTEGER): INTEGER is
            -- finds the first copy of the entry the element or item of which
            -- associated with key 'k' in the table
         do
            Result := guide.forth (pos)

            if Result /= 0 and then
                  not k.is_equal (guide.get_key (Result)) then

               Result := 0
            end
         end

      guide: SIMPLE_ORDER_[H] -- the ordering structure of the container

end
