indexing

   title:      "Stacks: dispensers implementing a last-in, first-out policy";
   cluster:    "containers";
   project:    "Visual Eiffel";
   date:       "$Date: 2005/02/15 16:21:22 $";
   revision:   "$Revision: 1.2 $";
   copyright:  "Copyright (C) 1996-2005 Object Tools Group";
   license:    "http://visual-eiffel.com/license";
   disclaimer: "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

class STACK_[G]

   inherit
      DISPENSER_[G]
         end


   feature -- Operations

      put (x: G) is
            -- add the new entry into the dispenser which immediately after
            -- becomes the "active entry" of the dispenser
         require else
            valid_item:  x /= Void
         do
            if pool = Void then
               !!pool.make (false)
               cs := pool.cursor
            end

            pool.put (x)
            item := x
         end

      remove is
            -- deletes the "active entry" and sets 'item' to the one most
            -- recently put into if at least one entry exists after the
            -- removal - if it does not then 'item' is set to Void or,
            -- to be exact, to the default value of the type ...
         do
            if not empty then
               pool.remove_last
               cs.last
               if empty then
                  item := species_default
               else
                  item := pool.item_at (cs)
                  cs.stop
               end
            else
               fault (f_remove, errhlg.sc_is_empty)
            end

         ensure then
            well_done: cs.is_finished
         end


   feature -- Queries

      list_of_elements: LIST_[G] is
            -- delivers all the entries of the dispenser in the order in
            -- which they would be repeatedly removed if it really happened
         do
            !!Result.make (false)
            if not empty then
               check pool /= Void and then count > 0 end

               from
                  Result.allocate_manually (count)
                  cs.last
               until
                  cs.is_finished
               loop
                  Result.put (pool.item_at (cs))
                  cs.back
               end

               Result.allocate_automatically
            end

         ensure then
            well_done: cs /= Void implies cs.is_finished
         end

end
