indexing

   title:      "Table iterators: tools for traversing the tables";
   cluster:    "containers";
   project:    "Visual Eiffel";
   date:       "$Date: 2005/02/15 16:21:22 $";
   revision:   "$Revision: 1.2 $";
   copyright:  "Copyright (C) 1996-2005 Object Tools Group";
   license:    "http://visual-eiffel.com/license";
   disclaimer: "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

class TABLE_ITERATOR_[G, H]

   inherit
      ITERATOR_
         end


   feature -- Operations

      attach (x: TABLE_[G, H]) is
            -- selects the target for traversal
         require
            proper_table: x /= Void
         do
            target := x
         ensure
            well_done: target /= Void
         end


   feature -- Queries

      extract_list: LIST_[G] is
            -- delivers all the elements or items of the table attached the
            -- entries of which including the keys are satisfied the
            -- conditions put by 'test'
         require
            proper_target: target /= Void
         do
            !!Result.make (false)
            from
               cs := target.cursor
            until
               cs.is_finished
            loop
               if test then
                  Result.put (item)
               end
               cs.forth
            end
         end

      extract_table: like target is
            -- delivers all the entries of the table attached which are
            -- satisfied the conditions put by 'test'
         require
            proper_target: target /= Void
         do
            !!Result.make (target.unique_keys)
            from
               cs := target.cursor
            until
               cs.is_finished
            loop
               if test then
                  Result.put (item, key)
               end
               cs.forth
            end
         end

      target: TABLE_[G, H]


   feature {NONE} -- Implementation

      item: G is
            -- delivers the element or item of the current entry for
            -- "being touched" in features 'action' and 'test' while
            -- traversing the table
         require
            proper_cursor: cs /= Void and then cs.target = target
         do
            Result := target.item_at (cs)
         end

      key: H is
            -- delivers the key of the current entry for "being touched"
            -- in features 'action' and 'test' while traversing the table
         require
            proper_cursor: cs /= Void and then cs.target = target
         do
            Result := target.key_at (cs)
         end

end
