indexing
   project    : "The Universal Simple Formatting Library (USFL)";
   copyright  : "Object Tools, 1996-2001";
   original   : "08 Sep 1996";
   version    : "1.2";
   last_change: "17 Jun 2001"

class G_COMMERCIAL_FORMAT

inherit

   G_SCIENTIFIC_FORMAT
      redefine
         make, parse_escape_sequence, raise_exception,
         set_default_format, sprint
      end

creation

   make

feature

   default_currency           : STRING
   default_decimal            : CHARACTER
   default_currency_is_dollar : STRING is "$"
   default_decimal_is_point   : CHARACTER is '.'
   Picture_chars              : STRING is "XIZLYHMSDM+$-9.*"
   Delimiter_chars            : STRING is ":/_,B"
   Special_chars              : STRING is "()[]"
   LRBracket                  : CHARACTER is '('
   RRBracket                  : CHARACTER is ')'
   LSBracket                  : CHARACTER is '['
   RSBracket                  : CHARACTER is ']'

  make is
   do
      parsed_picture := ""
      Precursor
      default_currency := default_currency_is_dollar
      default_decimal  := default_decimal_is_point
   end -- make

  is_picture_char (c: CHARACTER): BOOLEAN is
     do
       Result := str_tools.contain  (Picture_chars, c) or else
       str_tools.contain (Delimiter_chars, c) or else
       str_tools.contain (Special_chars, c)
     end -- is_picture_char

   set_default_format (format: STRING) is
   local
      format_wo_def_escape: STRING
   do
      if format @ 1 = default_escape then
         format_wo_def_escape := format.substring (2, format.count);
      else
         format_wo_def_escape := format
      end
      picture_gallery.search_by_picture (format_wo_def_escape)
      if picture_gallery.found then
         picture_gallery.found_item.set_default_format (format);
         formatters.dyna_types.make_default (picture_gallery.found_item);
      else
         Precursor (format);
      end
   end -- set_default_format

   sprint (para: ANY): STRING is
   -- print argument to string using default format string
   local
      pic_fmt_el : G_PICTURE_FORMATTER
   do
      formatters.dyna_types.search_by_example (para);
      if formatters.dyna_types.found then
         pic_fmt_el ?= formatters.dyna_types.found_item
         if pic_fmt_el /= Void then
            Result := sprintf (pic_fmt_el.default_format, <<para>>);
         else
            Result := Precursor (para);
         end
      else
         raise_exception ("Don't know how to print argument");
      end
   end -- sprint

   set_currency (currency: STRING) is
   do
      default_currency := currency
   end -- set_currency

   set_decimal (point: CHARACTER) is
   do
      default_decimal := point
   end -- set_decimal

feature {NONE} -- implementation

   parse_escape_sequence: STRING is
   -- parse escape sequence
   local
      i                    : INTEGER
      end_of_escape_seq,
      all_in_brackets      : BOOLEAN
      pic_fmt              : G_ABSTRACT_COMMERCIAL_FORMATTER
      this_is_posix_format : BOOLEAN
     -- promote REAL to DOUBLE
      -- for a bad HACK :(
     double_ref : DOUBLE_REF;
     real_ref : REAL_REF;
     args_current_item: ANY;
   do
      from
         i := cursor
      until
         i > format_str.count
      loop
         if not is_posix_spec_char (format_str @ i) then
            formatters.search_by_typechar (format_str @ i);
            this_is_posix_format := formatters.found
            i := format_str.count + 1 -- finish cycle
         else
            i := i + 1
         end
      end

      if not this_is_posix_format and then
         is_picture_char (format_str @ cursor) then

         if args.current_item = Void then
            raise_exception ("Argument is Void");
         end
       -- HACK! 1999-08-06 promoting REAL to DOUBLE because a DOUBLE is
       -- not a REAL and vice versa. Problem is: Every Eiffel-Compiler
       -- takes his own point-of-view what an blank floating point number
       -- is, so `1.23' may be a REAL on one Compiler and DOUBLE on another
       -- it seems that SE takes REAL as default and ISE-Eiffel eg DOUBLE
       --
       args_current_item := args.current_item;
       real_ref ?= args_current_item
        if real_ref /= Void then
         !!double_ref;
         double_ref.set_item(real_ref.item + 0.0);
         args_current_item := double_ref;
       end;
         -- Argument determine type of picture
         -- Picture formater is responsible for picture conformance
       picture_gallery.search_by_example(args_current_item);

       -- /HACK!

--     debug
--          io.put_string("args_current_item: ");
--          any_print(args_current_item);
--          io.put_new_line;
--          io.put_string("generator : ");
--          io.put_string(args_current_item.generator);
--          io.put_new_line;
--          io.put_string("picture_gallary.found = ");
--          if picture_gallery.found then
--             io.put_string("TRUE");
--          else
--             io.put_string("FALSE");
--          end;
--          io.put_new_line;
--          io.put_string("picture_gallery.found_item.sample: ");
--          any_print(picture_gallery.found_item.sample);
--          io.put_new_line;
--          io.put_string("generator : ");
--          io.put_string(picture_gallery.found_item.sample.generator);
--          io.put_new_line;
--     end;

         if not picture_gallery.found or else
            not args_current_item.conforms_to
                  (picture_gallery.found_item.sample)
        then
            raise_exception
         ("Argument does not conform to any known picture format.");
         end
         pic_fmt ?= picture_gallery.found_item.fmt
         check pic_fmt /= Void end
         pic_fmt.make (args_current_item);
         --
         all_in_brackets := format_str @ cursor = LRBracket

         if all_in_brackets then
            cursor := cursor + 1
         end
         from
            i := cursor
       until
            end_of_escape_seq or else
            i > format_str.count or else
            not is_picture_char (format_str @ i)
         loop
            inspect format_str @ i
         when LSBracket then      -- expext " XXX ] "
            i := parse_options (i, pic_fmt);
         when LRBracket then      -- expext " N) "
                  i := parse_repeatance (i);
                  was_repeatance := true
         when RSBracket then      -- ] is error
            end_of_escape_seq := true

         when RRBracket then
            if all_in_brackets then
              i := i + 1
            end
            end_of_escape_seq := true -- in any case
            else
               parsed_picture.append_character (format_str @ i);
            --             debug
            --               io.put_string("parsed_picture now: ");
            --               io.put_string(parsed_picture);
            --               io.put_new_line;
            --             end;
               was_repeatance := false
               i := i + 1
            end -- inspect
       end -- loop i
       cursor := i

         -- -----------------------------------------
         -- final check
         -- -----------------------------------------

         if parsed_picture.count = 0 then
            raise_exception ("Empty picture");
         elseif all_in_brackets and not end_of_escape_seq then
            raise_exception ("Missed final ')'");
         end

         -- -----------------------------------------
         -- interpret format
         -- -----------------------------------------
         pic_fmt.set_picture (parsed_picture);
         pic_fmt.set_currency (default_currency);
         pic_fmt.set_point (default_decimal);
         pic_fmt.set_delimiters (Delimiter_chars);
         Result := pic_fmt.output
      else
         Result := Precursor
      end
   -- cleaning string
   parsed_picture.wipe_out;
     end -- parse_escape_sequence

   parse_repeatance (i : INTEGER) : INTEGER is
   require
      correct_call: i > 0 and then i <= format_str.count and then
                     format_str @ i = LRBracket
   local
      j,
      repeat_count : INTEGER
      tail_str     : STRING
   do
      if was_repeatance then
         raise_exception ("Twice repeatance");
      end
      from j := i + 1
      until j > format_str.count or else
            format_str @ j = RRBracket
      loop
         if not str_tools.is_digit (format_str, j) then
            raise_exception ("Repeatance number (non digit etc)");
         end
         j := j + 1
      end
      --
      if i + 1 < j and then format_str @ j = RRBracket then
         repeat_count := str_tools.x_s2i (format_str.substring (i+1, j-1));
         if parsed_picture.count /= 0 then
            if repeat_count > 0 then
            tail_str := str_tools.fill_string((repeat_count -1),
                                     parsed_picture @ parsed_picture.count);
               parsed_picture.append_string (tail_str);
            elseif repeat_count = 0 then
               -- any number of repeatance
               inspect parsed_picture @ parsed_picture.count
                  when 'X' then
                     parsed_picture.put ('x', parsed_picture.count);
                  when 'I' then
                     parsed_picture.put ('i', parsed_picture.count);
                  else
                     raise_exception
                        ("Such repeatance unsupported in shareware version");
               end
            end
         else
            raise_exception ("Repeatance number (negative etc)");
         end
      else
         raise_exception ("Missed number or tail ')'");
      end
      Result := j + 1
     -- parsed_picture.wipe_out;
   ensure
      format_str @ (Result - 1)  = RRBracket
   end -- parse_repeatance

   parse_options(i : INTEGER;
             pic_fmt: G_ABSTRACT_COMMERCIAL_FORMATTER) : INTEGER is
   require
      correct_call: i > 0 and then i <= format_str.count and then
                     format_str @ i = LSBracket
   local
      j, option_pos    : INTEGER
      option           : STRING
   do
      from
         j := i + 1
      until j > format_str.count or else
            format_str @ j = RSBracket
      loop
         j := j + 1
      end
      if i + 1 < j and then j <= format_str.count and then
                           format_str @ j = RSBracket then

         option := format_str.substring (i+1, j-1);
         option_pos := str_tools.find_itself (option, option_list);
         if option_pos >= option_list.lower then

            inspect (option_pos + 1) // 2
               when BLANK_WHEN_ZERO        then
                  pic_fmt.set_blank_when_zero
               when JUSTIFIED_RIGHT        then
                  pic_fmt.set_justified_right
               when JUSTIFIED_LEFT         then
                  pic_fmt.set_justified_left
               when UPPER_CHARACTER        then
                  pic_fmt.set_upper_character
               when LOWER_CHARACTER        then
                  pic_fmt.set_lower_character
               when SIGN_LEADING           then
                  pic_fmt.set_sign_leading
               when SIGN_TRAILING          then
                  pic_fmt.set_sign_trailing
               when SIGN_LEADING_SEPARATE  then
                  pic_fmt.set_sign_leading_separate
               when SIGN_TRAILING_SEPARATE then
                  pic_fmt.set_sign_trailing_separate
            else
               raise_exception ("Unknown option constant");
            end -- inspect
         else
            raise_exception ("Unknown option in []");
         end
      else
         raise_exception ("[] or missing ]");
      end
      Result := j + 1
   ensure
      format_str @ (Result - 1)  = RSBracket
   end -- parse_options

   BLANK_WHEN_ZERO        : INTEGER is 1
   JUSTIFIED_RIGHT        : INTEGER is 2
   JUSTIFIED_LEFT         : INTEGER is 3
   UPPER_CHARACTER        : INTEGER is 4
   LOWER_CHARACTER        : INTEGER is 5
   SIGN_LEADING           : INTEGER is 6
   SIGN_TRAILING          : INTEGER is 7
   SIGN_LEADING_SEPARATE  : INTEGER is 8
   SIGN_TRAILING_SEPARATE : INTEGER is 9

   option_list : ARRAY [STRING] is
   once
      Result := <<
         "BLANK WHEN ZERO",        "BZ",
         "JUSTIFIED RIGHT",        "JR",
         "JUSTIFIED LEFT",         "JL",
         "UPPER CHARACTER",        "UC",
         "LOWER CHARACTER",        "LC",
         "SIGN LEADING",           "SL",
         "SIGN TRAILING",          "ST",
         "SIGN LEADING SEPARATE",  "SLS",
         "SIGN TRAILING SEPARATE", "STS"
      >>
   end -- option_list

feature {NONE}


   picture_gallery: G_PICTURE_FORMAT_TABLE [G_PICTURE_FORMATTER] is
   local
      fmt_el : G_PICTURE_FORMATTER
      afmt   : G_ABSTRACT_COMMERCIAL_FORMATTER
      i      : INTEGER
     -- i_double: DOUBLE;
   once
      !!Result.make
     -- add hoc: (formatters is once-method)
      i := formatters.count
      check i > 0 end

      -- integer
      !G_INTEGER_COMMERCIAL_FORMATTER!afmt.make (1);
      !!fmt_el.make ("9Z", 1, afmt, "999");
      Result.put_at_end (fmt_el);
      formatters.dyna_types.put_at_end (fmt_el);

      -- real
      !G_REAL_COMMERCIAL_FORMATTER!afmt.make (i_double);
      !!fmt_el.make ("9Z.", i_double, afmt, "9.99");
      Result.put_at_end (fmt_el);
      formatters.dyna_types.put_at_end (fmt_el);

      -- string
      !G_STRING_COMMERCIAL_FORMATTER!afmt.make ("abc");
      !!fmt_el.make ("XI", "a", afmt, "XXX");
      Result.put_at_end (fmt_el);
      formatters.dyna_types.put_at_end (fmt_el);

      -- boolean
      !G_LOGICAL_COMMERCIAL_FORMATTER!afmt.make (true);
      !!fmt_el.make ("LY", true, afmt, "LLL");
      Result.put_at_end (fmt_el);
      formatters.dyna_types.put_at_end (fmt_el);
   end -- picture_gallery

   raise_exception (msg: STRING) is
   local
      m : STRING
   do
      !!m.make_from_string ("Bad picture (");
      m.append_string (msg);
      m.append_character (')');
      exc.report ("G_COMMERCIAL_FORMAT", m, 0, Current);
   end -- raise_exception

   parsed_picture : STRING

   was_repeatance : BOOLEAN

end -- class G_COMMERCIAL_FORMAT

