indexing

   copyright:   "Object Tools, 1996-2002";
   original:    "08 Sep 1996";
   version:     "1.4";
   last_change: "03 Sep 2002";
   licence:     "Eiffel-forum freeware licence";

class G_FIXED_POINT_FORMATTER

inherit

   G_ABSTRACT_JUSTIFY_FORMATTER
      redefine
         make, set_align_char, set_sign
      end

   G_EXTENDED_MATH

creation

   make, make_real

feature {G_ABSTRACT_FORMATTER}

   fsign      : INTEGER
   int_part   : INTEGER
   fract_part : DOUBLE
   zero_strip : BOOLEAN
   -- formatter for integer part. formatted on-fly
   i_f        : G_INTEGER_FORMATTER -- for integer part formatting

feature

   make (f : DOUBLE_REF) is
   -- make split number on parts to simplify formatting
   require else
      format_real: f /= Void
   local
      fa: DOUBLE
   do
      fa := f.item.abs
      fsign := double_sign (f.item);
      int_part := fa.floor
      fract_part := fa - int_part
      !!i_f.make (int_part*fsign);
      Precursor (fa);
      set_precision (6);
      zero_strip := FALSE
      !!f_f.make (0)
      f_f.set_align_left
      f_f.set_align_char ('0');
   end -- make

   make_real (r: REAL_REF) is
     require
       r_not_void: r /= Void;
     local
       double_of_r : DOUBLE;
     do
       double_of_r := r.item;
       make(double_of_r);
     end;

   set_item (f: DOUBLE) is
         -- Set the new value without changing the format
      local
         fa: DOUBLE
      do
         fa := f.abs
         fsign := double_sign (f)
         int_part := f.floor
         fract_part := fa - int_part
         i_f.set_item (int_part*fsign)
      end

   set_sign (s : CHARACTER) is
   do
      i_f.set_sign (s);
   end -- set_sign

   set_align_char (c : CHARACTER) is
   do
      i_f.set_align_char (c);
   end -- set_align_char

   set_zero_strip (zs : BOOLEAN) is
   do
      zero_strip := zs
   end -- set_zero_strip

   output: STRING is
      do
       Result := make_fract_part
       -- make integer part and finish alignment
       inspect alignment
       when align_left then
         i_f.set_width (1);
         Result.prepend (i_f.output);
         Result := right_pad (Result, ' ', width);
       when align_right then
         i_f.set_width (width-Result.count);
         i_f.set_align_right
         Result.prepend (i_f.output);
       when align_center then
         i_f.set_width (1);
         Result.prepend (i_f.output);
         Result := center_pad (Result, ' ', width);
       end
     end -- output

feature {NONE}

   make_fract_part : STRING is
   -- this function entered as replacing of simple expression
   -- !!f_f.make (floor((fract_part*10^(prec))+0.5))
   -- which fails in case of large precision
   -- now we obtain fraction part with pieces of decimal_digit_count  digits
   local
      i: INTEGER
      fp, fff: DOUBLE
      sd: G_SYSTEM_DEPENDENCE
   do
      from
         !!sd
         i := prec
         fp := fract_part
         Result := ""
      until
         i <= sd.decimal_digit_count
      loop
         fff := fp*(10^sd.decimal_digit_count) -- .truncated_to_real
         x_append (Result, produce_piece (fff.floor, sd.decimal_digit_count));
         fp := fff - fff.floor
         i := i - sd.decimal_digit_count
      end -- loop
      x_append (Result, produce_piece (((fp*10^i)+0.5).floor, i));
      if zero_strip then
         Result := strip_zeros (Result);
      end -- if
         check
            Result.count = prec or else Result.count = prec + 1
         end
      if Result.count = prec then
         i_f.make (int_part * fsign)
      else
         -- There is overflow
         -- We should add 1 to the int_part ...
         i_f.make ((int_part + 1) * fsign)
         -- ... and remove extranenous '1'
         Result.remove (1)
      end
      if not Result.is_equal ("") then
         Result.precede ('.')
      end -- if
   end -- make_fract_part

   produce_piece (i : INTEGER; p : INTEGER) : STRING is
   do
      f_f.set_item (i)
      f_f.set_precision (p)
      Result := f_f.output
   end -- produce_piece

feature {NONE} -- Implementation

   f_f: G_UNSIGNED_DECIMAL_INTEGER_FORMATTER

end -- class G_FIXED_POINT_FORMATTER
