indexing

   copyright:   "Object Tools, 1996-2002";
   original:    "08 Sep 1996";
   version:     "1.5";
   last_change: "20 Nov 2002";
   licence:     "Eiffel-forum freeware licence";

class G_UNSIGNED_INTEGER_FORMATTER

inherit

   G_ABSTRACT_JUSTIFY_FORMATTER
      redefine
         make, set_align_char
      end

creation

   make, make_with_base

feature {G_ABSTRACT_FORMATTER}

   align_char: CHARACTER
   inum: INTEGER
   base: INTEGER

feature -- Creation

   make (i: INTEGER_REF) is
      require else
         format_number: i /= Void
         positive_argument: i >=0
      do
         Precursor (i)
         align_char := ' '
         inum := i.item
         base := 10
      end

   make_with_base (i:INTEGER_REF; b: INTEGER) is
      require
         base_is_legal: is_valid_base (b)
      do
         make (i)
         set_base (b)
      end

feature

   set_base (b: INTEGER) is
      require
         is_valid_base (b)
      do
         base := b
      ensure
         base = b
      end

   is_valid_base (b: INTEGER): BOOLEAN is
      do
         Result := b > 1 and b < 37
      ensure
         Result implies (1 < b and b < 37)
      end

   set_item (i: INTEGER) is
         -- Set the new value without changing the format
      do
         inum := i
      end

feature {G_ABSTRACT_FORMATTER}

    set_align_char (c: CHARACTER) is
       require else
           legal_align_char: c = ' ' or else c = '0'
       do
           align_char := c
       end

    digits: STRING is "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"

    get_plain_number_image (i: INTEGER): STRING is
       local
           n: INTEGER -- current value of the integer to convert
           d: INTEGER -- digit (starting at 0 for '0')
       do
         if i = 0 then
            Result := "0"
         else
            Result := ""
            if i < 0 then
               -- The first digit is calculated using floating point
               -- arithmetic to avoid sign extension problems
               -- TODO: replace the magic constant with the portable
               -- definition
               n := ((4294967296.0 + i) / base).truncated_to_integer
               d := i - n * base
               Result.precede (digits.item (d + 1))
            else
               n := i
            end
            from
            until
               n = 0
            loop
               d := n \\ base
               n := n // base
               Result.precede (digits.item (d + 1))
            end
         end
      end

   output: STRING is
      local
         str: STRING
      do
         if prec = 0 and then inum = 0 then
            Result := ""
         else
            str := left_pad (get_plain_number_image (inum), '0', prec)
            inspect alignment
            when align_left then
               Result := right_pad (str, align_char, width)
            when align_right then
               Result := left_pad (str, align_char, width)
            when align_center then
               Result := center_pad (str, align_char, width)
            end
         end
      end

invariant

   is_valid_base (base)

end -- class G_UNSIGNED_INTEGER_FORMATTER
