--
-- SUMMARY:      
--
-- AUTHOR:       Friedrich Dominicus
-- ORG:          No Org
-- ORIG-DATE:     9-Aug-99 at 18:13:35
-- LAST-MOD:     10-Aug-99 at 10:07:36 by Friedrich Dominicus
-- DESCRIPTION:  
-- DESCRIP-END.
indexing
   title:       "Additional math service";
   project:     "The Universal Simple Formatting Library (USFL)";
   copyright:   "SIG Computer, 1995";
   author:      "Igor Nesterof (nest@eiffel.ru)";
   original:    "08,Sep,1995";
   version:     1.2;
   last_modified: "1999-08-09";
   done_at:     "SIG Computer (info@eiffel.ru)";
   modified:    "has to be adjusted to be usable for SmallEiffel, there is%
                 % no class SINGLE_MATH available and all values are promoted %
                 % to Double";
   comment:     "Interface adopted to fit the DOUBLE_MATH class of ISE- %
                 % Eiffel and Visual-Eiffel";
   last_modified: "1999-08-09";
   maintainer: "Friedrich Dominicus";



class G_EXTENDED_MATH 


   

feature

   log_2 (v: DOUBLE): DOUBLE is
		 -- Base 2 logarithm of `v'
	  require
		 v > 0.0
	  do
		 Result := v.log / (2.0).log 
	  end;

   cosine (v: DOUBLE): DOUBLE is
		 -- Trigonometric cosine of radian `v' approximated
		 -- in the range [-pi/4, +pi/4]
	  do
		 Result := v.cos;
	  end;

   arc_cosine (v: DOUBLE): DOUBLE is
		 -- Trigonometric arccosine of `v'
	  do
		 Result := v.acos;
	  end;
   
   sine (v: DOUBLE): DOUBLE is
		 -- Trigonometric sine of radian `v' approximated
		 -- in range [-pi/4, +pi/4]
	  do
		 Result := v.sin;
	  end;
   
   arc_sine (v: DOUBLE): DOUBLE is
		 -- Trigonometric arcsine of `v'
	  do
		 Result := v.asin;
	  end;
   
   tangent (v: DOUBLE): DOUBLE is
		 -- Trigonometric tangent of radian `v' approximated
		 -- in range [-pi/4, +pi/4]
	  do
		 Result := v.tan;
	  end;
   
   arc_tangent (v: DOUBLE): DOUBLE is
		 -- Trigonometric arctangent of `v'
	  do
		 Result := v.atan;
	  end;
   
   sqrt (v: DOUBLE): DOUBLE is
		 -- Square root of `v'
	  require
		 v >= 0.0
	  do
		 Result := v.sqrt;
	  end;
   
   log (v: DOUBLE): DOUBLE is
		 -- Natural logarithm of `v'
	  require
		 v > 0.0
	  do
		 
		 Result := v.log;
	  end;
   
   log10 (v: DOUBLE): DOUBLE is
		 -- Base 10 logarithm of `v'
	  require
		 v > 0.0
	  do
		 Result := v.log10;
	  end;
   
   floor (v: DOUBLE): DOUBLE is
		 -- Greatest integral value less than or equal to `v'
	  do
		 Result := v.floor.to_double;
	  end;
   
   ceiling (v: DOUBLE): DOUBLE is
		 -- Least integral value greater than or equal to `v'
	  do
		 Result := v.ceiling.to_double;
	  end;
   
   
   
   dabs (v: DOUBLE): DOUBLE is
		 -- Absolute value of `v'
	  do
		 if v < 0 then
			Result := -v;
		 else
			Result := v;
		 end;
	  end;

--------------------------------------------------------------------- int_abs
   int_abs (i: INTEGER): INTEGER is
   do
      if i >= 0 then
         Result :=  i
      else
         Result := -i
      end
   end -- int_abs
------------------------------------------------------------------- real_sign
   double_sign (r: DOUBLE): INTEGER is
   do
      if r > 0.0 then
         Result :=  1
      elseif r = 0.0 then
         Result := 0
      else
         Result := -1
      end
   end -- real_sign
-------------------------------------------------------------------- int_sign
   int_sign (i: INTEGER): INTEGER is
   do
      if i > 0 then
         Result :=  1
      elseif i = 0 then
         Result := 0
      else
         Result := -1
      end
   end -- int_sign
----------------------------------------------------------------------- log10
--    log10 (x: REAL):REAL is
--    local
--       temp1 : REAL ;
--       temp2 : REAL ;
--    do
--       temp1 := log (x) ;
--       temp2 := log (10)
--       Result := temp1/temp2;
--    end -- log10
--    log10(x:DOUBLE): DOUBLE is
-- 	  local
-- 		 temp1 : DOUBLE;
-- 		 temp2 : DOUBLE;
   
------------------------------------------------------------------- ten_power
   ten_power (x: INTEGER): DOUBLE is
      do
		 if x < 0 then
			Result := 1.0 / (10.0^(-x));
		 else
			Result := 10.0^x
		 end
	  end -- ten_power





end
   
