indexing

   title:       "Supplies some additional string service";
   project:     "The Universal Simple Formatting Library (USFL)";
   old_copyright: "SIG Computer, 1995";
   copyright:   "Eiffel-forum free licence with permission from Object Tools";
   author:      "Andy Malakov and Igor Nesterof";
   original:    "08 Sep 1995";
   version:     "1.2";
   last_change: "17 Jun 2001";
   done_at:     "SiG Computer (www.object-tools.com)";
   copyed_by:   "Friedrich Dominicus";
   history: "1999, August: Renamed to G_STRING_TOOLS. Adapted for to use %
            %with gepp for ISE-Eiffel, Halstenbach-Eiffel, Visual Eiffel and %
            %SmallEiffel; %
            %some cleanups e.g in fill_string, x_s2i",
            "2001, June: Adjustments to avoid using obsolete features under %
            %Visual Eiffel";
   how_to_generate_Eiffel_files: "just run %
      %gepp -DCOMPILER_SWITCH g_string_tools.ge ../COMPILER_DIR/g_string_tools.e";
   
class G_STRING_TOOLS

feature

   fill_string (nr_blanks: INTEGER; pc: CHARACTER): STRING is
      do
         !!Result.blank(nr_blanks)
         Result.fill_with(pc)
      end

   left_pad (s: STRING;
             pc: CHARACTER;
             l: INTEGER): STRING is
      local
         sl: INTEGER
      do
         sl := s.count
         if sl >=l then
            !!Result.make_from_string(s)
         else
            !!Result.make_from_string(fill_string(l-sl,pc))
            Result.append(s)
         end
      end

   right_pad (s: STRING;
              pc: CHARACTER;
              l: INTEGER): STRING is
      do
         !!Result.make_from_string(s);
         if s.count < l then
            Result.append(fill_string(l-s.count,pc))
         end
      end

   center_pad (s: STRING;
               pc: CHARACTER;
               l: INTEGER): STRING is
      local
         sl, half: INTEGER
      do
         sl := s.count
         if sl >=l then
            !!Result.make_from_string(s)
         else
            half := (l-sl)//2
            !!Result.make_from_string(fill_string(half,pc))
            Result.append(s)
            Result.append(fill_string(l-sl-half,pc))
         end
      end

   pad_string (s: STRING; 
               pc: CHARACTER; 
               l: INTEGER; 
               from_left: BOOLEAN): STRING is
      do
         if from_left then
            !!Result.make_from_string(left_pad(s,pc,l))
         else
            !!Result.make_from_string(right_pad(s,pc,l))
         end
      end

   cut_here (s: STRING;
             fin_len: INTEGER): STRING is
         -- make string not more then fin_len characters
      require
         positive_len : fin_len >=0
      do
         if s.count > fin_len then
            !!Result.make_from_string(x_substring(s,1,fin_len))
         else
            !!Result.make_from_string(s)
         end
      end

   strip_zeros (s: STRING): STRING is
         -- strip trailing 0's
      local
         i: INTEGER;
         sub_index: INTEGER;
         done: BOOLEAN;
         -- Rewritten non recursive
      do
         if s.is_equal("") then
            Result := s;
         else
            from i := s.count
            until done
            loop
               if s.item(i) = '0' then
                  i := i - 1;
               else
                  done := True;
                  sub_index := i;
               end;
            end;
            Result := s.substring(1,sub_index)
         end
      end

         
   ifc (b: BOOLEAN; t,f: CHARACTER): CHARACTER is
         -- C-like style ? : operator
      do
         if b then
            Result := t
         else
            Result := f
         end
      end

   is_digit (s: STRING; indx: INTEGER): BOOLEAN is
      local c: CHARACTER
      do
         c := s.item ( indx )
         if c >= '0' and then c <= '9' then
            Result := true
         end
      end

   is_letter (s: STRING; indx: INTEGER): BOOLEAN is
      local c: CHARACTER
      do
         c := s.item ( indx )
         if ( c >= 'a' and then c <= 'z') or else
            ( c >= 'A' and then c <= 'Z') then
            Result := true
         end
      end

   index (s: STRING; pc: CHARACTER): INTEGER is
         -- return pc position or zero if it not found
      require
         s /= Void
      local
         i : INTEGER
      do
         from i := 1 until i > s.count loop
            if s.item (i) = pc then
               Result := i
               i := s.count + 1
            else
               i := i + 1
            end
         end
      end

   contain (s: STRING; pc: CHARACTER): BOOLEAN is
      do
         Result := index (s, pc) /= 0
      end

   x_substring (s: STRING; 
                from_index: INTEGER;
                to_index: INTEGER): STRING is
         -- extend substring with empty string
      require
         sub_interval: 1 <= from_index and then
         from_index <= to_index + 1 and then
         to_index <= s.count
      do
         if from_index = to_index + 1 then
            !!Result.make_from_string ("")
         else
            Result := s.substring (from_index, to_index)
         end
      ensure
         Result /= Void
      end

   find_itself (s: STRING; list: ARRAY [STRING]): INTEGER is
         -- find s in strings list
      local
         i : INTEGER
         found : BOOLEAN
      do
         from i := list.lower
         until found or else i > list.upper
         loop
            if s.is_equal ( list @ i ) then
               found := true
            else
               i := i + 1
            end
         end
         if found then
            Result := i
         else
            Result := list.lower - 1
         end
      end

   Zero: CHARACTER is '0';

   x_s2i (s: STRING): INTEGER is
         -- convert without checking
      require
         s /= Void
      do
         Result := s.to_integer;
      end

   -- Portability STUBS

   x_append (s, other: STRING) is
      do
         s.append (other)
      end


   x_extend (s: STRING; c: CHARACTER) is
      do
         s.extend (c)
      end

   x_adapt (s, other: STRING) is
      do
         s.copy (other)
      end

   x_prepend (s, other: STRING) is
      do
         s.prepend (other)
      end

end -- class G_STRING_TOOLS
