/*
You may be able to redistribute and/or modify this program.
See the README for details.
Copyright (C) 1999 Andrew Klochkov, Eugene Melekhov
*/

#include <gtk/gtk.h>
#include <vegtk.h>

static int _strlen(char * str)
{
  int l=0;
  while(*str++)
    l++;
  return l;
}

static int _strcmp (char *s1,char *s2)
{
 while (*s1 == *s2 && *s1 != '\0')
   {
    s1++;
    s2++;
   }
 return *s1 - *s2;
}

int ext_signal_connect(void *widget, char * name, EIF_REFERENCE cb_data)
{
  int name_len;
  VEGTK_EIF_PROC cb_data_set_is_event;
  int cb_id;
  VEGTK_EIF_PROC cb_data_set_callback_id;

#ifdef DEBUG_CALLBACKS
  g_print("signal_connect\nwidget = %x\n",widget);
  g_print("signal = %s\n",name);
  g_print("callback_data = %x\n",cb_data);
#endif

  cb_data_set_is_event = eif_dynamic_routine(cb_data,"set_is_event");
  cb_data_set_callback_id = eif_dynamic_routine(cb_data,"set_signal_id");
  eif_adopt(cb_data);

  name_len = _strlen (name);
  if ((name_len > 6 && _strcmp (&name [name_len - 6], "_event") == 0)
      || _strcmp (name, "event") == 0 )
    (cb_data_set_is_event)(cb_data, 1);

  cb_id = gtk_signal_connect (widget, name,0,(gpointer)cb_data);

  (cb_data_set_callback_id)(cb_data, cb_id);
  return cb_id;
}

int ext_signal_connect_after(void *widget,char * name,EIF_REFERENCE cb_data)
{
  int name_len;
  VEGTK_EIF_PROC cb_data_set_is_event;
  int cb_id;
  VEGTK_EIF_PROC cb_data_set_callback_id;

#ifdef DEBUG_CALLBACKS
  g_print("signal_connect\nwidget = %x\n",widget);
  g_print("signal = %s\n",name);
  g_print("callback_data = %x\n",cb_data);
#endif

  cb_data_set_is_event = eif_dynamic_routine(cb_data,"set_is_event");
  cb_data_set_callback_id = eif_dynamic_routine(cb_data,"set_signal_id");
  eif_adopt(cb_data);

  name_len = _strlen (name);
  if ((name_len > 6 && _strcmp (&name [name_len - 6], "_event") == 0)
      || _strcmp (name, "event") == 0 )
    (cb_data_set_is_event)(cb_data, 1);

  cb_id = gtk_signal_connect_after (widget, name,0,(gpointer)cb_data);

  (cb_data_set_callback_id)(cb_data, cb_id);
  return cb_id;
}

void marshal_func(GtkObject      *object,
                  gpointer        data,
                  int            nparams,
                  GtkArg         *args,
                  GtkType        *arg_types,
                  GtkType         return_type)
{
  EIF_REFERENCE cb_data;
  VEGTK_EIF_PROC cb_data_proc;
  VEGTK_EIF_PROC cbd_routine;
  EIF_REFERENCE cbd_object;
  EIF_REFERENCE cbd_data;

  if (data)
    {
      cb_data = (EIF_REFERENCE) data;

      cb_data_proc = eif_dynamic_routine(cb_data,"set_nparams");
      (cb_data_proc)(cb_data, nparams);
      cb_data_proc = eif_dynamic_routine(cb_data,"set_args");
      (cb_data_proc)(cb_data, args);
      cb_data_proc = eif_dynamic_routine(cb_data,"set_arg_types");
      (cb_data_proc)(cb_data, arg_types);
      cb_data_proc = eif_dynamic_routine(cb_data,"set_return_type");
      (cb_data_proc)(cb_data, return_type);

      cb_data_proc = eif_dynamic_routine(cb_data,"set_return_arg");
      if (return_type == GTK_TYPE_NONE)
        (cb_data_proc)(cb_data, NULL);
      else
        (cb_data_proc)(cb_data, &args[nparams]);

  /* Call Eiffel */
#ifdef DEBUG_CALLBACKS
      g_print("marshaller\ncb_data = %x\n",cbd);
      g_print("cb_data.object = %x\n",cbd->object);
      g_print("cb_data.routine = %x\n",cbd->routine);
      g_print("cb_data.widget = %x\n",cbd->widget);
      g_print("cb_data.data = %x\n",cbd->data);
      g_print("cb_data.cb_id = %x\n",cbd->callback_id);
      g_print("args = %x\n",cbd->args);
      g_print("arg_types = %x\n",cbd->arg_types);
#endif

      cbd_routine = eif_field(cb_data,"fld_routine",VEGTK_EIF_PROC);
      cbd_object = eif_field(cb_data,"fld_object",EIF_REFERENCE);
      cbd_data = eif_field(cb_data,"fld_data",EIF_REFERENCE);
//      g_print("Marshaller: routine = %x object = %x data = %x\n",cbd_routine,cbd_object,cbd_data);
      (cbd_routine)(cbd_object,cbd_data,cb_data);
    }
}

void destroy_func(gpointer data)
{
 EIF_REFERENCE cb_data = (EIF_REFERENCE) data;
 eif_wean(data);
}

GtkType ext_get_arg_type(EIF_REFERENCE data,int num) {
  GtkType* arg_types;

  arg_types = (GtkType*) eif_field(data,"fld_arg_types",EIF_POINTER);
  return arg_types[num-1];
}

EIF_INTEGER ext_get_arg_int(EIF_REFERENCE data,int n) {
  GtkArg* args;
  GtkType type;
  int ret=-1;

  type = ext_get_arg_type(data,n);
  if (type != GTK_TYPE_INT &&
      type != GTK_TYPE_UINT) {
    g_print("Warning: saved arg is not of type integer\n");
  }
  else
  {
    args = (GtkArg*) eif_field(data,"fld_args",EIF_POINTER);
    ret=GTK_VALUE_INT(args[n-1]);
  }
  return ret;
}

EIF_POINTER ext_get_arg_pointer(EIF_REFERENCE data,int n) {
  GtkArg* args;

  args = (GtkArg*) eif_field(data,"fld_args",EIF_POINTER);
  return GTK_VALUE_POINTER(args[n-1]);
}

EIF_REAL ext_get_arg_float(EIF_REFERENCE data, int n) {
  GtkArg* args;
  if (ext_get_arg_type(data,n) != GTK_TYPE_FLOAT ) {
    g_print("Warning: saved arg not of type float\n");
  }

  args =(GtkArg*) eif_field((EIF_REFERENCE)eif_access(data),"fld_args",EIF_POINTER);
  return (EIF_REAL)GTK_VALUE_FLOAT(args[n-1]);
}

EIF_REAL ext_get_arg_double(EIF_REFERENCE data, int n) {
  GtkArg* args;
  if (ext_get_arg_type(data,n) != GTK_TYPE_DOUBLE ) {
    g_print("Warning: saved arg not of type double\n");
  }

  args =(GtkArg*) eif_field((EIF_REFERENCE)eif_access(data),"fld_args",EIF_POINTER);
  return (EIF_REAL)GTK_VALUE_DOUBLE(args[n-1]);
}

void ext_set_return_arg_integer(EIF_REFERENCE data, EIF_INTEGER value) {
   GtkArg* ret_arg;

   ret_arg = (GtkArg*) eif_field(data, "fld_return_arg", EIF_POINTER);
   *GTK_RETLOC_INT(*ret_arg) = value;
}

void ext_set_return_arg_boolean(EIF_REFERENCE data, EIF_BOOLEAN value) {
   GtkArg* ret_arg;

   ret_arg = (GtkArg*) eif_field(data, "fld_return_arg", EIF_POINTER);
   *GTK_RETLOC_BOOL(*ret_arg) = value;
}
