/*
You may be able to redistribute and/or modify this program.
See the README for details.
Copyright (C) 1999 Andrew Klochkov, Eugene Melekhov
*/


#include <gtk/gtk.h>
#include <eif_callback.h>

void c_gtk_ctree_node_moveto(GtkCTree *ctree,GtkCTreeNode *node,EIF_INTEGER column,EIF_REAL row_align,EIF_REAL col_align)
{
 gtk_ctree_node_moveto(ctree,node,column,row_align,col_align);
}

/* ------------------------- ACR functions --------------------------- */

EIF_INTEGER gtk_ctree_tree_spacing_acr(GtkCTree *ctree)
{
 return ctree->tree_spacing;
}

EIF_INTEGER gtk_ctree_tree_column_acr(GtkCTree *ctree)
{
 return ctree->tree_column;
}

EIF_INTEGER gtk_ctree_tree_indent_acr(GtkCTree *ctree)
{
 return ctree->tree_indent;
}

GtkCTreeLineStyle gtk_ctree_line_style_acr (GtkCTree *ctree)
{
 return ctree->line_style;
}

/* this type is an Eiffel analog of GTK type GtkCTreeGNodeFunc */
/* (added first parameter - "Current") */
typedef EIF_BOOLEAN (*EiffelGtkCTreeGNodeFunc) (void *current,
                                            GtkCTree     *ctree,
                                            EIF_INTEGER  depth,
                                            GNode        *gnode,
                                            GtkCTreeNode *cnode,
                                            gpointer      data);


/* this function is actually called from gtk_ctree_export_to_gnode and, */
/* in turn, call Eiffel function */
gboolean ctree2gnode(GtkCTree        *ctree,
                        gint         depth,
                        GNode        *gnode,
                        GtkCTreeNode *cnode,
                        gpointer     data)
{
  eif_call_data *e_data = (eif_call_data*) data;
  if ( e_data && e_data->eiffel_func && e_data->eiffel_object)
    return ((VEGTK_EIF_FN_BOOL)e_data->eiffel_func)((EIF_REFERENCE)eif_access(e_data->eiffel_object),ctree,depth,
                                               gnode,cnode,(EIF_REFERENCE)(e_data->data?eif_access(e_data->data):0));
  else
    return 0;
}


GNode *c_gtk_ctree_export_to_gnode                (GtkCTree          *ctree,
                                                  GNode             *parent,
                                                  GNode             *sibling,
                                                  GtkCTreeNode      *node,
                                                  EIF_REFERENCE object,
                                                  VEGTK_EIF_FN_BOOL func,
                                                  EIF_REFERENCE data)
{
  GNode *result=0;
  eif_call_data *e_data = eif_call_data_new(object,func,data);
  if (e_data){
    result = gtk_ctree_export_to_gnode(ctree, parent, sibling, node, (GtkCTreeGNodeFunc)ctree2gnode, e_data);
    //g_print("export_to_gnode result = %p gnode data = %p\n",result,result->data);
    free_cb_data(e_data);
  }
  return result;
}

GtkCTreeNode * c_gtk_ctree_insert_gnode(GtkCTree             *ctree,
                                        GtkCTreeNode         *parent,
                                        GtkCTreeNode         *sibling,
                                        GNode                *node,
                                        EIF_REFERENCE        object,
                                        VEGTK_EIF_FN_BOOL          func,
                                        EIF_REFERENCE        data)
{
  GtkCTreeNode *result;
  eif_call_data *e_data = eif_call_data_new(object,func,data);

  //g_print("inserting...");

  if (e_data){
    //g_print(" calling callback...");
    result = gtk_ctree_insert_gnode(ctree, parent, sibling, node, (GtkCTreeGNodeFunc)ctree2gnode, e_data);
    //g_print(" call done...");
    free_cb_data(e_data);
  }
  return result;
}

typedef void (*EiffelGtkCTreeFunc) (void *current,
                                    GtkCTree     *ctree,
                                    GtkCTreeNode *node,
                                    gpointer      data);

void ctreefunc(GtkCTree *ctree, GtkCTreeNode *node, gpointer data)
{
  eif_call_data *e_data = (eif_call_data*) data;
  if (e_data && e_data->eiffel_object && e_data->eiffel_func)
    ((VEGTK_EIF_PROC)e_data->eiffel_func)((EIF_REFERENCE)eif_access(e_data->eiffel_object),ctree,node,
                                    (EIF_REFERENCE)(e_data->data?eif_access(e_data->data):0));
}

void c_gtk_ctree_pre_recursive                   (GtkCTree     *ctree,
                                                  GtkCTreeNode  *node,
                                                  EIF_REFERENCE object,
                                                  void          *func,
                                                  EIF_REFERENCE data)
{
  eif_call_data *e_data = eif_call_data_new(object,func,data);
  if (e_data){
    gtk_ctree_pre_recursive(ctree,node,ctreefunc,e_data);
    free_cb_data(e_data);
  }
}

void c_gtk_ctree_post_recursive                   (GtkCTree     *ctree,
                                                  GtkCTreeNode  *node,
                                                  EIF_REFERENCE object,
                                                  void          *func,
                                                  EIF_REFERENCE data)
{
  eif_call_data *e_data = eif_call_data_new(object,func,data);
  if (e_data){
    gtk_ctree_post_recursive(ctree,node,ctreefunc,e_data);
    free_cb_data(e_data);
  }
}

void
c_gtk_ctree_node_set_row_data_full (GtkCTree         *ctree,
                                    GtkCTreeNode     *node,
                                    EIF_REFERENCE    data,
                                    EIF_REFERENCE    object,
                                    void             *func)
{
  eif_call_data *e_data = eif_call_data_new(object,func,data);
  if (e_data)
    gtk_ctree_node_set_row_data_full(ctree, node, e_data, cb_destroy_call_eiffel);
}

void
c_gtk_ctree_node_set_row_data (GtkCTree         *ctree,
                               GtkCTreeNode     *node,
                               EIF_REFERENCE    data)
{
 c_gtk_ctree_node_set_row_data_full(ctree, node, data, NULL, NULL);
}

EIF_REFERENCE c_gtk_ctree_node_get_row_data(GtkCTree* ctree, GtkCTreeNode *node)
{
  eif_call_data *e_data = (eif_call_data*) GTK_CTREE_ROW(node)->row.data;
  if ( e_data && e_data->data)
    return (EIF_REFERENCE)eif_access(e_data->data);
  else
    return 0;
}



GtkCTreeNode *
c_gtk_ctree_find_by_row_data_aux (GtkCTree     *ctree,
                                  GtkCTreeNode *node,
                                  EIF_REFERENCE c_data)
{
  GtkCTreeNode *work;
  eif_call_data *e_data;

  if (!node)
    node = GTK_CTREE_NODE (GTK_CLIST (ctree)->row_list);

  while (node)
    {
      e_data = (eif_call_data *)GTK_CTREE_ROW (node)->row.data;

      if (e_data){
        if (e_data->data){
          if (((EIF_REFERENCE)eif_access(e_data->data)) == c_data)
            return node;
        }else if (c_data == 0)
          return node;
      } else if (c_data == 0)
          return node;

      if (GTK_CTREE_ROW (node)->children &&
	  (work = c_gtk_ctree_find_by_row_data_aux
	   (ctree, GTK_CTREE_ROW (node)->children, c_data)))
	return work;
      node = GTK_CTREE_ROW (node)->sibling;
    }
  return NULL;
}
GtkCTreeNode *
c_gtk_ctree_find_by_row_data (GtkCTree     *ctree,
                              GtkCTreeNode *node,
                              EIF_REFERENCE data)
{
  GtkCTreeNode *result;
  eif_call_data *e_data;
  EIF_REFERENCE c_data;

#ifdef VEGTK_ISE
  if (data)
    c_data = eif_adopt(data);
  else
    c_data =0;
#else
  c_data = data;
#endif
  result = c_gtk_ctree_find_by_row_data_aux(ctree,node,c_data);
#ifdef VEGTK_ISE
  if (c_data)
    eif_wean(c_data);
#endif
  return result;
}
