indexing
    title       : "GTK ";
    cluster     : "VEGTK";
    project     : "VEGTK library v 1.2.1 GTK+ 1.2.x";
    copyright   : "Object Tools, 1999-2000";
    key         : "GTK", "GUI";
    cvs         : "$Id: gtk_object.e,v 1.5 2000/03/16 14:03:52 eugenem Exp $";

class GTK_OBJECT
inherit
   GTK_OBJECT_EXTERNALS
   GTK_OBJECT_NP
   VEGTK_EXTERNAL_OBJECT
      redefine
         from_external
      end
   GTK_CONSTANTS
   GTK_SIGNAL_EXTERNALS

creation
   from_external,
   make_for_register

feature {ANY} -- Creation

   from_external (c_object : POINTER) is
      require else
         not_null: c_object /= default_pointer
      do
         Precursor (c_object)
         register
      ensure then
         has_c_object: to_external = c_object
      end

feature {GTK_OBJECT} -- Creation for registration in object_factory

   make_for_register is
      do
         register_without_external(gtk_object_get_type)
      ensure
         for_register_purposes_only: register_purposes
         has_not_c_object: to_external = default_pointer
      end

feature {ANY} -- Object registration in VEGTK_OBJECT_FACTORY

   register is
      do
         object_factory.register_object (Current, type)
      end

   register_without_external (c_type : INTEGER) is
      do
         register_purposes := True
         object_factory.register_without_external (Current, c_type)
      end

feature {ANY} -- Queries

   type : GTK_TYPE is
      do
         !!Result.from_external(gtk_object_type_acr(to_external))
      end

   get_user_data : ANY is
      do
         Result := np_gtk_object_get_user_data(to_external)
      end

feature {ANY} -- Operations

   destroy is
      do
         gtk_object_destroy (to_external);
      end

   set_user_data(data : ANY) is
      do
         np_gtk_object_set_user_data(to_external, data)
      end

   sink is
      do
         gtk_object_sink(to_external)
      end

   ref is
      do
         gtk_object_ref (to_external)
      end

   unref is
      do
         gtk_object_unref (to_external)
      end

   ref_count : INTEGER is
      do
         Result := gtk_object_ref_count_acr(to_external)
      end

--   dispose is
--      do
--         if ref_count = 0 then
--            destroy
--         end
--      end

feature -- eGTK-like callbacks

   add_action, signal_connect (signal_name: STRING; cmd: GTK_COMMAND_TYPE) is
         -- add a command object to handle a specific signal
      require
         signal_name_not_void: signal_name /= Void
         cmd_not_void: cmd /= Void
         cmd_not_connected: cmd.parent = Void
      do
         if actions = Void then
            !!actions.make (1, 0)
         end
         -- keep track of cmd objects
         actions.force (cmd, actions.count + 1)
         -- call C function to connect the signal handler
         cmd.connect (signal_name, Current)
      ensure
         cmd_connected: cmd.parent = Current
      end

   action_by_name (signal_name: STRING): GTK_COMMAND_TYPE is
         -- return the command object by signal name
      require
         signal_name_not_void: signal_name /= Void
      local
         cmd: GTK_COMMAND_TYPE
         i: INTEGER
      do
         from i := 1
         until (i > actions.count) or (Result /= Void)
         loop
            cmd := actions.item (i)
            if cmd /= Void and then
               signal_name.is_equal (cmd.event_name)
            then
               Result := cmd
            end
            i := i + 1
         end
      end

   remove_action (old_cmd: GTK_COMMAND_TYPE) is
         -- remove callback
      require
         old_cmd_not_void: old_cmd /= Void
      local
         i: INTEGER
         cmd: GTK_COMMAND_TYPE
      do
         from i := 1
         until i > actions.count
         loop
            cmd := actions.item (i)
            if cmd = old_cmd then
               cmd.disconnect
               actions.put (Void, i)
            end
            i := i + 1
         end
      end

   remove_all_signal_handlers is
         -- remove all action handlers
      do
         gtk_signal_handlers_destroy (to_external)
         actions := Void
      end

feature {NONE}

   actions: ARRAY [GTK_COMMAND_TYPE]

   object_factory : VEGTK_OBJECT_FACTORY is
      once
         !!Result;
      end

feature {ANY} -- Flag

   register_purposes : BOOLEAN
      -- True, if eiffel object has not corresponding C-object, ie
      -- to_external = default_pointer, only VEGTK_OBJECT_FACTORY can
      -- create such objects
      -- False, if eiffel object has corresponding C-object, is
      -- to_external /= default_pointer

invariant

   gtk_object_has_external: (not register_purposes) implies
                            to_external /= default_pointer

end -- class GTK_OBJECT
