indexing
    title       : "GTK ";
    cluster     : "VEGTK";
    project     : "VEGTK library v 1.2.1 GTK+ 1.2.x";
    copyright   : "Object Tools, 1999-2000";
    key         : "GTK", "GUI";
    cvs         : "$Id: g_list.e,v 1.3 2000/03/16 14:03:55 eugenem Exp $";

deferred class G_LIST [G]
inherit
   VEGTK_EXTERNAL_OBJECT
   G_LIST_EXTERNALS

feature -- Creation
  make is
     do
        from_external(default_pointer);
     end

feature -- access

   empty : BOOLEAN is
      do
         Result := to_external = default_pointer;
      end

feature -- operations

   next : like Current is
      do
         Result := clone(Current);
         Result.from_external(g_list_next_acr(to_external))
      end

   prev : like Current is
      do
         Result := clone(Current);
         Result.from_external(g_list_prev_acr(to_external))
      end

   append (arg_data : G) : like Current  is
      do
         Result := clone(Current);
         Result.from_external (g_list_append (to_external, data_to_pointer(arg_data)));
      end

   prepend (arg_data : G) : like Current is
      do
         Result := clone(Current);
         Result.from_external (g_list_prepend (to_external, data_to_pointer(arg_data)));
      end

   insert (arg_data : G; arg_position : INTEGER) : like Current is
      do
         Result := clone(Current);
         Result.from_external (g_list_insert (to_external, data_to_pointer(arg_data),
                                              arg_position));
      end

--   concat (list2 : G_LIST[ANY]) : like Current is
--      do
--         Result := clone(Current);
--         Result.from_external (g_list_concat (to_external,list2.to_external));
--      end

   remove (arg_data : G) : like Current is
      do
         Result := clone(Current);
         Result.from_external(g_list_remove (to_external, data_to_pointer(arg_data)));
      end

   reverse : like Current is
      do
         Result := clone(Current);
         Result.from_external(g_list_reverse  (to_external))
      end

   make_copy : like Current is
      do
         Result := clone(Current);
         Result.from_external (g_list_copy (to_external));
      end

   nth (n : INTEGER) : like Current is
      do
         Result := clone(Current);
         Result.from_external (g_list_nth (to_external, n));
      end

   find (arg_data : G) : like Current is
      do
         Result := clone(Current);
         Result.from_external (g_list_find(to_external, data_to_pointer(arg_data)));
      end

   index (arg_data : G) : INTEGER is
      do
         Result := g_list_index(to_external, data_to_pointer(arg_data));
      end

   last : like Current is
      do
         Result := clone(Current);
         Result.from_external (g_list_last (to_external));
      end

   first : like Current is
      do
         Result := clone(Current);
         Result.from_external(g_list_first  (to_external));
      end

   length : INTEGER is
      do
         if empty then
            Result := 0;
         else
            Result:= g_list_length  (to_external);
         end -- if
      end

   item : G is
      do
         if not empty then
            Result := pointer_to_data (g_list_data_acr(to_external))
         end
      end

   nth_item (n : INTEGER) : G is
      do
         if not empty then
            Result := pointer_to_data (g_list_nth_data(to_external,n))
         end
      end

feature {ANY} -- Conversion

   to_array : ARRAY[G] is
      local
         i,len : INTEGER;
         list : like Current;
      do
         from
            list := clone (Current);
            len := length
            !!Result.make (1,len)
            i := 1;
         until
            i > len
         loop
            Result.put (list.item, i);
            i := i+1;
            list := list.next;
         end; -- loop
      ensure
         Result /= Void
         Result.count = length
      end

   from_array (array : ARRAY[G]) is
      require
         is_empty_yet: to_external = default_pointer
      local
         i,last_item : INTEGER;
      do
         if array /= Void and then array.count > 0 then
            from
               i := array.lower;
               last_item := array.upper;
            until
               i > last_item
            loop
               from_external (g_list_append (to_external,
                                             data_to_pointer(array.item(i))));
               i := i+1;
            end; -- loop
         end; -- if
      ensure
         array /= Void implies to_external /= default_pointer
      end

feature -- Destroy
   free is
      do
         if to_external /= default_pointer then
            g_list_free (to_external);
         end
      end
feature { NONE } --

   pointer_data : POINTER is
   do
      Result := g_list_data_acr(to_external);
   end

   data_to_pointer (arg_data :G)  : POINTER is
   deferred
   end

   pointer_to_data (ptr : POINTER)  : G is
   deferred
   end

end -- class G_LIST
