indexing
    title       : "GTK ";
    cluster     : "VEGTK";
    project     : "VEGTK library v 1.2.1 GTK+ 1.2.x";
    copyright   : "Object Tools, 1999-2000";
    key         : "GTK", "GUI";
    cvs         : "$Id: vegtk_object_factory.e,v 1.6 2000/03/16 14:03:55 eugenem Exp $";

class VEGTK_OBJECT_FACTORY

feature -- Commands

   register_object (object : GTK_OBJECT; type : GTK_TYPE) is
      require
         object_exist: object /= Void and then
                       object.to_external /= default_pointer
--         not_registered: not has_type(type.value)
      do
         if not has_type(type) then
            add_type (object,  type.value)
         end -- if
      ensure
         has_type(type)
      end
   register_without_external (object : GTK_OBJECT; c_type : INTEGER) is
      require
         object_exist: object /= Void
      local
         e_type : GTK_TYPE;
      do
         !!e_type.from_external(c_type)
         if not has_type(e_type) then
            add_type (object,  e_type.value)
         end -- if
      end

   make_object_from_pointer (c_object : POINTER) : GTK_OBJECT is
      local
         type : GTK_TYPE
      do
         if c_object /= default_pointer then
            !!type.from_external (ext_object_type(c_object))
            Result := make_object_from_pointer_type(c_object,type);
         end
      end

   make_object_from_pointer_type (c_object : POINTER; arg_type : GTK_TYPE) : GTK_OBJECT is
      require
         pointer_not_null : c_object /= default_pointer
         has_type : has_type(arg_type)
      local
         type : INTEGER;
      do
         from
            type := arg_type.value;
         until
            object_dict.has(type) or type = 0
         loop
            type := ext_type_parent(type)
         end
         if type /= 0 then
            object_dict.search(type)
            if object_dict.found then
               Result := clone(object_dict.found_item)
               Result.from_external(c_object)
            end
         end
      end

feature -- Queries

   object_dict : HASH_TABLE [GTK_OBJECT, INTEGER] is
      once
         !!the_dict.make(50)
         Result := the_dict
      end

   has_type (type : GTK_TYPE) : BOOLEAN is
      do
         Result := object_dict.has(type.value)
      end

feature {NONE} -- Private

   the_dict : HASH_TABLE [GTK_OBJECT, INTEGER]

   add_type (gtk_object : GTK_OBJECT; object_type : INTEGER) is
      do
         if object_type = 0 then
            print("Bad stuff: ")
            print(object_type)
            print("%N")
         end
         object_dict.put(gtk_object, object_type)
      end

   ext_object_type (c_object : POINTER) : INTEGER is
      external "C"
      alias "ext_object_type"
      end

   ext_type_parent (type : INTEGER) : INTEGER is
      external "C"
      alias "gtk_type_parent"
      end

end
