indexing
    title       : "GTK ";
    cluster     : "VEGTK";
    project     : "VEGTK library v 1.2.1 GTK+ 1.2.x";
    copyright   : "Object Tools, 1999-2000";
    key         : "GTK", "GUI";
    cvs         : "$Id: vegtk_object_factory.e,v 1.6 2000/03/16 14:03:55 eugenem Exp $";

class VEGTK_OBJECT_FACTORY

feature -- Commands

   register_object (object : GTK_OBJECT; type : GTK_TYPE) is
      require
         object_exist: object /= Void and then
                       object.to_external /= default_pointer
--         not_registered: not has_type(type.value)
      do
         if not has_type(type) then
            add_type (object,  type.value)
         end -- if
      ensure
         has_type(type)
      end
   register_without_external (object : GTK_OBJECT; c_type : INTEGER) is
      require
         object_exist: object /= Void
      local
         e_type : GTK_TYPE;
      do
         !!e_type.from_external(c_type)
         if not has_type(e_type) then
            add_type (object,  e_type.value)
         end -- if
      end

   make_object_from_pointer (c_object : POINTER) : GTK_OBJECT is
      local
         type : GTK_TYPE
      do
         if c_object /= default_pointer then
            !!type.from_external (ext_object_type(c_object))
            Result := make_object_from_pointer_type(c_object,type);
         end
      end

   make_object_from_pointer_type (c_object : POINTER; arg_type : GTK_TYPE) : GTK_OBJECT is
      require
         pointer_not_null : c_object /= default_pointer
         has_type : has_type(arg_type)
      local
         type : INTEGER;
      do
         from
            type := arg_type.value;
         until
            object_dict.has_key(type) or type = 0
         loop
            type := ext_type_parent(type)
         end
         if type /= 0 then
            object_dict.search(type)
            if object_dict.found then
               Result := clone(object_dict.found_item)
               Result.from_external(c_object)
            end
         end
      end

feature -- Queries

   object_dict : HASH_TABLE_ [GTK_OBJECT, INTEGER] is
      once
         !!the_dict.make(True, 50)
         Result := the_dict
      end

   has_type (type : GTK_TYPE) : BOOLEAN is
      do
         Result := object_dict.has_key(type.value)
      end

feature {NONE} -- Private

   the_dict : HASH_TABLE_ [GTK_OBJECT, INTEGER]

   add_type (gtk_object : GTK_OBJECT; object_type : INTEGER) is
      do
         if object_type = 0 then
            print("Bad stuff: ")
            print(object_type)
            print("%N")
         end
         object_dict.put(gtk_object, object_type)
      end

feature {GTK_OBJECT} -- Type queries

   ext_object_type (c_object : POINTER) : INTEGER is
      external "C"
      alias "ext_object_type"
      end

   ext_type_parent (type : INTEGER) : INTEGER is
      external "C"
      alias "gtk_type_parent"
      end

   gtk_accel_label_get_type : INTEGER is
      external "C"
      alias "gtk_accel_label_get_type"
      end
   gtk_adjustment_get_type : INTEGER is
      external "C"
      alias "gtk_adjustment_get_type"
      end
   gtk_alignment_get_type : INTEGER is
      external "C"
      alias "gtk_alignment_get_type"
      end
   gtk_arrow_get_type : INTEGER is
      external "C"
      alias "gtk_arrow_get_type"
      end
   gtk_aspect_frame_get_type : INTEGER is
      external "C"
      alias "gtk_aspect_frame_get_type"
      end
   gtk_button_box_get_type : INTEGER is
      external "C"
      end
   gtk_bin_get_type : INTEGER is
      external "C"
      end
   gtk_box_get_type : INTEGER is
      external "C"
      end
   gtk_button_get_type : INTEGER is
      external "C"
      end
   gtk_check_button_get_type : INTEGER is
      external "C"
      end
   gtk_check_menu_item_get_type : INTEGER is
      external "C"
      end
   gtk_clist_get_type : INTEGER is
      external "C"
      end
   gtk_color_selection_get_type : INTEGER is
      external "C"
      end
   gtk_color_selection_dialog_get_type : INTEGER is
      external "C"
      end
   gtk_combo_get_type : INTEGER is
      external "C"
      end
   gtk_container_get_type : INTEGER is
      external "C"
      end
   gtk_ctree_get_type : INTEGER is
      external "C"
      end
   gtk_curve_get_type : INTEGER is
      external "C"
      end
   gtk_data_get_type : INTEGER is
      external "C"
      end
   gtk_dialog_get_type : INTEGER is
      external "C"
      end
   gtk_drawing_area_get_type : INTEGER is
      external "C"
      end
   gtk_editable_get_type : INTEGER is
      external "C"
      end
   gtk_entry_get_type : INTEGER is
      external "C"
      end
   gtk_event_box_get_type : INTEGER is
      external "C"
      end
   gtk_file_selection_get_type : INTEGER is
      external "C"
      end
   gtk_fixed_get_type : INTEGER is
      external "C"
      end
   gtk_font_selection_get_type : INTEGER is
      external "C"
      end
   gtk_font_selection_dialog_get_type : INTEGER is
      external "C"
      end
   gtk_frame_get_type : INTEGER is
      external "C"
      end
   gtk_gamma_curve_get_type : INTEGER is
      external "C"
      end
   gtk_handle_box_get_type : INTEGER is
      external "C"
      end
   gtk_hbutton_box_get_type : INTEGER is
      external "C"
      end
   gtk_hbox_get_type : INTEGER is
      external "C"
      end
   gtk_hpaned_get_type : INTEGER is
      external "C"
      end
   gtk_hruler_get_type : INTEGER is
      external "C"
      end
   gtk_hscale_get_type : INTEGER is
      external "C"
      end
   gtk_hscrollbar_get_type : INTEGER is
      external "C"
      end
   gtk_hseparator_get_type : INTEGER is
      external "C"
      end
   gtk_image_get_type : INTEGER is
      external "C"
      end
   gtk_input_dialog_get_type : INTEGER is
      external "C"
      end
   gtk_item_get_type : INTEGER is
      external "C"
      end
--   gtk_item_factory_get_type : INTEGER is -- MEV ??? Where is it
--      external "C"
--      end
   gtk_label_get_type : INTEGER is
      external "C"
      end
   gtk_list_get_type : INTEGER is
      external "C"
      end
   gtk_list_item_get_type : INTEGER is
      external "C"
      end
   gtk_menu_get_type : INTEGER is
      external "C"
      end
   gtk_menu_bar_get_type : INTEGER is
      external "C"
      end
   gtk_menu_item_get_type : INTEGER is
      external "C"
      end
   gtk_menu_shell_get_type : INTEGER is
      external "C"
      end
   gtk_misc_get_type : INTEGER is
      external "C"
      end
   gtk_notebook_get_type : INTEGER is
      external "C"
      end
   gtk_object_get_type : INTEGER is
      external "C"
      end
   gtk_option_menu_get_type : INTEGER is
      external "C"
      end
   gtk_packer_get_type : INTEGER is
      external "C"
      end
   gtk_paned_get_type : INTEGER is
      external "C"
      end
   gtk_pixmap_get_type : INTEGER is
      external "C"
      end
   gtk_preview_get_type : INTEGER is
      external "C"
      end
   gtk_progress_get_type : INTEGER is
      external "C"
      end
   gtk_progress_bar_get_type : INTEGER is
      external "C"
      end
   gtk_radio_button_get_type : INTEGER is
      external "C"
      end
   gtk_radio_menu_item_get_type : INTEGER is
      external "C"
      end
   gtk_range_get_type : INTEGER is
      external "C"
      end
   gtk_ruler_get_type : INTEGER is
      external "C"
      end
   gtk_scale_get_type : INTEGER is
      external "C"
      end
   gtk_scrollbar_get_type : INTEGER is
      external "C"
      end
   gtk_scrolled_window_get_type : INTEGER is
      external "C"
      end
   gtk_separator_get_type : INTEGER is
      external "C"
      end
   gtk_spin_button_get_type : INTEGER is
      external "C"
      end
   gtk_statusbar_get_type : INTEGER is
      external "C"
      end
   gtk_table_get_type : INTEGER is
      external "C"
      end
   gtk_text_get_type : INTEGER is
      external "C"
      end
   gtk_tips_query_get_type : INTEGER is
      external "C"
      end
   gtk_toggle_button_get_type : INTEGER is
      external "C"
      end
   gtk_toolbar_get_type : INTEGER is
      external "C"
      end
   gtk_tooltips_get_type : INTEGER is
      external "C"
      end
   gtk_tree_get_type : INTEGER is
      external "C"
      end
   gtk_tree_item_get_type : INTEGER is
      external "C"
      end
--   gtk_identifier_get_type : INTEGER is -- MEV ??? Where is it
--      external "C"
--      end
   gtk_vbutton_box_get_type : INTEGER is
      external "C"
      end
   gtk_vbox_get_type : INTEGER is
      external "C"
      end
   gtk_viewport_get_type : INTEGER is
      external "C"
      end
   gtk_vpaned_get_type : INTEGER is
      external "C"
      end
   gtk_vruler_get_type : INTEGER is
      external "C"
      end
   gtk_vscale_get_type : INTEGER is
      external "C"
      end
   gtk_vscrollbar_get_type : INTEGER is
      external "C"
      end
   gtk_vseparator_get_type : INTEGER is
      external "C"
      end
   gtk_widget_get_type : INTEGER is
      external "C"
      end
   gtk_window_get_type : INTEGER is
      external "C"
      end

end
