indexing

   title:       "Reference class for BOOLEAN";
   cluster:     "kernel";
   project:     "Eiffel Kernel Library";
   date:        "$Date: 2005/02/01 10:20:12 $";
   revision:    "$Revision: 1.1 $";
   copyright:   "Copyright (C) 1996-2005 Object Tools Group";
   license:     "http://visual-eiffel.com/license";
   disclaimer:  "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

class BOOLEAN_REF

inherit

   HASHABLE
      redefine
         out
      end

feature -- Access

   item: BOOLEAN
         -- Boolean value

   hash_code: INTEGER is
         -- Hash code value
         -- (From HASHABLE)
      do
         Result := item.hash_code
      end

feature -- Element change

   set_item (b: BOOLEAN) is
         -- Make 'b' the associated boolean value
      do
         item := b
      ensure
         item_set: item = b
      end

feature -- Basic operations

   infix "and" (other: BOOLEAN): BOOLEAN is
         -- Boolean conjunction with 'other'
      require
         other_exists: other /= Void
      do
         Result := item and other
      ensure
         result_exists: Result /= Void
         de_morgan: Result = not (not Current or (not other))
         commutative: Result = (other and item)
         consistent_with_semi_strict: Result implies (Current and then other)
      end

   infix "and then" (other: BOOLEAN): BOOLEAN is
         -- Boolean semi-strict conjunction with 'other'
      require
         other_exists: other /= Void
      do
         Result := item and then other
      ensure
         result_exists: Result /= Void
         de_morgan: Result = not (not Current or else (not other))
      end

   infix "implies" (other: BOOLEAN): BOOLEAN is
         -- Boolean implication of 'other' (semi-strict)
      require
         other_exists: other /= Void
      do
         Result := item implies other
      ensure
         definition: Result = (not Current or else other)
      end

   prefix "not": BOOLEAN is
         -- Negation
      do
         Result := not item
      end

   infix "or" (other: BOOLEAN): BOOLEAN is
         -- Boolean disjunction with 'other'
      require
         other_exists: other /= Void
      do
         Result := item or other
      ensure
         result_exists: Result /= Void
         de_morgan: Result = not (not Current and (not other))
         commutative: Result = (other or item)
         consistent_with_semi_strict: Result implies (Current or else other)
      end

   infix "or else" (other: BOOLEAN): BOOLEAN is
         -- Boolean semi-strict disjunction with 'other'
      require
         other_exists: other /= Void
      do
         Result := item or else other
      ensure
         result_exists: Result /= Void
         de_morgan: Result = not (not Current and then (not other))
      end

   infix "xor" (other: BOOLEAN): BOOLEAN is
         -- Boolean exclusive or with 'other'
      require
         other_exists: other /= Void
      do
         Result := item xor other
      ensure
         definition: Result = ((Current or other) and not (Current and other))
      end

feature -- Output

   out: STRING is
         -- Printable representation of boolean
      do
         if item then
            Result := "True"
         else
            Result := "False"
         end
      end

end