indexing

   title:       "Reference class for CHARACTER.";
   cluster:     "kernel";
   project:     "Eiffel Kernel Library";
   date:        "$Date: 2005/02/01 10:20:12 $";
   revision:    "$Revision: 1.1 $";
   copyright:   "Copyright (C) 1996-2005 Object Tools Group";
   license:     "http://visual-eiffel.com/license";
   disclaimer:  "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

class CHARACTER_REF

inherit

   HASHABLE
      undefine
         is_equal
      redefine
         out
      end

   COMPARABLE
      redefine
         out
      end

feature -- Access

   item: CHARACTER
         -- Character value

   code: INTEGER is
         -- Associated integer value
      do
         Result := item.code
      end

   hash_code: INTEGER is
         -- Hash code value
         -- (From HASHABLE)
      do
         Result := item.code
      end

feature -- Comparison

   infix "<" (other: like Current): BOOLEAN is
         -- Is current character less than 'other'?
         -- (From COMPARABLE)
      do
         Result := item < other.item
      end

feature -- Element change

   set_item (c: CHARACTER) is
         -- Make 'c' the associated character value
      do
         item := c
      ensure
         item_set: item = c
      end

feature -- Eiffel/S 1.3 compatibility

   to_integer: INTEGER is
      obsolete "Eiffel/S 1.3 compatibility, use code"
      do
         Result := item.code
      end

   from_integer (i: INTEGER) is
      do
         item.from_integer (i)
      end

   to_lower: CHARACTER is
      obsolete "Eiffel/S 1.3 compatibility, use as_lower"
         -- Lower case equivalent
      do
         Result := as_lower
      ensure
         definition: Result = as_lower
      end

   to_upper: CHARACTER is
      obsolete "Eiffel/S 1.3 compatibility, use as_upper"
         -- Upper case equivalent
      do
         Result := as_upper
      ensure
         definition: Result = as_upper
      end

feature -- Conversion

   as_lower: CHARACTER is
         -- Lower case equivalent
      do
         Result := item
         inspect
            Result
         when 'A' .. 'Z' then
            Result.from_integer (code - ('A').code + ('a').code)
         else
         end
      ensure
         converted:
            ("ABCDEFGHIJKLMNOPQRSTUVWXYZ").has (item)
            implies Result = ("abcdefghijklmnopqrstuvwxyz").item
            (("ABCDEFGHIJKLMNOPQRSTUVWXYZ").index_of (item, 1))
         unchanged:
            not ("ABCDEFGHIJKLMNOPQRSTUVWXYZ").has (item)
            implies Result = item
      end

   as_upper: CHARACTER is
         -- Upper case equivalent
      do
         Result := item
         inspect
            Result
         when 'a' .. 'z' then
            Result.from_integer (code - ('a').code + ('A').code)
         else
         end
      ensure
         converted:
            ("abcdefghijklmnopqrstuvwxyz").has (item)
            implies Result = ("ABCDEFGHIJKLMNOPQRSTUVWXYZ").item
            (("abcdefghijklmnopqrstuvwxyz").index_of (item, 1))
         unchanged:
            not ("abcdefghijklmnopqrstuvwxyz").has (item)
            implies Result = item
      end

feature -- Output

   out: STRING is
         -- Printable representation of character
         -- (From GENERAL)
      do
         if item <  ' '  and then
            item /= '%T' and then -- Tab
            item /= '%F' and then -- FF
            item /= '%R' and then -- CR
            item /= '%N'          -- LF
         then
            Result := "."
         else
            create Result.make_filled (item, 1)
         end
      end

   code_sequence: STRING is
         -- Printable representation of the character
         -- replaced by the special character code when required
         -- Examples:
         --   code_sequence ('A')      = "A"       -- printed as A
         --   code_sequence ('%B')     = "%%B"     -- printed as %B
         --   code_sequence ('%A')     = "@"       -- printed as @
         --   code_sequence ('%%')     = "%%%%"    -- printed as %%
         --   code_sequence ('%"')     = "%%%""    -- printed as %"
         --   code_sequence ('%/130/') = "%%/130/" -- printed as %/130/
         -- So, the output enclosed in quotes gives the original character
      do
         inspect item
      -- when '%A' then Result := "%%A" -- @
         when '%B' then Result := "%%B" -- %/8/
      -- when '%C' then Result := "%%C" -- ^
      -- when '%D' then Result := "%%D" -- $
         when '%F' then Result := "%%F" -- %/12/
      -- when '%H' then Result := "%%H" -- \
      -- when '%L' then Result := "%%L" -- ~
         when '%N' then Result := "%%N" -- %/13/
      -- when '%Q' then Result := "%%Q" -- `
         when '%R' then Result := "%%R" -- %/10/
      -- when '%S' then Result := "%%S" -- #
         when '%T' then Result := "%%T" -- %/9/
         when '%U' then Result := "%%U" -- %/0/
      -- when '%V' then Result := "%%V" --
         when '%%' then Result := "%%%%" -- %
         when '%'' then Result := "%%%'" -- '
         when '%"' then Result := "%%%"" -- "
      -- when '%(' then Result := "%%(" -- [
      -- when '%)' then Result := "%%)" -- ]
      -- when '%<' then Result := "%%<" -- {
      -- when '%>' then Result := "%%>" -- }
         when '%/1/'   .. '%/7/',
              '%/11/',
              '%/14/'  .. '%/31/',
              '%/127/' .. '%/255/'
         then
            Result := "%%/"
            Result.append_integer (code)
            Result.append_character ('/')
         else
            create Result.make_filled (item, 1)
         end
      ensure
         non_void_result: Result /= Void
         long_enough: Result.count >= 1
      end

end