indexing

   title:       "Objects that may be compared to a total order relation.",
                "Note: The basic operation is '<' (less than); others %
                %are defined in terms of this operation and 'is_equal'.";
   cluster:     "kernel";
   project:     "Eiffel Kernel Library";
   date:        "$Date: 2005/02/01 10:20:12 $";
   revision:    "$Revision: 1.1 $";
   copyright:   "Copyright (C) 1996-2005 Object Tools Group";
   license:     "http://visual-eiffel.com/license";
   disclaimer:  "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

deferred class COMPARABLE

inherit

   ANY
      redefine
         is_equal
      end

feature -- Comparison

   infix "<" (other: like Current): BOOLEAN is
         -- Is current object less than 'other'?
      require
         other_exists: other /= Void
      deferred
      ensure
         asymmetric: Result implies not (other < Current)
      end

   infix "<=" (other: like Current): BOOLEAN is
         -- Is current object less than or equal to 'other'?
      require
         other_exists: other /= Void
      do
         Result := not (other < Current)
      ensure
         definition: Result = (Current < other or is_equal (other))
      end

   infix ">=" (other: like Current): BOOLEAN is
         -- Is current object greater than or equal to 'other'?
      require
         other_exists: other /= Void
      do
         Result := not (Current < other)
      ensure
         definition: Result = (other <= Current)
      end

   infix ">" (other: like Current): BOOLEAN is
         -- Is current object greater than 'other'?
      require
         other_exists: other /= Void
      do
         Result := other < Current
      ensure
         definition: Result = (other < Current)
      end

   is_equal (other: like Current): BOOLEAN is
         -- Is 'other' attached to an object considered equal
         -- to current object?
      do
         Result := (not (Current < other) and not (other < Current))
      ensure then
         trichotomy: Result = (not (Current < other) and not (other < Current))
      end

   max (other: like Current): like Current is
         -- The greater of current object and 'other'
      require
         other_exists: other /= Void
      do
         if Current < other then
            Result := other
         else
            Result := Current
         end
      ensure
         current_if_not_smaller: (Current >= other) implies Result.is_equal (Current) -- (Result = Current)
         other_if_smaller: (Current <  other) implies Result.is_equal (other) -- (Result = other)
      end

   min (other: like Current): like Current is
         -- The smaller of current object and 'other'
      require
         other_exists: other /= Void
      do
         if Current <= other then
            Result := Current
         else
            Result := other
         end
      ensure
         current_if_not_greater: (Current <= other) implies Result.is_equal (Current) -- (Result = Current)
         other_if_greater: (Current >  other) implies Result.is_equal (other) -- (Result = other)
      end

   three_way_comparison (other: like Current): INTEGER is
         -- If current object equal to 'other', 0; if smaller, -1; if greater, 1
      require
         other_exists: other /= Void
      do
         if Current < other then
            Result := -1
         elseif Current > other then
            Result := 1
         end
      ensure
         equal_zero: (Result =  0) = is_equal (other)
         smaller_negative: (Result = -1) = (Current < other)
         greater_positive: (Result =  1) = (Current > other)
      end

feature -- Eiffel/S 1.3 compatibility

   compare (other: like Current): INTEGER is
      obsolete "Eiffel/S 1.3 compatibility, use three_way_comparison"
         -- Compare Current with 'other'.
         -- '<' <==> Result < 0
         -- '>' <==> Result > 0
         -- Otherwise Result = 0
      do
         Result := three_way_comparison (other)
      end

invariant

   irreflexive_comparison: not (Current < Current)

end