indexing

   title:       "Facilities for adapting the exception handling mechanism.",
                "This class may be used as ancestor by classes needing %
                %its facilities.";
   cluster:     "kernel";
   project:     "Eiffel Kernel Library";
   date:        "$Date: 2005/02/01 10:20:12 $";
   revision:    "$Revision: 1.1 $";
   copyright:   "Copyright (C) 1996-2005 Object Tools Group";
   license:     "http://visual-eiffel.com/license";
   disclaimer:  "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

class EXCEPTIONS

feature -- Developer exception name

   developer_exception_name: STRING is
         -- Name of last developer-raised exception
      require
         applicable: is_developer_exception
      external "C"
         alias "c_last_etext"
      end

feature -- Access

   Check_instruction: INTEGER is 10006
         -- Exception code for violated check

   Class_invariant: INTEGER is 10003
         -- Exception code for violated class invariant

   Incorrect_inspect_value: INTEGER is 10007
         -- Exception code for inspect value which is not one
         -- of the inspect constants, if there is no Else_part

   Loop_invariant: INTEGER is 10005
         -- Exception code for violated loop invariant

   Loop_variant: INTEGER is 10004
         -- Exception code for non-decreased loop variant

   No_more_memory: INTEGER is 10008
         -- Exception code for failed memory allocation

   Postcondition: INTEGER is 10002
         -- Exception code for violated postcondition

   Precondition: INTEGER is 10001
         -- Exception code for violated precondition

   Routine_failure: INTEGER is 10063
         -- Exception code for failed routine

   Void_attached_to_expanded: INTEGER is 10032
         -- Exception code for attachment of void value
         -- to expanded entity

   Void_call_target: INTEGER is 10009
         -- Exception code for violated check

feature -- Status report

   assertion_violation: BOOLEAN is
         -- Is last exception originally due to a violated
         -- assertion or non-decreasing variant?
      do
         inspect exception
            when
               Check_instruction,
               Class_invariant,
               Loop_invariant,
               Loop_variant,
               Postcondition,
               Precondition
            then
               Result := True
            else
         end
      end

   exception: INTEGER is
         -- Code of last exception that occured
      external "C"
         alias "c_last_ecode"
      end

   is_developer_exception: BOOLEAN is
         -- Is the last exception originally due to a developer exception?
      do
         inspect exception
            when
               Check_instruction,
               Class_invariant,
               Incorrect_inspect_value,
               Loop_invariant,
               Loop_variant,
               No_more_memory,
               Postcondition,
               Precondition,
               Routine_failure,
               Void_attached_to_expanded,
               Void_call_target
            then
            else
               Result := True
         end
      end

   is_signal: BOOLEAN is
         -- Is last exception originally due to an external event
         -- (operating system signal)?
      external "C"
         alias "isSignal"
      end

feature -- Basic operations

   die (code: INTEGER) is
         -- Terminate execution with exit status 'code',
         -- without triggering an exception
      external "C"
      end

   raise (name: STRING) is
         -- Raise a developer exception of name 'name'
      require
         name_non_void: name /= Void
      do
         ex_raise (name, 20000)
      end

feature {NONE} -- Implementation

   ex_raise (name: STRING; code: INTEGER) is
      external "C"
         alias "e_raise"
      end

end