indexing

   title:       "Reference class for INTEGER.";
   cluster:     "kernel";
   project:     "Eiffel Kernel Library";
   date:        "$Date: 2005/02/01 10:20:12 $";
   revision:    "$Revision: 1.1 $";
   copyright:   "Copyright (C) 1996-2005 Object Tools Group";
   license:     "http://visual-eiffel.com/license";
   disclaimer:  "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

class INTEGER_REF

inherit

   HASHABLE
      undefine
         is_equal
      redefine
         out
      end

   NUMERIC
      rename
         infix "/" as infix "//"
      undefine
         is_equal
      redefine
         out
      end

   COMPARABLE
      redefine
         out
      end

feature -- Access

   item: INTEGER
         -- Integer value

   hash_code: INTEGER is
         -- Hash code value
         -- (From HASHABLE)
      do
         if item < 0 then
            Result := - item
         else
            Result := item
         end
      end

feature -- Comparison

   infix "<" (other: like Current): BOOLEAN is
         -- Is current integer less than 'other'?
         -- (From COMPARABLE)
      do
         Result := item < other.item
      end

feature -- Element change

   set_item (i: INTEGER) is
         -- Make 'i' the associated integer value
      do
         item := i
      ensure
         item_set: item = i
      end

   one: like Current is
         -- Neutral element for "*" and "/"
         -- (From NUMERIC)
      do
         Result := clone (Current)
         Result.set_item (1)
      ensure then
         value: Result.item = 1
      end

   sign: INTEGER is
         -- Sign value (0, -1 or 1)
      do
         Result := three_way_comparison (zero)
      ensure
         three_way: Result = three_way_comparison (zero)
      end

   zero: like Current is
         -- Neutral element for "+" and "-"
         -- (From NUMERIC)
      do
         Result := clone (Current)
         Result.set_item (0)
      ensure then
         value: Result.item = 0
      end

feature -- Status report

   divisible (other: like Current): BOOLEAN is
         -- May current object be divided by 'other'?
         -- (From NUMERIC)
      do
         Result := other.item /= 0
      ensure then
         value: Result implies (other.item /= 0)
      end

   exponentiable (other: DOUBLE): BOOLEAN is
         -- May current object be elevated to the power 'other'?
         -- (From NUMERIC)
      do
         if not Result and then item ^ other.item = 0.0 then
            -- do_nothing
         end
         Result := not Result
      rescue
         Result := True
         retry
      end

feature -- Basic operations

   abs: like Current is
         -- Absolute value
      do
         if Current < zero then Result := - Current
         else                   Result := Current
         end
      ensure
         non_negative: Result.item >= 0
         same_absolute_value: standard_equal (Result,   Current) or
                              standard_equal (Result, - Current)
      end

   infix "*" (other: like Current): like Current is
         -- Product by 'other'
         -- (From NUMERIC)
      do
         Result := clone (Current)
         Result.set_item (item * other.item)
      end

   infix "+" (other: like Current): like Current is
         -- Sum with 'other'
         -- (From NUMERIC)
      do
         Result := clone (Current)
         Result.set_item (item + other.item)
      end

   infix "-" (other: like Current): like Current is
         -- Result of subtracting 'other'
         -- (From NUMERIC)
      do
         Result := clone (Current)
         Result.set_item (item - other.item)
      end

   infix "/" (other: like Current): DOUBLE is
         -- Division by 'other'
      do
         Result := item / other.item
      end

   infix "//" (other: like Current): like Current is
         -- Integer division of Current by 'other'
         -- (From "/" in NUMERIC)
      do
         Result := clone (Current)
         Result.set_item (item // other.item)
      end

   infix "\\" (other: like Current): like Current is
         -- Remainder of the integer division of Current by 'other'
      do
         Result := clone (Current)
         Result.set_item (item \\ other.item)
      end

   infix "^" (other: DOUBLE): DOUBLE is
         -- Integer power of Current by 'other'
         -- (From NUMERIC)
      do
         Result := item ^ other
      end

   prefix "+": like Current is
         -- Unary plus
         -- (From NUMERIC)
      do
         Result := clone (Current)
         Result.set_item (+ item)
      end

   prefix "-": like Current is
         -- Unary minus
         -- (From NUMERIC)
      do
         Result := clone (Current)
         Result.set_item (- item)
      end

feature -- Output

   out: STRING is
         -- Printable representation of integer value
         -- (From GENERAL)
      do
         create Result.make_empty
         Result.append_integer (item)
      end

end