indexing

   title:       "Input stream that reads items from array";
   cluster:     "kernel.io";
   project:     "Eiffel Kernel Library: input and output";
   date:        "$Date: 2005/02/15 16:22:03 $";
   revision:    "$Revision: 1.2 $";
   copyright:   "Copyright (C) 1996-2005 Object Tools Group";
   license:     "http://visual-eiffel.com/license";
   disclaimer:  "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

class ARRAYED_INPUT_STREAM [G]

inherit

   INPUT_STREAM [G]

create

   make

feature {NONE} -- Creation

   make (data: ARRAY [G]) is
         -- Create an object attached to array `data'.
      require
         non_void_data: data /= Void
      do
         storage := data
         index := storage.lower - 1
      ensure
         is_open: is_open
         not_end_of_stream: not end_of_stream
      end

feature -- Status report

   is_open: BOOLEAN is
         -- Is stream open?
      do
         Result := storage /= Void
      end

   end_of_stream: BOOLEAN is
         -- Has end of stream been detected?
      do
         Result := not storage.valid_index (index)
      end

feature -- Status setting

   close is
         -- Close stream.
         -- Set `is_open' to false on success.
      do
         storage := Void
      end

feature -- Access

   item: G
         -- Last item read from the stream

feature -- Read

   read is
         -- Read a next item from the stream if
         -- possible and make it available in `item'.
         -- Set `end_of_stream' to true if there are
         -- no more items in the stream.
      do
         index := index + 1
         if not end_of_stream then
            item := storage.item (index)
         end
      end

feature {NONE} -- Data

   storage: ARRAY [G]
         -- Data to be streamed

   index: INTEGER
         -- Index of the last item read

end