indexing

   title:       "Input stream of data in binary format";
   cluster:     "kernel";
   project:     "Eiffel Kernel Library: input and output";
   date:        "$Date: 2005/02/15 16:22:03 $";
   revision:    "$Revision: 1.2 $";
   copyright:   "Copyright (C) 1996-2005 Object Tools Group";
   license:     "http://visual-eiffel.com/license";
   disclaimer:  "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

class BINARY_INPUT_STREAM

inherit

   DATA_INPUT

   MEMORY

   PROXY_BYTE_INPUT_STREAM
      redefine
         make
      end

create

   make

feature {NONE} -- Creation

   make (input: BYTE_INPUT_STREAM) is
      do
         Precursor (input)
         create last_string.make_empty
      end

feature -- Input

   read_character is
         -- Read a new character.
         -- Make result available in 'last_character'.
      local
         tmp: CHARACTER
      do
         if read_buffer ($tmp, 1) = 1 then
            last_character := tmp
         end
      end

   read_double is
         -- Read the binary representation of a new double from file.
         -- Make result available in 'last_double'.
      local
         tmp: DOUBLE
      do
         if complete_read_buffer ($tmp, 8) = 8 then
            last_double := tmp
         end
      end

   read_integer is
         -- Read the binary representation of a new integer from file.
         -- Make result available in 'last_integer'.
      local
         tmp: INTEGER
      do
         if complete_read_buffer ($tmp, 4) = 4 then
            last_integer := tmp
         end
      end

   read_line is
         -- Read a string until new line or end of file.
         -- Make result available in 'last_string'.
         -- New line will be consumed but not part of 'last_string'.
      local
         exit: BOOLEAN
         buffered_character: CHARACTER
      do
         last_string.wipe_out
         from
            exit := read_buffer ($buffered_character, 1) = 0
         until
            exit
         loop
            if buffered_character = '%N' then
               if last_string.count > 0 and then
                  last_string.item (last_string.count) = '%R'
               then
                  last_string.resize (last_string.count - 1)
               end
               exit := True
            else
               last_string.append_character (buffered_character)
               exit := read_buffer ($buffered_character, 1) = 0
            end
         end
      end

   read_real is
         -- Read the binary representation of a new real from file.
         -- Make result available in 'last_real'.
      local
         tmp: REAL
      do
         if complete_read_buffer ($tmp, 8) = 8 then
            last_real := tmp
         end
      end

   read_stream (nb_char: INTEGER) is
         -- Read a string of a most 'nb_char' bound characters or until end of file.
         -- Make result available in 'last_string'.
      local
         l: INTEGER
         gc_off: BOOLEAN
      do
         last_string.resize (nb_char)
         if nb_char > 0 then
            collection_off
            gc_off := True
            l := complete_read_buffer (last_string.to_c, nb_char)
            collection_on
            gc_off := False
            if l < nb_char then
               last_string.resize (l)
            end
         end
      rescue
         if gc_off then
            gc_off := False
            collection_on
         end
      end

feature {NONE} -- Implementation

   complete_read_buffer (buffer: POINTER; amount: INTEGER): INTEGER is
         -- Read at most 'amount' bytes into 'buffer' at current position
         -- repeatedly until 'amount' is reached or there is no more data.
         -- Return actual number of bytes read.
      require
         is_open: is_open
         buffer_exists: buffer /= default_pointer
         non_negative_amount: amount >= 0
      local
         buf: POINTER
         buf_size: INTEGER
         act_size: INTEGER
      do
         from
            buf := buffer
            buf_size := amount
            act_size := amount
         until
            buf_size = 0 or else act_size = 0
         loop
            act_size := read_buffer (buf, buf_size)
            Result := Result + act_size
            buf := buf + act_size
            buf_size := buf_size - act_size
         end
      ensure
         non_negative_result: Result >= 0
         valid_result: Result <= amount
      end

invariant

   non_void_last_string: last_string /= Void

end