indexing

   title:       "Facilities for tuning up the garbage collection mechanism.",
                "This class may be used as ancestor by classes needing %
                %its facilities.";
   cluster:     "kernel";
   project:     "Eiffel Kernel Library";
   date:        "$Date: 2005/02/01 10:20:12 $";
   revision:    "$Revision: 1.1 $";
   copyright:   "Copyright (C) 1996-2005 Object Tools Group";
   license:     "http://visual-eiffel.com/license";
   disclaimer:  "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

class MEMORY

feature -- Status report

   collecting: BOOLEAN is
         -- Is garbage collection enabled?
      external "C"
         alias "GCEnabled"
      end

feature -- Status setting

   collection_off is
         -- Disable garbage collection
      external "C"
         alias "_disableGC"
      end

   collection_on is
         -- Enable garbage collection
      external "C"
         alias "_EnableGC"
      end

feature -- Removal

   dispose is
         -- Action to be executed just before garbage collection
         -- reclaims an object.
         -- Default version does nothing; redefine in descendants
         -- to perform specific dispose actions. Those actions
         -- should only take care of freeing external resources
         -- they should not perform remote calls on other objects
         -- since these may also be dead and reclaimed
      do
         do_nothing
      end

   full_collect is
         -- Force a full collection cycle if garbage
         -- collection is enabled; do nothing otherwise
      external "C"
         alias "_forceGC"
      end

feature -- X/Special: Garbage Collector controling

   set_threshold (new_threshold_size: INTEGER) is
      external "C"
         alias "_set_threshold"
      end

   set_threshold_delta (new_threshold_delta: INTEGER) is
      external "C"
         alias "_set_threshold_delta"
      end

feature -- Visual Eiffel specific: external memory controlling

   mem_copy (dest, source: POINTER; bytes_number: INTEGER) is
      external "C"
         alias "_mem_copy"
      end

   mem_cmp (left, right: POINTER; bytes_number: INTEGER): INTEGER is
      external "C"
         alias "_memcmp"
      end

feature -- Visual Eiffel specific: memory profiling

   class_memory (object: ANY): INTEGER is
      external "C"
         alias "_class_memory"
      end

   deep_class_memory (object: ANY): INTEGER is
      external "C"
         alias "_deep_class_memory"
      end

   gc_calls: INTEGER is
      external "C"
         alias "_gc_calls"
      end

   gc_times: INTEGER is
      external "C"
         alias "_gc_times"
      end

   gc_total_memory: INTEGER is
      external "C"
         alias "_gc_total_memory"
      end

   heap_profile is
      external "C"
         alias "_zone_profile"
      end

   classes_profile is
      external "C"
         alias "_classes_profile"
      end

end