indexing

   title:       "Objects to which numerical operations are applicable.",
                "Note: The model is that of a commutative ring.";
   cluster:     "kernel";
   project:     "Eiffel Kernel Library";
   date:        "$Date: 2005/02/01 10:20:12 $";
   revision:    "$Revision: 1.1 $";
   copyright:   "Copyright (C) 1996-2005 Object Tools Group";
   license:     "http://visual-eiffel.com/license";
   disclaimer:  "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

deferred class NUMERIC

feature -- Access

   one: like Current is
         -- Neutral element for "*" and "/"
      deferred
      ensure
         result_exists: Result /= Void
      end

   zero: like Current is
         -- Neutral element for "+" and "-"
      deferred
      ensure
         result_exists: Result /= Void
      end

feature -- Status report

   divisible (other: like Current): BOOLEAN is
         -- May current object be divided by 'other'?
      require
         other_exists: other /= Void
      deferred
      end

   exponentiable (other: DOUBLE): BOOLEAN is
         -- May current object be elevated to the power 'other'?
      require
         other_exists: -- other /= Void
      deferred
      end

feature -- Basic operations

   infix "+" (other: like Current): like Current is
         -- Sum with 'other' (commutative)
      require
         other_exists: other /= Void
      deferred
      ensure
         result_exists: Result /= Void
         commutative: equal (Result, other + Current)
      end

   infix "-" (other: like Current): like Current is
         -- Result of subtracting 'other'
      require
         other_exists: other /= Void
      deferred
      ensure
         result_exists: Result /= Void
      end

   infix "*" (other: like Current): like Current is
         -- Product by 'other'
      require
         other_exists: other /= Void
      deferred
      ensure
         result_exists: Result /= Void
      end

   infix "/" (other: like Current): like Current is
         -- Division by 'other'
      require
         other_exists: other /= Void
         good_divisor: divisible (other)
      deferred
      ensure
         result_exists: Result /= Void
      end

   infix "^" (other: DOUBLE): DOUBLE is
         -- Current object to the power 'other'
      require
         other_exists: -- other /= Void
         good_exponent: -- exponentiable (other)
      deferred
      ensure
         result_exists: -- Result /= Void
      end

   prefix "+": like Current is
         -- Unary plus
      deferred
      ensure
         result_exists: Result /= Void
      end

   prefix "-": like Current is
         -- Unary minus
      deferred
      ensure
         result_exists: Result /= Void
      end

invariant

   neutral_addition: equal (Current + zero, Current)
   self_subtraction: equal (Current - Current, zero)
   neutral_multiplication: equal (Current * one, Current)
   self_division: divisible (Current) implies equal (Current / Current, one)

end