indexing

   title:       "Platform-dependent properties.",
                "This class may be used as ancestor by classes needing %
                %its facilities.";
   cluster:     "kernel";
   project:     "Eiffel Kernel Library";
   date:        "$Date: 2005/02/01 10:20:12 $";
   revision:    "$Revision: 1.1 $";
   copyright:   "Copyright (C) 1996-2005 Object Tools Group";
   license:     "http://visual-eiffel.com/license";
   disclaimer:  "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

class PLATFORM

feature -- Access

   Boolean_bits: INTEGER is
         -- Number of bits in a value of type BOOLEAN
      once
         Result := 1
      ensure
         meaningful: Result >= 1
      end

   Character_bits: INTEGER is
         -- Number of bits in a value of type CHARACTER
      once
         Result := 8
      ensure
         meaningful: Result >= 1
         large_enough: 2 ^ Result >= Maximum_character_code
      end

   Double_bits: INTEGER is
         -- Number of bits in a value of type DOUBLE
      once
         Result := Real_bits
      ensure
         meaningful: Result >= 1
         meaningful: Result >= Real_bits
      end

   Integer_bits: INTEGER is
         -- Number of bits in a value of type INTEGER
      once
         Result := 32
      ensure
         meaningful: Result >= 1
         large_enough: 2 ^ Result >= Maximum_integer
         large_enough_for_negative: 2 ^ Result >= -Minimum_integer
      end

   Integer_8_bits: INTEGER is
         -- Number of bits in a value of type INTEGER_8
      once
         Result := 8
      ensure
         meaningful: Result >= 1
         large_enough: 2 ^ Result >= Maximum_integer_8
         large_enough_for_negative: 2 ^ Result >= -Minimum_integer_8
      end

   Integer_16_bits: INTEGER is
         -- Number of bits in a value of type INTEGER_16
      once
         Result := 16
      ensure
         meaningful: Result >= 1
         large_enough: 2 ^ Result >= Maximum_integer_16
         large_enough_for_negative: 2 ^ Result >= -Minimum_integer_16
      end

   Maximum_character_code: INTEGER is
         -- Largest supported code for CHARACTER values
      once
         Result := 255
      ensure
         meaningful: Result >= 127
      end

   Maximum_integer: INTEGER is
         -- Largest supported code of type INTEGER
      once
         Result := 2147483647
      ensure
         meaningful: Result >= 0
      end

   Maximum_integer_8: INTEGER is
         -- Largest supported code of type INTEGER_8
      once
         Result := 127
      ensure
         meaningful: Result >= 0
      end

   Maximum_integer_16: INTEGER is
         -- Largest supported code of type INTEGER_16
      once
         Result := 32767
      ensure
         meaningful: Result >= 0
      end

   Minimum_character_code: INTEGER is
         -- Smallest supported code for CHARACTER values
      once
         Result := 0
      ensure
         meaningful: Result <= 0
      end

   Minimum_integer: INTEGER is
         -- Smallest supported code of type INTEGER
      once
         Result := - 2147483648
      ensure
         meaningful: Result <= 0
      end

   Minimum_integer_8: INTEGER is
         -- Smallest supported code of type INTEGER_8
      once
         Result := - 128
      ensure
         meaningful: Result <= 0
      end

   Minimum_integer_16: INTEGER is
         -- Smallest supported code of type INTEGER_16
      once
         Result := - 32768
      ensure
         meaningful: Result <= 0
      end

   Pointer_bits: INTEGER is
         -- Number of bits in a value of type POINTER
      once
         Result := Integer_bits
      ensure
         meaningful: Result >= 1
      end

   Real_bits: INTEGER is
         -- Number of bits in a value of type REAL
      once
         Result := 64
      ensure
         meaningful: Result >= 1
      end

feature -- Access

   Boolean_bytes: INTEGER is 1
         -- Number of bytes occupied by a value of type BOOLEAN

   Character_bytes: INTEGER is 1
         -- Number of bytes occupied by a value of type CHARACTER

   Double_bytes: INTEGER is 8
         -- Number of bytes occupied by a value of type DOUBLE

   Integer_bytes: INTEGER is 4
         -- Number of bytes occupied by a value of type INTEGER

   Integer_8_bytes: INTEGER is 1
         -- Number of bytes occupied by a value of type INTEGER_8

   Integer_16_bytes: INTEGER is 2
         -- Number of bytes occupied by a value of type INTEGER_16

   Pointer_bytes: INTEGER is 4
         -- Number of bytes occupied by a value of type POINTER

   Real_bytes: INTEGER is 8
         -- Number of bytes occupied by a value of type REAL

end