indexing

   title:       "Reference class for POINTER.";
   cluster:     "kernel";
   project:     "Eiffel Kernel Library";
   date:        "$Date: 2005/02/01 10:20:12 $";
   revision:    "$Revision: 1.1 $";
   copyright:   "Copyright (C) 1996-2005 Object Tools Group";
   license:     "http://visual-eiffel.com/license";
   disclaimer:  "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

class POINTER_REF

inherit

   HASHABLE
      redefine
         out
      end

feature -- Access

   item: POINTER
         -- Pointer value

   hash_code: INTEGER is
         -- Hash code value
         -- (From HASHABLE)
      do
         Result := item.hash_code
      end

feature -- Element change

   set_item (p: POINTER) is
         -- Make 'p' the associated pointer value
      do
         item := p
      ensure
         item_set: item = p
      end

feature -- Output

   out: STRING is
         -- Printable representation of pointer value
         -- (From GENERAL)
      local
         b: BIT 32
         i: INTEGER
      do
         b.from_integer (to_integer)
         create Result.make_filled ('%U', 8)
         from
            -- i := 0
         until
            i = 8
         loop
            i := i + 1
            Result.put (hexadecimal_digits.item ((b ^ 28).to_integer + 1), i)
            b := b ^ -4
         end
      end

feature -- Conversion

   to_integer: INTEGER is
         -- The corresponding integer value of current pointer value
      do
         Result := item.to_integer
      end

   from_integer (value: INTEGER) is
         -- The corresponding integer value of current pointer value
      do
         item.from_integer (value)
      end

feature -- Arithmetic operations

   infix "+" (offset: INTEGER): POINTER is
      do
         Result.from_integer (item.to_integer + offset)
      end

feature {NONE} -- Implementation

   hexadecimal_digits: STRING is
      once
         Result := "0123456789ABCDEF"
      end

end