indexing

   title:       "View of the memory region as an array of objects. %
                %Should not be used with type 'G' tuned to class with %
                %reference attributes";
   cluster:     "Run_Time_System";
   project:     "Visual Eiffel Library";
   date:        "$Date: 2005/02/01 10:20:14 $";
   revision:    "$Revision: 1.1 $";
   copyright:   "Copyright (C) 1996-2005 Object Tools Group";
   license:     "http://visual-eiffel.com/license";
   disclaimer:  "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

deferred class RTS_MEMORY_MAP [G]

inherit

   RTS_MEMORY
      end

feature -- Access

   infix "@", item ( i : INTEGER ) : G is
         -- Entry at index 'i' if in index range
      require
         good_key: valid_index (i)
      do
         memory.mem_copy ($Result, item_area (i), item_size)
      end

feature -- Measurement

   size: INTEGER is
      do
         Result := count * item_size
      end

   count: INTEGER is
         -- Actual number of items
      deferred
      ensure
         non_negative_count: count >= 0
      end

feature -- Status report

   valid_index (i: INTEGER): BOOLEAN is
         -- Is 'i' within the bounds of the memory region?
      do
         Result := i > 0 and i <= count
      end

feature -- Element change

   put (v: G; i: INTEGER) is
         -- Replace item at index 'i' by 'v'
      require
         good_key: valid_index (i)
      do
         memory.mem_copy (item_area (i), $v, item_size)
      ensure
         -- insertion_done: item (i) = v
      end

   fill (v: G; start_index: INTEGER; end_index: INTEGER) is
         -- Replace every item having index in range 'start_index'..'end_index'
         -- with 'v'
      require
         valid_start_index: 1 <= start_index
         valid_end_index:        end_index <= count
         valid_index_range: start_index <= end_index + 1
      local
         i: INTEGER
      do
         from
            i := start_index
         invariant
            -- for every j in start_index..i item (j) = v
         variant
            end_index - i + 1
         until
            i > end_index
         loop
            put (v, i)
            i := i + 1
         end
      ensure
         definition: -- for every i in start_index..end_index item (i) = v
      end

   move (lower_index: INTEGER; upper_index: INTEGER; distance: INTEGER) is
         -- Move range 'lower_index'..'upper_index' by 'distance'
         -- positions. Negative 'distance' moves towards lower indices.
         -- Free places do not change
      require
         valid_lower_index: 1 <= lower_index
         valid_upper_index:      upper_index <= count
         valid_index_range: lower_index <= upper_index + 1
         inside_bounds: 1 <= lower_index + distance and
                                  upper_index + distance <= count
      do
         if distance /= 0 and then lower_index <= upper_index then
            memory.mem_copy(
               item_area (lower_index + distance),
               item_area (lower_index),
               item_size * (upper_index - lower_index + 1)
            )
         end
      end

feature {NONE} -- Implementation

   item_size: INTEGER is
      local
         pair: RTS_MEMORY_MAP_ITEM_PAIR_STUB [G]
      do
         create pair
         Result := pair.item_size
      end

   item_area (i: INTEGER): POINTER is
         -- Address of entry at index 'i'
      require
         good_key: valid_index (i)
      do
         Result.from_integer (area.to_integer + (i - 1) * item_size)
      end

invariant

   non_negative_count: count >= 0

end

class RTS_MEMORY_MAP_ITEM_PAIR_STUB [G]

feature -- Access

   item_size: INTEGER is
      local
         p1: POINTER
         p2: POINTER
      do
         p1.set_item ($dummy_item1)
         p2.set_item ($dummy_item2)
         Result := (p1.to_integer - p2.to_integer).abs
      end

feature {NONE} -- Implementation

   dummy_item1: G
   dummy_item2: G

end