indexing

   project:     "Visual Eiffel run-time system";
   date:        "$Date: 2005/02/01 10:20:15 $";
   revision:    "$Revision: 1.1 $";
   copyright:   "Copyright (C) 1996-2005 Object Tools Group";
   license:     "http://visual-eiffel.com/license";
   disclaimer:  "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

class RTS_CLASS_INFO

inherit

   RTS_CLASS_INFO_
      redefine
         out
      end

creation

   make

feature

   instance_length: INTEGER is
      do
         memory.mem_copy ($Result, area + 4, 4)
      end

   out: STRING is
      do
         Result := name
         Result.append_string (", is_extended: ")
         Result.append_string (is_extended_ci_format.out)
      end

   name: STRING is
      do
         !!Result.make_empty
         Result.from_c (area + name_pool_offset)
      ensure
         non_void_result: Result /= Void
         valid_result: Result.count > 0
      end

   name_pool: STRING is
      do
         !!Result.make_filled ('%U', sizeof - name_pool_offset)
         memory.mem_copy (Result.to_c, area + name_pool_offset, Result.count)
      end

   attribute_name (att_id: INTEGER): STRING is
      require
         valid_attribute_id: att_id > 0 and then att_id <= number_of_attributes
      do
         Result := name_from_pool (att_id + 1, name_pool)
      ensure
         non_void_result: Result /= Void
         valid_result: Result.count > 0
      end

   has_ancestor (aname: STRING): BOOLEAN is
      require
         non_void_name: aname /= Void
         valid_name: aname.count > 0
      local
         i, n: INTEGER
         p: POINTER
         ci: RTS_CLASS_INFO
         parents: RTS_MEMORY_MAPPED_AREA_ARRAY [POINTER]
      do
         from
            n := number_of_parents
            parents := ancestors_table
         until
            i = n or else Result
         loop
            i := i + 1
            p := parents.item (i)
            if p /= default_pointer then
               ci := class_info_from_union (p)
               if ci.name.is_equal (aname) then
                  Result := True
               end
            end
         end
      end

   class_id: INTEGER is
      do
         Result := (ci_flags ^ 16).to_integer
      ensure
         acceptable_class_id :
            class_id > 0 and then class_id <= rts_main_data.compiler_data.max_id
      end

   ci_flags: BIT 32 is
      do
         memory.mem_copy ($Result, area + 22, 4)
      ensure
         flags_consistency :
            (is_predefined implies is_expanded) and
            (is_predefined implies not is_deferred) and
            (is_deferred implies not is_expanded)
      end

feature -- Status

   is_expanded: BOOLEAN is
      do
         Result := ci_flags.item (is_expanded_item)
      end

   is_deferred: BOOLEAN is
      do
         Result := ci_flags.item (is_deferred_item)
      end

   is_predefined: BOOLEAN is
      do
         Result := ci_flags.item (is_predefined_item)
      end

   is_bit: BOOLEAN is
      do
         Result := ci_flags.item (is_bit_item)
      end

   is_require_on: BOOLEAN is
      do
         Result := ci_flags.item (require_on_item)
      end

   is_ensure_on: BOOLEAN is
      do
         Result := ci_flags.item (ensure_on_item)
      end

   is_class_invariant_on: BOOLEAN is
      do
         Result := ci_flags.item (class_invariant_on_item)
      end

   is_loop_invariant_on: BOOLEAN is
      do
         Result := ci_flags.item (loop_invariant_on_item)
      end

   is_loop_variant_on: BOOLEAN is
      do
         Result := ci_flags.item (loop_variant_on_item)
      end

   is_check_on: BOOLEAN is
      do
         Result := ci_flags.item (check_on_item)
      end

   is_extended_ci_format: BOOLEAN is
      do
         Result := ci_flags.item (new_ci_format_item)
      end

feature -- Numbers

   gc_index: INTEGER is
      do
         Result := (ci_flags and 00001111b).to_integer
      end

   number_of_parents: INTEGER is
      do
         memory.mem_copy ($Result, area + 8, 2)
      end

   number_of_references: INTEGER is
      do
         memory.mem_copy ($Result, area + 10, 2)
      end

   number_of_expanded_creators: INTEGER is
      do
         memory.mem_copy ($Result, area + 12, 2)
      end

   number_of_own_routines: INTEGER is
      do
         memory.mem_copy ($Result, area + 14, 2)
      end

   number_of_attributes: INTEGER is
      do
         memory.mem_copy ($Result, area + 16, 2)
      end

   number_of_callbacks: INTEGER is
      do
         memory.mem_copy ($Result, area + 18, 2)
      end

   number_of_string_constants: INTEGER is
      do
         memory.mem_copy ($Result, area + 20, 2)
      end

   class_check_summa: INTEGER is
      do
         memory.mem_copy ($Result, area + 26, 4)
      end

   array_item_class_info: RTS_CLASS_INFO is
      local
         p: POINTER
      do
         memory.mem_copy ($p, area + 30, Pointer_bytes)
         if p /= default_pointer then
            Result := class_info_from_union (p)
         end
      end

   reverse_class_info: RTS_CLASS_INFO is
         -- returns reference class info for expanded type and vice versa
      local
         p: POINTER
      do
         memory.mem_copy ($p, area + 34, Pointer_bytes)
         if p /= default_pointer then
            Result := class_info_from_union (p)
         end
      end

feature -- Tables

   routine_info_table: ARRAY [RTS_ROUTINE_INFO] is
      require
         positive_number_of_own_routines: number_of_own_routines > 0
      local
         i, n: INTEGER
         table: RTS_MEMORY_MAPPED_AREA_ARRAY [POINTER]
         p: POINTER
      do
         from
            table := routine_address_table
            n := table.count
            !!Result.make (1, n)
         until
            i = n
         loop
            i := i + 1
            p := table.item (i)
            check
               valid_routine_address: p /= default_pointer
            end
            Result.put (routine_address2routine_info (p), i)
         end
      ensure
         valid_result :
            Result /= Void and then Result.count = number_of_own_routines
      end

   routine_address_table: RTS_MEMORY_MAPPED_AREA_ARRAY [POINTER] is
      local
         p: POINTER
      do
         memory.mem_copy ($p, area + 38, Pointer_bytes)
         if p /= default_pointer then
            !!Result.make (p, number_of_own_routines)
         end
      ensure
         valid_result :
            number_of_own_routines > 0 implies
            Result /= Void and then Result.count = number_of_own_routines
      end

   ancestors_table: RTS_MEMORY_MAPPED_AREA_ARRAY [POINTER] is
      do
         !!Result.make (area + ancestors_table_offset, number_of_parents)
      end

   attribute_id_table: RTS_MEMORY_MAPPED_AREA_ARRAY [POINTER] is
      do
         !!Result.make (area + attributes_id_table_offset, number_of_attributes)
      end

   attribute_offsets_table: RTS_MEMORY_MAPPED_AREA_ARRAY [INTEGER] is
      do
         !!Result.make(
            area + attributes_offsets_table_offset, number_of_attributes
         )
      end

   attribute_description_table: ARRAY [RTS_ATTRIBUTE] is
      local
         i, n: INTEGER
         a: RTS_ATTRIBUTE
      do
         n := number_of_attributes
         if n > 0 then
            from
               !!Result.make (1, n)
            until
               i = n
            loop
               i := i + 1
               !!a.make (Current, i)
               Result.put (a, i)
            end
         end
      ensure
         valid_table :
            number_of_attributes > 0 implies
            Result /= Void and then Result.count = number_of_attributes
      end

feature -- Addresses

   default_rescue_address: POINTER is
      do
         memory.mem_copy ($Result, area + 42, Pointer_bytes)
      end

   dispose_address: POINTER is
      do
         memory.mem_copy ($Result, area + 46, Pointer_bytes)
      end

   class_invariant_address: POINTER is
      do
         memory.mem_copy ($Result, area + 50, Pointer_bytes)
      end

feature -- Flags constants

   is_predefined_item:      INTEGER is  5
   loop_invariant_on_item:  INTEGER is  6
   is_expanded_item:        INTEGER is  7
   is_debug_item:           INTEGER is  8
   require_on_item:         INTEGER is  9
   ensure_on_item:          INTEGER is 10
   class_invariant_on_item: INTEGER is 11
   check_on_item:           INTEGER is 12
   loop_variant_on_item:    INTEGER is 13
   is_deferred_item:        INTEGER is 14
   new_ci_format_item:      INTEGER is 15
   is_bit_item:             INTEGER is 16

feature {NONE} -- implementation

   ancestors_table_offset: INTEGER is 58

   references_table_offset: INTEGER is
      do
         Result :=
            ancestors_table_offset + number_of_parents * Pointer_bytes
      end

   expanded_currents_table_offset: INTEGER is
      do
         Result :=
            references_table_offset + number_of_references * 4
      end

   expanded_creators_table_offset: INTEGER is
      do
         Result :=
            expanded_currents_table_offset +
            number_of_expanded_creators * Pointer_bytes
      end

   attributes_id_table_offset: INTEGER is
      do
         Result :=
            expanded_creators_table_offset +
            number_of_expanded_creators * Pointer_bytes
      end

   attributes_offsets_table_offset: INTEGER is
      do
         Result :=
            attributes_id_table_offset +
            number_of_attributes * Pointer_bytes
      end

   callbacks_table_offset: INTEGER is
      do
         Result :=
            attributes_offsets_table_offset +
            number_of_attributes * 4
      end

   string_constants_offsets_table_offset: INTEGER is
      do
         Result :=
            callbacks_table_offset + number_of_callbacks * Pointer_bytes
      end

   string_constants_table_offset: INTEGER is
      do
         Result :=
            string_constants_offsets_table_offset +
            number_of_string_constants * 4
      end

   name_pool_offset: INTEGER is
      do
         Result :=
            string_constants_table_offset +
            number_of_string_constants * Pointer_bytes
      end

 --struct _CIFLAGS
 -- {
 --  unsigned gc_index: 4;   1
 --  unsigned is_predefined: 1;   5  // 16
 --  unsigned loop_invariant_on: 1;   6  // 32
 --  unsigned is_expanded: 1;   7  // 64
 --  unsigned is_debug: 1;   8  // 128
 --  unsigned require_on: 1;   9  // 256
 --  unsigned ensure_on: 1;  10  // 512
 --  unsigned class_invariant_on: 1;  11  // 1024
 --  unsigned check_on: 1;  12  // 2048
 --  unsigned loop_variant_on: 1;  13  // 4096
 --  unsigned is_deferred: 1;  14  // 8192
 --  unsigned is_new_format: 1;  15  // 16384
 --  unsigned is_bit: 1;  16  // 32768
 --  unsigned magic_number: 16;     // the field for run-time system magic number
 --};                     // The class flags
 --
 --
 --struct _CLASS_INFO
 --  {
 -- 0  int     size_of;     // length of the class info (including 'size_of')
 -- 4  DWORD   len;         // instance length
 -- 8  USHORT  pcnt;        // parents           counter
 -- 10 USHORT  ocnt;        // reference objects counter
 -- 12 USHORT  ecnt;        // expanded  objects counter
 -- 14 USHORT  rcnt;        // routines          counter
 -- 16 USHORT  acnt;        // attributes        counter
 -- 18 USHORT  bcnt;        // callbacks         counter
 -- 20 USHORT  scnt;        // string constants  counter
 -- 22 CIFLAGS flags
 -- 26 DWORD   hsum;        // check sum (high 32 bits)
 -- 30 MAGIC   atype;       // array item or generic type(magic number OR pointer to corresponding class info)
 --                         // or number of bits
 -- 34 MAGIC   rtype;       // 'reverse' type (reference for expanded or vice versa)(magic number OR pointer to corresponding class info)
 -- 38 DWORD   fpntr;       // pointer to first routine in Routine' Table
 -- 42 DWORD   rpntr;       // pointer to 'default_rescue' (or Void if does not exist)
 -- 46 DWORD   dpntr;       // pointer to 'dispose' (or Void if does not exist)
 -- 50 DWORD   ipntr;       // pointer to the class invariant (or Void if does not exist)
 -- 54 DWORD   java_1
 -- 58 MAGIC   parray [1];  // array of the parrents(magic numbers OR pointers to corresponding class infos)
 --
 --// //MAGIC  parray [pcnt]; see above
 --//   DWORD  oarray [ocnt]; array of offsets of all references in the class(including references in expanded objects)
 --//   DWORD  carray [ecnt]; array of offsets of all expanded attributes for wich it's necessary to invoke creation procedure
 --//   void   *earray [ecnt]; array of the above creation procedures
 --//   MAGIC  aarray [acnt]; array of the attributes(magic numbers OR pointers to corresponding class infos)
 --//   DWORD  farray [acnt]; array of the above attributes offsets
 --//   void  *barray [bcnt]; array of the all callback routines in the class
 --//   DWORD  sarray [scnt]; array of the all ofssets of all string constants in the class
 --//   char  *marray [scnt]; array of the all addresses of all string constants in the class
 --//   char   names  [~];    It's a pool of the '\0'-terminated strings: the first is the class name; others are attribute names
 -- }

end