indexing

   project:     "Visual Eiffel run-time system";
   date:        "$Date: 2005/02/01 10:20:15 $";
   revision:    "$Revision: 1.1 $";
   copyright:   "Copyright (C) 1996-2005 Object Tools Group";
   license:     "http://visual-eiffel.com/license";
   disclaimer:  "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

class RTS_ROUTINE_INFO

inherit

   RTS_MEMORY_AREA
      rename
         size as sizeof
      end

   RTS_SERVER
      undefine
         is_equal
      end

creation

   make

feature -- Names & types

   name: STRING is
      do
         !!Result.make_empty
         Result.from_c (area + routine_name_offset)
      ensure
         non_void_result: Result /= Void
         valid_result: Result.count > 0
      end

   argloc_name (name_id: INTEGER): STRING is
      require
         valid_name_id :
            number_of_args_locs > 0 and then
            name_id > 0 and then name_id <= number_of_args_locs
      do
         Result := name_from_pool (name_id, name_pool)
      ensure
         non_void_result: Result /= Void
         valid_result: Result.count > 0
      end

   result_class_type: RTS_CLASS_INFO is
      local
         p: POINTER
      do
         memory.mem_copy ($p, area + 28, Pointer_bytes)
         if p /= default_pointer then
            Result := class_info_from_union (p)
         end
      end

   source_class_type: RTS_CLASS_INFO is
      local
         p: POINTER
      do
         memory.mem_copy ($p, area + 4, Pointer_bytes)
         check
            valid_class_type: p /= default_pointer
         end
         Result := class_info_from_union (p)
      ensure
         non_void_result: Result /= Void
      end

feature -- Description tables

   precursors_description_table: ARRAY [RTS_ROUTINE_INFO] is
      local
         n, i: INTEGER
         table: RTS_MEMORY_MAPPED_AREA_ARRAY [POINTER]
         ri: RTS_ROUTINE_INFO
      do
         table := precursors_table
         if table /= Void then
            from
               n := table.count
               !!Result.make (1, n)
            until
               i = n
            loop
               i := i + 1
               !!ri.make (table.item (i))
               Result.put (ri, i)
            end
         end
      ensure
         valid_result :
            number_of_precursors > 0 implies
            Result /= Void and then Result.count = number_of_precursors
      end

   args_description_table: ARRAY [RTS_ARG_LOC] is
      local
         off_table: RTS_MEMORY_MAPPED_AREA_ARRAY [INTEGER_16]
         i, n: INTEGER
         a: RTS_ARG_LOC
      do
         n := number_of_args_locs
         if n > 0 then
            off_table := args_locs_offsets_table
            if off_table /= Void and then off_table.item (1).as_integer > 0 then
               from
                  !!Result.make (1, 0)
               until
                  i = n
               loop
                  i := i + 1
                  if off_table.item (i).as_integer > 0 then
                     !!a.make (Current, i)
                     Result.force (a, Result.upper + 1)
                  else
                     i := n
                  end
               end
            end
         end
      ensure
         valid_result :
            Result /= Void implies Result.count <= number_of_args_locs
      end

   args_locs_description_table: ARRAY [RTS_ARG_LOC] is
      require
         valid_argloc_number: number_of_args_locs > 0
      local
         i,
         n: INTEGER
         a: RTS_ARG_LOC
      do
         n := number_of_args_locs
         if n > 0 then
            from
               !!Result.make (1, n)
            until
               i = n
            loop
               i := i + 1
               !!a.make (Current, i)
               Result.put (a, i)
            end
         end
      ensure
         valid_result :
            number_of_args_locs > 0 implies
            Result /= Void and then Result.count = number_of_args_locs
      end

feature -- Addresses

   rescue_address: POINTER is
      do
         memory.mem_copy ($Result, area + 8, Pointer_bytes)
      end

   once_result_address: POINTER is
      do
         memory.mem_copy ($Result, area + 32, Pointer_bytes)
      end

   callback_tab_address: POINTER is
      do
         memory.mem_copy ($Result, area + 36, Pointer_bytes)
      end

   profiler_structure_address: POINTER is
      do
         memory.mem_copy ($Result, area + 42, Pointer_bytes)
      end

feature -- Lengths

   sizeof: INTEGER is
      do
         memory.mem_copy ($Result, area, 4)
      ensure then
         acceptable_size: sizeof > 0
      end

   arguments_length: INTEGER is
      do
         memory.mem_copy ($Result, area + 14, 2)
      end

   locals_length: INTEGER is
      do
         memory.mem_copy ($Result, area + 16, 4)
      end

feature -- Numbers

   number_of_references: INTEGER is
      do
         memory.mem_copy ($Result, area + 20, 2)
      end

   number_of_args_locs: INTEGER is
      do
         memory.mem_copy ($Result, area + 22, 2)
      end

   number_of_exp_locs: INTEGER is
      do
         memory.mem_copy ($Result, area + 24, 2)
      end

   number_of_constant_strings: INTEGER is
      do
         memory.mem_copy ($Result, area + 26, 2)
      end

   number_of_precursors: INTEGER is
      local
         o: INTEGER
      do
         o := extended_information_offset
         if o > 0 then
            memory.mem_copy ($Result, area + o, 2)
         end
      end

feature -- Pools

   precursors_table: RTS_MEMORY_MAPPED_AREA_ARRAY [POINTER] is
      local
         n: INTEGER
      do
         n := number_of_precursors
         if n > 0 then
            !!Result.make (area + extended_information_offset + 16, n)
         end
      ensure
         valid_result :
            number_of_precursors > 0 implies
            Result /= Void and then Result.count = number_of_precursors
      end

   args_locs_offsets_table: RTS_MEMORY_MAPPED_AREA_ARRAY [INTEGER_16] is
      local
         n: INTEGER
      do
         n := number_of_args_locs
         if n > 0 then
            check
               valid_extended_information_offset :
                  extended_information_offset > 0
            end
            !!Result.make(
               area + extended_information_offset + 16 +
               number_of_precursors * Pointer_bytes,
               n
            )
         end
      ensure
         valid_result :
            number_of_args_locs > 0 implies
            Result /= Void and then Result.count = number_of_args_locs
      end

   args_locs_types_table: RTS_MEMORY_MAPPED_AREA_ARRAY [POINTER] is
      local
         n: INTEGER
      do
         n := number_of_args_locs
         if n > 0 then
            check
               valid_extended_information_offset :
                  extended_information_offset > 0
            end
            !!Result.make(
               area + extended_information_offset + 16 +
               number_of_precursors * Pointer_bytes +
               n * 2,
               n
            )
         end
      ensure
         valid_result :
            number_of_args_locs > 0 implies
            Result /= Void and then Result.count = number_of_args_locs
      end

   name_pool: STRING is
      local
         n, o: INTEGER
      do
         n := number_of_args_locs
         if n > 0 then
            check
               valid_extended_information_offset :
                  extended_information_offset > 0
            end
            o :=
               extended_information_offset + 16 +
               number_of_precursors * Pointer_bytes +
               n * 2 + n * Pointer_bytes
            !!Result.make_filled ('%U', sizeof - o)
            memory.mem_copy (Result.to_c, area + o, Result.count)
         end
      ensure
         valid_name_pool:
            number_of_args_locs > 0 implies
            Result /= Void and then Result.count > 0
      end

feature {NONE} -- Implementation

   ri_flags: BIT 32 is
      do
         memory.mem_copy ($Result, area + 12, 2)
      end

   is_ref_result: INTEGER is 1
   is_native_code: INTEGER is 2
   is_once: INTEGER is 3
   initialized_exp_result: INTEGER is 4
   is_expanded_code: INTEGER is 6
   is_trace_on: INTEGER is 7
   is_profile_on: INTEGER is 8
   is_time_on: INTEGER is 9

   references_table_offset: INTEGER is 46

   expanded_locals_table_offset: INTEGER is
      do
         Result :=
            references_table_offset +
            number_of_references * 2
      end

   expanded_creators_table_offset: INTEGER is
      do
         Result :=
            expanded_locals_table_offset +
            number_of_exp_locs * 2
      end

   string_constants_offsets_table_offset: INTEGER is
      do
         Result :=
            expanded_creators_table_offset +
            number_of_exp_locs * Pointer_bytes
      end

   string_constants_table_offset: INTEGER is
      do
         Result :=
            string_constants_offsets_table_offset +
            number_of_constant_strings * 2
      end

   routine_name_offset: INTEGER is
      do
         Result :=
            string_constants_table_offset +
            number_of_constant_strings * Pointer_bytes
      end

   extended_information_offset: INTEGER is
      do
         memory.mem_copy ($Result, area + 40, 2)
      end

 --struct _Routine_Descriptor
 --   {
 -- 0  int   length;        // length of the routine descriptor (including 'length')
 -- 4  MAGIC classmn;       // The routine class magic number or pointer to corresponding Class Info
 -- 8  Code  rescue;        // Pointer to the rescue
 -- 12 short flags;         // Result is reference, native code
 -- 14 short arglen;        // Arguments size
 -- 16 int   loclen;        // Locals size
 -- 20 short refcnt;        // Number of references (n)
 -- 22 short alcnt;         // Number of arguments & locals (memory)
 -- 24 short expcnt;        // Number of expanded locs for which it's necessary to call creation procedure
 -- 26 short strcnt;        // Number of all constant strings in the routine info
 -- 28 MAGIC resmn;         // Result magic number or pointer to corresponding Class Info
 -- 32 void  *once_result;  // Pointer to once result
 -- 36 void  *callback_tab; // Pointer to address of callback table
 -- 40 short prs_cntr_off;  // precursors' counter offset
 --
 -- 42 DWORD prof;          // pointer to profiler support structure
 --
 -- 46 short oarray [1];    // Offsets of all references
 --   /*
 --    short  oarray  [refcnt];   // see above
 --    short  carray  [expcnt];   // array of offsets of expanded locs for wich it's necessary to invoke creation procedure
 --    DWORD  earray  [expcnt];   // array of the pointers to the above creation procedures
 --    short  sarray  [strcnt];   // array of offsets of all constant strings in the routine info
 --    char   *varray [strcnt];   // array of values of all constant strings in the routine info (they are placed in class info)
 --    char   rtn_name [~];       // the routine name
 --    // DEBUG information
 -- 0  USHORT precursors_cnt
 -- 2  void  *req_ptr
 -- 6  void  *ens_ptr
 -- 10 void  *old_ptr
 -- 14 short regs_off
 -- 16 DWORD  precursors_array [precursors_cnt]
 --    // DEBUG information (filled if alcnt >= 0)
 --    short  farray [alcnt];     // array of offsets of all args&locs in the routine
 --    MAGIC  marray [alcnt];     // array of the above args&locs types (magic numbers OR pointers to corresponding class infos)
 --    char   names  [~];         // a pool of the '\0'-terminated strings: each string is a name of an attribute or local.
 --   */
 --  }

end
