indexing

   project:     "Visual Eiffel run-time system";
   date:        "$Date: 2005/02/01 10:20:14 $";
   revision:    "$Revision: 1.1 $";
   copyright:   "Copyright (C) 1996-2005 Object Tools Group";
   license:     "http://visual-eiffel.com/license";
   disclaimer:  "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

class RTS_SERVER

inherit

   PLATFORM
      end

feature -- Run-time reflection

   rts_main_data: RTS_MAIN_DATA is
      once
         !!Result.make
      end

   class_info_from_union (ci: POINTER): RTS_CLASS_INFO is
      require
         valid_pointer: ci /= default_pointer
      do
         if ci.to_integer > Maximum_integer_16 then
            create Result.make (ci)
         else
            check
               valid_class_id: valid_id (ci.to_integer)
            end
            create Result.make(
                  rts_main_data.compiler_data.class_info_table.item (ci.to_integer)
            )
         end
      end

   class_info_by_name (class_name: STRING): RTS_CLASS_INFO is
      require
         non_void_class_name: class_name /= Void
         valid_class_name: class_name.count > 0
      local
         i, n: INTEGER
         p: POINTER
         ci: RTS_CLASS_INFO
      do
         from
            n := rts_main_data.compiler_data.max_id
         until
            i = n or else Result /= Void
         loop
            i := i + 1
            p := rts_main_data.compiler_data.class_info_table.item (i)
            if p /= default_pointer then
               !!ci.make (p)
               if ci.name.is_equal (class_name) then
                  Result := ci
               end
            end
         end
      end

   name_from_pool (name_id: INTEGER; pool: STRING): STRING is
      require
         valid_name_id: name_id > 0
         non_void_pool: pool /= Void
         valid_pool: pool.count > 0
      local
         pos: INTEGER
         id: INTEGER
      do
         if name_id = 1 then
            Result := pool
         else
            from
               id := 1
            until
               id = name_id
            loop
               pos := pos + 1
               id  := id + 1
               pos := pool.index_of ('%U', pos)
            end
            Result := pool.substring (pos + 1, pool.count)
         end

         pos := Result.index_of ('%U', 1)
         if pos /= 0 then
            Result := Result.substring (1, pos - 1)
         end
      ensure
         non_void_result: Result /= Void
         valid_result: Result.count > 0
      end

   routine_address2routine_info (p: POINTER): RTS_ROUTINE_INFO is
      require
         valid_pointer: p /= default_pointer
         valid_address: rts_main_data.module_data.is_eiffel_code (p)
      local
         memory: expanded MEMORY
      do
         memory.mem_copy ($p, p + (- Pointer_bytes), Pointer_bytes)
         create Result.make (p)
      ensure
         non_void_result: Result /= Void
      end

   object2class_id (obj: ANY): INTEGER is
      require
         non_void_object: obj /= Void
      local
         p: POINTER
         memory: expanded MEMORY
      do
         p.from_integer (obj.object_id - 4)
         memory.mem_copy ($Result, p, 2)
      ensure
         valid_class_id: valid_id (Result)
      end

   object2class_info (obj: ANY): RTS_CLASS_INFO is
      require
         non_void_object: obj /= Void
      do
         Result := id2class_info (object2class_id (obj))
      ensure
         non_void_result: Result /= Void
      end

   id2class_info (id: INTEGER): RTS_CLASS_INFO is
      require
         valid_id: valid_id (id)
      do
         !!Result.make (rts_main_data.compiler_data.class_info_table.item (id))
      ensure
         non_void_result: Result /= Void
      end

   valid_id (id: INTEGER): BOOLEAN is
      do
         Result := id > 0 and then id <= rts_main_data.compiler_data.max_id
         if Result then
            Result :=
               rts_main_data.compiler_data.class_info_table.item (id) /=
               default_pointer
         end
      end

feature -- External memory managment

   allocate_memory (size: INTEGER): POINTER is
      require
         valid_memory_size: size > 0
      external "C"
         alias "malloc"
      end

   release_memory (p: POINTER) is
      require
         valid_memory_address: p /= default_pointer
      external "C"
         alias "free"
      end

feature -- Conversion

   as_pointer: POINTER is
         -- Address of the current object data
      do
         Result.set_item ($Current)
      end

feature -- Reflection

   root_instance: ANY is
         -- The root object of the system
      external "C"
      end

end
