indexing

   title:       "Commonly used input and output mechanisms.",
                "This class may be used as either ancestor or supplier %
                %by classes needing its facilities.";
   cluster:     "kernel";
   project:     "Eiffel Kernel Library";
   date:        "$Date: 2005/02/01 10:20:12 $";
   revision:    "$Revision: 1.1 $";
   copyright:   "Copyright (C) 1996-2005 Object Tools Group";
   license:     "http://visual-eiffel.com/license";
   disclaimer:  "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

class STD_FILES

feature -- Access

   default_output: FILE is
         -- Default output
      do
         inspect default_output_mode.item
         when default_output_mode_none   then
         when default_output_mode_output then Result := output
         when default_output_mode_error  then Result := error
         end
      end

   error: FILE is
         -- Standard error file
      once
         create Result.make_open_stderr
      end

   input: FILE is
         -- Standard input file
      once
         create Result.make_open_stdin
      end

   output: FILE is
         -- Standard output file
      once
         create Result.make_open_stdout
      end

   standard_default: FILE is
         -- Return the default_output or output if default_output is Void
      do
         if default_output = Void then
            Result := output
         else
            Result := default_output
         end
      end

feature -- Status report

   last_character: CHARACTER is
         -- Last character read by 'read_character'
      do
         Result := input.last_character
      end

   last_double: DOUBLE is
         -- Last double read by 'read_double'
      do
         Result := input.last_double
      end

   last_integer: INTEGER is
         -- Last integer read by 'read_integer'
      do
         Result := input.last_integer
      end

   last_real: REAL is
         -- Last real read by 'read_real'
      do
         Result := input.last_real
      end

   last_string: STRING is
         -- Last string read by 'read_line', 'read_stream', or 'read_word'
      do
         Result := input.last_string
      end

feature -- Element change

   put_new_line is
         -- Write line feed at end of default output
      do
         if default_output = Void then
            set_output_default
         end
         default_output.put_string (new_line)
      end

   put_boolean (b: BOOLEAN) is
         -- Write 'b' at end of default output
      do
         if default_output = Void then
            set_output_default
         end
         default_output.put_boolean (b)
      end

   put_character (c: CHARACTER) is
         -- Write 'c' at end of default output
      do
         if default_output = Void then
            set_output_default
         end
         default_output.put_character (c)
      end

   put_double (d: DOUBLE) is
         -- Write 'd' at end of default output
      do
         if default_output = Void then
            set_output_default
         end
         default_output.put_double (d)
      end

   put_integer (i: INTEGER) is
         -- Write 'i' at end of default output
      do
         if default_output = Void then
            set_output_default
         end
         default_output.put_integer (i)
      end

   put_real (r: REAL) is
         -- Write 'r' at end of default output
      do
         if default_output = Void then
            set_output_default
         end
         default_output.put_real (r)
      end

   put_string (s: STRING) is
         -- Write 's' at end of default output
      require
         string_exists: s /= Void
      do
         if default_output = Void then
            set_output_default
         end
         default_output.put_string (s)
      end

   set_error_default is
         -- Use standard error as default output
      do
         default_output_mode.copy (default_output_mode_error)
      end

   set_output_default is
         -- Use standard output as default output
      do
         default_output_mode.copy (default_output_mode_output)
      end

feature -- Input

   to_next_line is
         -- Move to next input line on standard input
      do
         input.to_next_line
      end

   read_character is
         -- Read a new character from standard input.
         -- Make result available in 'last_character'
      do
         input.read_character
      end

   read_double is
         -- Read a new double from standard input.
         -- Make result available in 'last_double'
      do
         input.read_double
      end

   read_integer is
         -- Read a new integer from standard input.
         -- Make result available in 'last_integer'
      do
         input.read_integer
      end

   read_line is
         -- Read a line from standard input.
         -- Make result available in 'last_string'.
         -- New line will be consumed but not part of 'last_string'
      do
         input.read_line
      end

   read_real is
         -- Read a new real from standard input.
         -- Make result available in 'last_real'
      do
         input.read_real
      end

   read_stream (nb_char: INTEGER) is
         -- Read a string of at most 'nb_char' bound characters from standard input.
         -- Make result available in 'last_string'
      do
         input.read_stream (nb_char)
      end

feature -- Eiffel/S 1.3 compatibility

   last_char: CHARACTER is
      obsolete "Eiffel/S 1.3 compatibility. Use 'last_character' instead"
         -- Last character read by 'read_character'
      do
         Result := last_character
      end

   last_int: INTEGER is
      obsolete "Eiffel/S 1.3 compatibility. Use 'last_integer' instead"
         -- Last integer read by 'read_integer'
      do
         Result := last_integer
      end

   get_string is
      obsolete "Eiffel/S 1.3 compatibility. Use 'read_line' instead"
         -- Read a line from standard input.
         -- Make result available in 'last_string'.
         -- New line will be consumed but not part of 'last_string'
      do
         read_line
      end

   get_int is
      obsolete "Eiffel/S 1.3 compatibility. Use 'read_integer' instead"
         -- Read a new integer from standard input.
         -- Make result available in 'last_integer'
      do
         read_integer
      end

   get_real is
      obsolete "Eiffel/S 1.3 compatibility. Use 'read_real' instead"
         -- Read a new real from standard input.
         -- Make result available in 'last_real'
      do
         read_real
      end

   get_double is
      obsolete "Eiffel/S 1.3 compatibility. Use 'read_double' instead"
         -- Read a new double from standard input.
         -- Make result available in 'last_double'
      do
         read_double
      end

   get_char is
      obsolete "Eiffel/S 1.3 compatibility. Use 'read_character' instead"
         -- Read a new character from standard input.
         -- Make result available in 'last_character'
      do
         read_character
      end

   put_bool (b: BOOLEAN) is
      obsolete "Eiffel/S 1.3 compatibility. Use 'put_boolean' instead"
         -- Write 'b' at end of default output
      do
         put_boolean (b)
      end

   put_char (c: CHARACTER) is
      obsolete "Eiffel/S 1.3 compatibility. Use 'put_character' instead"
         -- Write 'c' at end of default output
      do
         put_character (c)
      end

   put_int (i: INTEGER) is
      obsolete "Eiffel/S 1.3 compatibility. Use 'put_integer' instead"
         -- Write 'i' at end of default output
      do
         put_integer (i)
      end

   put_newline is
      obsolete "Eiffel/S 1.3 compatibility. Use 'put_new_line' instead"
         -- Write line feed at end of default output
      do
         put_new_line
      end

feature {NONE} -- Implementation

   default_output_mode_none,
   default_output_mode_output,
   default_output_mode_error: INTEGER is unique

   default_output_mode: INTEGER_REF is
         -- Does default output goes to error stream?
      once
         Result := default_output_mode_none
      end

   new_line: STRING is
      once
         Result := "%N";
      end

end