indexing
   title:       "Commonly used mathematical constants and functions, %
                %random numbers generator";
   cluster:     "pool";
   project:     "Visual Eiffel Library";
   date:        "$Date: 2005/02/01 10:20:16 $";
   revision:    "$Revision: 1.1 $";
   copyright:   "Copyright (C) 1996-2005 Object Tools Group";
   license:     "http://visual-eiffel.com/license";
   disclaimer:  "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

class MATH

feature -- Constants

   pi: REAL is 3.14159265358979323846

   e: REAL is 2.71828182845904523536

   deg: REAL is 57.29577951308232087680
         -- deg/radian

   phi: REAL is 1.61803398874989484820
         -- golden ratio

feature -- Functions

   arccos (x: REAL): REAL is
      require
         legal_argument: -1.0 <= x and then x <= 1.0
      external "C"
         alias "xacos"
      end

   arccosh (x: REAL): REAL is
      require
         non_negative_argument: x >= 1.0
      external "C"
         alias "xacosh"
      end

   arcsin (x: REAL): REAL is
      require
         legal_argument: -1.0 <= x and then x <= 1.0
      external "C"
         alias "xasin"
      end

   arcsinh (x: REAL): REAL is
      external "C"
         alias "xasinh"
      end

   arctan (x: REAL): REAL is
         -- Result in the range [-pi/2, pi/2]
      external "C"
         alias "xatan"
      end

   arctan2 (y, x: REAL): REAL is
         -- Inverse tangent of y/x in range (-pi, pi]
         -- Quadrant depends on signs of x, y
      external "C"
         alias "xatan2"
      end

   arctanh (x: REAL): REAL is
      require
         --non_negative_argument: x >= 0.0
      external "C"
         alias "xatanh"
      end

   cos (x: REAL): REAL is
      external "C"
         alias "xcos"
      end

   cosh (x: REAL): REAL is
      external "C"
         alias "xcosh"
      end

   exp (x: REAL): REAL is
      external "C"
         alias "xexp"
      end

   log (x: REAL): REAL is
      require
         positive_argument: x > 0.0
      external "C"
         alias "xlog"
      end

   sin (x: REAL): REAL is
      external "C"
         alias "xsin"
      end

   sinh (x: REAL): REAL is
      external "C"
         alias "xsinh"
      end

   sqrt (x: REAL): REAL is
      require
         non_negative_argument: x >= 0.0
      external "C"
         alias "xsqrt"
      end

   tan (x: REAL): REAL is
      external "C"
         alias "xtan"
      end

   tanh (x: REAL): REAL is
      external "C"
         alias "xtanh"
      end

feature -- Random numbers

   rand: INTEGER is
      external "C"
      end

   srand (seed: INTEGER) is
      external "C"
      end

end -- class MATH
