indexing
   description: "Some useful facilities on objects of basic types            ";
   title      : "EiffelBase: library of reusable components for Eiffel.      ",
                "Based on EiffelBase library for ISE Eiffel,                 ",
                "used with permission.                                       ";
   cluster    : EiffelBase;
   project    : "EiffelBase Library";
   copyright  : "(c) 1986-1997 Interactive Software Engineering Inc.         ",
                "Modifications and extensions:                               ",
                "(c) 1997-1998 Object Tools                                  ",
                "All rights reserved. Duplication and distribution prohibited",
                "May be used only with Visual Eiffel under terms of user     ",
                "license                                                     ";
   address    : "mail_to:support@object-tools.com                            ",
                "http://www.object-tools.com                                 ";

class BASIC_ROUTINES

inherit
   ISE_GENERAL
   end

feature -- Conversion

   charconv( i : INTEGER ) : CHARACTER is
      -- Character corresponding to ascii code `i'
   do
      Result.from_integer( i )
   end -- charconv

   charcode( c : CHARACTER) : INTEGER is
   obsolete "Instead of charcode (c) use c.code"
      -- Integer ascii code corresponding to `c'
   do
      Result := c.code
   end -- charcode

   integer_to_real( n : INTEGER ) : REAL is
   obsolete "Use target := `n'"
      -- Real conversion of `n'
   do
      Result := n
   end -- integer_to_real

   real_to_integer( r : REAL ) : INTEGER is
   obsolete "Instead of real_to_integer (r) use r.truncated_to_integer"
      -- Integer conversion (truncation) of `r'
   do
      Result := r.truncated_to_integer
   end -- real_to_integer

   double_to_real( d : DOUBLE ) : REAL is
   obsolete "Instead of double_to_real (d) use d.truncated_to_real"
      -- Real conversion (truncation) of `d'
   do
      Result := d.truncated_to_real
   end -- double_to_real

   real_to_double( r : REAL ) : DOUBLE is
   obsolete "Use target := `r' instead."
      -- Double conversion of `r'
   do
      Result := r
   end -- real_to_double

   double_to_integer( d : DOUBLE ) : INTEGER is
   obsolete "Instead of double_to_integer (d) use d.truncated_to_integer"
      -- Integer conversion (truncation) of `d'
   do
      Result := d.truncated_to_integer
   end -- double_to_integer

feature -- Basic operations

   abs( n : INTEGER ) : INTEGER is
   obsolete "Use n.abs instead"
      -- Absolute value of `n'
   do
      Result := n.abs
   ensure
      non_negative_result: Result >= 0
   end -- abs

   sign( n : INTEGER ) : INTEGER is
   obsolete "Use n.sign instead"
      -- Sign of `n':
      -- -1 if `n' < 0
      --  0 if `n' = 0
      -- +1 if `n' > 0
   do
      Result := n.sign
   ensure
      correct_negative: (n < 0) = (Result = -1);
      correct_zero: (n = 0) = (Result = 0);
      correct_positive: (n > 0) = (Result = +1)
   end -- sign

   max( n1, n2 : INTEGER ) : INTEGER is
   obsolete "Replace max (a, b) with a.max (b)"
      -- Maximum of `n1' and `n2'
   do
      Result := n1.max (n2)
   ensure
      is_maximum: (n2 >= n1) = (Result = n2) or else (n1 > n2) = (Result = n1)
   end -- max

   min( n1, n2 : INTEGER ) : INTEGER is
   obsolete "Replace min (a, b) with a.min (b)"
      -- Minimum of `n1' and `n2'
   do
      Result := n1.min (n2)
   ensure
      is_minimum: (n2 <= n1) = (Result = n2) or else (n1 < n2) = (Result = n1)
   end -- min

   rsign( r : REAL ) : INTEGER is
   obsolete "Use r.sign instead"
      -- Sign of `r':
      -- -1 if `r' < 0
      --  0 if `r' = 0
      -- +1 if `r' > 0
   do
      Result := r.sign
   ensure
      correct_negative: (r < 0) = (Result = -1);
      correct_zero: (r = 0) = (Result = 0);
      correct_positive: (r > 0) = (Result = +1)
   end -- rsign

   rmax( r1, r2 : REAL ) : REAL is
   obsolete "Replace rmax (a, b) with a.max (b)"
      -- Maximum of `r1' and `r2'
   do
      Result := r1.max (r2)
   ensure
      is_maximum: (r2 >= r1) = (Result = r2) or else (r1 > r2) = (Result = r1)
   end -- rmax

   rmin( r1, r2 : REAL ) : REAL is
   obsolete "Replace rmin (a, b) with a.min (b)"
      -- Minimum of `r1' and `r2'
   do
      Result := r1.min (r2)
   ensure
      is_minimum: (r2 <= r1) = (Result = r2) or else (r1 < r2) = (Result = r1)
   end -- rmin

   bottom_int_div( n1, n2 : INTEGER ) : INTEGER is
      -- Greatest lower bound of the integer division of `n1' by `n2'
   do
      Result := n1 // n2;
      if Result < 0 and then ( n2 * Result ).abs < n1.abs then
         Result := Result - 1;
      end; -- if
   end -- bottom_int_div

   up_int_div( n1, n2 : INTEGER ) : INTEGER is
      -- Least upper bound of the integer division
      -- of `n1' by `n2'
   do
      Result := n1 // n2;
      if Result > 0 and then ( n2 * Result ).abs < n1.abs then
         Result := Result + 1;
      end; -- if
   end -- up_int_div

end -- class BASIC_ROUTINES
