indexing
   description: "File name abstraction";
   title      : "EiffelBase: library of reusable components for Eiffel.      ",
                "Based on EiffelBase library for ISE Eiffel,                 ",
                "used with permission.                                       ";
   cluster    : EiffelBase;
   project    : "EiffelBase Library";
   copyright  : "(c) 1986-1997 Interactive Software Engineering Inc.         ",
                "Modifications and extensions:                               ",
                "(c) 1997-2001 Object Tools                                  ",
                "All rights reserved. Duplication and distribution prohibited",
                "May be used only with Visual Eiffel under terms of user     ",
                "license                                                     ";
   address    : "mail_to:support@object-tools.com                            ",
                "http://www.object-tools.com                                 ";

class FILE_NAME

inherit

   PATH_NAME

creation

   make, make_from_string

feature

   set_file_name (file_name: STRING) is
         -- Set the value of the file name part.
      require
         string_exists: file_name /= Void
         valid_file_name: is_file_name_valid (file_name)
      local
         new_size: INTEGER
         str1, str2: ANY
      do
         new_size := count + file_name.count + 5;
         --if capacity < new_size then
         if count < new_size then
            resize (new_size)
         end
         str1 := to_c;
         str2 := file_name.to_c;
         c_append_file_name ($Current, $str1, $str2);
      ensure
         valid_file_name: is_valid
      end

   add_extension (ext: STRING) is
         -- Append the extension `ext' to the file name
      require
         string_exists: ext /= Void
         non_empty_extension: not ext.is_empty
         valid_extension: is_extension_valid (ext)
      do
         append_character ('.');
         append (ext)
      end

feature

   is_valid: BOOLEAN is
         -- Is the file name valid for the operating system?
      local
         any: ANY
      do
         any := to_c
         Result := c_is_file_valid ($any)
      end

   is_file_name_valid (f_name: STRING): BOOLEAN is
         -- Is `f_name' a valid file name part for the operating system?
      local
         any: ANY
      do
         any := f_name.to_c;
         Result := c_is_file_name_valid ($any)
      end

   is_extension_valid (ext: STRING): BOOLEAN is
         -- Is `ext' a valid extension for the operating system?
      local
         any: ANY
      do
         any := ext.to_c;
         Result := c_is_extension_valid ($any)
      end

feature {NONE} -- Externals

   c_append_file_name (s, p, v: POINTER) is
      local
         except : expanded EXCEPTIONS
      do
         except.raise( "FILE_NAME: is not implemented.%N" )
      end

   c_is_file_name_valid (p: POINTER): BOOLEAN is
      local
         except : expanded EXCEPTIONS
      do
         except.raise( "FILE_NAME: is not implemented.%N" )
      end

   c_is_extension_valid (p: POINTER): BOOLEAN is
      local
         except : expanded EXCEPTIONS
      do
         except.raise( "FILE_NAME: is not implemented.%N" )
      end

   c_is_file_valid (p: POINTER): BOOLEAN is
      local
         except : expanded EXCEPTIONS
      do
         except.raise( "FILE_NAME: is not implemented.%N" )
      end

end -- class FILE_NAME
