indexing
   description: "Sequences of values, all of the same type or of a conforming",
                "one, accessible through integer indices in a contiguous     ",
                "interval                                                    ";
   title      : "EiffelBase: library of reusable components for Eiffel.      ",
                "Based on EiffelBase library for ISE Eiffel,                 ",
                "used with permission.                                       ";
   cluster    : EiffelBase;
   project    : "EiffelBase Library";
   copyright  : "(c) 1986-1997 Interactive Software Engineering Inc.         ",
                "Modifications and extensions:                               ",
                "(c) 1997 Object Tools                                       ",
                "All rights reserved. Duplication and distribution prohibited",
                "May be used only with Visual Eiffel under terms of user     ",
                "license                                                     ";
   address    : "mail_to:support@object-tools.com                            ",
                "http://www.object-tools.com                                 ";

class ISE_ARRAY [G] inherit

   RESIZABLE [G]
       undefine
          copy, is_equal
       redefine
          full, --copy, is_equal,
          consistent, setup
       end;

   INDEXABLE [G, INTEGER]
       undefine
          copy, is_equal
       redefine
          --copy, is_equal,
          consistent, setup
       end;

   ARRAY[ G ]
      rename
         area   as ARRAY_area,
         to_c   as ARRAY_to_c,
         move   as ARRAY_move,
         insert as ARRAY_insert,
         remove as ARRAY_remove,
	 append as ARRAY_append
      undefine
            empty
      end

        --TO_SPECIAL [G]
        --        export
        --                {ARRAY} set_area
        --        redefine
        --                copy, is_equal,
        --                consistent, setup
        --        end

creation

   make, make_from_array

feature

   setup (other: like Current) is
                   -- Perform actions on a freshly created object so that
                   -- the contents of `other' can be safely copied onto it.
           do
                   make_from_array( other )
                   --make_area (other.capacity)
           end;

feature -- Access

        has (v: G): BOOLEAN is
                        -- Does `v' appear in array?
                        -- (Reference or object equality,
                        -- based on `object_comparison'.)
                local
                        i: INTEGER
                do
                        if object_comparison then
                                if v = void then
                                        i := upper + 1
                                else
                                        from
                                                i := lower
                                        until
                                                i > upper or else (item (i) /= Void and then item (i).is_equal(v))
                                        loop
                                                i := i + 1;
                                        end;
                                end
                        else
                                from
                                        i := lower
                                until
                                        i > upper or else (item (i) = v)
                                loop
                                        i := i + 1;
                                end;
                        end
                        Result := not (i > upper);
                end;

feature -- Measurement

        capacity: INTEGER is
                        -- Number of available indices
                do
                        Result := upper - lower + 1
                end;

        occurrences (v: G): INTEGER is
                        -- Number of times `v' appears in structure
                local
                        i: INTEGER
                do
                        if object_comparison then
                                if v /= Void then
                                        from
                                                i := lower
                                        until
                                                i > upper
                                        loop
                                                if item (i) /= Void and then v.is_equal (item (i)) then
                                                        Result := Result + 1
                                                end
                                                i := i + 1
                                        end
                                end
                        else
                                from
                                        i := lower
                                until
                                        i > upper
                                loop
                                        if item (i) = v then
                                                Result := Result +1
                                        end;
                                        i := i + 1
                                end
                        end;
                end;

feature -- Status report

        consistent (other: like Current): BOOLEAN is
                                -- Is object in a consistent state so that `other'
                                -- may be copied onto it? (Default answer: yes).
                do
                        Result := (capacity = other.capacity)
                end;

        full: BOOLEAN is
                        -- Is structure filled to capacity? (Answer: yes)
                do
                        Result := true
                end;

        extendible: BOOLEAN is
                        -- May items be added?
                        -- (Answer: no, although array may be resized.)
                do
                        Result := false
                end;

        prunable: BOOLEAN is
                        -- May items be removed? (Answer: no.)
                do
                        Result := false
                end;

feature -- Resizing

        grow (i: INTEGER) is
                        -- Change the capacity to at least `i'.
                do
                        if i > capacity then
                                resize (lower, upper + i - capacity)
                        end
                end;

feature -- Conversion

        to_c: ANY is
                        -- Address of actual sequence of values,
                        -- for passing to external (non-Eiffel) routines.
                do
                        Result := ARRAY_to_c
                end;

        linear_representation: LINEAR [G] is
                        -- Representation as a linear structure
                local
                        temp: ARRAYED_LIST [G];
                        i: INTEGER;
                do
                        !! temp.make (capacity);
                        from
                                i := lower;
                        until
                                i > upper
                        loop
                                temp.extend (item (i));
                                i := i + 1;
                        end;
                        Result := temp;
                end;


feature {NONE} -- Inapplicable

        prune (v: G) is
                        -- Remove first occurrence of `v' if any.
                        -- (Precondition is false.)
                do
                end;

        extend (v: G) is
                        -- Add `v' to structure.
                        -- (Precondition is false.)
                do
                end;

feature {NONE} -- Implementation

   area_item ( i : INTEGER ) : G is
   require
      valid_index : 0 <= i and then i <= count - 1
   do
      Result := item( i + lower );
   end; -- area_item

   area_put ( v : G; i : INTEGER ) is
   require
      valid_index : 0 <= i and then i <= count - 1
   do
      put( v, i + lower );
   end; -- area_put

end -- class ISE_ARRAY
