indexing
   description: "``Active'' data structures, which at every stage have       ",
                "a possibly undefined ``current item''.                      ",
                "Basic access and modification operations apply to the       ",
                "current item.                                               ";
   title      : "EiffelBase: library of reusable components for Eiffel.      ",
                "Based on EiffelBase library for ISE Eiffel,                 ",
                "used with permission.                                       ";
   cluster    : EiffelBase;
   project    : "EiffelBase Library";
   copyright  : "(c) 1986-1997 Interactive Software Engineering Inc.         ",
                "Modifications and extensions:                               ",
                "(c) 1997 Object Tools                                       ",
                "All rights reserved. Duplication and distribution prohibited",
                "May be used only with Visual Eiffel under terms of user     ",
                "license                                                     ";
   key        : active, access;
   address    : "mail_to:support@object-tools.com                            ",
                "http://www.object-tools.com                                 ";

deferred class ACTIVE [G] inherit

   BAG [G]

feature -- Access

   item: G is
         -- Current item
      require
         readable: readable
      deferred
      end;

feature -- Status report

   readable: BOOLEAN is
         -- Is there a current item that may be read?
      deferred
      end;

   writable: BOOLEAN is
         -- Is there a current item that may be modified?
      deferred
      end;

feature -- Element change

   replace (v: G) is
         -- Replace current item by `v'.
      require
         writable: writable
      deferred
      ensure
         item_replaced: item = v
      end;

feature -- Removal

   remove is
         -- Remove current item.
      require
         prunable: prunable;
         writable: writable
      deferred
      end;

invariant

   writable_constraint: writable implies readable;
   empty_constraint: empty implies (not readable) and (not writable)

end -- class ACTIVE
