indexing
   description: "General container data structures,                          ",
                "characterized by the membership properties of their items.  ";
   title      : "EiffelBase: library of reusable components for Eiffel.      ",
                "Based on EiffelBase library for ISE Eiffel,                 ",
                "used with permission.                                       ";
   cluster    : EiffelBase;
   project    : "EiffelBase Library";
   copyright  : "(c) 1986-1997 Interactive Software Engineering Inc.         ",
                "Modifications and extensions:                               ",
                "(c) 1997 Object Tools                                       ",
                "All rights reserved. Duplication and distribution prohibited",
                "May be used only with Visual Eiffel under terms of user     ",
                "license                                                     ";
   key        : collection, access;
   address    : "mail_to:support@object-tools.com                            ",
                "http://www.object-tools.com                                 ";

deferred class COLLECTION [G] inherit

   CONTAINER [G]

feature -- Status report

   extendible: BOOLEAN is
         -- May new items be added?
      deferred
      end;

   prunable: BOOLEAN is
         -- May items be removed?
      deferred
      end;

feature -- Element change

   put, extend (v: G) is
         -- Ensure that structure includes `v'.
      require
         extendible: extendible
      deferred
      ensure
         item_inserted: has (v)
      end;

   fill (other: CONTAINER [G]) is
         -- Fill with as many items of `other' as possible.
         -- The representations of `other' and current structure
         -- need not be the same.
      require
         other_not_void: other /= Void;
         extendible
      local
         lin_rep: LINEAR [G]
      do
         lin_rep := other.linear_representation;
         from
            lin_rep.start
         until
            not extendible or else lin_rep.off
         loop
            extend (lin_rep.item);
            lin_rep.forth
         end
      end;

feature -- Removal

   prune (v: G) is
         -- Remove one occurrence of `v' if any.
         -- (Reference or object equality,
         -- based on `object_comparison'.)
      require
         prunable: prunable
      deferred
      end;

   prune_all (v: G) is
         -- Remove all occurrences of `v'.
         -- (Reference or object equality,
         -- based on `object_comparison'.)
         --|Default implementation, usually inefficient.
      require
         prunable
      do
         from
         until not has (v) loop
            prune (v)
         end
      ensure
         no_more_occurrences: not has (v)
      end;

   wipe_out is
         -- Remove all items.
      require
         prunable
      deferred
      ensure
         wiped_out: empty
      end;

end -- class COLLECTION
