indexing
   description: "Data structures of the most general kind,                   ",
                "used to hold zero or more items.                            ";
   title      : "EiffelBase: library of reusable components for Eiffel.      ",
                "Based on EiffelBase library for ISE Eiffel,                 ",
                "used with permission.                                       ";
   cluster    : EiffelBase;
   project    : "EiffelBase Library";
   copyright  : "(c) 1986-1997 Interactive Software Engineering Inc.         ",
                "Modifications and extensions:                               ",
                "(c) 1997 Object Tools                                       ",
                "All rights reserved. Duplication and distribution prohibited",
                "May be used only with Visual Eiffel under terms of user     ",
                "license                                                     ";
   key        : access;
   address    : "mail_to:support@object-tools.com                            ",
                "http://www.object-tools.com                                 ";

deferred class
   CONTAINER [G]
inherit
        ISE_GENERAL
        end

feature -- Access

        has (v: G): BOOLEAN is
                        -- Does structure include `v'?
                        -- (Reference or object equality,
                        -- based on `object_comparison'.)
                deferred
                ensure
                        not_found_in_empty: Result implies not empty
                end;

feature -- Status report

        empty: BOOLEAN is
                        -- Is there no element?
                deferred
                end;

        object_comparison: BOOLEAN;
                        -- Must search operations use `equal' rather than `='
                        -- for comparing references? (Default: no, use `='.)

        changeable_comparison_criterion: BOOLEAN is
                        -- May `object_comparison' be changed?
                        -- (Answer: yes by default.)
                do
                        Result := True
                end;

feature -- Status setting

        compare_objects is
                        -- Ensure that future search operations will use `equal'
                        -- rather than `=' for comparing references.
                require
                        changeable_comparison_criterion
                do
                        object_comparison := True
                ensure
                        object_comparison
                end;

        compare_references is
                        -- Ensure that future search operations will use `='
                        -- rather than `equal' for comparing references.
                require
                        changeable_comparison_criterion
                do
                        object_comparison := False
                ensure
                        reference_comparison: not object_comparison
                end;

feature -- Conversion

        linear_representation: LINEAR [G] is
                        -- Representation as a linear structure
                deferred
                end;

end -- class CONTAINER
