indexing
   description: "Cursor trees in linked representation                       ";
   title      : "EiffelBase: library of reusable components for Eiffel.      ",
                "Based on EiffelBase library for ISE Eiffel,                 ",
                "used with permission.                                       ";
   cluster    : EiffelBase;
   project    : "EiffelBase Library";
   copyright  : "(c) 1986-1997 Interactive Software Engineering Inc.         ",
                "Modifications and extensions:                               ",
                "(c) 1997 Object Tools                                       ",
                "All rights reserved. Duplication and distribution prohibited",
                "May be used only with Visual Eiffel under terms of user     ",
                "license                                                     ";
   key        : linked_cursor_tree, cursor_tree;
   address    : "mail_to:support@object-tools.com                            ",
                "http://www.object-tools.com                                 ";

class LINKED_CURSOR_TREE [G] inherit

        RECURSIVE_CURSOR_TREE [G]
                redefine
                        active, cursor_anchor, is_leaf
                end

creation

        make, make_root

feature -- Initialization

        make is
                        -- Create an empty tree.
                local
                        dummy: G
                do
                        !! above_node.make (dummy);
                        active := above_node
                ensure
                        is_above: above;
                        is_empty: empty
                end;

        make_root (v: G) is
                        -- Create a tree with `v' as root.
                do
                        make;
                        put_root (v)
                end;

feature -- Status report

        full: BOOLEAN is false;
                        -- Is tree filled to capacity? (Answer: no.)

        prunable: BOOLEAN is true

        is_leaf: BOOLEAN is
                do
                        if not off then
                                Result := not below and then active.arity = 0
                        end
                end

feature -- Element change

        put_right (v: G) is
                        -- Add `v' to the right of cursor position.
                do
                        if below then
                                active.child_put_right (v);
                                active.child_forth;
                                active_parent := active;
                                active := active_parent.child;
                                below := false
                        elseif before then
                                active_parent.child_put_left (v);
                                active_parent.child_back;
                                active := active_parent.child
                        else
                                active_parent.child_put_right (v)
                        end
                end;

        put_root (v: G) is
                        -- Put `v' as root of an empty tree.
                require
                        is_empty: empty
                do
                        above_node.child_put_right (v);
                        active_parent := above_node;
                        active := active_parent.child
                ensure
                        is_root: is_root;
                        count = 1
                end;

        put_child (v: G) is
                        -- Put `v' as child of a leaf.
                require
                        is_leaf: is_leaf
                do
                        down (0)
                        put_right (v)
                end

feature {LINKED_CURSOR_TREE} -- Implementation

        new_tree: like Current is
                        -- A newly created instance of the same type.
                        -- This feature may be redefined in descendants so as to
                        -- produce an adequately allocated and initialized object.
                do
                        !! Result.make
                end;

feature {NONE} -- Implementation

        cursor_anchor: LINKED_TREE_CURSOR [G];
                        -- Anchor for definitions concerning cursors

        active: LINKED_TREE [G];
                        -- Current node

end -- class LINKED_CURSOR_TREE
