indexing
   description: "Stacks implemented by resizable arrays";
   title      : "EiffelBase: library of reusable components for Eiffel.      ",
                "Based on EiffelBase library for ISE Eiffel,                 ",
                "used with permission.                                       ";
   cluster    : EiffelBase;
   project    : "EiffelBase Library";
   copyright  : "(c) 1986-1997 Interactive Software Engineering Inc.         ",
                "Modifications and extensions:                               ",
                "(c) 1997 Object Tools                                       ",
                "All rights reserved. Duplication and distribution prohibited",
                "May be used only with Visual Eiffel under terms of user     ",
                "license                                                     ";
   key        : dispenser, array;
   address    : "mail_to:support@object-tools.com                            ",
                "http://www.object-tools.com                                 ";

class ARRAYED_STACK [G] inherit

   STACK [G]
      undefine
         copy, is_equal, consistent, setup, prune_all, replace
      select
         remove, extend, put
      end;

   ARRAYED_LIST [G]
      rename
         put as al_put,
         extend as al_extend,
         force as al_force,
         remove as al_remove,
         start as finish,
         finish as start,
         forth as back,
         back as forth,
         after as before,
         before as after
      export
         {NONE} all;
         {ANY}
            count, readable, writable, extendible,
             make, wipe_out
         {STACK} start, finish, forth, back
      undefine
         readable, writable,
         append, fill
      redefine
         linear_representation
      end;

creation

   make

feature -- Element change

   extend, put, force (v: G) is
         -- Push `v' on top.
      do
         al_extend (v);
         start
      end;

feature -- Removal

   remove is
         -- Remove top item.
      require else
         not_empty: count /= 0
      do
         al_remove;
         start
      end;

feature -- Conversion

   linear_representation: ARRAYED_LIST [G] is
         -- Representation as a linear structure
         -- (in the reverse order of original insertion)
      local
         i: INTEGER
      do
         from
            !! Result.make (count);
            i := count
         until
            i < 1
         loop
            Result.extend (i_th (i));
            i := i - 1
         end
      end;

end -- class ARRAYED_STACK
