indexing
   description: "Unbounded queues implemented as linked lists";
   title      : "EiffelBase: library of reusable components for Eiffel.      ",
                "Based on EiffelBase library for ISE Eiffel,                 ",
                "used with permission.                                       ";
   cluster    : EiffelBase;
   project    : "EiffelBase Library";
   copyright  : "(c) 1986-1997 Interactive Software Engineering Inc.         ",
                "Modifications and extensions:                               ",
                "(c) 1997 Object Tools                                       ",
                "All rights reserved. Duplication and distribution prohibited",
                "May be used only with Visual Eiffel under terms of user     ",
                "license                                                     ";
   key        : linked_queue, dispenser, linked_list;
   address    : "mail_to:support@object-tools.com                            ",
                "http://www.object-tools.com                                 ";

class LINKED_QUEUE [G] inherit

        QUEUE [G]
                undefine
                        empty
                redefine
                        prune_all
                select
                        item, put
                end;

        LINKED_LIST [G]
                rename
                        item as ll_item,
                        remove as ll_remove,
                        make as ll_make,
                        remove_left as remove,
                        put as ll_put
                export
                        {NONE}
                                all;
                        {ANY}
                                writable, extendible, wipe_out,
                                readable
                undefine
                        fill, append, prune,
                        readable, writable, prune_all, extend,
                        force
                redefine
                        duplicate, linear_representation
                select
                        remove
                end;

creation

        make

feature -- Initialization

        make is
                do
                        after := true
                end;

feature -- Access

        item: G is
                        -- Oldest item
                require else
                        not empty
                do
                                check
                                        after and not empty implies (active = last_element)
                                end;
                        Result := active.item
                end;

feature -- Element change

        put, extend, force (v: G) is
                        -- Add `v' as newest item.
                do
                        put_front (v)
                        before := false
                        after := true
                ensure then
                        (old empty) implies (item = v);
                end;

feature -- Conversion

        linear_representation: ARRAYED_LIST [G] is
                        -- Representation as a linear structure
                        -- (order is same as original order of insertion)
                do
                        !!Result.make_filled (count);
                        from
                                start;
                                Result.finish;
                        until
                                after
                        loop
                                Result.replace (item);
                                forth;
                                Result.back
                        end;
                end;

feature -- Duplication

        duplicate (n: INTEGER): like Current is
                        -- New queue containing the `n' oldest items in current queue.
                        -- If `n' is greater than `count', identical to current queue.
                require else
                        positive_argument: n > 0
                do
                        start;
                        from
                                !! Result.make;
                                start
                        until
                                after or Result.count = n
                        loop
                                Result.extend (ll_item);
                                forth;
                        end;
                        finish
                end;

feature {NONE} -- Not applicable

        prune (v: G) is
                        -- Remove one occurrence of `v'.
                        -- Not available.
                do
                        -- Do nothing
                end;

        prune_all (v: G) is
                        -- Remove all occurrences of `v'.
                        -- Not available
                do
                        -- Do nothing
                end;

invariant

        is_always_after: not empty implies after

end -- class LINKED_QUEUE
