indexing
   description: "Stacks (last-in, first-out dispensers), without commitment  ",
                "to a particular representation                              ";
   title      : "EiffelBase: library of reusable components for Eiffel.      ",
                "Based on EiffelBase library for ISE Eiffel,                 ",
                "used with permission.                                       ";
   cluster    : EiffelBase;
   project    : "EiffelBase Library";
   copyright  : "(c) 1986-1997 Interactive Software Engineering Inc.         ",
                "Modifications and extensions:                               ",
                "(c) 1997 Object Tools                                       ",
                "All rights reserved. Duplication and distribution prohibited",
                "May be used only with Visual Eiffel under terms of user     ",
                "license                                                     ";
   key        : stack, dispenser;
   address    : "mail_to:support@object-tools.com                            ",
                "http://www.object-tools.com                                 ";

deferred class STACK [G] inherit

        DISPENSER [G]
                export
                        {NONE} prune, prune_all
                redefine
                        extend, force, put, fill
                end;

feature -- Element change

        extend, force, put (v: G) is
                        -- Push `v' onto top.
                deferred
                ensure then
                        item_pushed: item = v
                end;

        replace (v: G) is
                        -- Replace top item by `v'.
                do
                        remove;
                        extend (v)
                end;

        fill (other: LINEAR [G]) is
                        -- Fill with as many items of `other' as possible.
                        -- Fill items with greatest index from `other' first.
                        -- Items inserted with lowest index (from `other') will
                        -- always be on the top of stack.
                        -- The representations of `other' and current structure
                        -- need not be the same.
                local
                        temp: ARRAYED_STACK [G]
                do
                        !! temp.make (0);
                        from
                                other.start
                        until
                                other.off
                        loop
                                temp.extend (other.item);
                                other.forth
                        end;
                        from
                        until
                                temp.empty or else not extendible
                        loop
                                extend (temp.item);
                                temp.remove
                        end
                end;

end -- class STACK
