indexing
   description: "Linkable cells with a reference to the left and right       ",
                "neighbors                                                   ";
   title      : "EiffelBase: library of reusable components for Eiffel.      ",
                "Based on EiffelBase library for ISE Eiffel,                 ",
                "used with permission.                                       ";
   cluster    : EiffelBase;
   project    : "EiffelBase Library";
   copyright  : "(c) 1986-1997 Interactive Software Engineering Inc.         ",
                "Modifications and extensions:                               ",
                "(c) 1997 Object Tools                                       ",
                "All rights reserved. Duplication and distribution prohibited",
                "May be used only with Visual Eiffel under terms of user     ",
                "license                                                     ";
   key        : bi_linkable, cell;
   address    : "mail_to:support@object-tools.com                            ",
                "http://www.object-tools.com                                 ";

class BI_LINKABLE [G] inherit

   LINKABLE [G]
      redefine
         put_right, forget_right
      end

feature -- Access

   left: like Current;
         -- Left neighbor

feature {CELL, CHAIN} -- Implementation

   put_right (other: like Current) is
         -- Put `other' to the right of current cell.
      do
         if right /= Void then
            right.simple_forget_left
         end;
         right := other;
         if (other /= Void) then
            other.simple_put_left (Current)
         end
      end;

   put_left (other: like Current) is
         -- Put `other' to the left of current cell.
      do
         if left /= Void then
            left.simple_forget_right
         end;
         left := other;
         if (other /= Void) then
            other.simple_put_right (Current)
         end
      ensure
         chained: left = other
      end;

   forget_right is
         -- Remove links with right neighbor.
      do
         if right /= Void then
            right.simple_forget_left;
            right := Void
         end
      ensure then
          right_not_chained:
             (old right /= Void) implies ((old right).left = Void)
      end;

   forget_left is
         -- Remove links with left neighbor.
      do
         if left /= Void then
            left.simple_forget_right;
            left := Void
         end
      ensure
         left_not_chained:
            left = Void;
             (old left /= Void) implies ((old left).right = Void)
      end;

feature {BI_LINKABLE, TWO_WAY_LIST} -- Implementation

   simple_put_right (other: like Current) is
         -- set `right' to `other'
      do
         if right /= Void then
            right.simple_forget_left;
         end;
         right := other
      end;

   simple_put_left (other: like Current) is
         -- set `left' to `other' is
      do
         if left /= Void then
            left.simple_forget_right
         end;
         left := other
      end;

   simple_forget_right is
         -- Remove right link (do nothing to right neighbor).
      do
         right := Void
      end;

   simple_forget_left is
         -- Remove left link (do nothing to left neighbor).
      do
         left := Void
      ensure
         not_chained: left = Void
      end;

invariant

   right_symmetry:
      (right /= Void) implies (right.left = Current);
   left_symmetry:
      (left /= Void) implies (left.right = Current)

end -- class BI_LINKABLE
