indexing
   description: "Sequential, dynamically modifiable lists,                   ",
                "without commitment to a particular representation           ";
   title      : "EiffelBase: library of reusable components for Eiffel.      ",
                "Based on EiffelBase library for ISE Eiffel,                 ",
                "used with permission.                                       ";
   cluster    : EiffelBase;
   project    : "EiffelBase Library";
   copyright  : "(c) 1986-1997 Interactive Software Engineering Inc.         ",
                "Modifications and extensions:                               ",
                "(c) 1997 Object Tools                                       ",
                "All rights reserved. Duplication and distribution prohibited",
                "May be used only with Visual Eiffel under terms of user     ",
                "license                                                     ";
   key        : dynamic_list, sequence;
   address    : "mail_to:support@object-tools.com                            ",
                "http://www.object-tools.com                                 ";

deferred class DYNAMIC_LIST [G] inherit

        LIST [G]
                undefine
                        prune,
                        sequential_index_of, sequential_has,
                        remove,
                        prune_all
                redefine
                        remove
                end;

        DYNAMIC_CHAIN [G]
                rename
                        wipe_out as chain_wipe_out
                export
                        {NONE} chain_wipe_out
                redefine
                        remove,
                        put_right,
                        remove_left, remove_right
                end;

        DYNAMIC_CHAIN [G]
                redefine
                        remove,
                        put_right,
                        remove_left, remove_right, wipe_out
                select
                        wipe_out
                end

feature -- Element change

        put_left (v: G) is
                        -- Add `v' to the left of cursor position.
                        -- Do not move cursor.
                local
                        temp: G
                do
                        if empty then
                                put_front (v)
                        elseif after then
                                back;
                                put_right (v);
                                move (2)
                        else
                                temp := item;
                                replace (v);
                                put_right (temp);
                                forth
                        end
                end;

        put_right (v: G) is
                        -- Add `v' to the right of cursor position.
                        -- Do not move cursor.
                deferred
                end;

        merge_left (other: like Current) is
                        -- Merge `other' into current structure before cursor
                        -- position. Do not move cursor. Empty `other'.
                do
                        from
                                other.start
                        until
                                other.empty
                        loop
                                put_left (other.item);
                                other.remove
                        end
                end;

        merge_right (other: like Current) is
                        -- Merge `other' into current structure after cursor
                        -- position. Do not move cursor. Empty `other'.
                do
                        from
                                other.finish
                        until
                                other.empty
                        loop
                                put_right (other.item);
                                other.back;
                                other.remove_right
                        end
                end;

feature -- Removal

        remove is
                        -- Remove current item.
                        -- Move cursor to right neighbor
                        -- (or `after' if no right neighbor).
                deferred
                ensure then
                        after_when_empty: empty implies after
                end;

        remove_left is
                        -- Remove item to the left of cursor position.
                        -- Do not move cursor.
                require else
                        not_before: not before
                deferred
                end;

        remove_right is
                        -- Remove item to the right of cursor position.
                        -- Do not move cursor.
                deferred
                end;

        wipe_out is
                        -- Remove all items.
                do
                        chain_wipe_out;
                        back
                ensure then
                        is_before: before
                end;

end -- class DYNAMIC_LIST
