indexing
   description: "Sequential lists whose items are sorted in ascending order  ",
                "according to the relational operators of PART_COMPARABLE    ";
   title      : "EiffelBase: library of reusable components for Eiffel.      ",
                "Based on EiffelBase library for ISE Eiffel,                 ",
                "used with permission.                                       ";
   cluster    : EiffelBase;
   project    : "EiffelBase Library";
   copyright  : "(c) 1986-1997 Interactive Software Engineering Inc.         ",
                "Modifications and extensions:                               ",
                "(c) 1997 Object Tools                                       ",
                "All rights reserved. Duplication and distribution prohibited",
                "May be used only with Visual Eiffel under terms of user     ",
                "license                                                     ";
   key        : sorted_list, sorted_struct, sequence;
   address    : "mail_to:support@object-tools.com                            ",
                "http://www.object-tools.com                                 ";

deferred class PART_SORTED_LIST [G -> COMPARABLE ] --PART_COMPARABLE] inherit
inherit
        LIST [G]
                redefine
                        has, extend
                end;

feature -- Access

        has (v: G): BOOLEAN is
                        -- Does structure include `v'?
                        -- (Reference or object equality,
                        -- based on `object_comparison'.)
                local
                        pos: CURSOR
                do
                        if not empty then
                                pos := cursor;
                                start;
                                search (v);
                                Result := not after;
                                go_to (pos)
                        end
                end;

        search_after (v: G) is
                        -- Go to first position with item greater
                        -- than or equal to `v'.
                do
                        from
                                start
                        until
                                after or else v <= item
                        loop
                                forth
                        end
                ensure
                        argument_less_than_item: (not after) implies (v <= item)
                end;

        search_before (v: G) is
                        -- Go to last position with item less
                        -- than or equal to `v'.
                do
                        from
                                finish
                        until
                                before or else v >= item
                        loop
                                back
                        end
                ensure
                        (not off) implies (item <= v)
                end;

feature -- Element change

        extend (v: G) is
                        -- Put `v' at proper position in list.
                        -- The cursor ends up on the newly inserted
                        -- item.
                deferred
                ensure then
                        remains_sorted: (old sorted) implies sorted;
                        item_inserted: item = v
                end;

        merge (other: LINEAR [G]) is
                        -- Add all items from `other' at their proper positions.
                do
                        from
                                other.start
                        until
                                other.off
                        loop
                                extend (other.item);
                                other.forth
                        end
                ensure
                        remains_sorted: (old sorted) implies sorted
                end;

feature -- Status report


        sorted: BOOLEAN is
                deferred
                end;
end -- class PART_SORTED_LIST
