indexing
   description: "Sets implemented by linked lists";
   title      : "EiffelBase: library of reusable components for Eiffel.      ",
                "Based on EiffelBase library for ISE Eiffel,                 ",
                "used with permission.                                       ";
   cluster    : EiffelBase;
   project    : "EiffelBase Library";
   copyright  : "(c) 1986-1997 Interactive Software Engineering Inc.         ",
                "Modifications and extensions:                               ",
                "(c) 1997 Object Tools                                       ",
                "All rights reserved. Duplication and distribution prohibited",
                "May be used only with Visual Eiffel under terms of user     ",
                "license                                                     ";
   key        : linked_set, set, linked_list;
   address    : "mail_to:support@object-tools.com                            ",
                "http://www.object-tools.com                                 ";

class LINKED_SET [G] inherit

   SUBSET [G]
      undefine
         prune_all
      select
         extend, put, prune
      end;

    LINKED_LIST [G]
      rename
         extend as ll_extend,
         put as ll_put,
         prune as ll_prune
      export
         {NONE} ll_extend, ll_put, ll_prune
      undefine
         changeable_comparison_criterion
      end;

creation

   make

feature -- Comparison

   is_subset (other: like Current): BOOLEAN is
         -- Is current set a subset of `other'?
      do
         if not other.empty and then count <= other.count then
            from
               start
            until
               off or else not other.has (item)
            loop
               forth
            end;
            if off then Result := true end
         elseif empty then
            Result := true
         end
      end;

feature -- Element change

   put, extend (v: G) is
         -- Ensure that set includes `v'.
      do
         if empty or else not has (v) then
            put_front (v)
         end
      end;

   merge (other: like Current) is
         -- Add all items of `other'.
      do
         from
            other.start
         until
            other.off
         loop
            extend (other.item);
            other.forth
         end
      end;
feature -- Removal

   prune (v : G) is
      -- Remove `v' if present.
      do
         start
         ll_prune (v)
      end

feature -- Basic operations

   intersect (other: like Current) is
         -- Remove all items not in `other'.
         -- No effect if `other' is `empty'.
      do
         if not other.empty then
            from
               start;
               other.start
            until
               off
            loop
               if other.has (item) then
                  forth
               else
                  remove
               end
            end
         else
            wipe_out
         end
      end;

   subtract (other: like Current) is
         -- Remove all items also in `other'.
      do
         if not (other.empty or empty) then
            from
               start;
               other.start
            until
               off
            loop
               if other.has (item) then
                  remove
               else
                  forth
               end
            end
         end
      end;

end -- class LINKED_SET
