indexing
   description: "Structures whose items are sorted according to a total order",
                "relation                                                    ";
   title      : "EiffelBase: library of reusable components for Eiffel.      ",
                "Based on EiffelBase library for ISE Eiffel,                 ",
                "used with permission.                                       ";
   cluster    : EiffelBase;
   project    : "EiffelBase Library";
   copyright  : "(c) 1986-1997 Interactive Software Engineering Inc.         ",
                "Modifications and extensions:                               ",
                "(c) 1997 Object Tools                                       ",
                "All rights reserved. Duplication and distribution prohibited",
                "May be used only with Visual Eiffel under terms of user     ",
                "license                                                     ";
   key        : sorted_struct, comparable_struct;
   address    : "mail_to:support@object-tools.com                            ",
                "http://www.object-tools.com                                 ";

deferred class SORTED_STRUCT [G -> COMPARABLE] inherit

   COMPARABLE_STRUCT [G]
      undefine
         search, off
      redefine
         min, max
      end;

   INDEXABLE [G, INTEGER]
      rename
         item as i_th,
         put as put_i_th,
         bag_put as putt
      redefine
         putt
      end;

   LINEAR [G]

feature -- Measurement

   min: G is
         -- Minimum item
      require else
         is_sorted: sorted
      do
         start;
         Result := item
      ensure then
      --   smallest:
      --       For every `i' in `first_position' .. `last_position':
      --            `Result <= i_th (i)';
      --       `Result' = `i_th' (`first_position')
      end;

   max: G is
         -- Maximum item
      require else
         is_sorted: sorted
      do
         finish;
         Result := item
      ensure then
      --   largest:
      --       For every `i' in `first_position' .. `last_position':
      --            `i_th (`i') <= `Result';
      --       `Result' = `i_th' (`last_position')
      end;

   median: G is
         -- Median element
      deferred
      ensure
         median_present: has (Result)
      --   median_definition:
      --      Result = i_th (first_position +
      --         (last_position - first_position) // 2)
      end;

feature -- Status report

   sorted: BOOLEAN is
         -- Is structure sorted?
      local
         m: G
      do
         if empty then
            Result := true
         else
            from
               start;
               m := item;
               forth
            until
               exhausted or else (item < m)
            loop
               m := item;
               forth
            end;
            Result := exhausted
         end
      end;

feature -- Transformation

   sort is
         -- Sort structure.
      deferred
      ensure
         is_sorted: sorted
      end;

feature {NONE} -- Inapplicable

   putt (v: G) is
      do
      end;

end -- class SORTED_STRUCT
