indexing
   description: "Finite structures whose item count is subject to change";
   title      : "EiffelBase: library of reusable components for Eiffel.      ",
                "Based on EiffelBase library for ISE Eiffel,                 ",
                "used with permission.                                       ";
   cluster    : EiffelBase;
   project    : "EiffelBase Library";
   copyright  : "(c) 1986-1997 Interactive Software Engineering Inc.         ",
                "Modifications and extensions:                               ",
                "(c) 1997 Object Tools                                       ",
                "All rights reserved. Duplication and distribution prohibited",
                "May be used only with Visual Eiffel under terms of user     ",
                "license                                                     ";
   key        : storage;
   address    : "mail_to:support@object-tools.com                            ",
                "http://www.object-tools.com                                 ";

deferred class RESIZABLE [G] inherit

        BOUNDED [G];

feature -- Measurement


        Growth_percentage: INTEGER is 50;
                        -- Percentage by which structure will grow automatically

        Minimal_increase: INTEGER is 5;
                        -- Minimal number of additional items

        additional_space: INTEGER is
                        -- Proposed number of additional items
                        --| Result is a reasonable value, resulting from a space-time tradeoff.
                do
                        Result := (capacity * Growth_percentage // 100).max (Minimal_increase)
                ensure
                        At_least_one: Result >= 1
                end;

feature -- Status report

        resizable: BOOLEAN is
                        -- May `capacity' be changed? (Answer: yes.)
                do
                        Result := true
                end;

feature -- Resizing

        automatic_grow is
                        -- Change the capacity to accommodate at least
                        -- `Growth_percentage' more items.
                        --| Trades space for time:
                        --| allocates fairly large chunks of memory but not very often.
                do
                        grow (capacity + additional_space)
                ensure
                        increased_capacity:
                                capacity >= old capacity + old capacity * Growth_percentage // 100
                end;

        grow (i: INTEGER) is
                        -- Ensure that capacity is at least `i'.
                deferred
                ensure
                        new_capacity: capacity >= i
                end;

invariant

        increase_by_at_least_one: Minimal_increase >= 1

end -- class RESIZABLE
