indexing
   description: "Structures that may be traversed forward and backward";
   title      : "EiffelBase: library of reusable components for Eiffel.      ",
                "Based on EiffelBase library for ISE Eiffel,                 ",
                "used with permission.                                       ";
   cluster    : EiffelBase;
   project    : "EiffelBase Library";
   copyright  : "(c) 1986-1997 Interactive Software Engineering Inc.         ",
                "Modifications and extensions:                               ",
                "(c) 1997 Object Tools                                       ",
                "All rights reserved. Duplication and distribution prohibited",
                "May be used only with Visual Eiffel under terms of user     ",
                "license                                                     ";
   key        : bidirectional, traversing;
   address    : "mail_to:support@object-tools.com                            ",
                "http://www.object-tools.com                                 ";

deferred class BILINEAR [G] inherit

        LINEAR [G]
                rename
                        search as sequential_search
                export
                        {NONE}
                                sequential_search
                redefine
                        off
                end;

        LINEAR [G]
                redefine
                        search, off
                select
                        search
                end

feature -- Access

        off: BOOLEAN is
                        -- Is there no current item?
                do
                        Result := before or after
                end;

feature -- Cursor movement

        before: BOOLEAN is
                        -- Is there no valid position to the left of current one?
                deferred
                end;

        back is
                        -- Move to previous position.
                require
                        not_before: not before
                deferred
                ensure
                        moved_back: index = old index - 1
                end;

        search (v: G) is
                        -- Move to first position (at or after current
                        -- position) where `item' and `v' are equal.
                        -- If structure does not include `v' ensure that
                        -- `exhausted' will be true.
                        -- (Reference or object equality,
                        -- based on `object_comparison'.)
                do
                        if before and not empty then
                                forth
                        end;
                        sequential_search (v)
                end;

invariant

        not_both: not (after and before);
        empty_property: empty implies (after or before);
        before_constraint: before implies off

end -- class BILINEAR
