indexing
   description: "Infinite sequences, indexed by integers";
   title      : "EiffelBase: library of reusable components for Eiffel.      ",
                "Based on EiffelBase library for ISE Eiffel,                 ",
                "used with permission.                                       ";
   cluster    : EiffelBase;
   project    : "EiffelBase Library";
   copyright  : "(c) 1986-1997 Interactive Software Engineering Inc.         ",
                "Modifications and extensions:                               ",
                "(c) 1997 Object Tools                                       ",
                "All rights reserved. Duplication and distribution prohibited",
                "May be used only with Visual Eiffel under terms of user     ",
                "license                                                     ";
   key        : countable_sequence, infinite_sequence;
   address    : "mail_to:support@object-tools.com                            ",
                "http://www.object-tools.com                                 ";

deferred class COUNTABLE_SEQUENCE [G]

inherit

        COUNTABLE [G]
                rename
                        item as i_th
                redefine
                        i_th
                end

        ACTIVE [G]
                export
                        {NONE}
                                        fill, prune_all, put,
                                        prune,
                                        wipe_out, replace, remove
                end

        LINEAR [G]
                redefine
                        linear_representation
                end

feature -- Access

        i_th (i: INTEGER): G is
                        -- Item of rank `i'
                deferred
                end;

        index: INTEGER
                        -- Index of current position

        item: G is
                        -- Item at current position
                do
                        Result := i_th (index)
                end;

feature -- Status report

        after: BOOLEAN is false;
                        -- Is current position past last item? (Answer: no.)

        extendible: BOOLEAN is false;
                        -- May items be added? (Answer: no.)

        prunable: BOOLEAN is false;
                        -- May items be removed? (Answer: no.)

        readable: BOOLEAN is true;
                        -- Is there a current item that may be read?
                        -- (Answer: yes.)

        writable: BOOLEAN is false
                        -- Is there a current item that may be written?
                        -- (Answer: no.)

feature -- Cursor movement

        forth is
                        -- Move to next position.
                do
                        index := index + 1
                end;

        start is
                        -- Move to first position.
                do
                        index := 1
                end;

feature {NONE} -- Inapplicable

        extend (v: G) is
                        -- Add `v' at end.
                do
                end;

        finish is
                        -- Move to last position.
                do
                ensure then
                        failure: false
                end;

        linear_representation: LINEAR [G] is
                        -- Representation as a linear structure
                do
                end;

        prune (v: G) is
                        -- Remove first occurrence of `v', if any.
                do
                end;

        put (v: G) is
                        -- Add `v' to the right of current position.
                do
                end;

        remove is
                        -- Remove item to the right of current position.
                do
                end;

        replace (v: G) is
                        -- Replace by `v' item at current position.
                do
                end;

        wipe_out is
                        -- Remove all items.
                do
                end;

end -- class COUNTABLE_SEQUENCE
