indexing
   description: "Basic mathematical operations, double-precision.           ";
   title      : "EiffelBase: library of reusable components for Eiffel.      ",
                "Based on EiffelBase library for ISE Eiffel,                 ",
                "used with permission.                                       ";
   cluster    : EiffelBase;
   project    : "EiffelBase Library";
   copyright  : "(c) 1986-1997 Interactive Software Engineering Inc.         ",
                "Modifications and extensions:                               ",
                "(c) 1997 Object Tools                                       ",
                "All rights reserved. Duplication and distribution prohibited",
                "May be used only with Visual Eiffel under terms of user     ",
                "license                                                     ";
   address    : "mail_to:support@object-tools.com                            ",
                "http://www.object-tools.com                                 ";

class DOUBLE_MATH

inherit
   MATH
      rename
         log  as math_log,
         sqrt as math_sqrt
      export
         {NONE} all
   end

feature -- Access

   log_2 (v: DOUBLE): DOUBLE is
         -- Base 2 logarithm of `v'
      do
         Result := log (v) / log (2.0)
      end;

   cosine (v: DOUBLE): DOUBLE is
         -- Trigonometric cosine of radian `v' approximated
         -- in the range [-pi/4, +pi/4]
      do
         Result := cos( v.truncated_to_real );
      end;

   arc_cosine (v: DOUBLE): DOUBLE is
         -- Trigonometric arccosine of radian `v'
         -- in the range [0, pi]
      do
         Result := arccos( v.truncated_to_real );
      end;

   sine (v: DOUBLE): DOUBLE is
         -- Trigonometric sine of radian `v' approximated
         -- in range [-pi/4, +pi/4]
      do
         Result := sin( v.truncated_to_real );
      end;

   arc_sine (v: DOUBLE): DOUBLE is
         -- Trigonometric arcsine of radian `v'
         -- in the range [-pi/2, +pi/2]
      do
         Result := arcsin( v.truncated_to_real );
      end;

   tangent (v: DOUBLE): DOUBLE is
         -- Trigonometric tangent of radian `v' approximated
         -- in range [-pi/4, +pi/4]
      do
         Result := tan( v.truncated_to_real );
      end;

   arc_tangent (v: DOUBLE): DOUBLE is
         -- Trigonometric arctangent of radian `v'
         -- in the range [-pi/2, +pi/2]
      do
         Result := arctan( v.truncated_to_real );
      end;

   sqrt (v: DOUBLE): DOUBLE is
         -- Square root of `v'
      do
         Result := math_sqrt( v.truncated_to_real );
      end;

   log (v: DOUBLE): DOUBLE is
         -- Natural logarithm of `v'
      do
         Result := math_log( v.truncated_to_real );
      end;

   log10 (v: DOUBLE): DOUBLE is
         -- Base 10 logarithm of `v'
      do
         Result := log( v ) / log( 10.0 );
      end;

   floor (v: DOUBLE): DOUBLE is
         -- Greatest integral less than or equal to `v'
      do
         Result := v.floor
      end;

   ceiling (v: DOUBLE): DOUBLE is
         -- Least integral greater than or equal to `v'
      do
         Result := v.ceiling
      end;

end -- class DOUBLE_MATH
