indexing
   description: "Basic mathematical operations, single-precision";
   title      : "EiffelBase: library of reusable components for Eiffel.      ",
                "Based on EiffelBase library for ISE Eiffel,                 ",
                "used with permission.                                       ";
   cluster    : EiffelBase;
   project    : "EiffelBase Library";
   copyright  : "(c) 1986-1997 Interactive Software Engineering Inc.         ",
                "Modifications and extensions:                               ",
                "(c) 1997 Object Tools                                       ",
                "All rights reserved. Duplication and distribution prohibited",
                "May be used only with Visual Eiffel under terms of user     ",
                "license                                                     ";
   address    : "mail_to:support@object-tools.com                            ",
                "http://www.object-tools.com                                 ";

class SINGLE_MATH
inherit
   MATH
      export
         {NONE} all
         {ANY}  log, sqrt
   end
feature -- Access

   log_2 (v: REAL): REAL is
         -- Base 2 logarithm of `v'
      do
         Result := log (v) / log (2.0)
      end;

   cosine (v: REAL): REAL is
         -- Trigonometric cosine of radian `v' approximated
         -- in the range [-pi/4, +pi/4]
      do
         Result := cos( v )
      end;

   arc_cosine (v: REAL): REAL is
         -- Trigonometric arccosine of `v'
      do
         Result := arccos(v)
      end;

   sine (v: REAL): REAL is
         -- Trigonometric sine of radian `v' approximated
         -- in range [-pi/4, +pi/4]
      do
         Result := sin(v)
      end;

   arc_sine (v: REAL): REAL is
         -- Trigonometric arcsine of `v'
      do
         Result := arcsin(v)
      end;

   tangent (v: REAL): REAL is
         -- Trigonometric tangent of radian `v' approximated
         -- in range [-pi/4, +pi/4]
      do
         Result := tan(v)
      end;

   arc_tangent (v: REAL): REAL is
         -- Trigonometric arctangent of `v'
      do
         Result := arctan(v)
      end;

   -- sqrt (v: REAL): REAL is
   --       -- Square root of `v'
   --    require
   --       v >= 0.0
   --    do
   --    end;

   -- log (v: REAL): REAL is
   --       -- Natural logarithm of `v'
   --    require
   --       v > 0.0
   --    do
   --    end;

   log10 (v: REAL): REAL is
         -- Base 10 logarithm of `v'
      require
         v > 0.0
      do
         Result := log (v)/ log (10.0);
      end;

   floor (v: REAL): REAL is
         -- Greatest integral value less than or equal to `v'
      do
         Result := v.floor
      end;

   ceiling (v: REAL): REAL is
         -- Least integral value greater than or equal to `v'
      do
         Result := v.ceiling
      end;

end -- class SINGLE_MATH
