/* LINTLIBRARY */

#include <ctype.h>

#include "eif_match.h"

#define MULTILINE	1	/* Flag for multi-line mode */
#define IGN_CASE	2	/* Flag for case insensitive search */

/* Macro like functions
 * they provide a portable interface for different Eiffel compilers
 * as the macro extension is not supported by all compilers
 */

EIF_INTEGER eif_match_ALPHASIZE(void)
{
	return ALPHASIZE;
}

EIF_INTEGER eif_match_SMALLPLEN(void)
{
	return SMALLPLEN;
}

EIF_INTEGER eif_match_MULTILINE(void)
{
	return MULTILINE;
}

EIF_INTEGER eif_match_IGN_CASE(void)
{
	return IGN_CASE;
}


typedef unsigned char DTableC[ALPHASIZE]; 	/* type of the small shift table */
typedef EIF_INTEGER DTableI[ALPHASIZE]; 	/* type of the large shift table */

/* routines for small shift table */

	/* Search backward case sensitive */
static EIF_POINTER s_search_b_cs_sl	(
	unsigned char *text, int tlen, unsigned char *pattern,
	int plen, int mm_max, DTableC *dt
);
	/* Search forward case sensitive */
static EIF_POINTER s_search_f_cs_sl	(
	unsigned char *text, int tlen, unsigned char *pattern,
	int plen, int mm_max, DTableC *dt
);
	/* Search backward case insensitive */
static EIF_POINTER s_search_b_ic_sl	(
	unsigned char *text, int tlen, unsigned char *pattern,
	int plen, int mm_max, DTableC *dt
);
	/* Search forward case insensitive */
static EIF_POINTER s_search_f_ic_sl	(
	unsigned char *text, int tlen, unsigned char *pattern,
	int plen, int mm_max, DTableC *dt
);

/* routines for large shift table */

	/* Search backward case sensitive */
static EIF_POINTER l_search_b_cs_sl	(
	unsigned char *text, int tlen, unsigned char *pattern,
	int plen, int mm_max, DTableI *dt
);
	/* Search forward case sensitive */
static EIF_POINTER l_search_f_cs_sl	(
	unsigned char *text, int tlen, unsigned char *pattern,
	int plen, int mm_max, DTableI *dt
);
	/* Search backward case insensitive */
static EIF_POINTER l_search_b_ic_sl	(
	unsigned char *text, int tlen, unsigned char *pattern,
	int plen, int mm_max, DTableI *dt
);
	/* Search forward case insensitive */
static EIF_POINTER l_search_f_ic_sl	(
	unsigned char *text, int tlen, unsigned char *pattern,
	int plen, int mm_max, DTableI *dt
);

/* ... and the same again for multi-line mode ... */

/* routines for small shift table */

	/* Search backward case sensitive */
static EIF_POINTER s_search_b_cs_ml	(
	unsigned char *text, int tlen, unsigned char *pattern,
	int plen, int mm_max, DTableC *dt
);
	/* Search forward case sensitive */
static EIF_POINTER s_search_f_cs_ml	(
	unsigned char *text, int tlen, unsigned char *pattern,
	int plen, int mm_max, DTableC *dt
);
	/* Search backward case insensitive */
static EIF_POINTER s_search_b_ic_ml	(
	unsigned char *text, int tlen, unsigned char *pattern,
	int plen, int mm_max, DTableC *dt
);
	/* Search forward case insensitive */
static EIF_POINTER s_search_f_ic_ml	(
	unsigned char *text, int tlen, unsigned char *pattern,
	int plen, int mm_max, DTableC *dt
);

/* routines for large shift table */

	/* Search backward case sensitive */
static EIF_POINTER l_search_b_cs_ml	(
	unsigned char *text, int tlen, unsigned char *pattern,
	int plen, int mm_max, DTableI *dt
);
	/* Search forward case sensitive */
static EIF_POINTER l_search_f_cs_ml	(
	unsigned char *text, int tlen, unsigned char *pattern,
	int plen, int mm_max, DTableI *dt
);
	/* Search backward case insensitive */
static EIF_POINTER l_search_b_ic_ml	(
	unsigned char *text, int tlen, unsigned char *pattern,
	int plen, int mm_max, DTableI *dt
);
	/* Search forward case insensitive */
static EIF_POINTER l_search_f_ic_ml	(
	unsigned char *text, int tlen, unsigned char *pattern,
	int plen, int mm_max, DTableI *dt
);

static int mismatches;					/* actual number of mismatches */

void eif_match_mis_compile	(
	EIF_POINTER pattern,			/* The pattern we want to compile */
	register1 EIF_INTEGER plen, 	/* The length of the pattern */
	EIF_POINTER dtable,				/* The shifting tables */
	EIF_INTEGER mis, 				/* Maximum number of mismatches allowed */
	register2 EIF_BOOLEAN ignore,	/* Is the search case insensitive? */
	EIF_BOOLEAN backward			/* Compile for backward search? */
)
{
	/* Compile `pattern' by computing the shifting tables and store them in
	 * dtable for later use. `ignore' says, if we shall ignore letter case.
	 * If that is the case, we depend on pattern being already downshifted!
	 * `backward' says, if the shifting table is for a backward search.
	 * Note: A shifting table can only be used for letter case and search
	 * direction it was compiled for.
	 */

	register4 int j;
	register5 int direction = 1;	/* default: forward = increment */
	register3 unsigned char *p;

	if(backward) {
		direction = -1;			/* backward = decrement */
		pattern = (char *) pattern + plen - 1;	/* go to end of pattern */
	}
	if(plen++ <= SMALLPLEN) {	/* increment `plen'!!! */

		/* If `plen' is small enough, shifting values will fit in a character,
		 * so we (hopefully) got a character array from the Eiffel side.
		 */
		register6 DTableC *dt = (DTableC *) dtable;

		while(mis-- >= 0) {
			for(j = 0; j < ALPHASIZE; j++) 
					/* init with pattern length + 1 */
				(*dt)[j] = (unsigned char) plen;
			plen--;					/* restore pattern length */
			for(
					p = (unsigned char *) pattern, j = plen;
					j > 0;
					p += direction, j--
			) {
				(*dt)[*p] = (unsigned char) j;
				if(ignore)
					(*dt)[toupper(*p)] = (unsigned char) j;
			}
			dt++;
		}
	} else {
		/* Length of `pattern' is too large that shifting values can be stored
		 * in a character array, so we assume that we got an integer array.
		 */
		register6 DTableI *dt = (DTableI *) dtable;

		while(mis-- >= 0) {
			for(j = 0; j < ALPHASIZE; j++) 
				(*dt)[j] = plen;	/* init with pattern length + 1 */
			plen--;					/* restore pattern length */
			for(
					p = (unsigned char *) pattern, j = plen;
					j > 0;
					p += direction, j--
			) {
				(*dt)[*p] = j;
				if(ignore)
					(*dt)[toupper(*p)] = j;
			}
			dt++;
		}
	}
}

EIF_BOOLEAN eif_match_mis_search_back	(
	EIF_POINTER text, 		/* The text we are searching through */
	EIF_INTEGER from,		/* The relevant range inside text */
	EIF_INTEGER to,			/* (from .. to) */
	EIF_POINTER pattern,	/* The pattern we are looking for */
	EIF_INTEGER plen,		/* The length of the pattern */
	EIF_POINTER dtable, 	/* The pattern in compiled form */
	EIF_INTEGER mis,		/* The maximum number of mismatches allowed */
	EIF_INTEGER flags,		/* Flags to control letter case and line mode */
	EIF_POINTER s_pos,		/* Pointer to the attribute `start_position' */
	EIF_POINTER e_pos, 		/* Pointer to the attribute `end_position' */
	EIF_POINTER mm			/* Pointer to the attribute `mismatches' */
)
{
	char *p;

	/* Dispatch function call */

	if(flags & MULTILINE)
		if(plen <= SMALLPLEN)
			if(flags & IGN_CASE) 
				p = s_search_b_ic_ml(
						(unsigned char *) text + to - 1, to - from + 1,
						(unsigned char *) pattern + plen - 1, plen, 
						mis, (DTableC *) dtable);
			else 
				p = s_search_b_cs_ml(
						(unsigned char *) text + to - 1, to - from + 1,
						(unsigned char *) pattern + plen - 1, plen, 
						mis, (DTableC *) dtable);
		else
			if(flags & IGN_CASE) 
				p = l_search_b_ic_ml(
						(unsigned char *) text + to - 1, to - from + 1,
						(unsigned char *) pattern + plen - 1, plen, 
						mis, (DTableI *) dtable);
			else 
				p = l_search_b_cs_ml(
						(unsigned char *) text + to - 1, to - from + 1,
						(unsigned char *) pattern + plen - 1, plen, 
						mis, (DTableI *) dtable);
	else
		if(plen <= SMALLPLEN)
			if(flags & IGN_CASE) 
				p = s_search_b_ic_sl(
						(unsigned char *) text + to - 1, to - from + 1,
						(unsigned char *) pattern + plen - 1, plen, 
						mis, (DTableC *) dtable);
			else 
				p = s_search_b_cs_sl(
						(unsigned char *) text + to - 1, to - from + 1,
						(unsigned char *) pattern + plen - 1, plen, 
						mis, (DTableC *) dtable);
		else
			if(flags & IGN_CASE) 
				p = l_search_b_ic_sl(
						(unsigned char *) text + to - 1, to - from + 1,
						(unsigned char *) pattern + plen - 1, plen, 
						mis, (DTableI *) dtable);
			else 
				p = l_search_b_cs_sl(
						(unsigned char *) text + to - 1, to - from + 1,
						(unsigned char *) pattern + plen - 1, plen, 
						mis, (DTableI *) dtable);
	
	if(p == NULL)
		return EIF_FALSE;

	if(e_pos != NULL)
		*((EIF_INTEGER *) e_pos) = 1 + (p - (char *) text);
	if(s_pos != NULL)
		*((EIF_INTEGER *) s_pos) = *((EIF_INTEGER *) e_pos) - plen + 1;
	if(mm != NULL)
		*((EIF_INTEGER *) mm) = mismatches;

	return EIF_TRUE;
}

EIF_BOOLEAN eif_match_mis_search_forw  (
	EIF_POINTER text, 		/* The text we are searching through */
	EIF_INTEGER from,		/* The relevant range inside text */
	EIF_INTEGER to,			/* (from .. to) */
	EIF_POINTER pattern,	/* The pattern we are looking for */
	EIF_INTEGER plen,		/* The length of the pattern */
	EIF_POINTER dtable, 	/* The pattern in compiled form */
	EIF_INTEGER mis,		/* The maximum number of mismatches allowed */
	EIF_INTEGER flags,		/* Flags to control letter case and line mode */
	EIF_POINTER s_pos,		/* Pointer to the attribute `start_position' */
	EIF_POINTER e_pos,		/* Pointer to the attribute `end_position' */
	EIF_POINTER mm			/* Pointer to the attribute `mismatches' */
)
{
	char *p;

	/* Dispatch function call */

	if(flags & MULTILINE)
		if(plen <= SMALLPLEN)
			if(flags & IGN_CASE) 
				p = s_search_f_ic_ml(
						(unsigned char *) text + from - 1, to - from + 1,
						(unsigned char *) pattern, plen, 
						mis, (DTableC *) dtable);
			else 
				p = s_search_f_cs_ml(
						(unsigned char *) text + from - 1, to - from + 1,
						(unsigned char *) pattern, plen, 
						mis, (DTableC *) dtable);
		else
			if(flags & IGN_CASE) 
				p = l_search_f_ic_ml(
						(unsigned char *) text + from - 1, to - from + 1,
						(unsigned char *) pattern, plen, 
						mis, (DTableI *) dtable);
			else 
				p = l_search_f_cs_ml(
						(unsigned char *) text + from - 1, to - from + 1,
						(unsigned char *) pattern, plen, 
						mis, (DTableI *) dtable);
	else
		if(plen <= SMALLPLEN)
			if(flags & IGN_CASE) 
				p = s_search_f_ic_sl(
						(unsigned char *) text + from - 1, to - from + 1,
						(unsigned char *) pattern, plen, 
						mis, (DTableC *) dtable);
			else 
				p = s_search_f_cs_sl(
						(unsigned char *) text + from - 1, to - from + 1,
						(unsigned char *) pattern, plen, 
						mis, (DTableC *) dtable);
		else
			if(flags & IGN_CASE) 
				p = l_search_f_ic_sl(
						(unsigned char *) text + from - 1, to - from + 1,
						(unsigned char *) pattern, plen, 
						mis, (DTableI *) dtable);
			else 
				p = l_search_f_cs_sl(
						(unsigned char *) text + from - 1, to - from + 1,
						(unsigned char *) pattern, plen, 
						mis, (DTableI *) dtable);
		
	if(p == NULL)
		return EIF_FALSE;

	if(s_pos != NULL)
		*((EIF_INTEGER *) s_pos) = 1 + (p - (char *) text);
	if(e_pos != NULL)
		*((EIF_INTEGER *) e_pos) = *((EIF_INTEGER *) s_pos) + plen - 1;
	if(mm != NULL)
		*((EIF_INTEGER *) mm) = mismatches;

	return EIF_TRUE;
}

/* Because there are lots of searching functions and they are all so similar we
 * generate them from a template instead of writing them by hand. This is much
 * more convenient, since we have to make changes only once in the template.
 */
 
/*
 * Generation of searching routines
 */
 
/* For small shifting tables ... */
 
#define DTABLE		  DTableC
#define BACKWARD
#define SEARCH		  s_search_b_cs_sl
#include "misspelled.t"
#undef BACKWARD
#undef SEARCH
#define SEARCH		  s_search_f_cs_sl
#include "misspelled.t"
#define IGNORE_CASE
#define BACKWARD
#undef SEARCH
#define SEARCH		  s_search_b_ic_sl
#include "misspelled.t"
#undef BACKWARD
#undef SEARCH
#define SEARCH		  s_search_f_ic_sl
#include "misspelled.t"

/* ... and again for the large ones ... */
 
#undef DTABLE
#define DTABLE		  DTableI
#undef IGNORE_CASE
#define BACKWARD
#undef SEARCH
#define SEARCH		  l_search_b_cs_sl
#include "misspelled.t"
#undef BACKWARD
#undef SEARCH
#define SEARCH		  l_search_f_cs_sl
#include "misspelled.t"
#define IGNORE_CASE
#define BACKWARD
#undef SEARCH
#define SEARCH		  l_search_b_ic_sl
#include "misspelled.t"
#undef BACKWARD
#undef SEARCH
#define SEARCH		  l_search_f_ic_sl
#include "misspelled.t"

/* ... and again for multi-line mode ... */
#define MULTI_LINE

/* ... small shifting tables ... */
 
#undef DTABLE
#define DTABLE		  DTableC
#undef IGNORE_CASE
#define BACKWARD
#undef SEARCH
#define SEARCH		  s_search_b_cs_ml
#include "misspelled.t"
#undef BACKWARD
#undef SEARCH
#define SEARCH		  s_search_f_cs_ml
#include "misspelled.t"
#define IGNORE_CASE
#define BACKWARD
#undef SEARCH
#define SEARCH		  s_search_b_ic_ml
#include "misspelled.t"
#undef BACKWARD
#undef SEARCH
#define SEARCH		  s_search_f_ic_ml
#include "misspelled.t"

/* ... and again for the large ones. */
 
#undef DTABLE
#define DTABLE		  DTableI
#undef IGNORE_CASE
#define BACKWARD
#undef SEARCH
#define SEARCH		  l_search_b_cs_ml
#include "misspelled.t"
#undef BACKWARD
#undef SEARCH
#define SEARCH		  l_search_f_cs_ml
#include "misspelled.t"
#define IGNORE_CASE
#define BACKWARD
#undef SEARCH
#define SEARCH		  l_search_b_ic_ml
#include "misspelled.t"
#undef BACKWARD
#undef SEARCH
#define SEARCH		  l_search_f_ic_ml
#include "misspelled.t"
