indexing

	description: 
		"General concept of a matcher which extracts a found pattern from a",
		"text source."

deferred class EXTRACTOR

inherit

	MATCHER

	MATCHER
		rename
			search_forward as matcher_search_forward,
			search_backward as matcher_search_backward,
			match as matcher_match,
			match_prefix as matcher_match_prefix,
			match_suffix as matcher_match_suffix,
			match_left_part as matcher_match_left_part,
			match_right_part as matcher_match_right_part
		select
			matcher_search_forward, matcher_search_backward, matcher_match,
			matcher_match_prefix, matcher_match_suffix, 
			matcher_match_left_part, matcher_match_right_part
		end

feature  -- Status report

	search_string: STRING
			-- String of last search or match operation.

	released: BOOLEAN is
			-- Is `search_string' released?
		do
			Result := search_string = Void
		end

	matched_string: STRING is
			-- Matched part of `search_string'.
		require
			successful_search: found
			not_released: not released
		do
			if end_position >= start_position then
				Result := search_string.substring(start_position, end_position)
			else
				Result := ""
			end
		end

feature -- Actions

	release is
			-- Release search string.
		do
			search_string := Void
		ensure
			released: released
		end

	search_forward (s: STRING; s_from, s_to: INTEGER) is
		do
			search_string := s
			matcher_search_forward(s, s_from, s_to)
		end

	search_backward (s: STRING; s_from, s_to: INTEGER) is
		do
			search_string := s
			matcher_search_backward(s, s_from, s_to)
		end

	match (s: STRING) is
		do
			search_string := s
			matcher_match(s)
		end

	match_prefix (s: STRING) is
		do
			search_string := s
			matcher_match_prefix(s)
		end

	match_suffix (s: STRING) is
		do
			search_string := s
			matcher_match_suffix(s)
		end

	match_left_part (s: STRING) is
		do
			search_string := s
			matcher_match_left_part(s)
		end

	match_right_part (s: STRING) is
		do
			search_string := s
			matcher_match_right_part(s)
		end

end  -- class EXTRACTOR
