indexing

	description:
		"General notion of a pattern matcher"

deferred class MATCHER

inherit

	SEARCHER

feature -- Matching actions

	match (s: STRING) is
			-- Match `pattern' against string `s'. The whole string must 
			-- match. Set attributes `found' and - if match succeeded -
			-- `start_position' and `end_position'.
		require
			s_exists: s /= Void;
			pattern_compiled: compiled;
		deferred
		ensure
			found_set: found = matches(s)
			position_set: found implies 
							(start_position = 1 and end_position = s.count)
		end

	match_prefix (s: STRING) is
			-- Match `pattern' against any prefix of string `s'.
			-- Set attributes `found' and - if match succeeded -
			-- `start_position' and `end_position'.
		require
			s_exists: s /= Void;
			pattern_compiled: compiled;
		deferred
		ensure
			found_set: found = matches_prefix(s)
			position_set: found implies 
							(start_position = 1 and end_position <= s.count)
		end

	match_suffix (s: STRING) is
			-- Match `pattern' against any suffix of string `s'.
			-- Set attributes `found' and - if match succeeded -
			-- `start_position' and `end_position'.
		require
			s_exists: s /= Void;
			pattern_compiled: compiled;
		deferred
		ensure
			found_set: found = matches_suffix(s)
			position_set: found implies 
							(start_position >= 1 and end_position = s.count)
		end

	match_left_part (s: STRING) is
			-- Match `pattern' against any part (substring) of string `s'.
			-- Matching is from left to right. Attributes `found' and -
			-- if matched - `start_position' and `end_position' are set.
		require
			s_exists: s /= Void;
			pattern_compiled: compiled;
		deferred
		ensure
			found_set: found = matches_suffix(s)
			position_set: found implies 
							(start_position >= 1 and end_position <= s.count)
		end

	match_right_part (s: STRING) is
			-- Match `pattern' against any part (substring) of string `s'.
			-- Matching is from right to left. Attributes `found' and -
			-- if matched - `start_position' and `end_position' are set.
		require
			s_exists: s /= Void;
			pattern_compiled: compiled;
		deferred
		ensure
			found_set: found = matches_suffix(s)
			position_set: found implies 
							(start_position >= 1 and end_position <= s.count)
		end

feature -- Matching questions

	matches (s: STRING): BOOLEAN is
			-- Does string `s' matches with pattern? 
			-- The whole string must match. 
		require
			s_exists: s /= Void;
			pattern_compiled: compiled;
		deferred
		end

	matches_prefix (s: STRING): BOOLEAN is
			-- Does any prefix of string `s' matches with pattern? 
		require
			s_exists: s /= Void;
			pattern_compiled: compiled;
		deferred
		end

	matches_suffix (s: STRING): BOOLEAN is
			-- Does any suffix of string `s' matches with pattern? 
		require
			s_exists: s /= Void;
			pattern_compiled: compiled;
		deferred
		end

	matches_left_part (s: STRING): BOOLEAN is
			-- Does any substring of `s' matches with pattern? 
			-- Matching is from left to right.
		require
			s_exists: s /= Void;
			pattern_compiled: compiled;
		deferred
		end

	matches_right_part (s: STRING): BOOLEAN is
			-- Does any substring of `s' matches with pattern? 
			-- Matching is from right to left.
		require
			s_exists: s /= Void;
			pattern_compiled: compiled;
		deferred
		end

end  -- class MATCHER
