indexing

	description:
		"Very fast substring searcher";
	store_and_retrieve: "yes"

class SUBSTRING

inherit

	SEARCHER


creation

	make, make_sensitive, make_insensitive

feature -- Modification

	set_sensitive is
			-- Make the searcher case-sensitive.
		local
			cp: like dtable
		do
			if insensitive then
				insensitive := false
				cp := saved_dtable
				saved_dtable := dtable
				dtable := cp
				cp := saved_dtable_b
				saved_dtable_b := dtable_b
				dtable_b := cp
				if dtable = Void then
					compile
				end
			end
		end

	set_insensitive is
			-- Make the searcher case-insensitive.
		local
			cp: like dtable
		do
			if not insensitive then
				insensitive := true
				cp := saved_dtable
				saved_dtable := dtable
				dtable := cp
				cp := saved_dtable_b
				saved_dtable_b := dtable_b
				dtable_b := cp
				if dtable = Void then
					compile
				end
			end
		end

feature -- Searching

	search_forward(s: STRING; s_from, s_to: INTEGER) is
            -- Search for `pattern' in string `s'. Start searching at index
            -- position `s_from' and search until the pattern matches (while
            -- ignoring any characters behind `s_to'). If pattern matches, set
            -- `found' to true and `start_position' and `end_position'
            -- to the corresponding index positions (relative to the beginning
            -- of the string). If the pattern doesn't match, set `found' to
            -- false, but do not modify `start_position' or `end_position'.
		local
			pattern_area: POINTER
			index, plen: INTEGER
		do
			found := false
			plen := pattern.count
			if insensitive then
				pattern_area := downshifted_pattern.to_c
			else
				pattern_area := pattern.to_c
			end
			index := sub_search(s.to_c, s_from, s_to, pattern_area, plen,
								dtable.to_c, insensitive, false)
			if (index > 0) then
				start_position := index
				end_position := index + plen - 1
				found := true;
			end
		end

	search_backward(s: STRING; s_from, s_to: INTEGER) is
            -- Search for `pattern' in string `s'. Start searching at index
            -- position `s_to' and search backward until the pattern matches
            -- (ignoring any characters before `s_from'). If pattern matches,
            -- set `found' to true and `start_position' and `end_position'
            -- to the corresponding index positions (relative to the beginning
            -- of the string). If the pattern doesn't match, set `found' to
            -- false, but do not modify `start_position' or `end_position'.
		local
			pattern_area: POINTER
			index, plen: INTEGER
		do
			if dtable_b = Void then
				compile_backw
			end
			if insensitive then
				pattern_area := downshifted_pattern.to_c
			else
				pattern_area := pattern.to_c
			end
			found := false
			plen := pattern.count
			index := sub_search(s.to_c, s_from, s_to, pattern_area, plen,
								dtable_b.to_c, insensitive, true)
			if (index > 0) then
				start_position := index
				end_position := index + plen - 1
				found := true;
			end
		end

feature {NONE} -- Implementation

	dtable, dtable_b, saved_dtable, saved_dtable_b: MEMORY_BLOCK
			-- Distance tables areas

	downshifted_pattern: like pattern
			-- Pattern for case-insensitive search

	wipe_out is
		do
			dtable := Void
			dtable_b := Void
			saved_dtable := Void
			saved_dtable_b := Void
			downshifted_pattern := Void
			compiled := false
		end

	compile is
			-- Compile `pattern' for a forward search.
		do
      	compile_it(false)
		end

	compile_backw is
			-- Compile `pattern' for a backward search.
		do
			compile_it(true)
		end

	compile_it(backw: BOOLEAN) is
			-- Compile `pattern'.
		local
			pattern_area: POINTER
			table: MEMORY_BLOCK
		do
			if insensitive then
				if downshifted_pattern = Void then
					downshifted_pattern := clone(pattern)
					downshifted_pattern.to_lower
				end
				pattern_area := downshifted_pattern.to_c
			else
				pattern_area := pattern.to_c
			end
			if (pattern.count <= Small_pattern_size) then
				!ARRAY_MEMORY_BLOCK[CHARACTER]!table.make(Alpha_size)
			else
				!ARRAY_MEMORY_BLOCK[INTEGER]!table.make(Alpha_size)
			end
			if (backw) then
				dtable_b := table
			else
				dtable := table
			end
			sub_compile(pattern_area, pattern.count, table.to_c,
				insensitive, backw)
			compiled := true;
		end

feature {NONE} -- Externals

	Alpha_size: INTEGER is
			-- Number of characters in the alphabet
		external
			"C"
		alias
			"eif_match_ALPHASIZE"
		end

	Small_pattern_size: INTEGER is
			-- Maximum pattern length for character array usage
		external
			"C"
		alias
			"eif_match_SMALLPLEN"
		end

	sub_compile (patt: POINTER; plen: INTEGER; dt: POINTER;
		ign, back: BOOLEAN) is
			-- Compile substring pattern `patt', which has length `plen'
			-- and store result in area pointed to by `dt'. `ign' says
			-- if we shall compile for case insensite, `back' if we compile
			-- for a backward search distance table.
		external
			"C"
		alias
			"eif_match_sub_compile"
		end

	sub_search (text: POINTER; t_from, t_to: INTEGER; patt: POINTER;
		plen: INTEGER; dt: POINTER; ign, back: BOOLEAN): INTEGER is
			-- Search in range `t_from' to `t_to' in `text' for pattern
			-- pointed to by `patt' (with length `plen') using distance
			-- table `dt'. `ign' says, if we shall ignore letter case,
			-- `back' says, if we shall do a backward search.
			-- Return index in `text' where pattern starts (if found) else 0.
		external
			"C"
		alias
			"eif_match_sub_search"
		end

invariant
	always_compiled: compiled

end  -- class SUBSTRING
