indexing

   title:       "JNI JDK1_1InitArgs (since JDK 1.1)";
   cluster:     "jni";
   project:     "Java Native Interface Library";
   date:        "$Date: 2005/02/01 10:20:41 $";
   revision:    "$Revision: 1.1 $";
   copyright:   "Copyright (C) 1996-2005 Object Tools Group";
   license:     "http://visual-eiffel.com/license";
   disclaimer:  "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

class JNI_JDK1_1InitArgs

inherit

    JNI_JavaVMInitArgs_Header
       redefine
          out
       end

creation

   make

feature {NONE} -- Creation

   make (peer: JNI_RAW_JDK1_1InitArgs) is
         -- Create arguments object
      require
         non_void_peer: peer /= Void
      do
         -- if peer.properties /= Void then
         --    !!raw_properties.make (peer.properties.lower, peer.properties.upper)
         --    from
         --       i := raw_properties.lower
         --    until
         --       i > raw_properties.upper
         --    loop
         --       raw_properties.put (peer.properties.item (i).to_c, i)
         --       i := i + 1
         --    end
         --    properties := raw_properties.to_c
         -- end
         checkSource := peer.checkSource /= 0
         nativeStackSize := peer.nativeStackSize
         javaStackSize := peer.javaStackSize
         minHeapSize := peer.minHeapSize
         maxHeapSize := peer.maxHeapSize
         verifyMode := peer.verifyMode
         !!classpath.make_empty
         if peer.classpath /= default_pointer then
            classpath.from_c (peer.classpath)
         end
         classpath.append_character (';')
         classpath.append_string (default_classpath)
         vfprintf := peer.vfprintf
         exit := peer.exit
         abort := peer.abort
         enableClassGC := peer.enableClassGC /= 0
         enableVerboseGC := peer.enableVerboseGC /= 0
         disableAsyncGC := peer.disableAsyncGC /= 0
         reserved0 := peer.reserved0
         reserved1 := peer.reserved1
         reserved2 := peer.reserved2
      end

feature -- Status

   version: INTEGER is 65537
         -- Java VM version: jni_version_1_1

   properties: ARRAY [STRING]
         -- System properties

   checkSource: BOOLEAN
         -- Whether to check the Java source files are newer than
         -- compiled class files

   nativeStackSize: INTEGER
         -- Maximum native stack size of Java-created threads

   javaStackSize: INTEGER
         -- Maximum Java stack size

   minHeapSize: INTEGER
         -- Initial heap size

   maxHeapSize: INTEGER
         -- Maximum heap size

   verifyMode: INTEGER
         -- Controls whether Java byte code should be verified:
         -- 0 -- none, 1 -- remotely loaded code, 2 -- all code

   classpath: STRING
         -- The local directory path for class loading

   vfprintf: POINTER
         -- A hook for a function that redirects all VM messages
         -- jint (*vfprintf)(FILE *fp, const char *format, va_list args)

   exit: POINTER
         -- A VM exit hook
         -- void (*exit)(jint code)

   abort: POINTER
         -- A VM abort hook
         -- void (*abort)()

   enableClassGC: BOOLEAN
         -- Whether to enable class GC

   enableVerboseGC: BOOLEAN
         -- Whether GC messages will appear

   disableAsyncGC: BOOLEAN
         -- Whether asynchronous GC is allowed

   reserved0: INTEGER
         -- Reserved field

   reserved1: INTEGER
         -- Reserved field

   reserved2: INTEGER
         -- Reserved field

feature -- Status setting

   set_properties (value: ARRAY [STRING]) is
         -- System properties
      do
         properties := value
      ensure
         properties_set: properties = value
      end

   set_checkSource (value: BOOLEAN) is
         -- Whether to check the Java source files are newer than
         -- compiled class files
      do
         checkSource := value
      ensure
         checkSource_set: checkSource = value
      end

   set_nativeStackSize (value: INTEGER) is
         -- Maximum native stack size of Java-created threads
      do
         nativeStackSize := value
      ensure
         nativeStackSize_set: nativeStackSize = value
      end

   set_javaStackSize (value: INTEGER) is
         -- Maximum Java stack size
      do
         javaStackSize := value
      ensure
         javaStackSize_set: javaStackSize = value
      end

   set_minHeapSize (value: INTEGER) is
         -- Initial heap size
      do
         minHeapSize := value
      ensure
         minHeapSize_set: minHeapSize = value
      end

   set_maxHeapSize (value: INTEGER) is
         -- Maximum heap size
      do
         maxHeapSize := value
      ensure
         maxHeapSize_set: maxHeapSize = value
      end

   set_verifyMode (value: INTEGER) is
         -- Controls whether Java byte code should be verified:
         -- 0 -- none, 1 -- remotely loaded code, 2 -- all code
      require
         valid_value: value = 0 or value = 1 or value = 2
      do
         verifyMode := value
      ensure
         verifyMode_set: verifyMode = value
      end

   set_classpath (value: STRING) is
         -- The local directory path for class loading
      do
         classpath := value
      ensure
         classpath_set: classpath = value
      end

   set_vfprintf (value: POINTER) is
         -- A hook for a function that redirects all VM messages
         -- jint (*vfprintf)(FILE *fp, const char *format, va_list args)
      do
         vfprintf := value
      ensure
         vfprintf_set: vfprintf = value
      end

   set_exit (value: POINTER) is
         -- A VM exit hook
         -- void (*exit)(jint code)
      do
         exit := value
      ensure
         exit_set: exit = value
      end

   set_abort (value: POINTER) is
         -- A VM abort hook
         -- void (*abort)()
      do
         abort := value
      ensure
         abort_set: abort = value
      end

   set_enableClassGC (value: BOOLEAN) is
         -- Whether to enable class GC
      do
         enableClassGC := value
      ensure
         enableClassGC_set: enableClassGC = value
      end

   set_enableVerboseGC (value: BOOLEAN) is
         -- Whether GC messages will appear
      do
         enableVerboseGC := value
      ensure
         enableVerboseGC_set: enableVerboseGC = value
      end

   set_disableAsyncGC (value: BOOLEAN) is
         -- Whether asynchronous GC is allowed
      do
         disableAsyncGC := value
      ensure
         disableAsyncGC_set: disableAsyncGC = value
      end

   set_reserved0 (value: INTEGER) is
         -- Reserved field
      do
         reserved0 := value
      ensure
         reserved0_set: reserved0 = value
      end

   set_reserved1 (value: INTEGER) is
         -- Reserved field
      do
         reserved1 := value
      ensure
         reserved1_set: reserved1 = value
      end

   set_reserved2 (value: INTEGER) is
         -- Reserved field
      do
         reserved2 := value
      ensure
         reserved2_set: reserved2 = value
      end

feature -- Output

   out: STRING is
      local
         i: INTEGER
      do
         Result := "version: "
         Result.append_integer (version // 0x00010000)
         Result.append_character ('.')
         Result.append_integer (version \\ 0x00010000)
         Result.append_character ('%N')
         if properties /= Void then
            Result.append_string ("Properties:%N")
            from
               i := properties.lower
            until
               i > properties.upper
            loop
               Result.append_string ("   ")
               Result.append_string (properties.item (i).out)
               Result.append_character ('%N')
               i := i + 1
            end
         end
         Result.append_string ("checkSource: ")
         Result.append_boolean (checkSource)
         Result.append_string ("%NnativeStackSize: ")
         Result.append_integer (nativeStackSize)
         Result.append_string ("%NjavaStackSize: ")
         Result.append_integer (javaStackSize)
         Result.append_string ("%NminHeapSize: ")
         Result.append_integer (minHeapSize)
         Result.append_string ("%NmaxHeapSize: ")
         Result.append_integer (maxHeapSize)
         Result.append_string ("%NverifyMode: ")
         Result.append_integer (verifyMode)
         Result.append_string (" (")
         inspect verifyMode
         when 0 then Result.append_string ("none")
         when 1 then Result.append_string ("remotely loaded code")
         when 2 then Result.append_string ("all code")
         else        Result.append_string ("?")
         end
         Result.append_string (")%Nclasspath: ")
         if classpath /= Void then
            Result.append_string (classpath)
         end
         Result.append_string ("%Nvfprintf: ")
         Result.append_string (vfprintf.out)
         Result.append_string ("%Nexit: ")
         Result.append_string (exit.out)
         Result.append_string ("%Nabort: ")
         Result.append_string (abort.out)
         Result.append_string ("%NenableClassGC: ")
         Result.append_boolean (enableClassGC)
         Result.append_string ("%NenableVerboseGC: ")
         Result.append_boolean (enableVerboseGC)
         Result.append_string ("%NdisableAsyncGC: ")
         Result.append_boolean (disableAsyncGC)
         Result.append_string ("%Nreserved0: ")
         Result.append_integer (reserved0)
         Result.append_string ("%Nreserved1: ")
         Result.append_integer (reserved1)
         Result.append_string ("%Nreserved2: ")
         Result.append_integer (reserved2)
      end

feature {JNI_JAVA_VIRTUAL_MACHINE} -- Implementation: raw arguments

   raw_data: JNI_RAW_JDK1_1InitArgs is
      do
         !!Result.make (Current)
      end

end