indexing

   title:       "Java object generator";
   cluster:     "jni_support";
   project:     "Java Native Interface Support Library";
   date:        "$Date: 2005/02/01 10:20:42 $";
   revision:    "$Revision: 1.1 $";
   copyright:   "Copyright (C) 1996-2005 Object Tools Group";
   license:     "http://visual-eiffel.com/license";
   disclaimer:  "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

deferred class JNI_OBJECT_FACTORY

inherit

   JNI_SERVER
      end

feature -- Object creation

   create_from_java_object (object: JNI_jobject): JNI_OBJECT is
         -- Creates object representing Java 'object'
      require
         non_void_object: object /= Void
      deferred
      ensure
         non_void_result: Result /= Void
      end

   frozen checked_create_from_java_object (
      object: JNI_jobject
   ): like create_from_java_object is
         -- Creates object representing Java 'object'
         -- Void 'object' yields Void Result
      do
         if object /= Void then
            Result := create_from_java_object (object)
         end
      ensure
         consistent_result: (object /= Void) = (Result /= Void)
      end

   frozen create_from_CallObjectMethod (
      object: JNI_OBJECT
      method_id: JNI_jmethodID
      args: ARRAY [ANY]
   ): like create_from_java_object is
      require
         non_void_object: object /=  Void
         valid_method_id: method_id /= Void
      do
         Result := checked_create_from_java_object(
            jni.CallObjectMethod(
               object.this,
               method_id,
               args
            )
         )
      end

   frozen create_from_GetObjectField (
      object: JNI_OBJECT
      field_id: JNI_jfieldID
   ): like create_from_java_object is
      require
         non_void_object: object /=  Void
         valid_field_id: field_id /= Void
      do
         Result := checked_create_from_java_object(
            jni.GetObjectField(
               object.this,
               field_id
            )
         )
      end

feature -- Conversion

   frozen create_array_from_java_object (
      object: JNI_jobject
   ): ARRAY [like create_from_java_object] is
         -- Creates ARRAY of objects using supplied Java array
      require
         non_void_object: object /= Void
         array_object: -- object is array
      local
         i: INTEGER
      do
         from
            i := jni.GetArrayLength (object)
            !!Result.make (1, i)
         variant
            i + 1
         until
            i <= 0
         loop
            Result.put(
               checked_create_from_java_object(
                  jni.GetObjectArrayElement (object, i - 1)
               ),
               i
            )
            i := i - 1
         end
      ensure
         non_void_result: Result /= Void
         consistent_result_count: Result.count = jni.GetArrayLength (object)
         consistent_result_items: -- for all i in 1..count
                                   --    jni.GetObjectArrayElement(
                                   --       object,
                                   --       i - 1
                                   --   ) =
                                   --    Result.item (i).this
      end

   frozen create_array_from_CallObjectMethod (
      object: JNI_OBJECT
      method_id: JNI_jmethodID
      args: ARRAY [ANY]
   ): ARRAY [like create_from_java_object] is
      require
         non_void_object: object /=  Void
         valid_method_id: method_id /= Void
      do
         Result := create_array_from_java_object(
            jni.CallObjectMethod(
               object.this,
               method_id,
               args
            )
         )
      end

end