indexing

   title:       "Access to JNI";
   cluster:     "jni_support";
   project:     "Java Native Interface Support Library";
   date:        "$Date: 2005/02/01 10:20:42 $";
   revision:    "$Revision: 1.1 $";
   copyright:   "Copyright (C) 1996-2005 Object Tools Group";
   license:     "http://visual-eiffel.com/license";
   disclaimer:  "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

class JNI_SERVER

feature -- Access

   jni: JNI_JAVA_NATIVE_INTERFACE is
         -- Access to JNI
      once
         if jvm /= Void then
            Result := jvm.jni
         end
      end

   to_internal_class_name (class_name: STRING): STRING is
         -- Converts "dot"ed class name to "slash"ed class name, e.g.
         -- java.lang.String yields java/lang/String
      require
         non_void_class_name: class_name /= Void
      local
         i: INTEGER
      do
         Result := clone (class_name)
         from
            i := Result.index_of ('.', 1)
         until
            i = 0
         loop
            Result.put ('/', i)
            i := Result.index_of ('.', i)
         end
      ensure
         converted: not Result.has ('.')
      end

   mandatory_find_class (class_name: STRING): JNI_jclass is
         -- Returns object, representing Java class
         -- Raises an exception if no class is found
      require
         non_void_class_name: class_name /= Void
      local
         message: STRING
         exceptions: EXCEPTIONS
      do
         if jni /= Void then
            Result := jni.FindClass (class_name)
            check_for_exception
         end
         if Result = Void then
            message := clone ("Cannot find java class %"")
            message.append_string (class_name)
            message.append_character ('"')
            !!exceptions
            exceptions.raise (message)
         end
      ensure
         non_void_result: Result /= Void
      end

   mandatory_get_method_id (
         jclass: JNI_jclass
         method_name: STRING
         method_signature: STRING
      ): JNI_jmethodID is
         -- Returns Java method identifier
         -- Raises an exception if no method is found
      require
         non_void_jclass: jclass /= Void
         non_void_method_name: method_name /= Void
         non_void_method_signature: method_signature /= Void
      local
         message: STRING
         exceptions: EXCEPTIONS
      do
         Result := jni.GetMethodID(
            jclass          ,
            method_name     ,
            method_signature
         )
         check_for_exception
         if Result = Void then
            message := clone ("Cannot find java method %"")
            message.append_string (method_name)
            message.append_character ('"')
            !!exceptions
            exceptions.raise (message)
         end
      ensure
         meaningful_result: Result /= Void
      end

   mandatory_get_field_id (
         jclass: JNI_jclass
         field_name: STRING
         field_signature: STRING
      ): JNI_jfieldID is
         -- Returns Java field identifier
         -- Raises an exception if no field is found
      require
         non_void_jclass: jclass /= Void
         non_void_field_name: field_name /= Void
         non_void_field_signature: field_signature /= Void
      local
         message: STRING
         exceptions: EXCEPTIONS
      do
         Result := jni.GetFieldID(
            jclass         ,
            field_name     ,
            field_signature
         )
         check_for_exception
         if Result = Void then
            message := clone ("Cannot find java field %"")
            message.append_string (field_name)
            message.append_character ('"')
            !!exceptions
            exceptions.raise (message)
         end
      ensure
         meaningful_result: Result /= Void
      end

   mandatory_get_static_field_id (
         jclass: JNI_jclass
         field_name: STRING
         field_signature: STRING
      ): JNI_jfieldID is
         -- Returns Java field identifier
         -- Raises an exception if no field is found
      require
         non_void_jclass: jclass          /= Void
         non_void_field_name: field_name      /= Void
         non_void_field_signature: field_signature /= Void
      local
         message: STRING
         exceptions: EXCEPTIONS
      do
         Result := jni.GetStaticFieldID(
            jclass         ,
            field_name     ,
            field_signature
         )
         check_for_exception
         if Result = Void then
            message := clone ("Cannot find java static field %"")
            message.append_string (field_name)
            message.append_character ('"')
            !!exceptions
            exceptions.raise (message)
         end
      ensure
         meaningful_result: Result /= Void
      end

   create_string_from_CallObjectMethod (
         object: JNI_OBJECT
         method_id: JNI_jmethodID
         args: ARRAY [ANY]
      ): STRING is
      require
         non_void_object: object    /= Void
         valid_method_id: method_id /= Void
      do
         Result := create_string_from_jobject_CallObjectMethod (
            object.this,
            method_id,
            args
         )
      end

   create_string_from_GetObjectField (
         object: JNI_OBJECT
         field_id: JNI_jfieldID
      ): STRING is
      require
         non_void_object: object   /= Void
         valid_field_id:  field_id /= Void
      local
         jstring_object: JNI_jobject
         jstring_pointer: POINTER
      do
         jstring_object := jni.GetObjectField(
            object.this,
            field_id
         )
         check_for_exception
         if jstring_object /= Void then
            jstring_pointer := jni.GetStringUTFChars (jstring_object, Void)
            check_for_exception
            if jstring_pointer /= default_pointer then
               !!Result.make_empty
               Result.from_c (jstring_pointer)
            end
            jni.ReleaseStringUTFChars (jstring_object, jstring_pointer)
            check_for_exception
         end
      end

   create_string_from_GetStaticObjectField (
         jclass: JNI_jclass
         field_id: JNI_jfieldID
      ): STRING is
      require
         non_void_jclass: jclass   /= Void
         valid_field_id:  field_id /= Void
      local
         jstring_object: JNI_jobject
         jstring_pointer: POINTER
      do
         jstring_object := jni.GetStaticObjectField(
            jclass,
            field_id
         )
         check_for_exception
         if jstring_object /= Void then
            jstring_pointer := jni.GetStringUTFChars (jstring_object, Void)
            check_for_exception
            if jstring_pointer /= default_pointer then
               !!Result.make_empty
               Result.from_c (jstring_pointer)
            end
            jni.ReleaseStringUTFChars (jstring_object, jstring_pointer)
            check_for_exception
         end
      end

   check_for_exception is
         -- Check whether there is no pending exception
         -- Raise an Eiffel exception otherwise
      local
         jexception: JNI_jobject
         exceptions: EXCEPTIONS
      do
         if jni /= Void then
            jexception := jni.ExceptionOccurred
            if jexception /= Void then
               jni.ExceptionClear
               !!exceptions
               exceptions.raise (
                  create_string_from_jobject_CallObjectMethod (
                     jexception,
                     mandatory_get_method_id (
                        mandatory_find_class ("java/lang/Object"),
                        "toString",
                        "()Ljava/lang/String;"
                     ),
                     Void
                  )
               )
            end
         end
      ensure
         no_pending_exception: jni /= Void implies jni.ExceptionOccurred = Void
      end

feature {NONE} -- Implementation

   create_string_from_jobject_CallObjectMethod (
         object: JNI_jobject
         method_id: JNI_jmethodID
         args: ARRAY [ANY]
      ): STRING is
      require
         non_void_object: object    /= Void
         valid_method_id: method_id /= Void
      local
         jstring_object: JNI_jobject
         jstring_pointer: POINTER
      do
         jstring_object := jni.CallObjectMethod(
            object,
            method_id,
            args
         )
         check_for_exception
         if jstring_object /= Void then
            jstring_pointer := jni.GetStringUTFChars (jstring_object, Void)
            check_for_exception
            if jstring_pointer /= default_pointer then
               !!Result.make_empty
               Result.from_c (jstring_pointer)
            end
            jni.ReleaseStringUTFChars (jstring_object, jstring_pointer)
            check_for_exception
         end
      end

   jvm: JNI_JAVA_VIRTUAL_MACHINE is
      once
         !!Result.make
      end

end