indexing

   cluster:     "velibc";
   project:     "Visual Eiffel libc Library";
   date:        "$Date: 2005/02/15 16:22:09 $";
   revision:    "$Revision: 1.2 $";
   copyright:   "Copyright (C) 1996-2005 Object Tools Group";
   license:     "http://visual-eiffel.com/license";
   disclaimer:  "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

class VELIBC_STAT

inherit

   VELIBC_BASE

   VELIBC_STAT_EXTERNALS

   VELIBC_STAT_CONSTANTS

feature

   mkdir (pathname: STRING; mode: BIT 32): INTEGER is
         -- create a directory
      require
         valid_pathname: pathname /= Void and then not pathname.is_empty
      do
         mem.collection_off;
         Result := ext_mkdir(sh.to_c(pathname), mode);
         mem.collection_on;
      end

   chmod (pathname: STRING; mode: BIT 32): INTEGER is
         -- change permissions of a file
      require
         valid_pathname: pathname /= Void and then not pathname.is_empty
      do
         mem.collection_off;
         Result := ext_chmod(sh.to_c(pathname), mode);
         mem.collection_on;
      end

   stat (pathname: STRING): VELIBC_STAT_STRUCT is
         -- get file status
         -- returns Void on error
      require
         valid_pathname: pathname /= Void and then not pathname.is_empty
      local
         res: INTEGER;
         stat_struct: VELIBC_STAT_STRUCT;
      do
         !!stat_struct.make;
         mem.collection_off;
         res := ext_stat (sh.to_c(pathname),stat_struct.to_c);
         mem.collection_on;
         if res = 0 then
            Result := stat_struct;
         end
      end

   lstat (pathname: STRING): VELIBC_STAT_STRUCT is
         -- get file status
         -- returns Void on error
      require
         valid_pathname: pathname /= Void and then not pathname.is_empty
      local
         res: INTEGER;
         stat_struct: VELIBC_STAT_STRUCT;
      do
         !!stat_struct.make;
         mem.collection_off;
         res := ext_lstat (sh.to_c(pathname),stat_struct.to_c);
         mem.collection_on;
         if res = 0 then
            Result := stat_struct;
         end
      end

   fstat (fd: INTEGER): VELIBC_STAT_STRUCT is
         -- get file status
      local
         res: INTEGER;
         stat_struct: VELIBC_STAT_STRUCT;
      do
         !!stat_struct.make;
         res := ext_fstat (fd,stat_struct.to_c);
         if res = 0 then
            Result := stat_struct;
         end
      end

   umask (new_mask: BIT 32): BIT 32 is
         -- set file creation mask
      external "C"
      alias "VELIBC_umask"
      end

end
