indexing

   cluster:     "velibc";
   project:     "Visual Eiffel libc Library";
   date:        "$Date: 2005/02/15 16:22:10 $";
   revision:    "$Revision: 1.2 $";
   copyright:   "Copyright (C) 1996-2005 Object Tools Group";
   license:     "http://visual-eiffel.com/license";
   disclaimer:  "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

class VELIBC_STDIO

inherit 

   VELIBC_BASE

   VELIBC_STDIO_EXTERNALS

feature
   
   rename_file (oldpath, newpath: STRING): INTEGER is
         -- Renames a file
         -- We can't name this feature 'rename' 'cause it's keyword
         -- in Eiffel.
      require
         non_void_oldpath: oldpath /= Void
         non_empty_oldpath: not oldpath.is_empty
         non_void_newpath: newpath /= Void
         non_empty_newpath: not newpath.is_empty
      do
         mem.collection_off;
         Result := ext_rename_file (sh.to_c(oldpath), sh.to_c(newpath));
         mem.collection_on;
      end
   
   remove (pathname: STRING): INTEGER is
         -- Removes a file from a directory
      require
         non_void_pathname: pathname /= Void
         non_empty_pathname: not pathname.is_empty
      do
         mem.collection_off;
         Result := ext_remove(sh.to_c(pathname));
         mem.collection_on;
      end

   tmpfile: POINTER is
         -- Create a temporary file
      external "C"
      alias "VELIBC_tmpfile"
      end

   tmpnam: STRING is
         -- Create a name for a temporary file
      do
         Result := sh.from_c(ext_tmpnam);
      end

   clearerr (stream: POINTER) is
         -- Clears end-of-file and error indicators for a stream
      require
         valid_stream: stream /= default_pointer
      external "C"
      alias "VELIBC_clearerr"
      end

   fclose (stream: POINTER): INTEGER is
         -- Closes an open stream
      require
         valid_stream: stream /= default_pointer
      external "C"
      alias "VELIBC_fclose"
      end

   feof (stream: POINTER): BOOLEAN is
         -- Tests the end-of-file indicator for a stream
      require
         valid_stream: stream /= default_pointer
      external "C"
      alias "VELIBC_feof"
      end
   
   ferror (stream: POINTER): BOOLEAN is
         -- Tests the error indicator for a stream
      require
         valid_stream: stream /= default_pointer
      external "C"
      alias "VELIBC_ferror"
      end

   fflush (stream: POINTER): INTEGER is
         -- Updates stream
         -- if stream is default_pointer all open streams are updated.
      external "C"
      alias "VELIBC_fflush"
      end
   
   fgetc (stream: POINTER): CHARACTER is
         -- Reads a character from a stream
      require
         valid_stream: stream /= default_pointer
      external "C"
      alias "VELIBC_fgetc"
      end

   fgetpos (stream, pos: POINTER): INTEGER is
         -- Gets the current file position
      require
         valid_stream: stream /= default_pointer
      external "C"
      alias "VELIBC_fgetpos"
      end

   fgets (buffer, stream: POINTER; n: INTEGER): POINTER is
         -- Reads at most one less than `n' characters from a stream.
      require
         valid_stream: stream /= default_pointer
         valid_buffer: buffer /= default_pointer
      external "C"
      alias "VELIBC_fgets"
      end
  
   fopen (pathname, mode: STRING): POINTER is
         -- Opens a stream
      require
         valid_mode: mode /= Void and then not mode.is_empty
         valid_pathname: pathname /=Void and then not pathname.is_empty
      do
         mem.collection_off;
         Result := ext_fopen(sh.to_c(pathname),sh.to_c(mode));
         mem.collection_on;
      end

   fdopen (fd: INTEGER; mode: STRING): POINTER is
         -- The fdopen function associates a stream with the existing
         -- file descriptor, fildes.  The mode of the stream (one of
         -- the values "r", "r+", "w", "w+", "a", "a+") must be
         -- compatible with the mode of the file descriptor.  The file
         -- position indicator of the new stream is set to that
         -- belonging to fildes, and the error and end-of-file
         -- indicators are cleared.  Modes "w" or "w+" do not cause
         -- truncation of the file.  The file descriptor is not
         -- dup'ed, and will be closed when the stream created by
         -- fdopen is closed.  The result of applying fdopen to a
         -- shared memory object is undefined.
      require
         valid_mode: mode /= Void and then not mode.is_empty
      do
         mem.collection_off;
         Result := ext_fdopen(fd,sh.to_c(mode));
         mem.collection_on;
      end

   fputc (c: CHARACTER; stream: POINTER): INTEGER is
         -- Writes a character to a stream
      require
         valid_stream: stream /= default_pointer
      external "C"
      alias "VELIBC_fputc"
      end

   fputs (s: STRING; stream: POINTER): INTEGER is
         -- Writes a stringn to a stream
      require
         valid_stream: stream /= default_pointer
         valid_string: s /= Void
      do
      end

   fread (ptr: POINTER; size, nmemb: INTEGER; stream: POINTER): INTEGER is
         -- The function fread reads nmemb elements of data, each size
         -- bytes long, from the stream pointed to by stream,  storing
         -- them at the location given by ptr.
      require
         valid_buffer: ptr /= default_pointer
         valid_stream: stream /= default_pointer
      external "C"
      alias "VELIBC_fread"
      end

   freopen (pathname, mode: STRING; stream: POINTER): POINTER is
         -- The freopen function opens the  file  whose  name  is  the
         -- string  pointed  to  by  path  and  associates  the stream
         -- pointed to by stream with it.  The original stream (if  it
         -- exists)  is  closed.  The mode argument is used just as in
         -- the fopen function.  The primary use of the freopen  func+
         -- tion is to change the file associated with a standard text
         -- stream (stderr, stdin, or stdout).
      require
         valid_pathname: pathname /= Void and then not pathname.is_empty
         valid_mode: mode /= Void and then not mode.is_empty
         valid_stream: stream /= default_pointer
      do
         mem.collection_off;
         Result := ext_freopen (sh.to_c(pathname),sh.to_c(mode),stream);
         mem.collection_on;
      end
   
   fseek (stream: POINTER; offset: INTEGER; whence: INTEGER): INTEGER is
         -- Sets file position
      require
         valid_stream: stream /= default_pointer
      external "C"
      alias "VELIBC_fseek"
      end

   fsetpos (stream: POINTER; pos: POINTER): INTEGER is
         -- Sets the file position for a stream
      require
         valid_stream: stream /= default_pointer
      external "C"
      alias "VELIBC_fsetpos"
      end

   ftell (stream: POINTER): INTEGER is
      -- Gets the position indicator for a stream
      require
         valid_stream: stream /= default_pointer
         -- file position should be less than LONG_MAX
      external "C"
      alias "VELIBC_ftell"
      end

   fwrite (ptr: POINTER; size, nmemb: INTEGER; stream: POINTER): INTEGER is
         -- The  function  fwrite  writes nmemb elements of data, each
         -- size bytes long, to  the  stream  pointed  to  by  stream,
         -- obtaining them from the location given by ptr.
      require
         valid_buffer: ptr /= default_pointer
         valid_stream: stream /= default_pointer
      external "C"
      alias "VELIBC_fwrite"
      end

   rewind (stream: POINTER) is
         -- Sets the file position to the beginning of the file
         -- the error indicator is also cleared
      require
         valid_stream: stream /= default_pointer
      external "C"
      alias "VELIBC_rewind"
      end
   
   setbuf (stream: POINTER; buf: POINTER) is
         -- Determines how a stream will be buffered
      require
         valid_stream: stream /= default_pointer
      external "C"
      alias "VELIBC_setbuf"
      end
   
   setvbuf (stream: POINTER; buf: POINTER; mode, size: INTEGER): INTEGER is
         -- Determines how a stream will be buffered
      require
         valid_stream: stream /= default_pointer
      external "C"
      alias "VELIBC_setvbuf"
      end
   
   ungetc (c: CHARACTER; stream: POINTER): INTEGER is
         -- Pushes a character back onto a stream
      require
         valid_stream: stream /= default_pointer
      external "C"
      alias "VELIBC_ungetc"
      end

   fileno (stream: POINTER): INTEGER is
         -- Maps a stream pointer to a file descriptor
      require
         valid_stream: stream /= default_pointer
      external "C"
      alias "VELIBC_fileno"
      end

   stdin: POINTER is
      external "C"
      alias "VELIBC_stdin"
      end

   stdout: POINTER is
      external "C"
      alias "VELIBC_stdout"
      end

   stderr: POINTER is
      external "C"
      alias "VELIBC_stderr"
      end

end
