indexing

   cluster:     "velibc";
   project:     "Visual Eiffel libc Library";
   date:        "$Date: 2005/02/15 16:22:10 $";
   revision:    "$Revision: 1.2 $";
   copyright:   "Copyright (C) 1996-2005 Object Tools Group";
   license:     "http://visual-eiffel.com/license";
   disclaimer:  "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

class VELIBC_STDLIB

inherit

   VELIBC_BASE

   VELIBC_STDLIB_EXTERNALS

feature

   calloc (num_of_elements, element_size: INTEGER): POINTER is
         -- Allocates and zeroes memory
      require
         valid_args: num_of_elements >= 0 and element_size >= 0
      external "C"
      alias "VELIBC_calloc"
      end

   free (p: POINTER) is
         -- Deallocates dynamic memory
      external "C"
      alias "VELIBC_free"
      end
   
   malloc (size: INTEGER): POINTER is
         -- Allocates dynamic memory
      require
         valid_size: size >= 0
      external "C"
      alias "VELIBC_malloc"
      end
   
   realloc(p: POINTER; size: INTEGER): POINTER is
         -- Changes the size of a memory object
      require
         valid_size: size >= 0
      external "C"
      alias "VELIBC_realloc"
      end

   abort is
         -- Causes abnormal process termination
      external "C"
      alias "VELIBC_abort"
      end

   exit (status: INTEGER) is

         -- Causes normal program termination and the value of status is
         -- returned to the parent.  All  functions registered with atexit()
         -- and on_exit() are called in the reverse order of  their
         -- registration, and all open streams are flushed and closed.
      external "C"
      alias "VELIBC_exit"
      end
   
   getenv (name: STRING): STRING is
         -- Gets the environment variable
      require
         valid_name: name /= Void and then not name.is_empty
      do
         mem.collection_off;
         Result := sh.from_c(ext_getenv(sh.to_c(name)));
         mem.collection_on;
      end

   setenv (name,value: STRING; overwrite: BOOLEAN): INTEGER is
         -- The setenv() function adds the variable name to the envi
         -- ronment -- with the value value, if name does not already
         -- exist.  If name does exist in the environment, then its
         -- value is changed to value if overwrite is TRUE; if
         -- overwrite is FALSE, then the value of name -- is not
         -- changed. Returns zero on success, or -1 if there was
         -- insufficient space in the environment.
      require
         valid_name: name /= Void and then not name.is_empty
         valid_value: value /= Void
      do
         mem.collection_off;
         Result := ext_setenv(sh.to_c(name),sh.to_c(value));
         mem.collection_on;
      end

   unsetenv (name: STRING) is
         -- The unsetenv() function deletes the variable name from the
         -- environment.
      require
         valid_name: name /= Void and then not name.is_empty
      do
         mem.collection_off;
         ext_unsetenv(sh.to_c(name));
         mem.collection_on;
      end

   system (string: STRING): INTEGER is
         -- system() executes a command specified in string by calling
         -- /bin/sh -c string, and returns after the command has  been
         -- completed.   During execution of the command, SIGCHLD will
         -- be blocked, and SIGINT and SIGQUIT will be ignored.
         -- The value returned is 127 if the execve() call for /bin/sh
         -- fails,  -1  if there was another error and the return code
         -- of the command otherwise.                                                            
         -- If the value of string is NULL, system()  returns  nonzero  
         -- if the shell is available, and zero if not.                 
      do
         mem.collection_off;
         Result := ext_system(sh.to_c(string));
         mem.collection_on;
      end

   canonicalize_file_name(name: STRING): STRING is
         -- Return a malloc'd string containing the canonical absolute name of the
         --  named file.  The last file name component need not exist, and may be a
         -- symlink to a nonexistent file.
      local
         res: POINTER;
      do
         mem.collection_off;
         res := ext_canonicalize_file_name(sh.to_c(name));
         mem.collection_on;
         Result := sh.from_c_with_free(res);
      end
end
