indexing

   cluster:     "velibc";
   project:     "Visual Eiffel libc Library";
   date:        "$Date: 2005/02/01 10:20:42 $";
   revision:    "$Revision: 1.1 $";
   copyright:   "Copyright (C) 1996-2005 Object Tools Group";
   license:     "http://visual-eiffel.com/license";
   disclaimer:  "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

class VELIBC_UNISTD

inherit

   VELIBC_BASE

   VELIBC_UNISTD_EXTERNALS

   VELIBC_UNISTD_CONSTANTS

feature   

   alarm (a_seconds: INTEGER) is
         -- Schedules an alarm
      external "C"
      alias "VELIBC_alarm"
      end
   
   getlogin: STRING is
         -- Gets user name
      do
         Result := sh.from_c(ext_getlogin);
      end
   
   pause: INTEGER is
         -- wait for signal
      external "C"
      alias "VELIBC_pause"
      end
   
   sleep (seconds: INTEGER): INTEGER is
         -- Delays process execution
      external "C"
      alias "VELIBC_sleep"
      end
   
   fork: INTEGER is
      -- creates a process
      external "C"
      alias "VELIBC_fork"
      end

   execv (path: STRING; argv: ARRAY[STRING]): INTEGER is
      require
         valid_path: path /= Void and then not path.is_empty
         valid_args: argv /= Void and then not argv.is_empty
      local
         ext_argv: VELIBC_EXTERNAL_STRING_ARRAY;
      do
         mem.collection_off;
         !!ext_argv.make(argv);
         Result := ext_execv(sh.to_c(path),ext_argv.to_external);
         mem.collection_on;
      end

   execvp (file: STRING; argv: ARRAY[STRING]): INTEGER is
      require
         valid_file: file /= Void and then not file.is_empty
         valid_args: argv /= Void and then not argv.is_empty
      local
         ext_argv: VELIBC_EXTERNAL_STRING_ARRAY;
      do
         mem.collection_off;
         !!ext_argv.make(argv);
         Result := ext_execvp(sh.to_c(file),ext_argv.to_external);
         mem.collection_on;
      end

   underscore_exit (status: INTEGER) is
         -- Terminates the calling process immediately. Any open file
         -- descriptors belonging to the process are closed;  any children
         -- of the process are inherited by process 1, init, and the
         -- process's parent is sent a SIGCHLD signal.
         -- status is returned to the parent process as the  process's
         -- exit  status,  and  can be collected using one of the wait
         -- family of calls.                                          
      external "C"
      alias "VELIBC__exit"
      end

   access (pathname: STRING; flags: BIT 32): INTEGER is
         -- check user's permissions for a file
      require
         valid_pathname: pathname /= Void and then not pathname.is_empty
      do
         mem.collection_off;
         Result := ext_access(sh.to_c(pathname),flags);
         mem.collection_on;
      end

   chdir (pathname: STRING): INTEGER is
         -- Changes the current working directory
      require
         valid_pathname: pathname /= Void and then not pathname.is_empty
      do
         mem.collection_off;
         Result := ext_chdir(sh.to_c(pathname));
         mem.collection_on;
      end

   chown (pathname: STRING; uid: INTEGER; guid: INTEGER): INTEGER is
         -- change ownership of a file
      require
         valid_pathname: pathname /= Void and then not pathname.is_empty
      do
         mem.collection_off;
         Result := ext_chown(sh.to_c(pathname),uid,guid);
         mem.collection_on;
      end

   getcwd: STRING is
         -- Gets current working directory
      local
         buf: STRING;
      do
         !!buf.make_filled('%U',255);
         mem.collection_off;
         Result := sh.from_c(ext_getcwd(sh.to_c(buf),255));
         mem.collection_on;
      end

   link (oldpath: STRING; newpath: STRING): INTEGER is
         -- make a new name for a file
      require
         valid_oldpath: oldpath /= Void and then not oldpath.is_empty
         valid_newpath: newpath /= Void and then not newpath.is_empty
      do
         mem.collection_off;
         Result := ext_link(sh.to_c(oldpath),sh.to_c(newpath));
         mem.collection_on;
      end

   unlink (pathname: STRING): INTEGER is
         --  delete a name and possibly the file it refers to
      require
         valid_pathname: pathname /= Void and then not pathname.is_empty
      do
         mem.collection_off;
         Result := ext_unlink(sh.to_c(pathname));
         mem.collection_on;
      end

   rmdir (pathname: STRING): INTEGER is
         --  delete a directory
      require
         valid_pathname: pathname /= Void and then not pathname.is_empty
      do
         mem.collection_off;
         Result := ext_rmdir(sh.to_c(pathname));
         mem.collection_on;
      end

   close (fd: INTEGER): INTEGER is
         -- close a file descriptor
      external "C"
      alias "VELIBC_close"
      end

   dup (fd: INTEGER): INTEGER is
         -- Duplicates an open file descriptor
      external "C"
      alias "VELIBC_dup"
      end

   dup2 (fd1, fd2: INTEGER): INTEGER is
         -- Duplicates an open file descriptor
      external "C"
      alias "VELIBC_dup2"
      end
   
   fdatasync (fd: INTEGER): INTEGER is
         -- Synchronize the data of a file
      external "C"
      alias "VELIBC_fdatasync"
      end   

   fsync (fd: INTEGER): INTEGER is
         -- Synchronize the state of a file
      external "C"
      alias "VELIBC_fsync"
      end   
   
   posix_fpathconf (fildes: INTEGER; name: INTEGER): INTEGER is
         -- Gets configuration variable for an open file
      external "C"
      alias "VELIBC_fpathconf"
      end

   isatty (fd: INTEGER): BOOLEAN is
         -- Determines if a file descriptor is associated with a terminal
      external "C"
      alias "VELIBC_isatty"
      end

   lseek (fd: INTEGER; offset: INTEGER; whence: INTEGER): INTEGER is
         -- reposition read/write file offset
      external "C"
      alias "VELIBC_lseek"
      end

   pipe(fds: ARRAY[INTEGER]): INTEGER is
         -- Creates an interprocess channel
      require
         valid_array: fds /= Void and then fds.count = 2
      do
         mem.collection_off;
         Result := ext_pipe(fds.to_c);
         mem.collection_on;
      end

   read (fd: INTEGER; buf: POINTER; count: INTEGER): INTEGER is
         -- read from a file descriptor
      external "C"
      alias "VELIBC_read"
      end

   write (fd: INTEGER; buf: POINTER; count: INTEGER): INTEGER is
         -- write to a file descriptor
      external "C"
      alias "VELIBC_write"
      end
   
   ttyname (fd: INTEGER): STRING is
         -- Determines a terminal pathname
      do
         Result := sh.from_c(ext_ttyname(fd));
      end

   getpid: INTEGER is
         -- get process  ID  of the current process
      external "C"
      alias "VELIBC_getpid"
      end

   getppid: INTEGER is
         --  get the process ID of the parent of the current process
      external "C"
      alias "VELIBC_getppid"
      end

   getpgrp: INTEGER is
         -- get the current process group
      external "C"
      alias "VELIBC_getpgrp"
      end

   getuid: INTEGER is
         -- Get the real user ID of the current process.
      external "C"
      alias "VELIBC_getuid"
      end
   
   geteuid: INTEGER is
         -- Get the effective user ID of the current process.
      external "C"
      alias "VELIBC_geteuid"
      end

   getgid: INTEGER is
         -- Get the real group ID of the current process.
      external "C"
      alias "VELIBC_getgid"
      end

   getegid: INTEGER is
         -- Get the efective group ID of the current process.
      external "C"
      alias "VELIBC_getegid"
      end
         
   setsid is
         -- creates a session and sets the process group ID
      external "C"
      alias "VELIBC_setsid"
      end

   setpgid (pid: INTEGER; pgid: INTEGER): INTEGER is
         -- Sets the process group ID of the process specified by pid 
         -- to pgid. If pid is zero, the process ID of the current process
         -- is used. If pgid is zero, the process ID of the process 
         -- specified by pid is used.
      external "C"
      alias "VELIBC_setpgid"
      end

   setuid (uid: INTEGER): INTEGER is
      external "C"
      alias "VELIBC_setuid"
      end

   seteuid (euid: INTEGER): INTEGER is
      external "C"
      alias "VELIBC_seteuid"
      end

   setgid (gid: INTEGER): INTEGER is
      external "C"
      alias "VELIBC_setgid"
      end

   setegid (egid: INTEGER): INTEGER is
      external "C"
      alias "VELIBC_setegid"
      end

   readlink (path: STRING): STRING is
         -- read value of a symbolic link
      require
         path /= Void and then not path.is_empty
      local
         res: POINTER;
      do
         mem.collection_off;
         res := ext_readlink(sh.to_c(path));
         mem.collection_on;
         Result := sh.from_c_with_free(res);
      end
end
