/*
   cluster:     "timedate";
   project:     "The Universal Time & Date UTDC";
   date:        "$Date: 2005/02/01 10:21:56 $";
   revision:    "$Revision: 1.1 $";
   copyright:   "Copyright (C) 1996-2005 Object Tools Group";
   license:     "http://visual-eiffel.com/license";
   disclaimer:  "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";
*/

/**********************************************************************
*     Features declaration for class SYSTEM_TIME
*
*     Functions:
*           TIME_DATE_now
*           TIME_DATE_sysclock
*           TIME_DATE_dst
*           TIME_DATE_offset
**********************************************************************/

#ifdef _OS_LINUX_
#include <time.h>
#include <sys/time.h>
#include <unistd.h>
#else
#include <windows.h>
#endif
#include "sys_time.h"

void
TIME_DATE_now (EIF_BOOLEAN local, /* specifies whether to return local time or UTC (GMT) */
               EIF_INTEGER *year,
               EIF_INTEGER *month,
               EIF_INTEGER *day,
	       EIF_INTEGER *hour,
	       EIF_INTEGER *minute,
	       EIF_INTEGER *second,
	       EIF_INTEGER *millisecond)
{
#ifdef _OS_LINUX_

    time_t local_time;
    struct tm *local_tm;
    struct timeval tv;

    gettimeofday (&tv, NULL);
    local_time = tv.tv_sec;
    if (local) local_tm = localtime (&local_time);
    else       local_tm = gmtime (&local_time);

    *year        = (EIF_INTEGER) local_tm->tm_year + 1900;
    *month       = (EIF_INTEGER) local_tm->tm_mon + 1;
    *day         = (EIF_INTEGER) local_tm->tm_mday;
    *hour        = (EIF_INTEGER) local_tm->tm_hour;
    *minute      = (EIF_INTEGER) local_tm->tm_min;
    *second      = (EIF_INTEGER) local_tm->tm_sec;
    *millisecond = (EIF_INTEGER) tv.tv_usec / 1000;

#else

    SYSTEMTIME st;

    if (local) GetLocalTime (&st);
    else       GetSystemTime (&st);

    *year        = (EIF_INTEGER) st.wYear;
    *month       = (EIF_INTEGER) st.wMonth;
    *day         = (EIF_INTEGER) st.wDay;
    *hour        = (EIF_INTEGER) st.wHour;
    *minute      = (EIF_INTEGER) st.wMinute;
    *second      = (EIF_INTEGER) st.wSecond;
    *millisecond = (EIF_INTEGER) st.wMilliseconds;

#endif
}

EIF_INTEGER
TIME_DATE_sysclock (void)
{
#ifdef _OS_LINUX_
  return (EIF_INTEGER) (clock () / (double) CLOCKS_PER_SEC * 1000);
#else
  return (EIF_INTEGER) (GetTickCount ());
#endif
}

EIF_BOOLEAN
TIME_DATE_dst (void)
{
#ifdef _OS_LINUX_

    time_t local_time;
    struct tm *local_tm;
    struct timeval tv;

    gettimeofday (&tv, NULL);
    local_time = tv.tv_sec;
    local_tm = localtime (&local_time);

    return local_tm -> tm_isdst;

#else

	TIME_ZONE_INFORMATION tz;

	return GetTimeZoneInformation (&tz) == TIME_ZONE_ID_DAYLIGHT;

#endif
}


EIF_INTEGER
TIME_DATE_offset (void)
{
#ifdef _OS_LINUX_

    time_t local_time;
    struct tm *local_tm;
    struct timeval tv;

    gettimeofday (&tv, NULL);
    local_time = tv.tv_sec;
    local_tm = localtime (&local_time);

    return local_tm->tm_gmtoff;

#else

	TIME_ZONE_INFORMATION tz;
	DWORD tz_id = GetTimeZoneInformation (&tz);

	if ((tz_id == TIME_ZONE_ID_STANDARD) ||
	    (tz_id == TIME_ZONE_ID_DAYLIGHT))
	{
		return - tz.Bias * 60;
	}

	return 0;

#endif
}
