indexing

   title:       "DURATION";
   cluster:     "timedate";
   project:     "The Universal Time & Date UTDC";
   date:        "$Date: 2005/02/01 10:21:56 $";
   revision:    "$Revision: 1.1 $";
   copyright:   "Copyright (C) 1996-2005 Object Tools Group";
   license:     "http://visual-eiffel.com/license";
   disclaimer:  "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

class DURATION

inherit

   SHORT_DURATION
      undefine
         infix "<",
         infix "+",
         infix "-",
         clear,
         plus,
         minus,
         ratio,
         divide_by_duration,
         multiply_by,
         divide_by
      redefine
         as_hours,
         as_minutes,
         as_seconds
      end

   LONG_DURATION
      undefine
         infix "<",
         infix "+",
         infix "-",
         clear,
         plus,
         minus,
         ratio,
         divide_by_duration,
         multiply_by,
         divide_by
      redefine
         as_years,
         as_months,
         as_days
      end

creation

   clear,
   set,
   set_from_compressed

feature

   set (yr, mh, dy, hr, me: INTEGER; sd: DOUBLE) is
      require
         date_time_valid: sd <  tt.seconds_in_minute and then
                           sd >  - tt.seconds_in_minute and then
                           me <  tt.minutes_in_hour and then
                           me >  - tt.minutes_in_hour and then
                           hr <  tt.hours_in_day and then
                           hr >  - tt.hours_in_day and then
                           dy <= tt.days_in_month_d and then
                           dy >= - tt.days_in_month_d and then
                           mh <= tt.months_in_year and then
                           mh >= - tt.months_in_year
      do
         set_date (yr, mh, dy)
         set_time (hr, me, sd)
      end

   clear is
      do
         milliseconds  := 0
         absolute_days := 0
      end

   as_seconds: DOUBLE is
      do
         Result := as_seconds_years (1)
      end

   as_minutes: DOUBLE is
      do
         Result := as_seconds_years (tt.seconds_in_minute)
      end

   as_hours: DOUBLE is
      do
         Result := as_seconds_years (tt.seconds_in_hour)
      end

   as_days: DOUBLE is
      do
         Result := as_seconds_years (tt.seconds_in_day)
      end

   as_months: DOUBLE is
      do
         Result := as_seconds_years (tt.seconds_in_month)
      end

   as_years: DOUBLE is
      do
         Result := as_seconds_years (tt.seconds_in_year)
      end

   infix "<" (ts: like Current): BOOLEAN is
      do
         Result :=
            (absolute_days < ts.absolute_days) or else
            (absolute_days = ts.absolute_days and then
              milliseconds  < ts.milliseconds)
      end

   infix "-",
   minus (d: like Current): like Current is
      do
         !!Result.set_from_compressed (
                  absolute_days - d.absolute_days,
                  milliseconds - d.milliseconds)
      end

   infix "+",
   plus (d: like Current): like Current is
      do
         !!Result.set_from_compressed (
                  absolute_days + d.absolute_days,
                  milliseconds + d.milliseconds)
      end

   multiply_by (n: INTEGER): like Current is
      do
         !!Result.set_from_compressed (
                  absolute_days * n,
                  milliseconds * n)
      end

   divide_by (n: INTEGER): like Current is
      require else
         non_zero_divider: n /= 0
      local
         dy, ms: INTEGER
      do
         dy := absolute_days // n
         ms := ((milliseconds + (absolute_days \\ n) *
                 tt.milliseconds_in_day) / n).rounded
         !!Result.set_from_compressed (dy, ms)
      end

   ratio,
   divide_by_duration (d: like Current): DOUBLE is
      require else
         non_zero_param: d.as_seconds /= 0
      do
         Result := as_seconds / d.as_seconds
      end

   set_from_compressed (day, millisec: INTEGER) is
      local
         ms, dy: INTEGER
      do
         ms := millisec
         dy := day
         -- normalize
         if ms >= tt.milliseconds_in_day or ms <= - tt.milliseconds_in_day then
            dy := dy + ms // tt.milliseconds_in_day
            ms := ms \\ tt.milliseconds_in_day
         end
         if dy > 0 and ms < 0 then
            dy := dy - 1
            ms := tt.milliseconds_in_day + ms
         elseif dy < 0 and ms > 0 then
            dy := dy + 1
            ms := ms - tt.milliseconds_in_day
         end

         absolute_days := dy
         milliseconds := ms
      end

feature {NONE}

   as_seconds_years (constant: INTEGER): DOUBLE is
      local
         ms, abs_day: DOUBLE
      do
         ms := milliseconds
         abs_day := absolute_days
         Result :=
            ms / (tt.milliseconds_in_second * constant) +
            (abs_day / constant) * tt.seconds_in_day
      end

end