indexing

   title:       "Long Duration - days or more";
   cluster:     "timedate";
   project:     "The Universal Time & Date UTDC";
   date:        "$Date: 2005/02/01 10:21:56 $";
   revision:    "$Revision: 1.1 $";
   copyright:   "Copyright (C) 1996-2005 Object Tools Group";
   license:     "http://visual-eiffel.com/license";
   disclaimer:  "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

class LONG_DURATION

inherit

   SIMPLE_DURATION

creation

   clear,
   set_days,
   set_date

feature

   absolute_days: INTEGER

   set_date (yr, mh, dy: INTEGER) is
      require
         date_valid: dy <= tt.days_in_month_d and then
                     dy >= - tt.days_in_month_d and then
                     mh <= tt.months_in_year and then
                     mh >= - tt.months_in_year
      do
         absolute_days := yr * tt.days_in_year_d +
                           mh * tt.days_in_month_d + dy
      end

   set_days (days: INTEGER) is
      do
         absolute_days := days
      end

   clear is
      do
         absolute_days := 0
      end

   year_part: INTEGER is
      do
         Result := absolute_days // tt.days_in_year_d
      end

   month_part: INTEGER is
      do
         Result :=
            (absolute_days - tt.days_in_year_d * year_part) //
            tt.days_in_month_d
      end

   day_part: INTEGER is
      do
         Result :=
            absolute_days - tt.days_in_year_d * year_part -
            month_part * tt.days_in_month_d
      end

   as_days: DOUBLE is
      do
         Result := absolute_days
      end

   as_months: DOUBLE is
      local
         abs_day: DOUBLE
      do
         abs_day := absolute_days
         Result  := abs_day / tt.days_in_month_d
      end

   as_years: DOUBLE is
      local
         abs_day: DOUBLE
      do
         abs_day := absolute_days
         Result := abs_day / tt.days_in_year_d
      end

   whole_days: INTEGER is
      do
         Result := as_days.floor
      end

   whole_months: INTEGER is
      do
         Result := as_months.floor
      end

   whole_years: INTEGER is
      do
         Result := as_years.floor
      end

   infix "<" (ts: like Current): BOOLEAN is
      do
         Result := absolute_days < ts.absolute_days
      end

   infix "+",
   plus (d: like Current): like Current is
      local
         sum: INTEGER
      do
         sum := absolute_days + d.absolute_days
         !!Result.set_days (sum)
      end

   infix "-",
   minus (d: like Current): like Current is
      local
         res: INTEGER
      do
         res := absolute_days - d.absolute_days
         !!Result.set_days (res)
      end

   multiply_by (n: INTEGER): like Current is
      do
         !!Result.set_days (absolute_days * n)
      end

   divide_by (n: INTEGER): like Current is
      require else
         non_zero_divider: n /= 0
      do
         !!Result.set_days (absolute_days // n)
      end

   ratio,
   divide_by_duration (d: like Current): DOUBLE is
      require else
         non_zero_param: d.absolute_days /= 0
      local
         abs_day: DOUBLE
      do
         abs_day := absolute_days
         Result := abs_day / d.absolute_days
      end

end