indexing

   title:       "Short_Duration";
   cluster:     "timedate";
   project:     "The Universal Time & Date UTDC";
   date:        "$Date: 2005/02/01 10:21:56 $";
   revision:    "$Revision: 1.1 $";
   copyright:   "Copyright (C) 1996-2005 Object Tools Group";
   license:     "http://visual-eiffel.com/license";
   disclaimer:  "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

class SHORT_DURATION

inherit

   SIMPLE_DURATION

creation

   clear,
   set_time,
   set_seconds,
   set_from_milliseconds

feature

   milliseconds: INTEGER

feature -- Commands

   set_time (hr, me: INTEGER; sd: DOUBLE) is
      require
         time_valid: sd < tt.seconds_in_minute and then
                     sd > - tt.seconds_in_minute and then
                     me < tt.minutes_in_hour and then
                     me > - tt.minutes_in_hour
      do
         milliseconds := ((hr * tt.seconds_in_hour + me * tt.seconds_in_minute + sd) *
                           tt.milliseconds_in_second).rounded
      end

   set_seconds (seconds: DOUBLE) is
      do
         milliseconds := (seconds * tt.milliseconds_in_second).rounded
      end

   set_from_milliseconds (n: INTEGER) is
      do
         milliseconds := n
      end

   clear is
      do
         milliseconds := 0
      end

feature -- Queries

   hour_part: INTEGER is
      do
         Result :=
         milliseconds // tt.milliseconds_in_second // tt.seconds_in_hour
      end

   minute_part: INTEGER is
      do
         Result :=
         (milliseconds //
         tt.milliseconds_in_second - tt.seconds_in_hour * hour_part) //
         tt.seconds_in_minute
      end

   second_part: DOUBLE is
      do
         Result := div_1000 (milliseconds) - tt.seconds_in_hour *
            hour_part - tt.seconds_in_minute * minute_part
      end

   as_seconds: DOUBLE is
      do
         Result := div_1000 (milliseconds)
      end

   as_minutes: DOUBLE is
      do
         Result := as_seconds / tt.seconds_in_minute
      end

   as_hours: DOUBLE is
      do
         Result := as_seconds / tt.seconds_in_hour
      end

   whole_seconds: INTEGER is
      do
         Result := as_seconds.floor
      end

   whole_minutes: INTEGER is
      do
         Result := as_minutes.floor
      end

   whole_hours: INTEGER is
      do
         Result := as_hours.floor
      end

   infix "<" (ts: like Current): BOOLEAN is
      do
         Result := milliseconds < ts.milliseconds
      end

   infix "+",
   plus (d: like Current): like Current is
      do
         !!Result.set_from_milliseconds (milliseconds + d.milliseconds)
      end

   infix "-",
   minus (d: like Current): like Current is
      do
         !!Result.set_from_milliseconds (milliseconds - d.milliseconds)
      end

   multiply_by (n: INTEGER): like Current is
      do
         !!Result.set_from_milliseconds (milliseconds * n)
      end

   divide_by (n: INTEGER): like Current is
      require else
         non_zero_divider: n /= 0
      do
         !!Result.set_from_milliseconds (milliseconds // n)
      end

   ratio,
   divide_by_duration (d: like Current): DOUBLE is
      require else
         non_zero_param: d.milliseconds /= 0
      local
         ms: DOUBLE
      do
         ms := milliseconds
         Result := ms / d.milliseconds
      end

feature {NONE} -- Implementation

   div_1000 (i: INTEGER): DOUBLE is
      local
         ii: INTEGER
         s: STRING
      do
         s := i.out
         if s.count < 4 and s.item (1) = '-' then
            from
               ii := s.count
            until
               ii = 4
            loop
               s.insert ("0", 2)
               ii := ii + 1
            end
         elseif s.count < 3 then
            from
               ii := s.count
            until
               ii = 3
            loop
               s.precede ('0')
               ii := ii + 1
            end
         end
         s.insert (".", s.count - 2)
         Result := s.to_real
      end

end