indexing

   title:       "Time";
   cluster:     "timedate";
   project:     "The Universal Time & Date UTDC";
   date:        "$Date: 2005/02/01 10:21:56 $";
   revision:    "$Revision: 1.1 $";
   copyright:   "Copyright (C) 1996-2005 Object Tools Group";
   license:     "http://visual-eiffel.com/license";
   disclaimer:  "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

class TIME

inherit

   SIMPLE_TIMESTAMP
      redefine
         out
      end

creation

   clear,
   set_time,
   set_to_now,
   set_local_time,
   set_from_milliseconds

feature

   milliseconds: INTEGER is
      do
         Result := (((hour * 60 + minute) * 60) + second) * 1000 + millisecond
      end

   set_time (hr, me: INTEGER; sd: DOUBLE) is
      require
         time_valid: tt.valid_time (hr, me, sd)
      do
         set_from_milliseconds (
            ((hr * tt.seconds_in_hour + me * tt.seconds_in_minute + sd) *
            tt.milliseconds_in_second).rounded)
      end

   set_to_now is
      local
         ts: TIMESTAMP
      do
         !!ts.set_to_now
         set_from_milliseconds (ts.milliseconds)
      end

   set_local_time is
      local
         ts: TIMESTAMP
      do
         !!ts.set_local_time
         set_from_milliseconds (ts.milliseconds)
      end

   set_from_milliseconds (m: INTEGER) is
      require
         milliseconds_valid: m >= 0 and then
            m < tt.seconds_in_day * tt.milliseconds_in_second
      do
         second := m // 1000
         millisecond := m - second * 1000
         minute := second // 60
         second := second - minute * 60
         hour := minute // 60
         minute := minute - hour * 60
      end

   clear is
      do
         set_from_milliseconds (0)
      end

   infix "<" (ts: like Current): BOOLEAN is
      do
         Result := milliseconds < ts.milliseconds
      end

feature

   infix "+",
   plus (d: SHORT_DURATION): TIMESTAMP is
      do
         !!Result.set_from_compressed
            (0, milliseconds + d.milliseconds)
      end

   infix "-",
   minus (d: SHORT_DURATION): TIMESTAMP is
      do
         !!Result.set_from_compressed
            (0, milliseconds - d.milliseconds)
      end

   difference (t: like Current): SHORT_DURATION is
      do
         !!Result.set_from_milliseconds (milliseconds - t.milliseconds)
      end

feature -- Status

   hour: INTEGER

   minute: INTEGER

   second: INTEGER

   millisecond: INTEGER

feature -- Status: compatibility features

   hour_part: INTEGER is
      obsolete "This feature will be removed soon: use hour instead"
         -- Hour
      do
         Result := hour
      end

   minute_part: INTEGER is
      obsolete "This feature will be removed soon: use minute instead"
         -- Minute
      do
         Result := minute
      end

   second_part: INTEGER is
      obsolete "This feature will be removed soon: use second instead"
         -- Second
      do
         Result := second
      end

   millisecond_part: INTEGER is
      obsolete "This feature will be removed soon: use millisecond instead"
         -- Millisecond
      do
         Result := millisecond
      end

feature -- Output
   out: STRING is
         -- New string containing printable represantation
         -- Format is HH:MM:SS[.F]
      local
         s: STRING
      do
         Result := "00:00:00"
         s := hour.out
         Result.replace_substring (s, 3 - s.count, 2)
         s := minute.out
         Result.replace_substring (s, 6 - s.count, 5)
         s := second.out
         Result.replace_substring (s, 9 - s.count, 8)
         if millisecond /= 0 then
            Result.append_string (".000")
            s := millisecond.out
            Result.replace_substring (s, 13 - s.count, 12)
         end
      end

invariant

   time_big_enough:  milliseconds >= 0
   time_small_enough:  milliseconds <  tt.milliseconds_in_day

end