indexing

   title:       "TIMETOOLS";
   cluster:     "timedate";
   project:     "The Universal Time & Date UTDC";
   date:        "$Date: 2005/02/01 10:21:56 $";
   revision:    "$Revision: 1.1 $";
   copyright:   "Copyright (C) 1996-2005 Object Tools Group";
   license:     "http://visual-eiffel.com/license";
   disclaimer:  "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

class TIME_TOOLS

feature -- Queries

   -- Days
   monday      : INTEGER is 1;
   tuesday     : INTEGER is 2;
   wednesday   : INTEGER is 3;
   thursday    : INTEGER is 4;
   friday      : INTEGER is 5;
   saturday    : INTEGER is 6;
   sunday      : INTEGER is 7;

   -- months
   january     : INTEGER is  1;
   february    : INTEGER is  2;
   march       : INTEGER is  3;
   april       : INTEGER is  4;
   may         : INTEGER is  5;
   june        : INTEGER is  6;
   july        : INTEGER is  7;
   august      : INTEGER is  8;
   september   : INTEGER is  9;
   october     : INTEGER is 10;
   november    : INTEGER is 11;
   december    : INTEGER is 12;

   -- constants
   months_in_year         : INTEGER is           12;
   days_in_week           : INTEGER is            7;
   days_in_month_d        : INTEGER is           30;
   days_in_year           : INTEGER is          365;
   days_in_year_d         : INTEGER is          360;
   hours_in_day           : INTEGER is           24;
   minutes_in_hour        : INTEGER is           60;
   minutes_in_day         : INTEGER is        1_440;
   seconds_in_minute      : INTEGER is           60;
   seconds_in_hour        : INTEGER is        3_600;
   seconds_in_day         : INTEGER is       86_400;
   seconds_in_month       : INTEGER is    2_592_000;
   seconds_in_year        : INTEGER is   31_104_000;
   milliseconds_in_second : INTEGER is        1_000;
   seconds_in_millisecond : DOUBLE  is        0.001;
   milliseconds_in_day    : INTEGER is   86_400_000;

feature -- Queries

   valid_time (hour, minute: INTEGER; second: DOUBLE) : BOOLEAN is
         -- Is the given time valid?
      do
         Result := second >= 0 and then second < seconds_in_minute and then
                   minute >= 0 and then minute < minutes_in_hour   and then
                   hour   >= 0 and then hour   < hours_in_day
      end

   valid_date (year, month, day: INTEGER) : BOOLEAN is
         -- Is the given date valid?
      do
         if year >= -4713 then
            if year /= 1582 or else
               month /= 10 or else
               day <= 4 or else
               day >= 15
            then
               -- The date is not in the range 05.Oct.1582 - 14.Oct.1582
               Result := valid_year_date (year, month, day, is_leap_year (year))
            end
         end
      end

   valid_julian_date (year, month, day: INTEGER) : BOOLEAN is
         -- Is the given date valid for Julian calendar?
      do
         if year >= -4713 then
            Result := valid_year_date (year, month, day, is_julian_leap_year (year))
         end
      end

   valid_gregorian_date (year, month, day: INTEGER) : BOOLEAN is
         -- Is the given date valid for Gregorian calendar?
      do
         if year > -4714 or else
            year = -4714 and then (month > 11 or else month = 11 and then day >= 24)
         then
            Result := valid_year_date (year, month, day, is_gregorian_leap_year (year))
         end
      end

   julian_date_days (year, month, day: INTEGER): INTEGER is
         -- Julian days calculated for Julian calendar
      require
         valid_date: valid_julian_date (year, month, day)
      local
         years: INTEGER
         month_: INTEGER
      do
         -- Make year always a positive number
         if year < 0 then
            years := year + 4801
         else
            years := year + 4800
         end
         -- Adjust the start of the year
         if month > 2 then
            month_ := month - 3
         else
            month_ := month + 9
            years := years - 1
         end
         Result := years * Days_per_4_years // 4 +
                   (month_ * Days_per_5_months + 2) // 5 +
                   day -
                   Julian_absolute_days_offset
      end

   gregorian_date_days (year, month, day: INTEGER): INTEGER is
         -- Julian days calculated for Gregorian calendar
      require
         valid_date: valid_gregorian_date (year, month, day)
      local
         years: INTEGER
         month_: INTEGER
      do
         -- Make year always a positive number
         if year < 0 then
            years := year + 4801
         else
            years := year + 4800
         end
         -- Adjust the start of the year
         if month > 2 then
            month_ := month - 3
         else
            month_ := month + 9
            years := years - 1
         end
         Result := ((years // 100) * Days_per_400_years) // 4 +
                   ((years \\ 100) * Days_per_4_years) // 4 +
                   (month_ * Days_per_5_months + 2) // 5 +
                   day -
                   Gregorian_absolute_days_offset
      end

   is_leap_year (year: INTEGER): BOOLEAN is
         -- Is the year leap?
      do
         if year < 1582 then
            Result := is_julian_leap_year (year)
         else
            Result := is_gregorian_leap_year (year)
         end
      end

   is_julian_leap_year (year: INTEGER): BOOLEAN is
         -- Is the year leap accordingly to Julian calendar?
      local
         years: INTEGER
      do
         -- Make year always a positive number
         if year < 0 then
            years := year + 4801
         else
            years := year + 4800
         end
         Result := years \\ 4 = 0
      end

   is_gregorian_leap_year (year: INTEGER): BOOLEAN is
         -- Is the year leap accordingly to Julian calendar?
      local
         years: INTEGER
      do
         -- Make year always a positive number
         if year < 0 then
            years := year + 4801
         else
            years := year + 4800
         end
         if years \\ 4 = 0 and then
            (years \\ 100 /= 0 or else years \\ 400 = 0)
         then
            -- Year divisible by 4 but not by 100 is leap
            -- Year divisible by 400 is leap
            Result := True
         end
      end

   day_name (n: INTEGER) : STRING is
      require
         non_valid_day_name: n >= 1 and then n <= days_in_week
      do
         Result := days.item (n)
      end

   month_name (n: INTEGER) : STRING is
      require
         non_valid_month_name: n >= 1 and then n <= months_in_year
      do
         Result := months.item (n);
      end

   days_in_month (n: INTEGER) : INTEGER is
      require
         non_valid_day_name: n >= 1 and then n <= months_in_year
      do
         Result := day_in_months.item (n);
      end

   days : ARRAY [STRING] is
      once
         Result := <<
            "MONDAY",
            "TUESDAY",
            "WEDNESDAY",
            "THURSDAY",
            "FRIDAY",
            "SATURDAY",
            "SUNDAY"
         >>;
      end

   months : ARRAY [STRING] is
      once
         Result := <<
            "JANUARY",
            "FEBRUARY",
            "MARCH",
            "APRIL",
            "MAY",
            "JUNE",
            "JULY",
            "AUGUST",
            "SEPTEMBER",
            "OCTOBER",
            "NOVEMBER",
            "DECEMBER"
         >>
      end

   day_in_months : ARRAY [INTEGER] is
      once
         Result := <<
            31,
            28,
            31,
            30,
            31,
            30,
            31,
            31,
            30,
            31,
            30,
            31
         >>;
      end

   days_in_months : ARRAY [INTEGER] is
      once
         Result := <<
            0,
            31,
            59,
            90,
            120,
            151,
            181,
            212,
            243,
            273,
            304,
            334,
            365
         >>
      end

feature -- System

   sysclock: INTEGER is
         -- High resolution system clock
         -- Result is in 1/1000 seconds.
         -- The value is wraps in about 49.7 days
         -- and can be used only to measure
         -- durations
      external "C"
         alias "TIME_DATE_sysclock"
      end


feature {NONE} -- Implementation

   valid_year_date (year, month, day: INTEGER; leap: BOOLEAN): BOOLEAN is
         -- Is date valid for this common/leap year?
      local
         max_days: INTEGER
      do
         if day   >  0              and then
            month >  0              and then
            month <= months_in_year
         then
            max_days := day_in_months.item (month)
            if month = 2 and then leap then
               max_days := max_days + 1
            end
            Result := day <= max_days
         end
      end

feature {DATE} -- Implementation: constants

   Days_per_400_years: INTEGER is 146097
   Days_per_4_years: INTEGER is 1461
   Days_per_5_months: INTEGER is 153
   Julian_absolute_days_offset: INTEGER is 32083
   Gregorian_absolute_days_offset: INTEGER is 32045

end