indexing

   title:       "TIMESTAMP";
   cluster:     "timedate";
   project:     "The Universal Time & Date UTDC";
   date:        "$Date: 2005/02/01 10:21:56 $";
   revision:    "$Revision: 1.1 $";
   copyright:   "Copyright (C) 1996-2005 Object Tools Group";
   license:     "http://visual-eiffel.com/license";
   disclaimer:  "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

class TIMESTAMP

inherit

   DATE
      undefine
         set_to_now,
         set_local_time,
         difference,
         infix "<",
         infix "+",
         plus,
         infix "-",
         minus
      redefine
         clear,
         out
      end

   TIME
      undefine
         set_to_now,
         set_local_time,
         difference,
         infix "<",
         infix "+",
         plus,
         infix "-",
         minus
      redefine
         clear,
         out
      end

creation

   clear,
   set,
   set_from_compressed,
   set_to_now,
   set_local_time

feature

   set (yr, mh, dy, hr, me: INTEGER; sd: DOUBLE) is
      require
         date_time_valid: tt.valid_date (yr, mh, dy) and then
                           tt.valid_time (hr, me, sd)
      do
         set_date (yr, mh, dy)
         set_time (hr, me, sd)
      end

   set_from_compressed (abs_day, millsec: INTEGER) is
         -- if necessary with normalisation
      local
         ms, dy: INTEGER
      do
         ms := millsec
         dy := abs_day
         if ms >= tt.milliseconds_in_day then
            dy := dy + ms // tt.milliseconds_in_day
            ms := ms \\ tt.milliseconds_in_day
         elseif ms < 0 then
            dy := dy + ms // tt.milliseconds_in_day - 1
            ms := ms \\ tt.milliseconds_in_day + 1
         end
         set_from_absolute_days (dy)
         set_from_milliseconds (ms)
      end

   set_to_now is
      do
         TIME_DATE_now (False, $year, $month, $day, $hour, $minute, $second, $millisecond)
      end

   set_local_time is
      do
         TIME_DATE_now (True, $year, $month, $day, $hour, $minute, $second, $millisecond)
      end

   clear is
      do
         Precursor {TIME}
         Precursor {DATE}
      end

   infix "<" (ts: like Current): BOOLEAN is
      do
         Result :=
            (absolute_days < ts.absolute_days) or else
            (absolute_days = ts.absolute_days and then
              milliseconds  < ts.milliseconds)
      end

   infix "+",
   plus (d: DURATION): like Current is
      require else
         date_time_valid: (milliseconds + d.milliseconds) // tt.milliseconds_in_day +
                           absolute_days + d.absolute_days  > 0
      local
         abs_day, millsec: INTEGER
      do
         abs_day := absolute_days + d.absolute_days
         millsec := milliseconds + d.milliseconds

         -- adjust carry
         if millsec < 0 then
            millsec := millsec + tt.milliseconds_in_day
            abs_day := abs_day - 1
         end

         if millsec >= tt.milliseconds_in_day then
            millsec := millsec - tt.milliseconds_in_day
            abs_day := abs_day + 1
         end
         !!Result.set_from_compressed (abs_day, millsec)
      end

   infix "-",
   minus (d: DURATION): like Current is
      require else
         date_time_valid: (milliseconds - d.milliseconds) //
            tt.milliseconds_in_day + absolute_days - d.absolute_days  > 0
      local
         abs_day, millsec: INTEGER
      do
         abs_day := absolute_days - d.absolute_days
         millsec := milliseconds - d.milliseconds
         -- adjust carry
         if millsec < 0 then
            millsec := millsec + tt.milliseconds_in_day
            abs_day := abs_day - 1
         end
         if millsec >= tt.milliseconds_in_day then
            millsec := millsec - tt.milliseconds_in_day
            abs_day := abs_day + 1
         end
         !!Result.set_from_compressed (abs_day, millsec)
      end

   difference (t: like Current): DURATION is
      local
         dy, me: INTEGER
      do
         dy := absolute_days - t.absolute_days
         me := milliseconds - t.milliseconds
         if me < 0 then
            me := me + tt.milliseconds_in_day
            dy := dy - 1
         end
         !!Result.set_from_compressed (dy, me)
      end

feature -- Output

   out: STRING is
         -- New string containing printable represantation
         -- Format is YYYY-MM-DD HH:MM:SS[.F]
      do
         Result := Precursor {DATE}
         Result.append_character (' ')
         Result.append_string (Precursor {TIME})
      end

feature {NONE} -- Implementation

   TIME_DATE_now (local_value: BOOLEAN; yy, mm, dd, hr, mn, sc, ms: POINTER) is
         -- Set the values to the current date/time:
         --    UTC        if `local_value = False'
         --    local time if `local_value = True'
      require
         yy /= Void
         mm /= Void
         dd /= Void
         hr /= Void
         mn /= Void
         sc /= Void
         ms /= Void
      external "C"
         alias "TIME_DATE_now"
      end

end