indexing

   title:       "Advanced string processing";
   cluster:     "gargs";
   project:     "Visual Eiffel Library";
   date:        "$Date: 2005/02/01 10:21:57 $";
   revision:    "$Revision: 1.1 $";
   copyright:   "Copyright (C) 1996-2005 Object Tools Group";
   license:     "http://visual-eiffel.com/license";
   disclaimer:  "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

class ADVANCED_STRING

feature -- Access

   chars_index (str1, str2: STRING; start: INTEGER): INTEGER is
         -- Position of first occurrence of any character from 'str2' in 'str1'
         -- at or after 'start' from 'str1'.
         -- Returns 0 if the two string arguments have no characters in common.
      require
         valid_str1: not_empty (str1)
         valid_str2: not_empty (str2)
         valid_start: start > 0 and start <= str1.count
      local
         i, cnt: INTEGER
      do
         from
            i := start
            cnt := str1.count
         until
            i > cnt
         loop
            if str2.has (str1.item (i)) then
               Result := i
               i := cnt
            end
            i := i + 1
         end
      end

   last_index (s: STRING; c: CHARACTER): INTEGER is
         -- Position of last occurence of 'c' in string 's'; 0 if none.
      require
         non_void_string: s /= Void
      local
         i: INTEGER
      do
         from
            i := s.count
         until
            i = 0
         loop
            if s.item (i) = c then
               Result := i
               i := 0
            else
               i := i - 1
            end
         end
      end

   sign_index (input: STRING; c: CHARACTER; start: INTEGER): INTEGER is
         -- Position of first occurrence of 'c' in 'input' at or after 'start'
         -- (but not within the string "..." !); 0 if none.
      require
         valid_input: not_empty (input)
         valid_start: start > 0 and start <= input.count
      local
         within_str: BOOLEAN
         i, cnt: INTEGER
      do
         from
            i := start
            cnt := input.count
         until
            i > cnt
         loop
            if input.item (i) = '"' then
               if within_str then
                  within_str := False
               else
                  within_str := True
               end
            elseif c = input.item (i) and then not within_str then
               Result := i
               i := cnt
            end
            i := i + 1
         end
      ensure
         non_negative_result: Result >= 0
      end

feature -- Conversion

   to_tokens (input, delimiters: STRING): ARRAY [STRING] is
         -- Converts the 'input' to array of tokens; Void if none.
         -- It examines any character from 'delimiters' as token delimiters.
      require
         valid_input: not_empty (input)
         valid_delimiters: not_empty (delimiters)
      local
         token: STRING
         i1, i2: INTEGER
      do
         from
            !!Result.make (1, 0)
            i1 := 1
            i2 := input.count
         until
            i2 = 0
         loop
            i2 := chars_index (input, delimiters, i1)
            if i2 = 0 then
               token := input.substring (i1, input.count)
            elseif i2 > i1 then
               token := input.substring (i1, i2 - 1)
            else -- if i2 = i1 then
               token := Void
            end

            if token /= Void then
               Result.force (token, Result.count + 1)
            end

            if i2 = input.count then
               i2 := 0
            else
               i1 := i2 + 1
            end
         end
      end

feature -- Validations

   not_empty (s: STRING): BOOLEAN is
         -- Is 's' non Void and not empty ?
      do
         Result := s /= Void and then not s.is_empty
      end

end
