indexing

   title:       "Processes command line";
   cluster:     "gargs";
   project:     "Visual Eiffel Library";
   date:        "$Date: 2005/02/01 10:21:57 $";
   revision:    "$Revision: 1.1 $";
   copyright:   "Copyright (C) 1996-2005 Object Tools Group";
   license:     "http://visual-eiffel.com/license";
   disclaimer:  "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

class COMMAND_LINE

feature -- Attributes

   has_error: BOOLEAN
   last_error: ARG_ERROR

   check_simple_sw (
      arg: ADVANCED_ARGUMENTS
      names: ARRAY[STRING]
      case_sens: ARRAY [BOOLEAN]
   ): BOOLEAN is
   local
      arr: ARRAY[STRING]
      str: STRING
      i, n: INTEGER
   do
      Result := True
      arr := arg.simple_switches
      if arr /= Void then
         from
            i := 1
            n := arr.count
         until
            i > n
         loop
            str := arr.item(i)
            if find(str, names, case_sens) = 0 then
               Result := False
               !!last_error.make (arge.unknown_switch_code, str, arge.simple, 0)
               has_error := True
            end
            i := i + 1
         end
      end
   end

   check_list_sw (
      arg: ADVANCED_ARGUMENTS
      names: ARRAY[STRING]
      case_sens: ARRAY [BOOLEAN]
   ): BOOLEAN is
   local
      arr: ARRAY[STRING]
      str: STRING
      i, n, num: INTEGER
   do
      Result := True

      arr := arg.list_switches

      if arr /= Void then
         from
            i := 1
            n := arr.count
         until
            i > n
         loop
            str := arr.item(i)
            num := find(str, names, case_sens)
            if num = 0 then
               Result := False
               !!last_error.make (arge.unknown_switch_code, str, arge.list, 0)
               has_error := True
            end
            i := i + 1
         end
      end
   end

   find (s: STRING; arr: ARRAY[STRING]; case_sens: ARRAY [BOOLEAN]): INTEGER is
   local
      i, n: INTEGER
      str, st: STRING
   do
      from
         i := 1
         n := arr.count
      until
         i > n
      loop
         if not case_sens.item (i) then
            st := clone(arr.item(i))
            st.to_lower
            str := clone(s)
            str.to_lower
         else
            st := arr.item(i)
            str := s
         end
         if str.is_equal(st) then
            Result := i
            i := n
         end
         i := i + 1
      end
   end

   value (
      arr: ARRAY [STRING]
      pos_val: ARRAY [STRING]
   ): ARRAY [STRING] is
   local
      tst: ARG_TESTER [STRING]
   do
      !!tst
      Result := arr
      if pos_val /= Void and then not tst.test (Result, pos_val) then
         Result := Void
         !!last_error.make (arge.invalid_value_code, Void, Void, 0)
         has_error := True
      end
   end

   int_value (
      arr: ARRAY [STRING]
      pos_val: ARRAY [INTEGER]
   ): ARRAY [INTEGER] is
   local
      i, n: INTEGER
      tst: ARG_TESTER [INTEGER]
      was_error: BOOLEAN
   do
      if not was_error then
         !!tst
         from
            i := 1
            n := arr.count
            !!Result.make(1, n)
         until
            i > n
         loop
            Result.put(arr.item(i).to_integer, i)
            i := i + 1
         end
         if pos_val /= Void and then not tst.test (Result, pos_val) then
            Result := Void
            !!last_error.make (arge.invalid_value_code, Void, Void, 0)
            has_error := True
         end
      end
   rescue
      Result := Void
      if not was_error then
         !!last_error.make (arge.invalid_parameter_code, arr.item(i), arge.list, i)
         has_error := True
         was_error := True
      end
      retry
   end

   real_value (
      arr: ARRAY [STRING]
      pos_val: ARRAY [REAL]
   ): ARRAY [REAL] is
   local
      i, n: INTEGER
      was_error: BOOLEAN
      tst: ARG_TESTER [REAL]
   do
      if not was_error then
         !!tst
         from
            i := 1
            n := arr.count
            !!Result.make(1, n)
         until
            i > n
         loop
            Result.put(arr.item(i).to_real, i)
            i := i + 1
         end
         if pos_val /= Void and then not tst.test (Result, pos_val) then
            Result := Void
            !!last_error.make (arge.invalid_value_code, Void, Void, 0)
            has_error := True
         end
      end
   rescue
      Result := Void
      if not was_error then
         !!last_error.make (arge.invalid_parameter_code, arr.item(i), arge.list, i)
         has_error := True
         was_error := True
      end
      retry
   end

   char_value (
      arr: ARRAY [STRING]
      pos_val: ARRAY [CHARACTER]
   ): ARRAY [CHARACTER] is
   local
      i, n: INTEGER
      str: STRING
      tst: ARG_TESTER [CHARACTER]
   do
      !!tst
      from
         i := 1
         n := arr.count
         !!Result.make(1, n)
      until
         i > n
      loop
         str := arr.item(i)
         if str = Void or else str.count /= 1 then
            Result := Void
            !!last_error.make (arge.invalid_parameter_code, arr.item(i), arge.list, i)
            has_error := True
            i := n
         end
         Result.put(str.item(1), i)
         i := i + 1
      end
      if pos_val /= Void and then not tst.test (Result, pos_val) then
         Result := Void
         !!last_error.make (arge.invalid_value_code, Void, Void, 0)
         has_error := True
      end
   end

   once_value (
      arr: ARRAY [STRING]
      pos_val: ARRAY [STRING]
      arg_name: STRING
   ): STRING is
   local
      tst: ARG_TESTER [STRING]
   do
      !!tst
      if singleton (arr) then
         Result := arr.item(1)
         if pos_val /= Void and then not tst.test_once (Result, pos_val) then
            Result := Void
            !!last_error.make (arge.invalid_value_code, arg_name, arge.once_str, 0)
            has_error := True
         end
      else
         !!last_error.make (arge.invalid_param_cnt_code, arg_name, arge.once_str, 0)
         has_error := True
      end
   end

   once_int_value (
      arr: ARRAY [STRING]
      pos_val: ARRAY [INTEGER]
      arg_name: STRING
   ): INTEGER is
   local
      was_error: BOOLEAN
      tst: ARG_TESTER [INTEGER]
   do
      if not was_error then
         !!tst
         if singleton (arr) then
            Result := arr.item (1).to_integer
            if pos_val /= Void and then not tst.test_once (Result, pos_val) then
               Result := Result.default
               !!last_error.make (arge.invalid_value_code, arg_name, arge.once_str, 0)
               has_error := True
            end
         else
            !!last_error.make (arge.invalid_param_cnt_code, arg_name, arge.once_str, 0)
            has_error := True
         end
      end
   rescue
      Result := 0
      if not was_error then
         !!last_error.make (arge.invalid_parameter_code, arr.item(1), arge.once_str, 1)
         has_error := True
         was_error := True
      end
      retry
   end

   once_char_value (
      arr: ARRAY [STRING]
      pos_val: ARRAY [CHARACTER]
      arg_name: STRING
   ): CHARACTER is
   local
      tst: ARG_TESTER [CHARACTER]
   do
      !!tst
      if singleton (arr) then
         if arr.item (1).count /= 1 then
            !!last_error.make (arge.invalid_parameter_code, arr.item(1), arge.once_str, 1)
            has_error := True
         else
            Result := arr.item (1).item (1)
            if pos_val /= Void and then not tst.test_once (Result, pos_val) then
               Result := Result.default
               !!last_error.make (arge.invalid_value_code, arg_name, arge.once_str, 0)
               has_error := True
            end
         end
      else
         !!last_error.make (arge.invalid_param_cnt_code, arg_name, arge.once_str, 0)
         has_error := True
      end
   end

   once_real_value (
      arr: ARRAY [STRING]
      pos_val: ARRAY [REAL]
      arg_name: STRING
   ): REAL is
   local
      was_error: BOOLEAN
      tst: ARG_TESTER [REAL]
   do
      if not was_error then
         !!tst
         if singleton (arr) then
            Result := arr.item (1).to_real
            if pos_val /= Void and then not tst.test_once (Result, pos_val) then
               Result := Result.default
               !!last_error.make (arge.invalid_value_code, arg_name, arge.once_str, 0)
               has_error := True
            end
         else
            !!last_error.make (arge.invalid_param_cnt_code, arg_name, arge.once_str, 0)
            has_error := True
         end
      end
   rescue
      Result := 0.0
      if not was_error then
         !!last_error.make (arge.invalid_parameter_code, arr.item(1), arge.once_str, 1)
         has_error := True
         was_error := True
      end
      retry
   end

   set_opt (
      arg: ADVANCED_ARGUMENTS
      case_sens: BOOLEAN
      expand_wildc: BOOLEAN
      recursive_find: BOOLEAN
      include_directories: BOOLEAN
      name_with_directory: BOOLEAN
      file_access: BOOLEAN
   ) is
   do
      arg.case_sensitive          (case_sens     )
      arg.wildcards               (expand_wildc   )
      arg.recursive_find          (recursive_find     )
      arg.include_directories     (include_directories)
      arg.set_name_with_directory (name_with_directory)
      arg.file_access             (file_access)
   end

   arge: ARG_ERROR is
   once
      !!Result.make (0, Void, Void, 0)
   end

feature {NONE} -- Implementation

   singleton (array: ARRAY [STRING]): BOOLEAN is
      -- Returns True iff array is not Void and contains exactly one element
      -- or all elements are the same
   local
      i: INTEGER
      p: STRING
      c: STRING
   do
      if array /= Void then
         i := array.count
         if i > 0 then
            Result := True
            if i > 1 then
               from
                  p := array.item (i)
               variant
                  i
               until
                  i <= 1
               loop
                  i := i - 1
                  c := array.item (i)
                  if not equal (c, p) then
                     Result := False
                     i := 0
                  end
                  p := c
               end
            end
         end
      end
   ensure
      definition: Result = (array /= Void and then array.count = 1)
   end

end


class ARG_TESTER [G]

feature

   test (values: ARRAY[G]; pos_val: ARRAY[G]): BOOLEAN is
   local
      i, n: INTEGER
   do
      Result := True
      from
         i := 1
         n := values.count
      until
         i > n
      loop
         if not test_once (values.item(i), pos_val) then
            Result := False
            i := n
         end
         i := i + 1
      end
   end

   test_once (val: G; pos_val: ARRAY[G]): BOOLEAN is
   local
      j, m: INTEGER
   do
      from
         j := 1
         m := pos_val.count
      until
         j > m
      loop
         if val.is_equal(pos_val.item(j)) then
            Result := True
         end
         j := j + 1
      end
   end

end


class ARG_ERROR

inherit

   ANY
      redefine
         out
      end

creation

   make

feature

   code: INTEGER
   name: STRING
   type: STRING
   param: INTEGER

   make (c: INTEGER; n, t: STRING; p: INTEGER) is
   do
      code := c
      name := n
      type := t
      param:= p
   end

   unknown_switch_code,
   invalid_value_code,
   invalid_parameter_code,
   invalid_param_cnt_code: INTEGER is unique

   list     : STRING is once Result := "LIST"   end
   simple   : STRING is once Result := "SIMPLE" end
   once_str : STRING is once Result := "ONCE"   end

feature -- Output

   out: STRING is
      do
         inspect code
         when unknown_switch_code then
            Result := "Unknown option: %"" + name + "%""
         when invalid_value_code then
            Result := "Invalid value in option"
            if name /= Void then
               Result.append_string (" %"" + name + "%"")
            end
         when invalid_parameter_code then
            Result := "Invalid parameter #" + param.out + " in option %"" + "name" + "%""
         when invalid_param_cnt_code then
            Result := "Invalid number of values in option %"" + name + "%""
         else
            Result := "Unknown error in options"
         end
      end

end
