indexing

   title:       "Reads the file data";
   cluster:     "gargs";
   project:     "Visual Eiffel Library";
   date:        "$Date: 2005/02/01 10:21:57 $";
   revision:    "$Revision: 1.1 $";
   copyright:   "Copyright (C) 1996-2005 Object Tools Group";
   license:     "http://visual-eiffel.com/license";
   disclaimer:  "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

class FILE_CMDLN

creation

   make

feature -- Initialization

   make (file: STRING) is
         -- Initialize from the characters of 'file'.
         -- Note1: The 'file' must pass without its prefix symbol (@).
         -- Note2: The recursive parsing of arguments files not supported.
      require
         valid_file_name : file /= Void and then not file.is_empty
      do
         file_name := file;
      end

feature -- Access

   to_string: STRING is
         -- Converts the file data to single string; Void if none.
      local
         text: TEXTFILE;
         i, n: INTEGER;
         err: BOOLEAN
      do
         if not err then
            !!text.make_open_read(file_name);
            Result := "";
            if text /= Void then
               from
                  i := 1;
                  n := text.count
               until
                  i > n
               loop
                  Result.append_string(text.item(i));
                  Result.append_character(' ');
                  i := i + 1
               end; -- loop
            end; -- if
         end; -- if
      rescue
         err := True;
         Result := Void;
         retry
      end

   to_tokens: ARRAY [STRING] is
         -- Converts the file data to array of tokens; Void if none.
      local
         input    : STRING;
         adv_str  : ADVANCED_STRING;
      do
         input := to_string;
         if input /= Void and then not input.is_empty then
            !!adv_str;
            Result := adv_str.to_tokens (input, token_delimiters);
         end;
      end

feature {NONE} -- Implementation

   file_name: STRING

   token_delimiters: STRING is
        -- The file token delimiters.
      once
         Result := " %T%R%N";
      end

end -- class FILE_CMDLN
