indexing

   title:       "To provide support for accessing the individual components",
                "of an arbitrary path name.";
   copyright:   "(c) Object Tools, 1997-2003";
   version:     "$Revision: 1.1 $";
   last_change: "$Date: 2005/02/01 10:21:57 $";
   done_at:     "Object Tools (www.object-tools.com)";

class FILE_NAME

creation

   make,
   make_path

feature -- Initialization

   make (path: STRING) is
         -- Initialize from the characters of 'path'.
      do
         full_path := path
      end

   make_path (path_drive, path_dir, path_fname, path_extension: STRING) is
         -- Creates a single path and stores it in 'full_path'.
      do
         if path_drive /= Void or else
            path_dir /= Void or else
            path_fname /= Void or else
            path_extension /= Void
         then
            if full_path = Void then
               !!full_path.make_empty
            else
               full_path.resize (0)
            end
            if path_drive /= Void then
               full_path.append_string (path_drive)
            end
            if path_dir /= Void then
               full_path.append_string (path_dir)
            end
            if path_fname /= Void then
               full_path.append_string (path_fname)
            end
            if path_extension /= Void then
               full_path.append_string (path_extension)
            end
         else
            full_path := Void
         end
      end

feature -- Status Report

   has_drive: BOOLEAN is
         -- Does 'full_path' include 'drive' ?
      do
         Result := not_empty (drive)
      end

   has_directory: BOOLEAN is
         -- Does 'full_path' include 'directory' ?
      do
         Result := not_empty (directory)
      end

   has_filename: BOOLEAN is
         -- Does 'full_path' include 'filename' ?
      do
         Result := not_empty (filename)
      end

   has_extension: BOOLEAN is
         -- Does 'full_path' include 'extension' ?
      do
         Result := not_empty (extension)
      end

   has_wildcards: BOOLEAN is
         -- Does 'full_path' include wildcards ?
      local
         li, cnt: INTEGER
      do
         if not_empty (full_path) then
            from
               cnt := full_path.count
            until
               li = cnt
            loop
               li := li + 1
               if wildcards_list.has (full_path.item (li)) then
                  Result := True
                  li := cnt
               end
            end
         end
      end

feature -- Access

   full_path: STRING
         -- Full path.

   drive: STRING is
         -- Drive component; Void if none.
      do
         if full_path /= Void and then
            full_path.count >= MAX_DRIVE and then
            full_path.item (MAX_DRIVE) = drive_postfix
         then
            Result := full_path.substring (1, MAX_DRIVE)
         end
      end

   directory: STRING is
         -- Directory component; Void if none.
      local
         start, last_slash, li, cnt: INTEGER
      do
         if not_empty (full_path) then
            if not_empty (drive) then
               start := drive.count
            end

            from
               start := start + 1
               li := start
               cnt := full_path.count
            until
               li > cnt
            loop
               if dir_delimiters_list.has (full_path.item (li)) then
                  last_slash := li
               end
               li := li + 1
            end

            if last_slash /= 0 then
               Result := full_path.substring (start, last_slash)
            end
         end
      end

   filename: STRING is
         -- Filename component; Void if none.
      local
         start, last_dot, li, cnt: INTEGER
      do
         if not_empty (full_path) then
            if not_empty (drive) then
               start := drive.count
            end
            if not_empty (directory) then
               start := start + directory.count
            end

            from
               start := start + 1
               li := start
               cnt := full_path.count
            until
               li > cnt
            loop
               if full_path.item (li) = extension_prefix then
                  last_dot := li
               end
               li := li + 1
            end

            if start <= cnt then
               if last_dot = 0 then
                  Result := full_path.substring (start, cnt)
               elseif last_dot /= start then
                  Result := full_path.substring (start, last_dot - 1)
               end
            end
         end
      end

   extension: STRING is
         -- Extension component; Void if none.
      local
         start, cnt: INTEGER
      do
         if not_empty (full_path) then
            if not_empty (drive) then
               start := drive.count
            end
            if not_empty (directory) then
               start := start + directory.count
            end
            if not_empty (filename) then
               start := start + filename.count
            end
            start := start + 1

            cnt := full_path.count
            if start <= cnt then
               Result := full_path.substring (start, cnt)
            end
         end
      end

   drive_dir: STRING is
         -- Drive and Directory; Void if none
      local
         dir: STRING
      do
         Result := clone (drive)
         dir := directory
         if Result = Void then
            Result := clone (dir)
         elseif dir /= Void then
            Result.append_string (dir)
         end
      end

   filename_ext: STRING is
         -- Filename and Extension; Void if none
      local
         ext: STRING
      do
         Result := clone (filename)
         ext := extension
         if Result = Void then
            Result := clone (ext)
         elseif ext /= Void then
            Result.append_string (ext)
         end
      end

feature {NONE} -- Validations

   not_empty (s: STRING): BOOLEAN is
      do
         Result := s /= Void and then not s.is_empty
      end

   MAX_DRIVE: INTEGER is 2; -- Maximum length of drive component
   drive_postfix: CHARACTER is ':'
   extension_prefix: CHARACTER is '.'

   dir_delimiters_list: STRING is
      once
         Result := "\/";
      end

   wildcards_list: STRING is
      once
         Result := "*?";
      end

end
