indexing

   title:       "MESSAGE";
   cluster:     "gmsgs";
   project:     "Visual Eiffel Library";
   date:        "$Date: 2005/02/15 16:22:42 $";
   revision:    "$Revision: 1.2 $";
   copyright:   "Copyright (C) 1996-2005 Object Tools Group";
   license:     "http://visual-eiffel.com/license";
   disclaimer:  "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

class MESSAGE

inherit

   COMPARABLE
      redefine
         out
      end

creation

   make

feature {NONE} -- Creation

   make (n: INTEGER; t: ARRAY [ANY]; resource: MESSAGE_HANDLER) is
      do
         id := n
         text := t
         handler := resource
      ensure
         id = n
         text = t
         handler = resource
      end

feature -- Access

   id: INTEGER
         -- Message identifier

   type: CHARACTER is
      do
         Result := handler.message_type (id)
      end

   message_name: STRING is
      do
         Result := handler.message_name (id)
      end

   message_code: STRING is
      do
         Result := handler.message_code (id)
      end

   is_error: BOOLEAN is
         -- Is it an error message?
      do
         -- False by default
      end

   infix "<" (other: like Current): BOOLEAN is
      local
         i: INTEGER
         exit: BOOLEAN
      do
         if id = other.id then
            if text = Void then
               Result := other.text /= Void
            elseif other.text /= Void then
               from
               until
                  exit
               loop
                  i := i + 1
                  if i >= text.count then
                     Result := i < text.count
                     exit := true
                  elseif i >= other.text.count then
                     exit := true
                  elseif text.item (i) = Void then
                     if other.text.item (i) /= Void then
                        Result := true
                        exit := true
                     end
                  elseif other.text.item (i) = Void then
                     exit := true
                  else
                     inspect
                        text.item (i).out.three_way_comparison
                        (other.text.item (i).out)
                     when -1 then
                        Result := true
                        exit := true
                     when 0 then
                     when 1 then
                        exit := true
                     end
                  end
               end
            end
         else
            Result := id < other.id
         end
      end

   to_add: BOOLEAN is
      do
         Result := True
      end

   out: STRING is
      do
         Result := ""
         if handler.is_valid_message_id (id) then
            Result.append_string (message_name)
            Result.append_character (' ')
            Result.append_string (message_code)
            Result.append_character (':')
            Result.append_character (' ')
         end
         Result.append_string (msg)
      end

   msg: STRING is
      do
         Result := handler.message_text (id, text, append_param)
      ensure
         Result /= Void
      end

feature {MESSAGE} -- Implementation

   text: ARRAY [ANY]

feature {NONE}

   append_param: BOOLEAN

   handler: MESSAGE_HANDLER
         -- Message resource

end
