indexing

   title:       "MESSAGE_HANDLER";
   cluster:     "gmsgs";
   project:     "Visual Eiffel Library";
   date:        "$Date: 2005/02/15 16:22:42 $";
   revision:    "$Revision: 1.2 $";
   copyright:   "Copyright (C) 1996-2005 Object Tools Group";
   license:     "http://visual-eiffel.com/license";
   disclaimer:  "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

class MESSAGE_HANDLER

inherit

   ABSTRACT_MESSAGE_CODES
      end

create

   make_open

feature {NONE} -- Creation

   make_open (message_file_name: STRING; message_codes: GENERATED_MESSAGE_CODES) is
      require
         message_file_name /= Void
         message_codes /= Void
      local
         err: BOOLEAN
         tf: TEXTFILE
      do
         file_name := message_file_name
         codes := message_codes
         error_code := No_error
         text := Void
         if err then
            error_code := Can_not_open_file
         else
            create tf.make_open_read(file_name)
            if not tf.is_open then
               tf := Void
               error_code := Can_not_open_file
            else
               if not load (tf) and then error_code = No_error then
                  error_code := Invalid_file_format
               end
               tf.close
            end
         end
      rescue
         err := True
         retry
      end

feature -- Status report

   has_error: BOOLEAN is
         -- Has error been encountered?
      do
         Result := error_code /= No_error
      end

   error: STRING is
         -- Error text
      require
         has_error
      do
         inspect error_code
         when Can_not_open_file then
            Result := "Cannot read message file %"" + file_name + "%"."
         when Invalid_str_count then
            Result := "Message file %"" + file_name + "%" is not actual."
         when Invalid_file_format then
            Result := "Message file %"" + file_name + "%" is in invalid format."
         end
      ensure
         Result /= Void
      end

   file_name: STRING
         -- Message file name

feature {NONE} -- Error codes

   error_code: INTEGER

   No_error: INTEGER is 1
   Can_not_open_file: INTEGER is 2
   Invalid_str_count: INTEGER is 3
   Invalid_file_format: INTEGER is 4

feature {NONE} -- Initialization

   load (tf: TEXTFILE): BOOLEAN is
      require
         codes /= Void
      local
         i, n,
         j, cnt,
         k, l: INTEGER
         exit: BOOLEAN
         arr: ARRAY [STRING]
         str1,
         str2: STRING
         c: CHARACTER
      do
         arr := codes.codes
         text := Void
         if arr.count /= tf.count then
            Result := False
            error_code := Invalid_str_count
            if arr.count > tf.count then
               n := tf.count
            else
               n := arr.count
            end
         else
            Result := True
            n := tf.count
         end

         from
            i := 1
            !!text.make (1, n)
         until
            i > n
         loop
            str1 := tf.item (i)
            if str1.item (1) = '{' then
               from
                  j := 2
                  cnt := str1.count
               until
                  j > cnt or else str1.item (j) = '}'
               loop
                  j := j + 1
               end
               if j > cnt then
                  Result := False
               else
                  if j > 2 then
                     str2 := str1.substring (2, j - 1)
                  else
                     str2 := ""
                  end
                  if arr.item (i).is_equal (str2) then
                     from
                        k := j + 1
                     until
                        k > cnt or else str1.item (k) = '%"'
                     loop
                        k := k + 1
                     end
                     k := k + 1
                     if k < cnt then
                        from
                           str2 := ""
                           l := k
                           exit := False
                        until
                           k > cnt or else exit
                        loop
                           inspect
                              str1.item (k)
                           when '%%' then
                              if k < cnt then
                                 c := str1.item (k + 1)
                                 if c = '%"' then
                                    if l < k then
                                       str2.append_string (str1.substring(l, k-1))
                                    end
                                    k := k + 1
                                    l := k
                                 end
                              else
                                 -- new line case
                              end
                              k := k + 1
                           when '%"' then

                              exit := True
                           else
                              k := k + 1
                           end

                        end

                        if k <= cnt then
                           if l < k then
                              str2.append_string (str1.substring(l, k-1))
                           else
                           end
                           text.put (str2, i)
                        else
                           Result := False
                        end
                     elseif k = l and then str1.item(k) = '%"' then
                        str2 := "";
                        text.put (str2, i)
                     else
                        Result := False
                     end
                  else
                     Result := False
                  end
               end
            else
               Result := False
            end
            i := i + 1
         end
      end

   close is
      do
         text := Void
      end

feature {MESSAGE} -- Access

   is_valid_message_id (id: INTEGER): BOOLEAN is
      do
         Result := codes.is_valid_message_id (id)
      end

   message_type (id: INTEGER): CHARACTER is
      do
         Result := codes.message_type (id)
      end

   message_code (id: INTEGER): STRING is
      do
         Result := codes.message_code (id)
      end

   message_name (id: INTEGER): STRING is
      do
         Result := codes.message_name (id)
      end

   message_text (msg: INTEGER; txt: ARRAY [ANY]; append: BOOLEAN): STRING is
         -- Textual representation of the given message
      do
         if text /= Void then
            if msg >= 1 and then msg <= text.count then
               Result := text.item (msg)
            end
         end
         if Result = Void then
            Result := prepare_message_string (empty_string, txt, True)
         else
            Result := prepare_message_string (Result, txt, append)
         end
      ensure
         Result /= Void
      end

feature {NONE}

   prepare_message_string(
      msg: STRING; txt: ARRAY [ANY]; append: BOOLEAN
   ): STRING is
      require
         not_void_message_string: msg /= Void
      local
         i: INTEGER
         j: INTEGER
         num: INTEGER
         txt_count: INTEGER
         value: ANY
         escape: BOOLEAN
         s: STRING
         ch: CHARACTER
      do
         Result := clone (msg)
         if txt /= Void then
            txt_count := txt.count
         end
         from
         until
            i >= Result.count
         loop
            i := i + 1
            ch := Result.item (i)
            if escape then
               escape := False
               inspect ch
               when 'A' then ch := '%A'
               when 'B' then ch := '%B'
               when 'C' then ch := '%C'
               when 'D' then ch := '%D'
               when 'F' then ch := '%F'
               when 'H' then ch := '%H'
               when 'L' then ch := '%L'
               when 'N' then ch := '%N'
               when 'Q' then ch := '%Q'
               when 'R' then ch := '%R'
               when 'S' then ch := '%S'
               when 'T' then ch := '%T'
               when 'U' then ch := '%U'
               when 'V' then ch := '%V'
               when '(' then ch := '%('
               when ')' then ch := '%)'
               when '<' then ch := '%<'
               when '>' then ch := '%>'
               when '1' .. '9' then
                  from
                     j := i + 1
                  until
                     j > Result.count or else '0' > Result.item (j) or else Result.item (j) > '9'
                  loop
                     j := j + 1
                  end
                  j := j - 1
                  s := empty_string
                  num := Result.substring (i, j).to_integer
                  if 1 <= num and then num <= txt_count then
                     value := txt.item (num)
                     if value /= Void then
                        s := value.out
                     end
                  end
                  Result.replace_substring (s, i, j)
                  ch := '1'
                  i := i * 2 - j + s.count - 1
               else
               end
               if ch /= '1' then
                  Result.put (ch, i)
               end
            elseif ch = '%%' then
               Result.remove (i)
               escape := True
               i := i - 1
            end
         end
         if append and then txt_count > 0 then
            from
               i := 0
            until
               i = txt_count
            loop
               i := i + 1
               value := txt.item(i)
               if value /= Void then
                  Result.append_string (value.out)
               end
            end
         end
      ensure
         Result /= Void
      end

   empty_string: STRING is
      once
         Result := ""
      end

   text: ARRAY [STRING]

   codes: GENERATED_MESSAGE_CODES
         -- Codes of the associated messages

invariant

   valid_error_code:
      error_code = No_error or
      error_code = Can_not_open_file or
      error_code = Invalid_str_count or
      error_code = Invalid_file_format

end
