indexing

   title:       "Support for project errors handling";
   cluster:     "gmsgs";
   project:     "Visual Eiffel Library";
   date:        "$Date: 2005/02/15 16:22:42 $";
   revision:    "$Revision: 1.2 $";
   copyright:   "Copyright (C) 1996-2005 Object Tools Group";
   license:     "http://visual-eiffel.com/license";
   disclaimer:  "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

class PROJECT_MESSAGE [G->MESSAGE]

creation

   make

feature

   make (file_name: STRING; message_codes: GENERATED_MESSAGE_CODES) is
   require
      file_name /= Void
      message_codes /= Void
   do
      errors_count := 0
      create messages.make(1, 0)
      create disabled_messages.make
      create disabled_types.make
      handler := create_handler (file_name, message_codes)
   ensure
      messages /= Void
   end

   create_handler (file_name: STRING; message_codes: GENERATED_MESSAGE_CODES): MESSAGE_HANDLER is
         -- Create message handler
      do
         create Result.make_open (file_name, message_codes)
         if Result.has_error then
            report_handler_error (Result)
         end
      end

   reset is
      do
         errors_count := 0
         !!messages.make(1, 0)
      end

   display (header, footer: STRING) is
   local
      i, n: INTEGER
      m: G
   do
      if header /= Void then
         print (header)
         print ("%N");
      end
      from
         i := 1
         n := messages.count
      until
         i > n
      loop
         m := messages.item(i)
         if not_ignore(m) then
            display_message (m)
         end
         print ("%N");
         i := i + 1
      end
      if footer /= Void then
         print (footer)
         print ("%N");
      end
   end

   display_message (m: G) is
   do
      print (m)
   end

   has_messages_for_print: BOOLEAN is
   local
      i, n: INTEGER
      m: G
   do
      from
         i := 1
         n := messages.count
      until
         i > n
      loop
         m := messages.item(i)
         if not_ignore(m) then
            Result := True
            i := n
         end
         i := i + 1
      end
   end

   add (m: G) is
   do
      if not_ignore (m) then
         if m.to_add then
            add_msg (m)
            if echo_on then
               print ("%N");
               display_message (m)
               print ("%N");
            end
            if m.is_error then
               errors_count := errors_count + 1
               if max_error_count > 0 and then
                  errors_count >= max_error_count
               then
                  stop_after_n_errors
               end
            end
         else
            print (m)
         end
      end
   end

   message (id: INTEGER; text: ARRAY [ANY]) is
   local
      m: MESSAGE
   do
      create m.make (id, text, handler)
      add (m)
   end

   messages_count: INTEGER is
   do
      Result := messages.count
   end

   enable (message_id: INTEGER) is
   do
      disabled_messages.remove(message_id)
   end

   disable (message_id: INTEGER) is
   do
      disabled_messages.add(message_id)
   end

   disable_type (type: CHARACTER) is
   do
      if type /= 'f' then
         disabled_types.add(type.code)
      end
   end

   enable_type (type: CHARACTER) is
   do
      disabled_types.remove(type.code)
   end

   set_echo (on: BOOLEAN) is
   do
      echo_on := on
   end

   set_max_error_count (cnt: INTEGER) is
   do
      max_error_count := cnt
   end

   stop_after_n_errors is
   do
   end

   messages: ARRAY [G]
   errors_count: INTEGER

   handler: MESSAGE_HANDLER
         -- Message file loader

feature {NONE}

   disabled_messages: SORTED_ARRAY [INTEGER]
   disabled_types: SORTED_ARRAY [INTEGER]
   echo_on: BOOLEAN
   max_error_count: INTEGER

   not_ignore (m: G): BOOLEAN is
   do
      Result := (disabled_messages.seek(m.id) <= 0) and then
                (disabled_types.seek(m.type.code) <= 0)
   end

   add_msg (m: G) is
   do
      messages.force(m, messages.count + 1)
   end

   report_handler_error (message_handler: MESSAGE_HANDLER) is
         -- Report error in message handler.
      require
         has_error: message_handler.has_error
      local
         m: MESSAGE
      do
         create {WARNING} m.make (0, <<message_handler.error>>, message_handler)
         add (m)
      end

invariant

   not_void_messages: messages /= Void

end
