indexing

   title:       "Extended output";
   cluster:     "gmsgs";
   project:     "Visual Eiffel Library";
   date:        "$Date: 2005/02/15 16:22:42 $";
   revision:    "$Revision: 1.2 $";
   copyright:   "Copyright (C) 1996-2005 Object Tools Group";
   license:     "http://visual-eiffel.com/license";
   disclaimer:  "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

class PROJECT_OUTPUT

creation

   make

feature

   make (fname: STRING; device: CHARACTER) is
      require
         valid_device: (device = 's') or (device = 'f') or (device = 'b') or (device = 'n')
         not_void_file_name: (device = 'f' or device = 'b') implies (fname /= Void)
      local
         was_error,
         name_flg ,
         old2file ,
         new2file: BOOLEAN
      do
         if was_error then
            target        := screen
            was_file_used := False
         else
            separator := default_separator
            if different(file_name,fname) then
               name_flg := TRUE
            end
            if is_to_file then
               old2file := TRUE
            end
            target := device
            if is_to_file then
               new2file := TRUE
            end

            if name_flg and then new2file and old2file then
               close
            end

            if new2file then
               file_name := fname
               if name_flg then
                  !!output.make (file_name)
               end
               was_file_used := True
            else
               was_file_used := False
            end
         end
      rescue
         if not was_error then
            was_error := True
            retry
         end
      end

   close is
      do
         if was_file_used then
            was_file_used := False
            target        := screen
            file_name     := Void
            output.close
         end
      end

   put (obj: ANY) is
      do
         put_string (to_string (obj))
      end

   put_character (c: CHARACTER) is
      do
         put_string (c.out)
      end

   put_integer (i: INTEGER) is
      do
         put_string (i.out)
      end

   put_real (r: REAL) is
      do
         put_string (r.out)
      end

   put_double (d: DOUBLE) is
      do
         put_string (d.out)
      end

   put_char (c: CHARACTER) is
      obsolete "Use 'put_character' instead"
      do
         put_character (c)
      end

   put_int (i: INTEGER) is
      obsolete "Use 'put_integer' instead"
      do
         put_integer (i)
      end

   put_string (s: STRING) is
      local
         str: STRING
      do
         if s /= Void then
            str := s
         else
            str := void_str
         end
         inspect
            target
         when screen then
            io.put_string (str)
         when file then
            output.put (str)
         when both then
            io.put_string (str)
            output.put (str)
         else
            -- do nothing ...
         end
      end

   to_string (obj: ANY): STRING is
      local
         i, n: INTEGER
         arr: ARRAY [ANY]
      do
         if obj = Void then
            Result := void_str
         else
            arr ?= obj
            if arr = Void then
               Result := obj.out
            else
               Result := "";
               from
                  i := 1
                  n := arr.count
               until
                  i > n
               loop
                  Result.append_string (arr.item(i).out)
                  if i /= n then
                     Result.append_string (separator)
                  end
                  i := i + 1
               end
            end
         end
      end

   xput (arr: ARRAY[ANY]) is
      local
         i, n: INTEGER
      do
         if arr = Void then
            put(void_str)
         else
            from
               i := 1
               n := arr.count
            until
               i > n
            loop
               put(arr.item(i))
               i := i + 1
            end
         end
      end

   putln (obj: ANY) is
      do
         put(obj)
         put_newline
      end

   xputln (arr: ARRAY[ANY]) is
      do
         xput(arr)
         put_newline
      end

   reset_line is
      do
         inspect
            target
         when screen then
            io.put_character('%R')
         when file   then
            output.put_newline
         when both   then
            io.put_character('%R')
            output.put_newline
         else
            -- do nothing ...
         end
      end

   put_newline is
      do
         inspect
            target
         when screen then
            io.put_new_line
         when file   then
            output.put_newline
         when both   then
            io.put_new_line
            output.put_newline
         else
            -- do nothing ...
         end
      end

   last_char: CHARACTER

   get_char is
      do
         io.read_character
         last_char := io.last_character
      end

   is_to_screen: BOOLEAN is
      do
         Result := target = screen or else target = both
      end

   is_to_file: BOOLEAN is
      do
         Result := target = file or else target = both
      end

   set_separator (s: STRING) is
      require
         s /= Void
      do
         separator := s
      end

feature

   screen: CHARACTER is 's' -- all output will be sent to the screen
   file: CHARACTER is 'f' -- all output will be sent to the disk file
   both: CHARACTER is 'b' -- all output will be sent to the screen and in
   -- parallel to the file
   null: CHARACTER is 'n' -- no output.
   separator: STRING

   default_separator: STRING is
      once
         Result := ", "
      end

feature {NONE}

   different (s1, s2: STRING): BOOLEAN is
      do
         if s1 = Void then
            Result := s2 /= Void
         elseif s2 = Void then
            Result := True
         else
            Result := not s1.is_equal (s2)
         end
      end

   target: CHARACTER
   output: XTEXT
   was_file_used: BOOLEAN
   file_name: STRING

   void_str: STRING is "Void"

end