indexing

   title:       "Support for project errors handling";
   cluster:     "gmsgs";
   project:     "Visual Eiffel Library";
   date:        "$Date: 2005/02/15 16:22:42 $";
   revision:    "$Revision: 1.2 $";
   copyright:   "Copyright (C) 1996-2005 Object Tools Group";
   license:     "http://visual-eiffel.com/license";
   disclaimer:  "[
      This program is free software; you can redistribute it and/or modify it 
      under the terms of the GNU General Public License as published by the 
      Free Software Foundation; either version 2 of the License, or (at your 
      option) any later version. This program is distributed in the hope that 
      it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
      See the GNU General Public License for more details. 
      You should have received a copy of the GNU General Public License along 
      with this program; if not, write to the Free Software Foundation, Inc., 
      59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   ]";
   restrictions: "[
      if you want to use this program under a different license type, contact 
      Object Tools http://www.object-tools.com
   ]";

class STANDARD_MESSAGES [G-> MESSAGE]

inherit

   PROJECT_MESSAGE [G]
      redefine
         make, print, display_message
      end

   OUTPUT_SERVER
      redefine
         print
      end

create

   make

feature

   make (file_name: STRING; message_codes: GENERATED_MESSAGE_CODES) is
      do
         width := 75
         Precursor (file_name, message_codes)
      end

   display_message (m: G) is
      local
         str: STRING
      do
         str := m.out
         if width > 40 then
            prepare_message (str , 0, width)
         end
         print (str)
      end

   set_width (w: INTEGER) is
      do
         width := w
      end

   set_output_mode (file_name: STRING; mode: CHARACTER) is
      require
         valid_mode: mode = 's' or mode = 'f' or mode = 'b' or mode = 'n'
         not_void_file_name: (mode = 'f' or mode = 'b') implies (file_name /= Void)
      do
         po.make(file_name, mode)
      end

   error (name: INTEGER; text: ARRAY [ANY]) is
      local
         m: ERROR
      do
         !!m.make (name, text, handler)
         add (m)
      end

   warning (name: INTEGER; text: ARRAY [ANY]) is
      local
         m: WARNING
      do
         !!m.make(name, text, handler)
         add (m)
      end

   info (name: INTEGER; text: ARRAY [ANY]) is
      local
         m: INFORMATION
      do
         !!m.make (name, text, handler)
         print (m)
      end

   print (obj: ANY) is
      do
         po.put (obj)
      end

feature {NONE}

   width: INTEGER

   prepare_message (str: STRING; shift, limit: INTEGER) is
      local
         i, b_marker, e_marker: INTEGER
         str_len: INTEGER
         exit: BOOLEAN
      do
         str_len := str.count
         from
            b_marker := 1
            e_marker := limit - shift
         until
            exit
         loop
            from
               i := b_marker
            until
               i > str_len or else i > e_marker
            loop
               if str.item(i) = '%N' then
                  b_marker := i + 1
                  e_marker := b_marker + limit - 1
               end
               i := i + 1
            end
            if e_marker >= str_len then
               exit := true
            else

               from
                  i := e_marker
               until
                  i <= b_marker or else str.item(i) = ' '
               loop
                  i := i - 1
               end

               if i > b_marker then
                  str.insert_character('%N', i)
                  b_marker := i + 1
               else
                  b_marker := e_marker + 1
                  str.insert_character('%N', e_marker)
               end
               e_marker := b_marker + limit - 1
            end
         end
      end

end
